c------------------------------------------------------------------------------c
c   For the computation of the O((alpha_t+lambda+kappa)^2) corrections         c
c   written by Thi Nhung Dao,  last update: 6/2021                             c
c------------------------------------------------------------------------------c
c uncomment it to printout details for checking
c#define printMT4
* subroutine ReHSelf_atlambda(renfin,regfac)
*  gives finite parts of the neutral Higgs selfenergies at (\alpha_t+lambda+kappa)^2 order
*  in the mass eigensates (h1,h2,h3,h4,h5) to connect with CalcMasses.F
* subroutine couplings_atlambda
*  calculates neccessary couplings used in the calculation
* subroutine LIBLOOP_atlambda
* computes all loop-integrals used in the calculation
* subroutine CT1loop_atlambda
* computes all one-loop counterterms 
* subroutine CTCLs_atlambda
* computes all one-loop counterterm couplings
* subroutine TadMT4_atlambda
* computes two-loop tadpoles
* subroutine CHSelfMT4, CHSelf_atlambda
* compute two-loop charged Higgs selfenergy
* subroutine UnselfHH_atat,  UnselfHH_atlambda1, UnselfHH_atlambda2
*  calculate unrenormalied Higgs selfenergies in the Interation basis (hd,hu,hs,ad,au,as)
* subroutine SelfTWW2L_MT4, SelfTWW2L_lambda
*  calculate two-loop W selfenergy
* subroutine SelfTZZ2L_MT4, SelfTZZ2L_lambda
*  calculate two-loop Z selfenergy
* subroutine counterM_atlambda
*  calculate two-loop counterterm matrix for neutral Higgs boson in interaction basis
*  (hd,hu,hs,ad,au,as)
c Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021.  
* mathematica_nxmmatrix and mathematica_list
*  write in  matrices and lists to .m files

      subroutine mathematica_nxmmatrix(filestream,str,mat,n,m)
            implicit none
            integer, Intent(in) :: n,m,filestream
            character(len=*),intent(in) :: str
            real(8), dimension(n,m), intent(in) :: mat
            integer i,j
            character(10000) strout,tmp
            strout =  trim(str)//trim(" = {")
            do i=1,n
                strout = trim(strout)//trim("{")
                do j=1,m
                    write(tmp,'(F24.14)') mat(i,j)
                    strout = trim(strout) // trim(tmp)
                    if(j.lt.m) then
                        strout = trim(strout) // trim(",")
                    endif

                enddo 
      
                if(i.eq.n) then
                    strout = trim(strout) // trim("}")
                else 
                    strout = trim(strout) // trim("},")
                endif
      
            enddo
            strout = trim(strout) // trim("};")
            write(filestream,*) trim(strout)
      end subroutine mathematica_nxmmatrix
      subroutine mathematica_list(filestream,str,list,n)
            implicit none
            integer, Intent(in) :: n,filestream
            character(len=*),intent(in) :: str
            real(8), dimension(n), intent(in) :: list
            integer i
            character(10000) strout,tmp
            strout =  trim(str)//trim(" = {")
            do i=1,n
                    write(tmp,'(F24.14)') list(i)
                    strout = trim(strout) // trim(tmp)
                    if(i.lt.n) then
                        strout = trim(strout) // trim(",")
                    endif
            enddo
            strout = trim(strout) // trim("};")
            write(filestream,*) trim(strout)
      end subroutine mathematica_list
      character*60 function mathematica_c(cplx)
            complex(8), intent(in) :: cplx
            character(1000) tmpr,tmpc
            write(tmpr,'(F26.14)') real(cplx)
            write(tmpc,'(F26.14)') aimag(cplx)
            mathematica_c = trim(tmpr) // trim("+ I*(")
            mathematica_c = trim(mathematica_c) // trim(tmpc)
            mathematica_c = trim(mathematica_c) // trim(")")
            return
      end
      subroutine mathematica_nxmmatrix_c(filestream,str,mat,n,m)
            implicit none
            integer, Intent(in) :: n,m,filestream
            character(len=*),intent(in) :: str
            complex(8), dimension(n,m), intent(in) :: mat
            integer i,j
            character(60) mathematica_c
            character(10000) strout
            strout =  trim(str)//trim(" = {")
            do i=1,n
               strout = trim(strout)//trim("{")
               do j=1,m
                  strout = trim(strout) // trim(mathematica_c(mat(i,j)))
                  if(j.lt.m) then
                      strout = trim(strout) // trim(",")
                  endif
               enddo 
      
                if(i.eq.n) then
                    strout = trim(strout) // trim("}")
                else 
                    strout = trim(strout) // trim("},")
                endif
      
            enddo
            strout = trim(strout) // trim("};")
            write(filestream,*) trim(strout)
      end subroutine mathematica_nxmmatrix_c
      subroutine mathematica_list_c(filestream,str,list,n)
            implicit none
            integer, Intent(in) :: n,filestream
            character(len=*),intent(in) :: str
            complex(8), dimension(n), intent(in) :: list
            integer i
            character(10000) strout
            character(60) mathematica_c
            strout =  trim(str)//trim(" = {")
            do i=1,n
                strout = trim(strout) // trim(mathematica_c(list(i)))
                if(i.lt.n) then
                    strout = trim(strout) // trim(",")
                endif
            enddo
            strout = trim(strout) // trim("};")
            write(filestream,*) trim(strout)
      end subroutine mathematica_list_c

c end Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021 

#define epsi 1D-10
      subroutine MSQ_MSU_AT_OStoDR_atlamda(dmsq4fin,dmsur4fin,dAt4fin,ierror)
      implicit none
      integer i,j,k,l,ierror
      double precision MHp2,MW2,MZ2,dmsq4fin,dmsur4fin
      double precision Qs
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      logical CPcon,MHpmInput,onelswitch,mQrun

      double complex IC,ZUN(2,2),AT,la,Ala,ka,Aka,MUE,expETA,expVS,dAt4fin
      common /scale/ Qs
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch

      IC=dcmplx(0,1D0)
      ZUN(1:2,1:2) = nZU(5:6,5:6)
      MW2=Mw**2; MZ2=Mz**2;MHp2=MHpm**2
      Ala= absAlambda*(cos(phiAl) + IC*Sin(phiAl))
      AT= absAu(3)*(cos(phiAu(3)) + IC*Sin(phiAu(3)))
      la= abslambda*(Cos(phil) + IC*Sin(phil))
      Aka= absAkappa*(cos(phiAk) + IC*Sin(phiAk))
      ka= abskappa*(Cos(phik) + IC*Sin(phik))
      expvs=(Cos(phis) + IC*Sin(phis))
      expETA=(Cos(phiu) + IC*Sin(phiu))
      MUE=vs*la*expvS/dsqrt(2d0)

      call couplings_atlambda(QS**2,MT,MHp2,v,vS,Tanb,MW2,MZ2,nMSu(5),
     -  nMSu(6),ZUN,AT,MUE,Ala,ka,AKa,expETA,expvS,MHpmInput,ierror)
      call CT1L_TB(dmsq4fin,dmsur4fin,dat4fin)

      end subroutine MSQ_MSU_AT_OStoDR_atlamda
c######################################################################
      subroutine ReHSelf_atlambda(renfin,regfac)
      implicit none
      integer i,j,k,l,ischeme
      double precision renfin(5,5),regfac,reself(5,5),MHp2
      double precision MW2,MZ2, A0k
      double precision Qs
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      logical CPcon,MHpmInput,onelswitch,mQrun

      double complex IC,ZUN(2,2),AT,la,Ala,ka,Aka,MUE,expETA,expVS
      common /scale/ Qs
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      IC=dcmplx(0,1D0)
      ZUN(1:2,1:2) = nZU(5:6,5:6)
      MW2=Mw**2; MZ2=Mz**2;MHp2=MHpm**2
      Ala= absAlambda*(cos(phiAl) + IC*Sin(phiAl))
      AT= absAu(3)*(cos(phiAu(3)) + IC*Sin(phiAu(3)))
      la= abslambda*(Cos(phil) + IC*Sin(phil))
      ka= abskappa*(Cos(phik) + IC*Sin(phik))
      Aka=absAkappa*(Cos(phiAk) + IC*Sin(phiAk))
      expvs=(Cos(phis) + IC*Sin(phis))
      expETA=(Cos(phiu) + IC*Sin(phiu))
      MUE=vs*la*expvS/dsqrt(2d0)
      if(mQrun)then
        ischeme=1
      else
        ischeme=3
      endif

      call selfHH_atlambda(Qs**2,mt,MHp2,v,vS,Tanb,MW2,MZ2,
     -   nMSu(5),nMSu(6),ZUN,AT,MUE,Ala,ka,Aka,expETA,expvS,
     -   reself,ischeme,MHpmInput,regfac)

      do i=1,5
      do j=1,5
      renfin(i,j) =0D0
       do k=1,5
       do l=1,5
      renfin(i,j) = renfin(i,j)+ZH5(i,k)*ZH5(j,l)*reself(k,l)
       enddo
       enddo
      enddo
      enddo
#ifdef printMT4
      write(*,*) "===================================================="
      write(*,*) "ZH5(1,1:5)", ZH5(1,1:5)
      write(*,*) "ZH5(2,1:5)", ZH5(2,1:5)
      write(*,*) "ZH5(3,1:5)", ZH5(3,1:5)
      write(*,*) "ZH5(4,1:5)", ZH5(4,1:5)
      write(*,*) "ZH5(5,1:5)", ZH5(5,1:5)
      write(*,*) " atlambda contributions in basis (a1,a2,h1,h2,h3)"
      write(*,*) "renfin(1,1:5)=",renfin(1,1:5)   
      write(*,*) "renfin(2,1:5)=",renfin(2,1:5)
      write(*,*) "renfin(3,1:5)=",renfin(3,1:5)     
      write(*,*) "renfin(4,1:5)=",renfin(4,1:5) 
      write(*,*) "renfin(5,1:5)=",renfin(5,1:5) 
#endif
      end subroutine ReHSelf_atlambda
c
c----------------------------------------------------------------------c
c
      subroutine selfHH_atlambda(Q2N,MTN,MHpm2N,vvN,vSN,TBN,
     -   MW2N,MZ2N,MST1,MST2,ZUN,ATN,MUEN,AlaN,kaN,AkaN,expETAN,expvSN,
     -   reself,ischeme,MHpmInput,regfac)
      implicit none
      integer i,j,l,k,gen2,Gen3,Gen4,Gen5,Gen6,h1,ischeme,ierror
      double precision reself(5,5),self(6,6),dMH2MT4(6,6),
     .     aTad(6),selfCT(6,6),TadCT(6),CHselfCT,dtad2MT4(6),
     .     selfIN(6,6),selfCTIN(6,6),selfINS(6,6),CHSELF,CHselfla,
     .     selfINla1(6,6),selfINla2(6,6),selfW,selfWla,selfZ,selfZla,
     .     dv42,regfac,dMWoMW,dMZoMZ

      double precision Q2N,MTN,MSt1,MSt2,MHpm2N,vvN,vSN,TBN,MW2N,MZ2N,
     .     S2B,C2B,dmsq4fin,dmsur4fin

      double complex ATN,ZUN(2,2),MUEN,AlaN,kaN,AkaN,expETAN,expvSN,dAt4fin
      logical MHpmInput

      integer modselval(10)
      character spinfo1*100,spinfo2*100
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval
      COMMON/vectorself/dMWoMW,dMZoMZ

#include "TLMT4.h"
#include "TLMT4lambda.h"
      if(regfac.lt.1D-10)then
      write(*,*) "warning!!! the regulator factor, ", regfac," , is two
     &   small. We use the default value R=1, MG^2=R*10^(-3) MUR^2"      
      regMass2= 1D-3*Q2N
      else
      regmass2=regfac*1D-3*Q2N
      endif
      call couplings_atlambda(Q2N,MTN,MHpm2N,vvN,vSN,TBN, MW2N,MZ2N,MST1,
     -  MST2,ZUN,ATN,MUEN,AlaN,kaN,AKaN,expETAN,expvSN,MHpmInput,ierror)

      S2B = 2*TB*CB2;          C2B= (CB - SB)*(CB + SB)
c two-loop integrals
      call LIBLOOP_atlambda

c one-loop counterterms

      call CT1loop_atlambda(ischeme,dmsq4fin,dmsur4fin,dAt4fin,MHpminput)

      call CTCLs_atlambda()
c tadpoles
      call TadMT4_atlambda(aTad)
c charged Higgs self
      call CHSelfMT4(CHself)
      call CHSelf_atlambda(CHselfla)
      CHself=CHself+CHselfla

c neutral Higgs self
      call UnselfHH_atat(selfIN)
      call UnselfHH_atlambda1(selfINla1)
      call UnselfHH_atlambda2(selfINla2)
c Martin Gabelmann changed to connect mathematica code for momentum dependent Higgs
c selfenergies 3/2021 
      if(modselval(9).gt.0) then
        OPEN(17,FILE='out.m', STATUS= 'UNKNOWN', ACCESS='APPEND')
        call mathematica_nxmmatrix(17,'Res[UnRenSelfhh]',selfINla1+selfINla2,6,6)
        call mathematica_nxmmatrix(17,'Res[UnRenSelfhhMSSM]',selfIN,6,6)
        write(17,*)'Res[MRegulator]=',sqrt(regmass2),';'
        close(17)
        call UnselfHH_atlambdap2(selfIN,selfINla1,selfINla2,modselval(9))
      endif
c end Martin Gabelmann changed to connect mathematica code for momentum dependent Higgs
c selfenergies 3/2021 
      selfIN(1:6,1:6)=selfIN(1:6,1:6)+selfINla1(1:6,1:6)+selfINla2(1:6,1:6)


c W-boson selfenergy
      call SelfTWW2L_MT4(selfW)
      call SelfTWW2L_lambda(selfWla)
      selfW=selfW+selfWla
c Z-boson selfenergy
      call SelfTZZ2L_MT4(selfZ)
      call SelfTZZ2L_lambda(selfZla)
      selfZ=selfZ+selfZla
c two-loop counterterms
c Martin Gabelmann changed to take into account one-loop squared terms 6/2021 
      dv42  =(-CW2/SW2/2.*(selfW - selfZ)+1/2.*selfW)*vv
     &    + (-dMWoMW**2 + 2*dMWoMW*dMZoMZ*(1 + SW2 - 2*SW2**2)
     &       + dMZoMZ**2*(-1 - 2*SW2 + 3*SW2**2))/(SW2**2*8D0)
c end Martin Gabelmann changed to take into account one-loop squared terms 6/2021 
      do i=1,6
       dtad2MT4(i)=0D0
      do gen3=1,6
       dtad2MT4(i)=dtad2MT4(i)+ZH(gen3,i)*(aTad(gen3))
      enddo
      enddo

        dtad2MT4(1) = dtad2MT4(1) - dtad1MT4(1,3) *(Zfactor1LIN(1,1))/2D0
        dtad2MT4(2) = dtad2MT4(2) - dtad1MT4(2,3) *(dZHu4+Zfactor1LIN(2,2))/2D0
        dtad2MT4(3) = dtad2MT4(3) - dtad1MT4(3,3) *(Zfactor1LIN(3,3))/2D0
        dtad2MT4(4) = dtad2MT4(4) - dtad1MT4(4,3) *(Zfactor1LIN(4,4))/2D0
        dtad2MT4(5) = dtad2MT4(5) - dtad1MT4(5,3) *(dZHu4+Zfactor1LIN(5,5))/2D0
        dtad2MT4(6) = dtad2MT4(6) - dtad1MT4(6,3) *(Zfactor1LIN(6,6))/2D0

#ifdef printMT4
 
      Write(*,*) "___________________________________________________"
      Write(*,*) "Neutral Higgs selfenergies in interaction state"
      Write(*,*) "Sum of 2loop diagram + 1 loop CT"
      Write(*,*) " finite part in interaction basis"

      Write(*,*) "selfIN(1,1->6)+selfCTIN(1,1->6)",selfIN(1,1:6)
      Write(*,*) "selfIN(2,2->6)+selfCTIN(2,2->6)",selfIN(2,2:6)
      Write(*,*) "selfIN(3,3->6)+selfCTIN(3,3->6)",selfIN(3,3:6)
      Write(*,*) "selfIN(4,4->6)+selfCTIN(4,4->6)",selfIN(4,4:6)
      Write(*,*) "selfIN(5,5->6)+selfCTIN(5,5->6)",selfIN(5,5:6)
      Write(*,*) "selfIN(6,6->6)+selfCTIN(6,6->6)",selfIN(6,6)
#endif     
      call counterM_atlambda(CHself,dtad2MT4,dMH2MT4,dv42)

#ifdef printMT4
      Write(*,*) "______Neutral Higgs CT matrix___________"
      Write(*,*) " finite part in interaction basis"

      Write(*,*) "dMH2MT4(1,1->6)",dMH2MT4(1,1:6)
      Write(*,*) "dMH2MT4(2,2->6)",dMH2MT4(2,2:6)
      Write(*,*) "dMH2MT4(3,3->6)",dMH2MT4(3,3:6)
      Write(*,*) "dMH2MT4(4,4->6)",dMH2MT4(4,4:6)
      Write(*,*) "dMH2MT4(5,5->6)",dMH2MT4(5,5:6)
      Write(*,*) "dMH2MT4(6,6->6)",dMH2MT4(6,6:6)
      Write(*,*) "___________________________________________________"

#endif   


      do i=1,6
       do j=i,6
      SelfINS(i,j) = SelfIN(i,j) - dMH2MT4(i,j)
       enddo
      enddo
      do i=1,6
       do j=i,6
      SelfINS(j,i) = SelfINS(i,j) 
       enddo
      enddo
c rotate to (a,as,hd,hu,hs) basis
#ifdef printMT4
      Write(*,*) "Goldstone mixing"
      Write(*,*)CB2*selfINS(4,5) - SB2*selfINS(4,5) + CB*SB*(selfINS(4,4) - selfINS(5,5))
      Write(*,*)CB*selfINS(4,6) -SB*selfINS(5,6)
      Write(*,*)CB*selfINS(1,4) - SB*selfINS(1,5)
      Write(*,*)CB*selfINS(2,4) - SB*selfINS(2,5)
      Write(*,*)CB*selfINS(3,4) - SB*selfINS(3,5)
#endif
      ReSelf(1,1)=SB2*selfINS(4,4)+S2B*selfINS(4,5)+CB2*selfINS(5,5)
      ReSelf(1,2)=SB*selfINS(4,6)+CB*selfINS(5,6)
      ReSelf(1,3)=SB*selfINS(1,4)+CB*selfINS(1,5)
      ReSelf(1,4)=SB*selfINS(2,4)+CB*selfINS(2,5)
      ReSelf(1,5)=SB*selfINS(3,4)+CB*selfINS(3,5)
      Reself(2,2)=selfINS(6,6)
      Reself(2,3)=selfINS(1,6)
      Reself(2,4)=selfINS(2,6)
      Reself(2,5)=selfINS(3,6)

      ReSelf(3,3:5)=selfINS(1,1:3)
      ReSelf(4,4:5)=selfINS(2,2:3)
      ReSelf(5,5)=selfINS(3,3)
      do i=1,5
       do j=i,5
         ReSelf(j,i)=ReSelf(i,j)
       enddo
      enddo
#ifdef printMT4
      Write(*,*) "------------------------------------------------"
      Write(*,*) "renormalized self in interaction state hd,hu,hs,au,ad,as"
      Write(*,*) "selfINS(1,1->6)",selfINS(1,1:6)
      Write(*,*) "selfINS(2,2->6)",selfINS(2,2:6)
      Write(*,*) "selfINS(3,3->6)",selfINS(3,3:6)
      Write(*,*) "selfINS(4,4->6)",selfINS(4,4:6)
      Write(*,*) "selfINS(5,5->6)",selfINS(5,5:6)
      Write(*,*) "selfINS(6,6->6)",selfINS(6,6)
      Write(*,*) "------------------------------------------------"
      Write(*,*) "renormalized self in interaction state a,as,hu,hd,hs"
      Write(*,*) "reself(1,1->6)",reself(1,1:5)
      Write(*,*) "reself(2,2->6)",reself(2,2:5)
      Write(*,*) "reself(3,3->6)",reself(3,3:5)
      Write(*,*) "reself(4,4->6)",reself(4,4:5)
      Write(*,*) "reself(5,5->6)",reself(5,5:5)

      do i=1,6
       do j=1,6 
       self(i,j)=0D0
      do gen3=1,6
        do gen2=1,6
       self(i,j)=self(i,j)+ZH(i,gen2)*ZH(j,gen3)*selfINS(gen2,gen3)
        enddo
      enddo
        enddo
      enddo
      Write(*,*) "renormalized self in mass eigenstate h1,h2,h3,h4,h5"
      Write(*,*) "self(1,1->6)",self(1,1:6)
      Write(*,*) "self(2,2->6)",self(2,2:6)
      Write(*,*) "self(3,3->6)",self(3,3:6)
      Write(*,*) "self(4,4->6)",self(4,4:6)
      Write(*,*) "self(5,5->6)",self(5,5:6)
      Write(*,*) "self(6,6->6)",self(6,6:6)
#endif 
c)
      end subroutine selfHH_atlambda
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine higgstreeha(v,beta,g1,g2,vs,abslambda,phil,
     &                      abskappa,phik,absAkappa,phiAk,absAlambda,
     &                      phiAl,phiu,phis,Mh,ZH,CPcon,ierror)
     
      implicit none
      
      double precision v,beta,g1,g2,vs,abslambda,phil,abskappa,phik,
     &                 absAkappa,phiAk,absAlambda,phiAl,phiu,phis,Mh(6)
     &                 ,ZH(6,6),Mh2(6),diff
      double precision CB2,SB2,MZ2,MW2,MHp2,la2,vv,vv2,vs2,Cosphiy,Sinphiy,
     &                 sign2,sqrt2,SW2,CW2,CB,SB,ka2
     
      double precision phi,phiIl,phiIk
      double complex MassH(6,6),U(6,6)
      
      integer i,j,ierror
      
      logical CPcon
      
      double complex Meven(3,3),Modd(2,2),Ueven(3,3),Uodd(2,2)
      double precision Mhh2(3),MAh2(2)
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror
    
      phi=-phik + phil - 2*phis + phiu
      phiIl=phis + phiAl + phil + phiu
      phiIk=3*phis + phiAk + phik 
      MZ2=(g1**2 + g2**2)*v**2/4D0
      MW2= g2**2*v**2/4D0
      la2=abslambda**2; vv=v;vv2=vv**2;SB=sin(beta);CB=cos(beta);
      CB2=CB**2; SB2=SB**2;vs2=vs**2
      Sinphiy=sin(phi); Cosphiy=cos(phi);sqrt2=dsqrt(2d0);CW2=MW2/MZ2
      SW2=1-CW2;ka2=abskappa**2
      sign2 =cos(phis + phiAk + phik)/abs(cos(phis + phiAk + phik))
      MHp2=((-2*abslambda**2 + g2**2)*v**2 + 
     -        4*abslambda*vs*(Sqrt(2d0)*absAlambda*cos(phiAl)+
     -        abskappa*vs*Cos(phik + 3*Phis))/Sin(2*beta)/
     -        Cos(phil + Phis + Phiu))/4.

      MassH(1:6,1:6)=0D0
      MassH(1,1)=CB2*MZ2 + MHp2*SB2 - MW2*SB2 + (la2*SB2*vv2)/2.
      MassH(1,2)=(CB*SB*(-2*MHp2 + 2*MW2 - 2*MZ2 + la2*vv2))/2.
      MassH(1,3)=-((CB*MHp2*SB2*vv)/vS) + (CB*MW2*SB2*vv)/vS  
     -  + CB*la2*vS*vv -(abskappa*abslambda*Cosphiy*SB*vS*vv)/2.
     -  - (CB*la2*SB2*vv**3)/(2.*vS)

      MassH(1,5)=(-3*abskappa*abslambda*SB*Sinphiy*vS*vv)/2.

      MassH(2,2)=CB2*MHp2 - CB2*MW2+MZ2*SB2+(CB2*la2*vv2)/2.
      MassH(2,3)= -((CB2*MHp2*SB*vv)/vS) + (CB2*MW2*SB*vv)/vS - 
     -  (abskappa*abslambda*CB*Cosphiy*vS*vv)/2. + 
     -  la2*SB*vS*vv - (CB2*la2*SB*vv**3)/(2.*vS)
      MassH(2,5)=(-3*abskappa*abslambda*CB*Sinphiy*vS*vv)/2.
      MassH(3,3)=2*ka2*vs2 + (CB2*la2*SB2*vv**4)/(2.*vS2) + 
     -  (absAkappa*abskappa*sign2*vS*Sqrt(1D0 - 
     -  (9*CB2*la2*SB2*Sinphiy**2*vv**4)/(2.*absAkappa**2*vS2)))/sqrt2- 
     -  (abskappa*abslambda*CB*Cosphiy*SB*vv2)/2. + 
     -  (CB2*MHp2*SB2*vv2)/vS2 - (CB2*MW2*SB2*vv2)/vS2
      MassH(3,4)=(abskappa*abslambda*Sinphiy*vS*vv)/2.
      MassH(3,5)=2*abskappa*abslambda*CB*SB*Sinphiy*vv2
      MassH(4,4)=(2*MHp2 - 2*MW2 + la2*vv2)/2.

      MassH(4,5)= (vv*(-3*abskappa*abslambda*Cosphiy*vS**2 + 
     -      CB**3*SB*(2*MHp2 - 2*MW2 + la2*vv**2) + CB*SB**3*
     -     (2*MHp2 - 2*MW2 + la2*vv**2)))/(2.*vS)

      MassH(5,5)=(-3*absAkappa*abskappa*sign2*vS*vS2*
     -     Sqrt(4 - (18*CB2*la2*SB2*Sinphiy**2*vv**4)/(absAkappa**2*vS2)) + 
     -    3*abskappa*abslambda*CB*Cosphiy*SB*sqrt2*vS2*vv2 + 
     -    CB2*SB2*sqrt2*(la2*vv**4 + 2*(MHp2 - MW2)*vv2))/(2.*sqrt2*vS2)         
      do i=1,5
       do j=i,5
        MassH(j,i)=MassH(i,j)
       enddo
      enddo

      call HEigensystem(5,MassH(1:5,1:5),5,Mh2(1:5), U(1:5,1:5),5,1)
         do i=1,5,1
            if(Mh2(i).lt.0 .OR. Mh2(i).ne.Mh2(i) ) then
              if(mainrun) then
              print*, '* Warning!! Negative or NAN Higgs mass Eigenvalues!! in the gaugeless limit'
              print*, "MH_1^2=",Mh2(i)
              print*, 'Using the option 6  3 in Block MODSEL of input file'   
              print*
              ierror=1
              return
              else
              calcmasseserror=.TRUE.
              end if
            end if
         end do

        Mh2(6)=0
        Mh=sqrt(Mh2)
        ZH(1:5,1:5)=realpart(U(1:5,1:5))
c check if degenerate case happens
      diff=Mh(2)-Mh(1)
      do i=3,5
         if (diff .gt. (Mh(i)-Mh(i-1)) )then
               diff = (Mh(i)-Mh(i-1))
         endif
      enddo
	  if (diff.lt.1D-3)then
        print*, '* Warning!! degenerate Higgs mass spectrum in the gaugeless limit'
        print*, 'Return to the option 6  3 in Block MODSEL of input file'   
        print*, 'that include O(alpha_t alpha_s + alpha_t^2) correction'
        ierror=1
        return
      endif

c
      do i=1,5
        ZH(i,6)=ZH(i,5)
        ZH(i,5)= CB*ZH(i,4)
        ZH(i,4)= SB*ZH(i,4)
      enddo
        ZH(6,6)=0; ZH(6,5)=-SB; ZH(6,4)= CB
      return
      end

c**********************************************************************
      subroutine couplings_atlambda(Q2N,MTN,MHpm2N,vvN,vSN,TBN,MW2N,MZ2N,
     -   MST1,MST2,ZUN,ATN,MUEN,AlaN,kaN,AKaN,expETAN,expvSN,MHpmInput,ierror)
      implicit none
      integer i,j,l,k,l1,gen1,gen2,gen3,gen4,gen5,i1,i2,i3,ierror
      double precision Q2N,MTN,MSt1,MSt2,MHpm2N,vvN,vSN,TBN,MW2N,MZ2N,
     .     S2B,C2B,am,diff
      double complex MassH(6,6),ZHN(6,6)
      double complex ATN,ZUN(2,2),MUEN,AlaN,kaN,AKaN,expETAN,expvSN,
     .     cm,yy(3,3)
      character(60) mathematica_c
      logical MHpmInput

      integer modselval(10)
      character spinfo1*100,spinfo2*100
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval



#include "TLMT4.h"
#include "TLMT4lambda.h"

      pi = 2d0*Asin(1d0)
      IC=dcmplx(0,1D0)
      sqrt2= sqrt(2D0)
      ZU(1:2,1:2)=ZUN(1:2,1:2)
      
      Q2=Q2N;vs=vSN;vS2=vS**2
      vv=vvN;vv2= vv**2;MW2=MW2N;MZ2=MZ2N
      CW2=MW2/MZ2; SW2=1-CW2;SW=sqrt(SW2);CW=Sqrt(CW2)     
      TB=TBN;      TB2= TB**2;	       CB2 = 1/(1 + TB2)
      SB2 = CB2*TB2;	       CB = sqrt(CB2);	SB = CB*TB
      S2B = 2*TB*CB2;          C2B= (CB - SB)*(CB + SB)
      ZP(1,1)=-CB; ZP(1,2)=SB; ZP(2,1)=SB;  ZP(2,2)= CB
      ATT=ATN
      expETA=expETAN;expvS=expvSN
      MUE=MUEN; MUEC=DCONJG(MUE)
      la=MUE/vs/expvs*sqrt2
      ihiggs=1;ineu=1
      ihiggse=6;ineue=3
      abslambda = abs(la); la2=abslambda**2;laC=DCONJG(la)
      kappa=kaN; abskappa=abs(kappa); ka2=abskappa**2
      kappaC=DCONJG(kappa)
      if (abslambda.gt.1D-10)then
      explam=la/abslambda
      else
      explam=1D0
      endif

      if (abskappa.gt.1D-10)then
      expkap=kappa/abskappa
      else
      expkap=1D0
      endif

      Aka=AkaN; ReAkappa= DBLE(Aka);absAkappa=abs(Aka)
      if (absAkappa.gt.1D-10)then
      expAka = Aka/absAkappa
      else
      expAka=1D0
      endif

      cosphiy = DBLE(expETA/expVS**(2)/expkap*explam)
      sinphiy = DIMAG(expETA/expVS**(2)/expkap*explam)
      cosk3s= DBLE(expkap*expVS**3)
      sink3s= DIMAG(expkap*expVS**3)
      sign2 =DBLE(expVS*expAka*expkap)/abs(DBLE(expVS*expAka*expkap))

      if (MHpmInput)then

        MHp2N=MHpm2N
       
        RAlambda=(-4*abskappa*abslambda*vs**2*cosk3s + 
     -      (4*(MHp2N-MW2*0) + (2*abslambda**2 )*vv2)
     -      * DBLE(expVS*expETA*explam)*S2B)/
     -      (4.*Sqrt2*abslambda*vs)

        IAlambda=(abskappa*vs/DBLE(expVS*expETA*explam)*
     -        DIMAG(expVS**(2)*expkap/explam/expETA))/Sqrt2 
     -       - RAlambda*DIMAG(expVS*expETA*explam)/DBLE(expVS*expETA*explam)  


        AlaN=RAlambda +IC*IAlambda 
      sign1 =  DBLE(expETA*expVS*AlaN/abs(AlaN)*explam)
     -    /abs(DBLE(expETA*expVS*AlaN/abs(AlaN)*explam))

      else
      sign1 =  DBLE(expETA*expVS*AlaN/abs(AlaN)*explam)
     -    /abs(DBLE(expETA*expVS*AlaN/abs(AlaN)*explam))
      sinphix =-((abskappa*Sinphiy*vS)/(abs(AlaN)*sqrt2))
      sinphix =-((abskappa*Sinphiy*vS)/(abs(AlaN)*sqrt2))
      sinphiz =(abslambda*CB*SB*(abs(AlaN)*Sinphix*sqrt2 
     -  - 2*abskappa*Sinphiy*vS)*vv2)/(absAkappa*abskappa*sqrt2*vs2)

      Cosphix = Sign1*Sqrt(1D0- Sinphix**2)
      Cosphiz = Sign2*Sqrt(1D0- Sinphiz**2)

      MHp2N= 0.0*MW2 + (Abs(AlaN)*abslambda*Cosphix*vS)/(CB*SB*sqrt2)+ 
     -     (abskappa*abslambda*Cosphiy*vs2)/(2.*CB*SB) - (la2*vv2)/2D0

      endif
      MHp2=MHp2N
      Ala=AlaN

      MHp2=MHp2N
      AlaC=DCONJG(Ala)
      absAlambda= Abs(Ala)
      if (absAlambda.gt.1D-10)then
      expAla = Ala/absAlambda
      else
      expAla=1D0
      endif
      sqrt1x=  Sqrt(2D0-0*(9*abslambda**2*CB2*SB2*Sinphiy**2*
     -      vv**4)/(absAkappa**2*vS2))
      sqrt2x= Sqrt(2D0 - (ka2*Sinphiy**2*vS2)/absAlambda**2)


      MassH(1,1)=MHp2*SB2  + (la2*SB2*vv2)/2.
      MassH(1,2)=(CB*SB*(-2*MHp2 + la2*vv2))/2.
      MassH(1,3)=-((CB*MHp2*SB2*vv)/vS)  + CB*la2*vS*vv
     -  -(abskappa*abslambda*Cosphiy*SB*vS*vv)/2.
     -  - (CB*la2*SB2*vv**3)/(2.*vS)
      MassH(1,4)=0D0;
      MassH(1,5)=(-3*abskappa*abslambda*SB*Sinphiy*vS*vv)/2.
      MassH(1,6)=0D0
      MassH(2,2)=CB2*MHp2 +(CB2*la2*vv2)/2.
      MassH(2,3)= -((CB2*MHp2*SB*vv)/vS)  - 
     -  (abskappa*abslambda*CB*Cosphiy*vS*vv)/2. + 
     -  la2*SB*vS*vv - (CB2*la2*SB*vv**3)/(2.*vS)
      MassH(2,4)=0D0
      MassH(2,5)=(-3*abskappa*abslambda*CB*Sinphiy*vS*vv)/2.
      MassH(2,6)=0D0
      MassH(3,3)=2*ka2*vs2 + (CB2*la2*SB2*vv**4)/(2.*vS2) + 
     -  (absAkappa*abskappa*sign2*vS*Sqrt(1D0 - 
     -  (9*CB2*la2*SB2*Sinphiy**2*vv**4)/(2.*absAkappa**2*vS2)))/sqrt2- 
     -  (abskappa*abslambda*CB*Cosphiy*SB*vv2)/2. + 
     -  (CB2*MHp2*SB2*vv2)/vS2 
      MassH(3,4)=(abskappa*abslambda*Sinphiy*vS*vv)/2.
      MassH(3,5)=2*abskappa*abslambda*CB*SB*Sinphiy*vv2
      MassH(3,6)=0D0
      MassH(4,4)=(2*MHp2 + la2*vv2)/2.
      MassH(4,5)= (vv*(-3*abskappa*abslambda*Cosphiy*vS**2 + 
     -      CB**3*SB*(2*MHp2 + la2*vv**2) + CB*SB**3*
     -     (2*MHp2  + la2*vv**2)))/(2.*vS)
      MassH(4,6)=0D0
      MassH(5,5)=(-3*absAkappa*abskappa*sign2*vS*vS2*
     -     Sqrt(4 - (18*CB2*la2*SB2*Sinphiy**2*vv**4)/(absAkappa**2*vS2)) + 
     -    3*abskappa*abslambda*CB*Cosphiy*SB*sqrt2*vS2*vv2 + 
     -    CB2*SB2*sqrt2*(la2*vv**4 + 2*(MHp2 )*vv2))/(2.*sqrt2*vS2)
      MassH(5,6)=0D0
      MassH(6,6)=0D0        
      MassH(2,1)=MassH(1,2)
      MassH(3,1)=MassH(1,3)
      MassH(3,2)=MassH(2,3)
      MassH(4,1)=MassH(1,4)
      MassH(4,2)=MassH(2,4)
      MassH(4,3)=MassH(3,4)
      MassH(5,1)=MassH(1,5)
      MassH(5,2)=MassH(2,5)
      MassH(5,3)=MassH(3,5)
      MassH(5,4)=MassH(4,5)
      MassH(6,1)=MassH(1,6)
      MassH(6,2)=MassH(2,6)
      MassH(6,3)=MassH(3,6)
      MassH(6,4)=MassH(4,6)
      MassH(6,5)=MassH(5,6)

      call HEIgensystem(5, MassH(1:5,1:5),5, Mah2(1:5), ZHN(1:5,1:5),5, 1)
      do i=1,5
        if (mah2(i).lt.0D0 .or. mah2(i).ne.mah2(i) ) then
        print*, '* Warning!! Negative or NAN Higgs mass Eigenvalues!! in the gaugeless limit'
        print*, "MH_1^2=",mah2(i)
        print*, 'Return to the option 6  3 in Block MODSEL of input file'   
        print*, 'that include O(alpha_t alpha_s + alpha_t^2) correction'
        ierror=1
        return
        else 
        Mah(i) = Sqrt(mah2(i))
        endif
      enddo
        ierror=0
        Mah(6)=0  
      do i=1,5
        ZHN(i,6)=ZHN(i,5)
        ZHN(i,5)= CB*ZHN(i,4)
        ZHN(i,4)= SB*ZHN(i,4)
      enddo
        ZHN(6,6)=0; ZHN(6,5)=-SB; ZHN(6,4)= CB
        ZH(1:6,1:6)=DBLE(ZHN(1:6,1:6))

c check if degenerate case happens
      diff=Mah(2)-Mah(1)
      do i=3,5
         if (diff .gt. (Mah(i)-Mah(i-1)) )then
               diff = (Mah(i)-Mah(i-1))
         endif
      enddo
	  if (diff.lt.1D-3)then
        print*, '* Warning!! degenerate Higgs mass spectrum in the gaugeless limit'
        print*, 'Return to the option 6  3 in Block MODSEL of input file'   
        print*, 'that include O(alpha_t alpha_s + alpha_t^2) correction'
        ierror=1
        return
      endif
c
      ihiggs=1;ineu=1
      yy(1,1)=0D0; yy(1,2)=-MUE; yy(1,3)=-la*vv*SB/sqrt2*expeta
      yy(2,1)=yy(1,2); yy(2,2)=0D0; yy(2,3)=-vv*CB*la/Sqrt2
      yy(3,1)=yy(1,3);yy(3,2)=yy(2,3);yy(3,3)=sqrt2*expvS*vS*kappa
      call TakagiFactor(3, yy, 3, MNeu, ZNeu, 3, 1)
      Do i=ineu,ineue 
        MNeu2(i)=MNeu(i)**2; 
        ZNeuC(i,1:3)=DCONJG(ZNeu(i,1:3))
      enddo
      ZUC(1:2,1:2)=DCONJG(ZU(1:2,1:2))
      M1=MTN;   M2=MSt1;  M4=MSt2
      M12=M1**2;M22=M2**2;M42=M4**2
      MassSu(1)=M2; MassSu(2)= M4
      MassSu2(1)=M22; MassSu2(2)= M42
      logm12=Log(M12/Q2); logM22=Log(M22/Q2);logM42=log(M42/Q2)

      MCha=Abs(MUE); MCha2=MCha**2;VCha=MUE/MCha;VChaC=DCONJG(VCha)
      ATC= DCONJG(ATT)

      MSQ2=-M12+M22*ZU(1,1)*ZUC(1,1)+M42*ZU(2,1)*ZUC(2,1)
      Mah2(1:6)= mah(1:6)**2
      masshpm(1) = 0D0; masshpm(2)=dsqrt(MHp2)
c Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021. 
      if(modselval(9).gt.0) then
      OPEN(17,FILE='out.m', STATUS= 'UNKNOWN')
      write(17,*) 'Res=<||>;'
      call mathematica_nxmmatrix(17,'Res[ZH]',ZH,6,6)
      call mathematica_nxmmatrix(17,'Res[ZP]',ZP,2,2)
      call mathematica_nxmmatrix_c(17,'Res[ZUN]',ZUN,2,2)
      call mathematica_nxmmatrix_c(17,'Res[ZN]',ZNeu,3,3)
      call mathematica_nxmmatrix_c(17,'Res[ZU]',ZU,6,6)

      call mathematica_list(17,'Res[Masshh]',mah,6)
      call mathematica_list(17,'Res[MassSu]',MassSu,2)
      call mathematica_list(17,'Res[MassHpm]',masshpm,2)
      call mathematica_list(17,'Res[MassChi]',MNeu,3)
      write(17,*)'Res[mueff]=',mathematica_c(MUE),';'
      
      write(17,*)'Res[InputScale2]=',Q2,';'
      write(17,*)'Res[Tbeta]=',TB,';'
      write(17,*)'Res[lam]=',mathematica_c(la),';'
      write(17,*)'Res[kap]=',mathematica_c(kappa),';'
      write(17,*)'Res[Alam]=',mathematica_c(AlaN),';'
      write(17,*)'Res[Akap]=',mathematica_c(Aka),';'
      write(17,*)'Res[At]=',mathematica_c(ATT),';'
      write(17,*)'Res[mtop]=',MTN,';'
      write(17,*)'Res[vs]=',vs,';'
      write(17,*)'Res[mst1]=',mst1,';'
      write(17,*)'Res[mst2]=',mst2,';'
      write(17,*)'Res[msq]=',sqrt(msq2),';'
      write(17,*)'Res[phix]=',asin(sinphix),';'
      write(17,*)'Res[phiy]=',asin(sinphiy),';'
      write(17,*)'Res[phiz]=',asin(sinphiz),';'
      CLOSE(17)
      endif
c end Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021. 
#ifdef printMT4
c     output for mathematica routines


        OPEN(16,FILE='slha_test.dat', STATUS= 'UNKNOWN')
      Write(16,*)'alambda', AlaN
      Write(16,*) 'akappa',Aka
	  Write(16,*)'kappa',kappa
      Write(16,*) 'sign1,sign2',sign1,sign2
      Write(16,*)'sinphix,cosphix',sinphix,cosphix  
      Write(16,*)'sinphiy,cosphiy',sinphiy,cosphiy   
      Write(16,*)'sinphiz,cosphiz',sinphiz,cosphiz

      Write(16,*)'Mh1=',mah(1)
      Write(16,*)'Mh2=',mah(2)
      Write(16,*)'Mh3=',mah(3)
      Write(16,*)'Mh4=',mah(4)
      Write(16,*)'Mh5=',mah(5)
      Write(16,*)'Goldstone boson Mh6=',mah(6)
      Write(16,*) "ZH(1,i), i=1,6",ZH(1,1),ZH(1,2),ZH(1,3),ZH(1,4),ZH(1,5),ZH(1,6)
      Write(16,*) "ZH(2,i), i=1,6",ZH(2,1),ZH(2,2),ZH(2,3),ZH(2,4),ZH(2,5),ZH(2,6)
      Write(16,*) "ZH(3,i), i=1,6",ZH(3,1),ZH(3,2),ZH(3,3),ZH(3,4),ZH(3,5),ZH(3,6)
      Write(16,*) "ZH(4,i), i=1,6",ZH(4,1),ZH(4,2),ZH(4,3),ZH(4,4),ZH(4,5),ZH(4,6)
      Write(16,*) "ZH(5,i), i=1,6",ZH(5,1),ZH(5,2),ZH(5,3),ZH(5,4),ZH(5,5),ZH(5,6)
      Write(16,*) "ZH(6,i), i=1,6",ZH(6,1),ZH(6,2),ZH(6,3),ZH(6,4),ZH(6,5),ZH(6,6)
      Write(16,*) "MNeu", Mneu(1:3), ZNeu(2,1:3),ZNeu(3,1:3)
      Write(16,*)"phase",Sqrt((MUE/abs(MUE)))/sqrt2
      Write(16,*)"ZUN(1,1),ZUN(1,2)", ZU(1,1),ZU(1,2)
      Write(16,*)"ZUN(2,1),ZUN(2,2)", ZU(2,1),ZU(2,2)
      Write(16,*)'mst1,mst2',mst1,mst2
      Write(16,*)'MSQ(3), MSU(3), ATT, MTN',sqrt(MSQ2), ATT, MTN
#endif


      do k=1,2
       do j=1,2
         do i=1,6
         CSSS(i,j,k)=ZH(i,5)*(ATT*expETA*IC*ZU(k,2)*ZUC(j,1) - 
     -   (IC*ATC*ZU(k,1)*ZUC(j,2))/expETA) + 
     -   vv*ZH(i,3)*(-((CB*laC*ZU(k,2)*ZUC(j,1))/(expVS*sqrt2)) -
     -   (CB*expVS*la*ZU(k,1)*ZUC(j,2))/sqrt2) + 
     -   vv*ZH(i,6)*((CB*IC*laC*ZU(k,2)*ZUC(j,1))/(expVS*sqrt2) - 
     -   (IC*CB*expVS*la*ZU(k,1)*ZUC(j,2))/sqrt2) + 
     -   ZH(i,1)*(-((MUEC*ZU(k,2)*ZUC(j,1))) - 
     -   (MUE*ZU(k,1)*ZUC(j,2))) + 
     -   ZH(i,4)*((IC*MUEC*ZU(k,2)*ZUC(j,1)) - 
     -   (IC*MUE*ZU(k,1)*ZUC(j,2))) + 
     -   ZH(i,2)*(ATT*expETA*ZU(k,2)*ZUC(j,1) +(ATC*ZU(k,1)*ZUC(j,2))
     -   /expETA + M1*(2*ZU(k,1)*ZUC(j,1) + 2*ZU(k,2)*ZUC(j,2)))
         CSSS(i,j,k)=CSSS(i,j,k)*M1/vv/SB
         enddo
        enddo
      enddo

      do i =1,6
        hy(i)=(M1*(ZH(i,2) + IC*ZH(i,5)))/(SB*vv)
        hyc(i)=(M1*(ZH(i,2) - IC*ZH(i,5)))/(SB*vv)
      enddo

      do k=1,2
        do l1=1,2
         do j=1,6
          do i=1,6
        CSSSS(i,j,l1,k)=ZH(i,3)*(-((laC*(ZH(j,1) - IC*ZH(j,4))*
     -  ZU(k,2)*ZUC(l1,1))/(expVS*sqrt2)) - (expVS*la*(ZH(j,1) + 
     -  IC*ZH(j,4))*ZU(k,1)*ZUC(l1,2))/sqrt2) + ZH(i,6)*
     -  (-((laC*((-IC)*ZH(j,1) - ZH(j,4))*ZU(k,2)*ZUC(l1,1))/
     -  (expVS*sqrt2)) - (IC*expVS*la*(ZH(j,1) + IC*ZH(j,4))*ZU(k,1)*
     -  ZUC(l1,2))/sqrt2) + ZH(i,1)*(-((laC*(ZH(j,3) 
     -  - IC*ZH(j,6))*ZU(k,2)*ZUC(l1,1))/(expVS*sqrt2)) - 
     -  (expVS*la*(ZH(j,3) + IC*ZH(j,6))*ZU(k,1)*ZUC(l1,2))/sqrt2) + 
     -  ZH(i,4)*((IC*laC*(ZH(j,3) - IC*ZH(j,6))*ZU(k,2)*ZUC(l1,1))/
     -  (expVS*sqrt2) - (IC*expVS*la*(ZH(j,3) + IC*ZH(j,6))*ZU(k,1)*
     -  ZUC(l1,2))/sqrt2) + M1*ZH(i,2)*((2*ZH(j,2)*ZU(k,1)*ZUC(l1,1))
     -  /(SB*vv) + (2*ZH(j,2)*ZU(k,2)*ZUC(l1,2))/(SB*vv)) + 
     -  M1*ZH(i,5)*((2*ZH(j,5)*ZU(k,1)*ZUC(l1,1))/(SB*vv) + 
     -  (2*ZH(j,5)*ZU(k,2)*ZUC(l1,2))/(SB*vv))
        CSSSS(i,j,l1,k)=CSSSS(i,j,l1,k)*M1/vv/SB
            enddo   
          enddo
        enddo
       enddo 


      do j=1,2
       do i=1,6
       CS1S3SS(i,2,j)=-((M1*(2*CB*M1*(ZH(i,2)+ IC*ZH(i,5))*ZUC(j,1)+ 
     -  expvS*la*SB2*sqrt2*vv*(ZH(i,3) +IC*ZH(i,6))*ZUC(j,2)))/
     -  (SB2*sqrt2*vv2))
       CS1S3SSC(i,2,j)=DCONJG(CS1S3SS(i,2,j))
       CS1S3SS(i,1,j)=-((M12*sqrt2*(ZH(i,2)+Ic*ZH(i,5))*ZUC(j,1))/(SB*
     - vv2))+(CB*expvS*la*M1*(ZH(i,3) + Ic*ZH(i,6))*ZUC(j,2))/(SB*vv)
       CS1S3SSC(i,1,j)=DCONJG(CS1S3SS(i,1,j))
       enddo   
       CS3S13S(2,j)=-((M12*sqrt2*ZUC(j,1))/(TB*vv)) -(ATC*M1*sqrt2*
     .  ZUC(j,2))/(expETA*TB*vv) - (MUE*sqrt2*M1*ZUC(j,2))/vv

       CS3SSS13(1,j)=(2*M12*ZU(j,2)*ZUC(1,2))/(TB2*vv**2)
       CS3SSS13(2,j)=(2*M12*ZU(j,2)*ZUC(2,2))/(TB2*vv**2)
       CS3S13SC(2,j)=DCONJG(CS3S13S(2,j))
c
       CS3S13S(1,j)=-((expETA*M12*SB*sqrt2*ZUC(j,1)+ATC*M1*SB*sqrt2* 
     - ZUC(j,2) -CB*expETA*MUE*sqrt2*M1*ZUC(j,2))/(expETA*SB*vv))
       CS3S13SC(1,j)=DCONJG(CS3S13S(1,j))

       CS3S13S14(j)=-((M12*sqrt2*ZUC(j,1))/(TB*vv)) -(ATC*M1*sqrt2*
     .  ZUC(j,2))/(expETA*TB*vv) - (MUE*sqrt2*M1*ZUC(j,2))/vv

        CS3S14S13(j)=-((M12*sqrt2*ZU(j,1))/(TB*vv)) -(ATT*expETA*M1* 
     .  sqrt2*ZU(j,2))/(TB*vv) - (MUEC*sqrt2*M1*ZU(j,2))/(vv)

      enddo
      CS3SSS14 = (2*M12)/(TB2*vv**2)

      CS3S3S14S14(2,2) = CS3SSS14
      CS3S3S14S14(2,1) = (2*CB*M12)/(SB*vv**2)
      CS3S3S14S14(1,2) =       CS3S3S14S14(2,1)
      CS3S3S14S14(1,1) = (2*M12)/vv**2

      CTBS3R   =-(sqrt2/(sqrt(expETA)*vv))* M1/TB
      CTBS3(2)= -(sqrt2/(sqrt(expETA)*vv))* M1/TB
      CTBS3C(2)= DCONJG(CTBS3(2))
      CTBS3(1)= -sqrt2*M1/vv/sqrt(expETA)
      CTBS3C(1)=DCONJG(CTBS3(1))

      do i=1,3
       do j=1,2
       CF11FSR(i,j)=(Sqrt(expETA)*M1*sqrt2*ZNeu(i,2)*
     .     ZU(j,1))/(SB*vv)
       CF11FSL(i,j)=(M1*sqrt2*ZNeuC(i,2)*ZU(j,2))
     .     /(Sqrt(expETA)*SB*vv)
       CF11FSRC(i,j)=DCONJG(CF11FSR(i,j))
       CF11FSLC(i,j)=DCONJG(CF11FSL(i,j))
       enddo
      enddo

      do l=1,2
       do k=1,2
         do j=1,2
           do i=1,2
            CS13S13S13S13MT4(i,j,k,l)=(2*M12*(ZU(k,2)*ZU(l,1)*ZUC(i,2)
     -      *ZUC(j,1)+ZU(k,1)*ZU(l,2)*ZUC(i,1)*ZUC(j,2)))/(SB2*vv2)

           enddo
         enddo
            CS14S13S14S13MT4(k,l)=(2*M12*ZU(l,2)*ZUC(k,2))/(SB2*vv2)
       enddo
      enddo

      CF12F3S= -((Sqrt(expETA)*M1*sqrt2)/(SB*vv))
      CF12F3SC= DCONJG(CF12F3S)
      do i=1,2
      CF12F4S(i) =-((M1*sqrt2*ZU(i,2))/(SB*vv))
      CF12F4SC(i)= DCONJG(CF12F4S(i))
      enddo

      do k=1,2
        do i=1,2
         do l=1,2
          do j=1,2
          CS3S3S13S13(k,i,l,j)=(2*M12*ZP(k,2)*ZP(i,2)
     &    *ZU(j,2)*ZUC(l,2))/(SB2*vv2)
          enddo
        enddo
       enddo
      enddo

      do gen5=1,6
       do gen4=1,3
         do gen3=1,3
       CFF11S1L(gen3,gen4,gen5)=-((expvS*ZH(gen5,3)*(la*ZNeuC(gen3,2)
     &  *ZNeuC(gen4,1)+la
     &  *ZNeuC(gen3,1)*ZNeuC(gen4,2)-kappa*sqrt2**2*ZNeuC(gen3,3)*ZNeuC
     &  (gen4,3))+expvS*Ic*ZH(gen5,6)*(la*ZNeuC(gen3,2)*ZNeuC(gen4,1)+
     &  la*ZNeuC(gen3,1)*ZNeuC(gen4,2)-kappa*sqrt2**2*ZNeuC(gen3,3)*ZNe
     &  uC(gen4,3))+la*(expETA*ZH(gen5,2)*(ZNeuC(gen3,3)*ZNeuC(gen4,1)
     &  +ZNeuC(gen3,1)*ZNeuC(gen4,3))+expETA*Ic*ZH(gen5,5)*(ZNeuC(gen3
     &  ,3)*ZNeuC(gen4,1)+ZNeuC(gen3,1)*ZNeuC(gen4,3))+(ZH(gen5,1)+Ic*
     &  ZH(gen5,4))*(ZNeuC(gen3,3)*ZNeuC(gen4,2)+ZNeuC(gen3,2)*ZNeuC(g
     &  en4,3))))/sqrt2)

       CFF11S1R(gen3,gen4,gen5)=(-(expETA*ZH(gen5,3)*(laC*ZNeu(gen3,2)
     &  *ZNeu(gen4,1)+laC*
     &  ZNeu(gen3,1)*ZNeu(gen4,2)-kappaC*sqrt2**2*ZNeu(gen3,3)*ZNeu(g
     &  en4,3)))+expETA*Ic*ZH(gen5,6)*(laC*ZNeu(gen3,2)*ZNeu(gen4,1)+l
     &  aC*ZNeu(gen3,1)*ZNeu(gen4,2)-kappaC*sqrt2**2*ZNeu(gen3,3)*ZNeu(
     &  gen4,3))+expvS*laC*(-(ZH(gen5,2)*(ZNeu(gen3,3)*ZNeu(gen4,1)+ZN
     &  eu(gen3,1)*ZNeu(gen4,3)))+Ic*ZH(gen5,5)*(ZNeu(gen3,3)*ZNeu(gen
     &  4,1)+ZNeu(gen3,1)*ZNeu(gen4,3))-expETA*(ZH(gen5,1)-Ic*ZH(gen5
     &  ,4))*(ZNeu(gen3,3)*ZNeu(gen4,2)+ZNeu(gen3,2)*ZNeu(gen4,3))))/(e
     &  xpETA*expvS*sqrt2)
         enddo
       enddo
       CFF12S1L(gen5)=(expvS*la*VChaC*(ZH(gen5,3)+Ic*ZH(gen5,6)))/sqrt2
       CFF12S1R(gen5)=(laC*VCha*(ZH(gen5,3)-Ic*ZH(gen5,6)))/(expvS*sqrt2)
      enddo

c trilinear Higgs self coupling
c for test 

       CS1S1S1IN(1:6,1:6,1:6)=0D0
       CS1S1S1IN(1,1,2)=la2*SB*vv
       CS1S1S1IN(1,1,3)=la2*vS
       CS1S1S1IN(1,2,2)=CB*la2*vv
       CS1S1S1IN(1,2,3)=(-(Ala*expETA**2*expvS**4*la*sqrt2)-2*expETA**
     &  2*expvS*kappaC*la*vS-2*expvS**5*kappa*laC*vS-expvS**2*laC*
     &  sqrt2*DCONJG(Ala))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,2,6)=(IC*(-(Ala*expETA**2*expvS**4*la*sqrt2)+2*
     &  expETA**2*expvS*kappaC*la*vS-2*expvS**5*kappa*laC*vS+expvS**2*
     &  laC*sqrt2*DCONJG(Ala)))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,3,3)=(4*CB*expETA*expvS**3*la2*vv-2*expETA**2*expvS*
     &  kappaC*la*SB*vv-2*expvS**5*kappa*laC*SB*vv)/(4.*expETA*expvS**
     &  3)
       CS1S1S1IN(1,3,5)=(IC*(-(Ala*expETA**2*expvS**4*la*sqrt2)-2*
     &  expETA**2*expvS*kappaC*la*vS+2*expvS**5*kappa*laC*vS+expvS**2*
     &  laC*sqrt2*DCONJG(Ala)))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,3,6)=(IC*(2*expETA**2*expvS*kappaC*la*SB*vv-2*expvS*
     &  *5*kappa*laC*SB*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,5,5)=CB*la2*vv
       CS1S1S1IN(1,5,6)=(Ala*expETA**2*expvS**4*la*sqrt2-2*expETA**2*
     &  expvS*kappaC*la*vS-2*expvS**5*kappa*laC*vS+expvS**2*laC*sqrt2*
     &  DCONJG(Ala))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,6,6)=(4*CB*expETA*expvS**3*la2*vv+2*expETA**2*expvS*
     &  kappaC*la*SB*vv+2*expvS**5*kappa*laC*SB*vv)/(4.*expETA*expvS**
     &  3)
       CS1S1S1IN(2,2,3)=la2*vS
       CS1S1S1IN(2,3,3)=(-2*CB*expETA**2*expvS*kappaC*la*vv-2*CB*expvS*
     &  *5*kappa*laC*vv+4*expETA*expvS**3*la2*SB*vv)/(4.*expETA*expvS**
     &  3)
       CS1S1S1IN(2,3,4)=(IC*(-(Ala*expETA**2*expvS**4*la*sqrt2)-2*
     &  expETA**2*expvS*kappaC*la*vS+2*expvS**5*kappa*laC*vS+expvS**2*
     &  laC*sqrt2*DCONJG(Ala)))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,3,6)=(IC*(2*CB*expETA**2*expvS*kappaC*la*vv-2*CB*
     &  expvS**5*kappa*laC*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,4,4)=la2*SB*vv
       CS1S1S1IN(2,4,6)=(Ala*expETA**2*expvS**4*la*sqrt2-2*expETA**2*
     &  expvS*kappaC*la*vS-2*expvS**5*kappa*laC*vS+expvS**2*laC*sqrt2*
     &  DCONJG(Ala))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,6,6)=(2*CB*expETA**2*expvS*kappaC*la*vv+2*CB*expvS**
     &  5*kappa*laC*vv+4*expETA*expvS**3*la2*SB*vv)/(4.*expETA*expvS**
     &  3)
       CS1S1S1IN(3,3,3)=(2*Aka*expETA*expvS**6*kappa*sqrt2+24*expETA*
     &  expvS**3*kappa*kappaC*vS+2*expETA*kappaC*sqrt2*DCONJG(Aka))
     &  /(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,4)=(IC*(-2*expETA**2*expvS*kappaC*la*SB*vv+2*
     &  expvS**5*kappa*laC*SB*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,5)=(IC*(-2*CB*expETA**2*expvS*kappaC*la*vv+2*CB*
     &  expvS**5*kappa*laC*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,6)=(IC*(2*Aka*expETA*expvS**6*kappa*sqrt2-2*
     &  expETA*kappaC*sqrt2*DCONJG(Aka)))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,4,4)=la2*vS
       CS1S1S1IN(3,4,5)=(Ala*expETA**2*expvS**4*la*sqrt2+2*expETA**2*
     &  expvS*kappaC*la*vS+2*expvS**5*kappa*laC*vS+expvS**2*laC*sqrt2*
     &  DCONJG(Ala))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,4,6)=(-2*expETA**2*expvS*kappaC*la*SB*vv-2*expvS**5*
     &  kappa*laC*SB*vv)/(4.*expETA*expvS**3)
       CS1S1S1IN(3,5,5)=la2*vS
       CS1S1S1IN(3,5,6)=(-2*CB*expETA**2*expvS*kappaC*la*vv-2*CB*expvS*
     &  *5*kappa*laC*vv)/(4.*expETA*expvS**3)
       CS1S1S1IN(3,6,6)=(-2*Aka*expETA*expvS**6*kappa*sqrt2+8*expETA*
     &  expvS**3*kappa*kappaC*vS-2*expETA*kappaC*sqrt2*DCONJG(Aka))
     &  /(4.*expETA*expvS**3)
       CS1S1S1IN(4,5,6)=(IC*(Ala*expETA**2*expvS**4*la*sqrt2-2*expETA**
     &  2*expvS*kappaC*la*vS+2*expvS**5*kappa*laC*vS-expvS**2*laC*
     &  sqrt2*DCONJG(Ala)))/(4.*expETA*expvS**3)
       CS1S1S1IN(4,6,6)=(IC*(2*expETA**2*expvS*kappaC*la*SB*vv-2*expvS*
     &  *5*kappa*laC*SB*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(5,6,6)=(IC*(2*CB*expETA**2*expvS*kappaC*la*vv-2*CB*
     &  expvS**5*kappa*laC*vv))/(4.*expETA*expvS**3)
       CS1S1S1IN(6,6,6)=(IC*(-2*Aka*expETA*expvS**6*kappa*sqrt2+2*
     &  expETA*kappaC*sqrt2*DCONJG(Aka)))/(4.*expETA*expvS**3)

       do i=1,6
         do j=i,6
          do k= j,6
            CS1S1S1IN(k,j,i) = CS1S1S1IN(i,j,k)
            CS1S1S1IN(k,i,j) = CS1S1S1IN(i,j,k)
            CS1S1S1IN(i,k,j) = CS1S1S1IN(i,j,k)
            CS1S1S1IN(j,k,i) = CS1S1S1IN(i,j,k)
            CS1S1S1IN(j,i,k) = CS1S1S1IN(i,j,k)
          enddo
         enddo
        enddo



       do i=1,6
         do j=1,6
          do k= 1,6
            CS1S1S1(i,j,k) =0D0
            do i1=1,6
            do i2=1,6
            do i3=1,6
            CS1S1S1(i,j,k)=CS1S1S1(i,j,k)+ZH(i,i1)*ZH(j,i2)*ZH(k,i3)*CS1S1S1IN(i1,i2,i3)
            enddo
            enddo
            enddo
          enddo
         enddo
        enddo


       CS1S1S1IN(1:6,1:6,1:6)=0D0
       CS1S1S1IN(1,1,2)=-(la2*SB*vv)/2.
       CS1S1S1IN(1,2,1)=-(la2*SB*vv)/2.
       CS1S1S1IN(2,1,2)=-(CB*la2*vv)/2.
       CS1S1S1IN(2,2,1)=-(CB*la2*vv)/2.
       CS1S1S1IN(3,1,1)=la2*vS
       CS1S1S1IN(3,1,2)=(2*Ala*expETA*expvS**3*la*sqrt2+4*expETA*
     &  kappaC*la*vS)/(4.*expvS**2)
       CS1S1S1IN(3,2,1)=(4*expvS**3*kappa*laC*vS+2*sqrt2*DCONJG(Ala)*
     &  DCONJG(la))/(4.*expETA*expvS)
       CS1S1S1IN(3,2,2)=la2*vS
       CS1S1S1IN(4,1,2)=(IC*la2*SB*vv)/2.
       CS1S1S1IN(4,2,1)=-(IC*la2*SB*vv)/2.
       CS1S1S1IN(5,1,2)=(CB*IC*la2*vv)/2.
       CS1S1S1IN(5,2,1)=-(CB*IC*la2*vv)/2.
       CS1S1S1IN(6,1,2)=-(IC*(-2*Ala*expETA**2*expvS**3*la*sqrt2+4*
     &  expETA**2*kappaC*la*vS))/(4.*expETA*expvS**2)
       CS1S1S1IN(6,2,1)=-(IC*(-4*expvS**4*kappa*laC*vS+2*expvS*sqrt2*
     &  DCONJG(Ala)*DCONJG(la)))/(4.*expETA*expvS**2)



       do i=1,6
         do j=1,2
          do k= 1,2

            CS1S3S3(i,j,k) =0D0
            do i1=1,6
            do i2=1,2
            do i3=1,2
            CS1S3S3(i,j,k)=CS1S3S3(i,j,k)+ZH(i,i1)*ZP(j,i2)*ZP(k,i3)*CS1S1S1IN(i1,i2,i3)
            enddo
            enddo
            enddo
          enddo
         enddo
        enddo

       CS1S1S1S1IN(1:6,1:6,1:2,1:2)=0D0
       CS1S1S1S1IN(1,2,1,2)=-la2/2.
       CS1S1S1S1IN(1,2,2,1)=-la2/2.
       CS1S1S1S1IN(1,5,1,2)=(IC*la2)/2.
       CS1S1S1S1IN(1,5,2,1)=-(IC*la2)/2.
       CS1S1S1S1IN(2,4,1,2)=(IC*la2)/2.
       CS1S1S1S1IN(2,4,2,1)=-(IC*la2)/2.
       CS1S1S1S1IN(3,3,1,1)=la2
       CS1S1S1S1IN(3,3,1,2)=(expETA*kappaC*la)/expvS**2
       CS1S1S1S1IN(3,3,2,1)=(expvS**2*kappa*laC)/expETA
       CS1S1S1S1IN(3,3,2,2)=la2
       CS1S1S1S1IN(3,6,1,2)=-((expETA*Ic*kappaC*la)/expvS**2)
       CS1S1S1S1IN(3,6,2,1)=(expvS**2*Ic*kappa*laC)/expETA
       CS1S1S1S1IN(4,5,1,2)=la2/2.
       CS1S1S1S1IN(4,5,2,1)=la2/2.
       CS1S1S1S1IN(6,6,1,1)=la2
       CS1S1S1S1IN(6,6,1,2)=-((expETA*kappaC*la)/expvS**2)
       CS1S1S1S1IN(6,6,2,1)=-((expvS**2*kappa*laC)/expETA)
       CS1S1S1S1IN(6,6,2,2)=la2
       
       do i=1,6
         do j=i,6
       CS1S1S1S1IN(j,i,1:2,1:2)=CS1S1S1S1IN(i,j,1:2,1:2)
	 enddo
       enddo

       do i=1,6
        do i1=1,6
         do j=1,2
          do k= 1,2
            CS1S1S3S3(i,i1,j,k) =0D0
            do gen1=1,6
            do gen2=1,6
            do gen3=1,2
            do gen4=1,2
            CS1S1S3S3(i,i1,j,k)=CS1S1S3S3(i,i1,j,k)+ZH(i,gen1)*ZH(i1,gen2)*
     &          ZP(j,gen3)*ZP(k,gen4)*CS1S1S1S1IN(gen1,gen2,gen3,gen4)
            enddo
            enddo
            enddo
            enddo
           enddo
          enddo
         enddo
        enddo

       CS1S1S1S1IN(1:6,1:6,1:6,1:6)=0D0
       CS1S1S1S1IN(1,1,2,2)=la2
       CS1S1S1S1IN(1,1,3,3)=la2
       CS1S1S1S1IN(1,1,5,5)=la2
       CS1S1S1S1IN(1,1,6,6)=la2
       CS1S1S1S1IN(1,2,3,3)=-(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(1,2,3,6)=(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(1,2,6,6)=(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(1,3,3,5)=-(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(1,3,5,6)=-(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(1,5,6,6)=(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(2,2,3,3)=la2
       CS1S1S1S1IN(2,2,4,4)=la2
       CS1S1S1S1IN(2,2,6,6)=la2
       CS1S1S1S1IN(2,3,3,4)=-(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(2,3,4,6)=-(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(2,4,6,6)=(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(3,3,3,3)=6*kappa*kappaC
       CS1S1S1S1IN(3,3,4,4)=la2
       CS1S1S1S1IN(3,3,4,5)=(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(3,3,5,5)=la2
       CS1S1S1S1IN(3,3,6,6)=2*kappa*kappaC
       CS1S1S1S1IN(3,4,5,6)=-(IC*(expETA**2*kappaC*la-expvS**4*kappa*
     &  laC))/(2.*expETA*expvS**2)
       CS1S1S1S1IN(4,4,5,5)=la2
       CS1S1S1S1IN(4,4,6,6)=la2
       CS1S1S1S1IN(4,5,6,6)=-(expETA**2*kappaC*la+expvS**4*kappa*laC)
     &  /(2.*expETA*expvS**2)
       CS1S1S1S1IN(5,5,6,6)=la2
       CS1S1S1S1IN(6,6,6,6)=6*kappa*kappaC



       do i=1,6
         do i1=i,6
         do j=i1,6
          do k= j,6
            CS1S1S1S1IN(k,i,i1,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i,j,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,j,i,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,j,i1,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i1,i,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i1,j,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(j,i,i1,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i,k,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,k,i,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,k,i1,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i1,i,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i1,k,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(i1,i,j,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,i,k,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,k,i,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,k,j,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,j,i,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,j,k,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(i,i1,k,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,k,i1,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,k,j,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,j,i1,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,j,k,i1) = CS1S1S1S1IN(i,i1,j,k)
          enddo
          enddo
         enddo
        enddo


       do i=1,6
        do i1=1,6
         do j=1,6
          do k= 1,6
            CS1S1S1S1(i,i1,j,k) =0D0
            do gen1=1,6
            do gen2=1,6
            do gen3=1,6
            do gen4=1,6
            CS1S1S1S1(i,i1,j,k)=CS1S1S1S1(i,i1,j,k)+ZH(i,gen1)*ZH(i1,gen2)*
     &          ZH(j,gen3)*ZH(k,gen4)*CS1S1S1S1IN(gen1,gen2,gen3,gen4)
            enddo
            enddo
            enddo
            enddo
           enddo
          enddo
         enddo
        enddo

       do i=1,2
        do i1=1,2
         do j=1,2
          do k= 1,2
       CS3S3S3S3(i,i1,j,k)=la2*ZP(i,2)*ZP(i1,1)*ZP(j,2)*ZP(k,1)+la2*
     &  ZP(i,1)*ZP(i1,2)*ZP(j,2)*ZP(k,1)+la2*ZP(i,2)*ZP(i1,1)*ZP(j,1)*
     &  ZP(k,2) +la2*ZP(i,1)*ZP(i1,2)*ZP(j,1)*ZP(k,2)
           enddo
          enddo
         enddo
        enddo

      do gen3=1,3
       do gen4=1,2
       CF11FSR(gen3,gen4)=(Sqrt(expETA)*M1*sqrt2*ZNeu(gen3,2)*
     .     ZU(gen4,1))/(SB*vv)
       CF11FSL(gen3,gen4)=(M1*sqrt2*ZNeuC(gen3,2)*ZU(gen4,2))
     .     /(Sqrt(expETA)*SB*vv)
       CF11FSRC(gen3,gen4)=DCONJG(CF11FSR(gen3,gen4))
       CF11FSLC(gen3,gen4)=DCONJG(CF11FSL(gen3,gen4))
       enddo
      enddo
c gauge boson couplings
        do i=1,3
       CF11F12V3R(i)=-(sqrt2*ZNeuC(i,2))/(2.*SW)
       CF11F12V3L(i)=(sqrt2*VChaC*ZNeu(i,1))/(2.*SW)
         do j=1,3
       CF11F11V2R(i,j)=-((ZNeu(j,1)*ZNeuC(i,1)-ZNeu(j,2)*
     &  ZNeuC(i,2)))/(2.*CW*SW)
       CF11F11V2L(i,j)=((ZNeu(i,1)*ZNeuC(j,1)-ZNeu(i,2)*
     &  ZNeuC(j,2)))/(2.*CW*SW)
         enddo
         do k=1,2
       CF11F12S3L(i,k)=expETA*la*VChaC*ZNeuC(i,3)*ZP(k,2)
       CF11F12S3R(i,k)=laC*ZNeu(i,3)*ZP(k,1)
         enddo
        enddo

       do i=1,2
       do j=1,2
       CS3S3V3V3(i,j)=-(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2))/(2.*SW2)
       CS3S3V2V2(i,j)=-((CW2-SW2)**2*(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2)))
     &      /(2.*CW2*SW2)
       enddo
       enddo
       do i=1,6
       do j=1,2
       CS1S3V3(i,j)=(Ic*ZH(i,1)*ZP(j,1)+ZH(i,4)*ZP(j,1)+(-(Ic*
     &  ZH(i,2))+ZH(i,5))*ZP(j,2))/(2.*SW)
       CS1S3V3C(i,j)=DCONJG(       CS1S3V3(i,j))
       enddo
       do j=1,6
       CS1S1V3V3(i,j)=-(ZH(i,1)*ZH(j,1)+ZH(i,2)*ZH(j,2)+ZH(i,4)*
     &  ZH(j,4)+ZH(i,5)*ZH(j,5))/(2.*SW2)
       CS1S1V2V2(i,j)=-(ZH(i,1)*ZH(j,1)+ZH(i,2)*ZH(j,2)+ZH(i,4)*ZH(j,4)
     &     +ZH(i,5)*ZH(j,5))/(2.*CW2*SW2)
       CS1S1V2(i,j)=((ZH(i,4)*ZH(j,1)-ZH(i,5)*ZH(j,2)-
     &  ZH(i,1)*ZH(j,4)+ZH(i,2)*ZH(j,5)))/(2.*CW*SW)
       enddo
       enddo
       CF12F12V2R=(-CW2+SW2)/(2.*CW*SW)
       CF12F12V2L=((-CW2+SW2))/(2.*CW*SW)
       CS3S3V2(1,1)= (CW2-SW2)/(2*CW*SW)
       CS3S3V2(2,2)= (CW2-SW2)/(2*CW*SW)
       CS3S3V2(2,1)=0D0 
       CS3S3V2(1,2)=0D0



      do gen3=1,2
        CS14S13V3(gen3)=ZU(gen3,1)/SW/sqrt2
      enddo

		CF4F3V3=1D0/sqrt2/SW/Sqrt(expeta)

	    CS14S14V3V3=-1/(2.*SW2)

        do i=1,2
        do j=1,2
       CS13S13V3V3(i,j)=-(ZU(j,1)*ZUC(i,1))/(2.*SW**2)
        enddo
        enddo

        do i=1,6
        do j=1,6
       CS1S1V3V3(i,j)=-(ZH(i,1)*ZH(j,1)+ZH(i,2)*ZH(j,2)+ZH(i,4)*ZH(j,4)
     &  +ZH(i,5)*ZH(j,5))/(2.*SW**2)
        enddo
        enddo

        do i=1,2
        do j=1,2
       CS3S3V3V3(i,j)=-(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2))/(2.*SW**2)
        enddo
        enddo

        do i=1,2
        do j=1,2
       CS13S13V2(i,j)=((3*CW**2-SW**2)*ZU(j,1)*ZUC(i,1)-4*SW**2*ZU(j,2)
     &  *ZUC(i,2))/(6.*CW*SW)
        enddo
        enddo

       CS14S14V2=-CW/(2.*SW)-SW/(6.*CW)

        do i=1,2
        do j=1,2
       CS13S13V2V2(i,j)=(-((3-4*SW**2)**2*ZU(j,1)*ZUC(i,1))-16*SW**4*
     &  ZU(j,2)*ZUC(i,2))/(18.*CW**2*SW**2)
        enddo
        enddo
       CS14S14V2V2=-(3-2*SW**2)**2/(18.*CW**2*SW**2)


#ifdef printMT4
      Write(16,*) '1)  1 Higgs i two stop j,k couplings, CSSS(i,j,k)'
      Write(16,*) 'CSSS(1,1,1),CSSS(1,1,2),CSSS(1,2,1),CSSS(1,2,2)',CSSS(1,1,1),CSSS(1,1,2),CSSS(1,2,1),CSSS(1,2,2)
      Write(16,*) 'CSSS(4,1,1),CSSS(4,1,2),CSSS(4,2,1),CSSS(4,2,2)',CSSS(4,1,1),CSSS(4,1,2),CSSS(4,2,1),CSSS(4,2,2)
      Write(16,*) 'CSSS(5,1,1),CSSS(5,1,2),CSSS(5,2,1),CSSS(5,2,2)',CSSS(5,1,1),CSSS(5,1,2),CSSS(5,2,1),CSSS(5,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '2)  2 Higgs i,j  two stop l,k couplings, CSSSS(i,j,l,k)'
      Write(16,*) "CSSSS(1,1,1,1),CSSSS(1,1,1,2)",CSSSS(1,1,1,1),CSSSS(1,1,1,2)
      Write(16,*) "CSSSS(1,1,2,1),CSSSS(1,1,2,2)",CSSSS(1,1,2,1),CSSSS(1,1,2,2)
      Write(16,*) "CSSSS(1,2,1,1),CSSSS(1,2,1,2)",CSSSS(1,2,1,1),CSSSS(1,2,1,2)
      Write(16,*) "CSSSS(1,2,2,1),CSSSS(1,2,2,2)",CSSSS(1,2,2,1),CSSSS(1,2,2,2)

      Write(16,*) "CSSSS(4,4,1,1),CSSSS(4,4,1,2)",CSSSS(4,4,1,1),CSSSS(4,4,1,2)
      Write(16,*) "CSSSS(4,4,2,1),CSSSS(4,4,2,2)",CSSSS(4,4,2,1),CSSSS(4,4,2,2)
      Write(16,*) "CSSSS(4,5,1,1),CSSSS(4,5,1,2)",CSSSS(4,5,1,1),CSSSS(4,5,1,2)
      Write(16,*) "CSSSS(4,5,2,1),CSSSS(4,5,2,2)",CSSSS(4,5,2,1),CSSSS(4,5,2,2)


      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '3)   1 Higgs i charged Higgs j, stop k, sbottom CS1S3SS(i,j,k) '
      Write(16,*) 'CS1S3SS(1,1,1),CS1S3SS(1,2,2)',CS1S3SS(1,1,1),CS1S3SS(1,2,2)
      Write(16,*) 'CS1S3SS(2,1,1),CS1S3SS(2,2,2)',CS1S3SS(2,1,1),CS1S3SS(2,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '4)   4 stop, CS13S13S13S13MT4(i,j,k,l)' 
      Write(16,*) 'CS13S13S13S13MT4(1,1,1,1)',CS13S13S13S13MT4(1,1,1,1)
      Write(16,*) 'CS13S13S13S13MT4(1,1,1,2)',CS13S13S13S13MT4(1,1,1,2)
      Write(16,*) 'CS13S13S13S13MT4(1,1,2,2)',CS13S13S13S13MT4(1,1,2,2)
      Write(16,*) 'CS13S13S13S13MT4(1,2,2,2)',CS13S13S13S13MT4(1,2,2,2)
      Write(16,*) 'CS13S13S13S13MT4(2,2,2,2)',CS13S13S13S13MT4(2,2,2,2)
      Write(16,*) 'CS13S13S13S13MT4(1,2,1,2)',CS13S13S13S13MT4(1,2,1,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '5)   2 stop, 2 sbottom, CS14S13S14S13MT4(i,j)' 
      Write(16,*) 'CS14S13S14S13MT4(1,1)',CS14S13S14S13MT4(1,1)
      Write(16,*) 'CS14S13S14S13MT4(1,2)',CS14S13S14S13MT4(1,2)
      Write(16,*) 'CS14S13S14S13MT4(2,2)',CS14S13S14S13MT4(2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '6) 1 neutralino i, top, stop j (right- and left-handed) CF11FSR(i,j), CF11FSL(i,j)'
	  Write(16,*) 'CF11FSR(1,1),CF11FSL(1,1)',CF11FSR(1,1),CF11FSL(1,1)
	  Write(16,*) 'CF11FSR(1,2),CF11FSL(1,2)',CF11FSR(1,2),CF11FSL(1,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '7) 1 chargino top sbottom CF12F3S:', CF12F3S
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '8) 1 chargino bottom, stop CF12F4S(i)'
      Write(16,*) 'CF12F4S(1), CF12F4S(2)',CF12F4S(1), CF12F4S(2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '9) 2 neutralino i,j, Higgs k (left and right) CFF11S1L(i,j,k), CFF11S1R(i,j,k)'
      Write(16,*) 'CFF11S1L(1,1,1),CFF11S1L(2,1,1),CFF11S1L(3,1,1)',CFF11S1L(1,1,1),CFF11S1L(2,1,1),CFF11S1L(3,1,1)
      Write(16,*) 'CFF11S1R(1,1,1),CFF11S1R(2,1,1),CFF11S1R(3,1,1)',CFF11S1R(1,1,1),CFF11S1R(2,1,1),CFF11S1R(3,1,1)
      Write(16,*) 'CFF11S1L(1,1,6),CFF11S1L(2,1,6),CFF11S1L(3,1,6)',CFF11S1L(1,1,6),CFF11S1L(2,1,6),CFF11S1L(3,1,6)
      Write(16,*) 'CFF11S1R(1,1,6),CFF11S1R(2,1,6),CFF11S1R(3,1,6)',CFF11S1R(1,1,6),CFF11S1R(2,1,6),CFF11S1R(3,1,6)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '10) 2 charginoHiggs i (left and right) CFF12S1L(i), CFF12S1R(i)'
      Write(16,*) 'CFF12S1L(1), CFF12S1R(1)',CFF12S1L(1), CFF12S1R(1)
      Write(16,*) 'CFF12S1L(6), CFF12S1R(6)',CFF12S1L(6), CFF12S1R(6)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '11) charged Higgs i, stop j, sbottom, CS3S13S(i,j)'
      Write(16,*) 'CS3S13S(1,1),CS3S13S(1,2)',CS3S13S(1,1),CS3S13S(1,2)
      Write(16,*) 'CS3S13S(2,1),CS3S13S(2,2)',CS3S13S(2,1),CS3S13S(2,2)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '12) charged Higgs i, top bottom CTBS3(i)'
      Write(16,*) 'CTBS3(1),CTBS3(2)',CTBS3(1),CTBS3(2)

      Write(16,*) '-----New couplings-----------------------------------------'
      Write(16,*) '1) Trilinear higgs self couplings'
      Write(16,*) "CS1S1S1(1,1,1:6)",CS1S1S1(1,1,1:6)
      Write(16,*) "CS1S1S1(1,2,1:6)",CS1S1S1(1,2,1:6)
      Write(16,*) "CS1S1S1(1,3,1:6)",CS1S1S1(1,3,1:6)
      Write(16,*) "CS1S1S1(1,4,1:6)",CS1S1S1(1,4,1:6)
      Write(16,*) "CS1S1S1(1,5,1:6)",CS1S1S1(1,5,1:6)
      Write(16,*) "CS1S1S1(1,6,1:6)",CS1S1S1(1,6,1:6)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '2) 1 neutral higgs - 2 charged Higgs'
      Write(16,*) "CS1S3S3(1:6,1,1)",CS1S3S3(1:6,1,1)
      Write(16,*) "CS1S3S3(1:6,1,2)",CS1S3S3(1:6,1,2)
      Write(16,*) "CS1S3S3(1:6,2,2)",CS1S3S3(1:6,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '3) 2 neutral higgs - 2 charged Higgs'
      Write(16,*) "CS1S1S3S3(1:6,1,1,1)",CS1S1S3S3(1:6,1,1,1)
      Write(16,*) "CS1S1S3S3(1:6,1,1,2)",CS1S1S3S3(1:6,1,1,2)
      Write(16,*) "CS1S1S3S3(1:6,1:6,2,2)",CS1S1S3S3(1,1,2,2),CS1S1S3S3(2,2,2,2),
     &             CS1S1S3S3(3,3,2,2),CS1S1S3S3(4,4,2,2),CS1S1S3S3(5,5,2,2),CS1S1S3S3(6,6,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '4) 4 neutral Higgs self couplings'
      Write(16,*) "CS1S1S1S1(1:6,1,1,1)",CS1S1S1S1(1:6,1,1,1)
      Write(16,*) "CS1S1S1S1(1:6,2,1,2)",CS1S1S1S1(1:6,2,1,2)
      Write(16,*) "CS1S1S1S1(1:6,3,2,2)",CS1S1S1S1(1:6,3,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '5) 4 charged Higgs self couplings'
      Write(16,*) "CS3S3S3S3(1:2,1,1,1)",CS3S3S3S3(1:2,1,1,1)
      Write(16,*) "CS3S3S3S3(1:2,2,1,2)",CS3S3S3S3(1:2,2,1,2)
      Write(16,*) "CS3S3S3S3(1:2,2,2,2)",CS3S3S3S3(1:2,2,2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '6) 1 neutralino, chargino charged Higgs couplings'
      Write(16,*) " CF11F12S3L(1:3,2)", CF11F12S3L(1:3,2)
      Write(16,*) "CF11F12S3R(1:3,2)",CF11F12S3R(1:3,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '7) 1 neutralino, chargino W couplings'
      Write(16,*) " CF11F12V3R(1:3)", CF11F12V3R(1:3)
      Write(16,*) "CF11F12V3L(1:3)",CF11F12V3L(1:3)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '8) 2 charged Higgs  2 W couplings'
      Write(16,*) "  CS3S3V3V3(2,2)", CS3S3V3V3(2,2)
      Write(16,*) " CS3S3V3V3(1,1)", CS3S3V3V3(1,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '9) 2 neutral Higgs  2 W couplings'
      Write(16,*) "  CS1S1V3V3(1:6,1)", CS1S1V3V3(1:6,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '10)  neutral Higgs charged Higgs  W couplings'
      Write(16,*) "  CS1S3V3(1:6,1)", CS1S3V3(1:6,1)
      Write(16,*) "  CS1S3V3(1:6,2)", CS1S3V3(1:6,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '11) 2 neutralino  Z couplings'
      Write(16,*) " CF11F11V2R(1:3,1)", CF11F11V2R(1:3,1)
      Write(16,*) " CF11F11V2L(1:3,2)", CF11F11V2L(1:3,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '12) 2 charged Higgs  2 Z couplings'
      Write(16,*) " CS3S3V2V2(1:2,1)", CS3S3V2V2(1:2,1)
      Write(16,*) " CS3S3V2V2(1:2,2)", CS3S3V2V2(1:2,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '13) 2 neutral Higgs  2 Z couplings'
      Write(16,*) " CS1S1V2V2(1:6,1)", CS1S1V2V2(1:6,1)
      Write(16,*) " CS1S1V2V2(1:6,2)", CS1S1V2V2(1:6,2)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '14) 2 chargino  Z couplings'
      Write(16,*) "  CF12F12V2R", CF12F12V2R
      Write(16,*) "  CF12F12V2L", CF12F12V2L
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '15) 2 neutral Higgs  Z couplings'
      Write(16,*) " CS1S1V2(1:6,1)", CS1S1V2(1:6,1)
      Write(16,*) " CS1S1V2(1:6,2)", CS1S1V2(1:6,2)


#endif

      end subroutine couplings_atlambda



c**********************************************************************
      subroutine LIBLOOP_atlambda
      implicit none
      integer i,j,k,gen3,gen4,gen5,h1
      double precision A0T,AEpT,B0T,BEpT,C0T,CEpT,MHpm2(2),K0T0N,
     .     MahN(6),a0th,aeth,M24,M44,mahs,FDMH(4,2,2),I0A,coefC(10),
     .     coef(5)
      double complex K0T0



#include "TLMT4.h"
#include "TLMT4lambda.h"

      ak0t0x3= K0T0(0D0,M22,M42,Q2)
      ak0t05=K0T0(0D0, M22, MSQ2, Q2)
      ak0t06=K0T0(0D0, M42, MSQ2, Q2)

      LogMSQ2=Log(MSQ2/Q2)
      logcha=Log(MCha2/Q2)
      A0T1=A0T(M12,Q2);   AET1=AEpT(M12,Q2)
      A0T2=A0T(M22,Q2);   AET2=AEpT(M22,Q2)
      A0T4=A0T(M42,Q2);   AET4=AEpT(M42,Q2)
      aA0TN(1)=A0T2;  aAETN(1)=AET2
      aA0TN(2)=A0T4;  aAETN(2)=AET4
      M24=M22**2;M44=M42**2

      do i=1,2
       do j=1,2
        aB0T1(j,i)= B0T(MassSu2(i),MassSu2(j),Q2)
        aBEpT1(j,i)=BEpT(MassSu2(i),MassSu2(j),Q2)
       enddo
      enddo

      do i=1,2
       do j=1,2
        Do k=1,2
       aC0T(i,j,k)=C0T(MassSu2(i),MassSu2(j),MassSu2(k),Q2)
       aCEpT(i,j,k)=CEpT(MassSu2(i),MassSu2(j),MassSu2(k),Q2)
        enddo
       enddo
      enddo

      do j=1,2
        aC0TH2(j)  = C0T(MSQ2,MSQ2,MassSu2(j),Q2)
        aCEpTH2(j) = CEpT(MSQ2,MSQ2,MassSu2(j),Q2)
        aB0TH1(j)  = B0T(MSQ2, MassSu2(j),Q2)
        aBEpTH1(j) = BEpT(MSQ2, MassSu2(j),Q2)
        do i=1,2
        aC0TH1(i,j)  = C0T(MSQ2,MassSu2(i),MassSu2(j),Q2)
        aCEpTH1(i,j) = CEpT(MSQ2,MassSu2(i),MassSu2(j),Q2)
        enddo
      enddo
      A0T5= A0T(MSQ2,Q2);      AET5= AEpT(MSQ2,Q2)
      aB0T5= B0T(MSQ2,MSQ2,Q2); aBEpT5=BEpT(MSQ2,MSQ2,Q2)
      aB0TN1=B0T(M12, M12, Q2); aBEpTN1=BEpT(M12,M12,Q2)


      do i=ihiggs,ihiggse
        A0TNH(i) =A0T(Mah2(i),Q2)
        AETNH(i)=AEpT(Mah2(i),Q2)
      enddo   

      A0THp= A0T(MHp2N,Q2);      AETHp= AEpT(MHp2N,Q2)
      A0THpN(2)=A0THp; AETHpN(2)=AETHp
      A0THpN(1)=0D0; AETHpN(1)=0D0


      do i=ihiggs,ihiggse
        Mahs=Mah2(i); a0th=A0TNH(i); aeth=AETNH(i)
        call NHiggs_TR(Mahs,M22,M42,aeth,aet2,aet4,a0th,a0t2,a0t4,
     .    Q2,FDMH)
       do gen4=1,2
        do gen5=1,2
       FD111MH(i,gen4,gen5)=FDMH(1,gen4,gen5)
       FD211MH(i,gen4,gen5)=FDMH(2,gen4,gen5)
       FD311MH(i,gen4,gen5)=FDMH(3,gen4,gen5)
       FD221MH(i,gen4,gen5)=FDMH(4,gen4,gen5)
        enddo
       enddo
      enddo
       Mahs=MHp2N; a0th=A0THp; aeth=AETHp
        call NHiggs_TR(Mahs,M22,MSQ2,aeth,aet2,aet5,a0th,a0t2,a0t5,
     .    Q2,FDMH)
        FD111CH(2,1)=FDMH(1,1,2)
        FD211CH(2,1)=FDMH(2,1,2)
        FD311CH(2,1)=FDMH(3,1,2)
        call NHiggs_TR(Mahs,M42,MSQ2,aeth,aet4,aet5,a0th,a0t4,a0t5,
     .    Q2,FDMH)
        FD111CH(2,2)=FDMH(1,1,2)
        FD211CH(2,2)=FDMH(2,1,2)
        FD311CH(2,2)=FDMH(3,1,2)
        FD111CH(1,1)=K0T0N(0D0,M22,MSQ2,0D0,aet2,aet5,Q2)
        FD211CH(1,1)=(a0t5*M22 +aet5*M22+M24+a0t2*(a0t5+2*M22-MSQ2)+ 
     -    aet2*MSQ2 + M22*MSQ2 + M22*FD111CH(1,1))/
     -  (M24 - M22*MSQ2)
        FD311CH(1,1)=-((M22 - MSQ2)*(M22*(a0t5 + M22 - MSQ2) + 
     -       a0t2*(-2*M22 + MSQ2)))/
     -  (2.*M24*(M22 - MSQ2)**2)

        FD111CH(1,2)=K0T0N(0D0,M42,MSQ2,0D0,aet4,aet5,Q2)
        FD211CH(1,2)=(a0t5*M42 +aet5*M42+M44+a0t4*(a0t5+2*M42-MSQ2)+ 
     -    aet4*MSQ2 + M42*MSQ2 + M42*FD111CH(1,2))/
     -  (M44 - M42*MSQ2)
        FD311CH(1,2)=-((M42 - MSQ2)*(M42*(a0t5 + M42 - MSQ2) + 
     -       a0t4*(-2*M42 + MSQ2)))/
     -  (2.*M44*(M42 - MSQ2)**2)

        do h1=ihiggs,ihiggse
         if (Mah2(h1).gt.0)then
         logmha(h1)=Log(Mah2(h1)/Q2)
         else
         logmha(h1)=0D0
         endif
        enddo

        logmhpm(1)=0
	    logmhpm(2)=Log(MHp2N/Q2)
      do i=ihiggs,ihiggse
       Mahs=Mah2(i); a0th=A0TNH(i); aeth=AETNH(i)
       call NHiggs_TB(M12,Mahs,aet1,aeth,a0t1,a0th,Q2,coef)
       do gen4=1,5
        FDTH(i,gen4)=coef(gen4)
       enddo 
      enddo

      call CHiggs_TB(M12,MHp2N,aet1,AETHp,a0t1,A0THp,Q2,coef)
      FDTCH(2,1)=coef(1);FDTCH(2,2)=coef(2)
      FDTCH(1,1)=coef(3);FDTCH(1,2)=coef(4)


      do i=ineu,ineue
      mahs=MNeu2(i)
      A0TNeu(i)= A0T(mahs,Q2);  AETNeu(i)= AEpT(mahs,Q2)
      a0th=A0TNeu(i);  aeth=AETNeu(i)
      call FD111C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            FD111Chi(1:10,i,1),0)

      call FD211C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            FD211Chi(1:10,i,1),0)

      call FD211C(M12,M22,mahs,a0t1,a0t2,a0th,aet1,aet2,aeth,Q2,
     .            FD211ChiM1(1:10,i,1),0)

      call FD221C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            FD221Chi(1:10,i,1),0)

      call FD311C(M22,M12,mahs,a0t2,a0t1,a0th,aet2,aet1,aeth,Q2,
     .            FD311Chi(1:10,i,1),0)

      call FD311C(M12,M22,mahs,a0t1,a0t2,a0th,aet1,aet2,aeth,Q2,
     .            FD311ChiM1(1:10,i,1),0)

      call FD111C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            FD111Chi(1:10,i,2),0)

      call FD211C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            FD211Chi(1:10,i,2),0)

      call FD211C(M12,M42,mahs,a0t1,a0t4,a0th,aet1,aet4,aeth,Q2,
     .            FD211ChiM1(1:10,i,2),0)

      call FD221C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            FD221Chi(1:10,i,2),0)

      call FD311C(M42,M12,mahs,a0t4,a0t1,a0th,aet4,aet1,aeth,Q2,
     .            FD311Chi(1:10,i,2),0)

      call FD311C(M12,M42,mahs,a0t1,a0t4,a0th,aet1,aet4,aeth,Q2,
     .            FD311ChiM1(1:10,i,2),0)


      call FD221C(Mahs,M12,M22,a0th,a0t1,a0t2,aeth,aet1,aet2,Q2,
     .            FD221ChiM1MS(1:10,i,1),0)

      call FD221C(Mahs,M12,M42,a0th,a0t1,a0t4,aeth,aet1,aet4,Q2,
     .            FD221ChiM1MS(1:10,i,2),0)

      call FD211C(Mahs,M22,M12,a0th,a0t2,a0t1,aeth,aet2,aet1,Q2,
     .            FD211ChiMSM1(1:10,i,1),0)

      call FD211C(Mahs,M42,M12,Q2,a0th,a0t4,a0t1,aeth,aet4,aet1,
     .            FD211ChiMSM1(1:10,i,2),0)

      call FD221C(Mahs,M22,M12,a0th,a0t2,a0t1,aeth,aet2,aet1,Q2,
     .            FD221ChiMSM1(1:10,i,1),0)

      call FD221C(Mahs,M42,M12,a0th,a0t4,a0t1,aeth,aet4,aet1,Q2,
     .            FD221ChiMSM1(1:10,i,2),0)

      call FD311C(Mahs,M12,M22,a0th,a0t1,a0t2,aeth,aet1,aet2,Q2,
     .            FD311ChiM1MS(1:10,i,1),0)

      call FD311C(Mahs,M12,M42,a0th,a0t1,a0t4,aeth,aet1,aet4,Q2,
     .            FD311ChiM1MS(1:10,i,2),0)

      call FD111C(MSQ2,M12,Mahs,a0t5,a0t1,a0th,aet5,aet1,aeth,Q2,
     .            FD111Chi5M1(1:10,i),0)

      enddo

      a0tcha=A0T(MCha2,Q2); aetcha=AEpT(MCha2,Q2)
      call FD311C(M12,MSQ2,MCha2,a0t1,a0t5,a0tcha,aet1,aet5,aetcha,Q2,
     .            FD311Cha(1:10),0)

      call FD311C(MCha2,M12,MSQ2,a0tcha,a0t1,a0t5,aetcha,aet1,aet5,Q2,
     .            FD311ChaM1(1:10),0)

      call FD221C(MCha2,M12,MSQ2,a0tcha,a0t1,a0t5,aetcha,aet1,aet5,Q2,
     .            FD221Cha(1:10),0)

      call FD211C(M12,MSQ2,MCha2,a0t1,a0t5,a0tcha,aet1,aet5,aetcha,Q2,
     .            FD211Cha(1:10),0)

      call FD211C(MSQ2,M12,MCha2,a0t5,a0t1,a0tcha,aet5,aet1,aetcha,Q2,
     .            FD211Cha5(1:10),0)

      call ChaFD(Mcha2,M22,M42,a0tcha,aetcha,a0t2,aet2,a0t4,aet4,Q2,
     .           coefc)
      do i=1,7
      FDCha(i)=coefc(i)
      enddo
c end fo atat contribution

c Alphat_lambda 2019
       regmass=sqrt(regmass2)
       logx=Log(regmass2/Q2)
       logx2=logx**2
       xinverse=1/Regmass2
       xinverse2=1/Regmass2**2
     
       B0IR= -Log(regMass2)
       B0epsIR= Log(regMass2)**2/2D0  
       C0IR=-1/2D0/regMass2
       C0epsIR=0D0   
       B1IR=1/2D0*Log(regMass2)
       B1epsIR= - 1/4D0*Log(regMass2)**2
       Do i=1,6 
         do j=1,6
          if ((mah2(i)+mah2(j)) .gt.0D0)then
          aB0TNH(i,j)= B0T(mah2(i),mah2(j),Q2)
          aBEpTNH(i,j)= BEpT(mah2(i),mah2(j),Q2)
          else
          aB0TNH(i,j)= B0T(regMass2,regMass2,Q2)
          aBEpTNH(i,j)= BEpT(regMass2,regMass2,Q2)
          endif
         enddo
         do j=1,2
          if ((mah2(i)+masshpm(j)) .gt.0D0)then
         aB0TNCH(i,j)=B0T(mah2(i),masshpm(j)**2,Q2)
         aBEpTNCH(i,j)=BEpT(mah2(i),masshpm(j)**2,Q2)
          else
          aB0TNCH(i,j)= B0T(regMass2,regMass2,Q2)
          aBEpTNCH(i,j)=BEpT(regMass2,regMass2,Q2)
          endif
         enddo

       Enddo

       do i=1,2
         do j=1,2
          if ((masshpm(i)+masshpm(j)) .gt.0D0)then
         aB0TCH(i,j)=B0T(masshpm(i)**2,masshpm(j)**2,Q2)
         aBEpTCH(i,j)=BEpT(masshpm(i)**2,masshpm(j)**2,Q2)
          else
          aB0TCH(i,j)= B0T(regMass2,regMass2,Q2)
          aBEpTCH(i,j)=BEpT(regMass2,regMass2,Q2)
          endif
         enddo
       enddo


       Do INH2=1,6
         call FD111C(M12,Mah2(INH2),M12,a0t1,A0TNH(INH2),a0t1,aet1,
     &        aetnh(INH2),aet1,Q2,FD111THT(1:10,INH2),0)
       enddo 
       Do INH2=1,6
         call FD111C(MCha2,Mah2(INH2),MCha2,a0tcha,A0TNH(INH2),a0tcha
     &        ,aetcha,aetnh(INH2),aetcha,Q2,FD111ChaHCha(1:10,INH2),0)
       enddo 
       Do INe2=1,3
       Do INH2=1,6
       Do INe3=1,3
         call FD111C(Mneu2(INe2),Mah2(INH2),Mneu2(INe3),A0TNeu(INe2)
     &      ,A0TNH(INH2),A0TNeu(INe3),AeTNeu(INe2),AeTNH(INH2),
     &      AeTNeu(INe3),Q2,FD111ChiHChi(1:10,INe2,INH2,INe3),0)
       enddo 
       enddo 
       enddo 
       Do INH2=1,6
       Do INH3=1,6
       Do INH4=1,6
         call FD111C(Mah2(INH2),Mah2(INH3),Mah2(INH4),A0TNH(INH2),
     &       A0TNH(INH3),A0TNH(INH4),AeTNH(INH2),AeTNH(INH3),
     &       AeTNH(INH4),Q2,FD111HHH(1:10,INH2,INH3,INH4),0)
       enddo 
       enddo 
       enddo 
       Do INH2=1,6
       Do ICH2=1,2
       Do ICH3=1,2
         call FD111C(Mah2(INH2),MassHpm(ICH2)**2,MassHpm(ICH3)**2,
     &        A0TNH(INH2),A0THpN(ICH2),A0THpN(ICH3),AeTNH(INH2),
     &       AeTHpN(ICH2),AeTHpN(ICH3),Q2,FD111HHpHp(1:10,INH2,ICH2,ICH3),0)
       enddo 
       enddo 
       enddo 
       Do INH2=1,6
       Do ISt2=1,2
       Do ISt3=1,2
         call FD111C(Mah2(INH2),MassSu2(ISt2),MassSu2(ISt3),A0TNH(INH2),
     &       aA0TN(ISt2),aA0TN(ISt3),AeTNH(INH2),aAeTN(ISt2),aAeTN(ISt3),Q2,
     &       FD111HSTST(1:10,INH2, ISt2, ISt3),0)
       enddo 
       enddo 
       enddo

       Do INH2=1,5
         call FD211C(Mah2(INH2),M12,M12,A0TNH(INH2),a0t1,a0t1,AeTNH(INH2),aet1,
     &        aet2,Q2,FD211HTT(1:10,INH2),0)
       enddo 
         INH2=6
         call FD211X(Mah2(INH2),M12,M12,A0TNH(INH2),a0t1,a0t1,AeTNH(INH2),aet1,
     &        aet2,Q2,FD211HTT(1:10,INH2),0)
       Do INH2=1,5
         call FD211C(Mah2(INH2),MCha2,MCha2,A0TNH(INH2),a0tcha,a0tcha,
     &        AeTNH(INH2),aetcha,aetcha,Q2,FD211HChaCha(1:10,INH2),0)
       enddo 
         INH2=6
         call FD211X(Mah2(INH2),MCha2,MCha2,A0TNH(INH2),a0tcha,a0tcha,
     &        AeTNH(INH2),aetcha,aetcha,Q2,FD211HChaCha(1:10,INH2),0)

       Do INe2=1,3
       Do INe3=1,3
       Do INH2=1,5
         call FD211C(Mah2(INH2),Mneu2(INe2),Mneu2(INe3),A0TNH(INH2),A0TNeu(INe2),
     &        A0TNeu(INe3),AeTNH(INH2),AeTNeu(INe2),
     &        AeTNeu(INe3),Q2,FD211HNeuNeu(1:10,INH2, INe2, INe3),0)
       enddo 
         INH2=6
         call FD211X(Mah2(INH2),Mneu2(INe2),Mneu2(INe3),A0TNH(INH2),A0TNeu(INe2),
     &        A0TNeu(INe3),AeTNH(INH2),AeTNeu(INe2),
     &        AeTNeu(INe3),Q2,FD211HNeuNeu(1:10,INH2, INe2, INe3),0)
       enddo 
       enddo 

       Do INH3=1,6
       Do INH4=1,6
       Do INH2=1,5
         call FD211C(Mah2(INH2),Mah2(INH3),Mah2(INH4),A0TNH(INH2),
     &       A0TNH(INH3),A0TNH(INH4),AeTNH(INH2),AeTNH(INH3),
     &       AeTNH(INH4),Q2,FD211HHH(1:10,INH2, INH3, INH4),0)
       enddo 
         INH2=6
         call FD211X(Mah2(INH2),Mah2(INH3),Mah2(INH4),A0TNH(INH2),
     &       A0TNH(INH3),A0TNH(INH4),AeTNH(INH2),AeTNH(INH3),
     &       AeTNH(INH4),Q2,FD211HHH(1:10,INH2, INH3, INH4),0)
       enddo 
       enddo 

       Do ICH2=1,2
       Do ICH3=1,2
       Do INH2=1,5
         call FD211C(Mah2(INH2),MassHpm(ICH2)**2,MassHpm(ICH3)**2,
     &        A0TNH(INH2),A0THpN(ICH2),A0THpN(ICH3),AeTNH(INH2),
     &        AeTHpN(ICH2),AeTHpN(ICH3),Q2,FD211HHpHp(1:10,INH2, ICH2, ICH3),0)
       enddo
         INH2=6
         call FD211X(Mah2(INH2),MassHpm(ICH2)**2,MassHpm(ICH3)**2,
     &        A0TNH(INH2),A0THpN(ICH2),A0THpN(ICH3),AeTNH(INH2),
     &        AeTHpN(ICH2),AeTHpN(ICH3),Q2,FD211HHpHp(1:10,INH2, ICH2, ICH3),0) 
       enddo 
       enddo 

       Do ISt2=1,2
       Do ISt3=1,2
       Do INH2=1,5
         call FD211C(Mah2(INH2),MassSu2(ISt2),MassSu2(ISt3),A0TNH(INH2),
     &        aA0TN(ISt2),aA0TN(ISt3),AeTNH(INH2),aAeTN(ISt2),aAeTN(ISt3)
     &       ,Q2,FD211HSTST(1:10,INH2, ISt2, ISt3),0)
       enddo 
        INH2=6
         call FD211X(Mah2(INH2),MassSu2(ISt2),MassSu2(ISt3),A0TNH(INH2),
     &        aA0TN(ISt2),aA0TN(ISt3),AeTNH(INH2),aAeTN(ISt2),aAeTN(ISt3)
     &       ,Q2,FD211HSTST(1:10,INH2, ISt2, ISt3),0)
       enddo 
       enddo



       Do INe2=1,3
       Do ICH2=1,2
         call FD111C(Mneu2(INe2),MassHpm(ICH2)**2,MCha2,A0TNeu(INe2),A0THpN(ICH2)
     &       ,a0tcha,AeTNeu(INe2),AeTHpN(ICH2),aetcha,Q2,FD111ChiHpCha(1:10,INe2,ICH2),0)
       enddo 
       enddo 

       Do ICH2=1,2
       Do ISt2=1,2
         call FD111C(MassHpm(ICH2)**2,MassSu2(ISt2),MSQ2,A0THpN(ICH2),aA0TN(ISt2),
     &    a0t5,AeTHpN(ICH2),aAeTN(ISt2),aet5,Q2,FD111HpSTSB(1:10,ICH2,ISt2),0)
       enddo 
       enddo 
       Do ICH2=1,2
         call FD111C(M12,MassHpm(ICH2)**2,0D0,a0t1,A0THpN(ICH2),0D0,aet1,AeTHpN(ICH2)
     &    ,0D0,Q2,FD111THpB(1:10,ICH2),0)
       enddo 

 
       Do INe2=1,3
       ICH2=2
         call FD211C(MassHpm(ICH2)**2,Mneu2(INe2),MCha2,A0THpN(ICH2),A0TNeu(INe2),
     &   a0tcha,AeTHpN(ICH2),Aetneu(INe2),aetcha,Q2,FD211HpChiCha(1:10,ICH2, INe2),0)     
       ICH2=1
         call FD211X(MassHpm(ICH2)**2,Mneu2(INe2),MCha2,A0THpN(ICH2),A0TNeu(INe2),
     &   a0tcha,AeTHpN(ICH2),Aetneu(INe2),aetcha,Q2,FD211HpChiCha(1:10,ICH2, INe2),0)
       enddo 

       Do INH2=1,6
       Do ICH3=1,2
       ICH2=2
         call FD211C(MassHpm(ICH2)**2,Mah2(INH2),MassHpm(ICH3)**2,A0THpN(ICH2),A0TNH(INH2)
     &    ,A0THpN(ICH3),AeTHpN(ICH2),AeTNH(INH2),AeTHpN(ICH3),Q2,FD211HpHHp(1:10,ICH2,INH2,ICH3),0) 

       ICH2=1
         call FD211X(MassHpm(ICH2)**2,Mah2(INH2),MassHpm(ICH3)**2,A0THpN(ICH2),A0TNH(INH2)
     &    ,A0THpN(ICH3),AeTHpN(ICH2),AeTNH(INH2),AeTHpN(ICH3),Q2,FD211HpHHp(1:10,ICH2,INH2,ICH3),0) 
       enddo 
       enddo 

       Do ISt2=1,2
       ICH2=2
         call FD211C(MassHpm(ICH2)**2,MassSu2(ISt2),MSQ2,A0THpN(ICH2),aA0TN(ISt2),
     &    a0t5,AeTHpN(ICH2),aAeTN(ISt2),aet5,Q2,FD211HpSTSB(1:10,ICH2,ISt2),0)
       ICH2=1
         call FD211X(MassHpm(ICH2)**2,MassSu2(ISt2),MSQ2,A0THpN(ICH2),aA0TN(ISt2),
     &    a0t5,AeTHpN(ICH2),aAeTN(ISt2),aet5,Q2,FD211HpSTSB(1:10,ICH2,ISt2),0)
       enddo 


       ICH2=2
         call FD211C(MassHpm(ICH2)**2,M12,0D0,A0THpN(ICH2),a0t1,0D0,
     &       AeTHpN(ICH2),aet1,0D0,Q2,FD211HpTB(1:10,ICH2),0)

       ICH2=1
         call FD211Z(MassHpm(ICH2)**2,M12,0D0,A0THpN(ICH2),a0t1,0D0,
     &       AeTHpN(ICH2),aet1,0D0,Q2,FD211HpTB(1:10,ICH2),0)


       Do INe2=1,3
       Do ISt2=1,2
         call FD111C(M12,Mneu2(INe2),MassSu2(ISt2),a0t1,A0TNeu(INe2),aA0TN(ISt2),aet1
     &       ,AeTNeu(INe2),aAeTN(ISt2),Q2,FD111TChiST(1:10,INe2, ISt2),0)
       enddo 
       enddo 
       Do INe2=1,3
       Do ICH2=1,2
         call FD111C(MCha2,Mneu2(INe2),MassHpm(ICH2)**2,a0tcha,A0TNeu(INe2),A0THpN(ICH2)
     &    ,aetcha,AeTNeu(INe2),AeTHpN(ICH2),Q2,FD111ChaChiHp(1:10,INe2, ICH2),0)
       enddo 
       enddo 
       Do INe2=1,3
       Do INe3=1,3
       Do INH2=1,6
         call FD111C(Mneu2(INe2),Mneu2(INe3),Mah2(INH2),A0TNeu(INe2),A0TNeu(INe3),A0TNH(INH2),
     &         AeTNeu(INe2),AeTNeu(INe3),AeTNH(INH2),Q2,FD111ChiChiH(1:10,INe2, INe3, INH2),0)
       enddo 
       enddo 
       enddo 
       Do INe2=1,3
       Do ISt2=1,2
         call FD111C(Mneu2(INe2),M12,MassSu2(ISt2),A0TNeu(INe2),a0t1,aA0TN(ISt2)
     &         ,AeTNeu(INe2),aet1,aAeTN(ISt2),Q2,FD111ChiTST(1:10,INe2, ISt2),0)
       enddo 
       enddo 
       Do INe2=1,3
       Do ICH2=1,2
         call FD111C(Mneu2(INe2),MCha2,MassHpm(ICH2)**2,A0TNeu(INe2),a0tcha,A0THpN(ICH2),
     &        AeTNeu(INe2),aetcha,AeTHpN(ICH2),Q2,FD111ChiChaHp(1:10,INe2, ICH2),0)
       enddo 
       enddo

       Do INe3=1,3
       Do INe2=1,3
       Do INH2=1,6
         call FD211C(Mneu2(INe3),Mneu2(INe2),Mah2(INH2),A0TNeu(INe3),A0TNeu(INe2),A0TNH(INH2),
     &         AeTNeu(INe3),AeTNeu(INe2),AeTNH(INH2),Q2,FD211ChiCHiH(1:10,INe3, INe2, INH2),0)
       enddo 
       enddo 
       enddo 

       Do INe2=1,3
       Do ISt2=1,2
         call FD211C(Mneu2(INe2),M12,MassSu2(ISt2),A0TNeu(INe2),a0t1,aA0TN(ISt2)
     &         ,AeTNeu(INe2),aet1,aAeTN(ISt2),Q2,FD211ChiTST(1:10,INe2, ISt2),0)
       enddo 
       enddo 
       Do INe2=1,3
       Do ICH2=1,2
         call FD211C(Mneu2(INe2),MCha2,MassHpm(ICH2)**2,A0TNeu(INe2),a0tcha,A0THpN(ICH2),
     &        AeTNeu(INe2),aetcha,AeTHpN(ICH2),Q2,FD211ChiChaHpm(1:10,INe2, ICH2),0)
       enddo 
       enddo  

       Do INH2=1,6
         call FD211C(MCha2,MCha2,Mah2(INH2),a0tcha,a0tcha,a0tNH(INH2),
     &        aetcha,aetcha,aetNH(INH2),Q2,FD211ChaChaH(1:10,INH2),0)
       enddo 
       Do INe2=1,3
       Do ICH2=1,2
         call FD211C(MCha2,Mneu2(INe2),MassHpm(ICH2)**2,a0tcha,a0tNeu(INe2),a0tHpN(ICH2),
     &        aetcha,aetNeu(INe2),aetHpN(ICH2),Q2,FD211ChaChiHp(1:10,INe2, ICH2),0)
       enddo 
       enddo 
       Do ISt2=1,2
         call FD211C(MCha2,0D0,MassSu2(ISt2),a0tcha,0D0,aA0TN(ISt2),aetcha,0D0,
     &        aAeTN(ISt2),Q2,FD211ChaBST(1:10,ISt2),0)
       enddo 
         call FD211C(MCha2,M12,MSQ2,a0tcha,a0t1,a0t5,aetcha,aet1,aet5,
     &        Q2,FD211ChaTSB(1:10),0)


       Do INH2=1,6
       Do INH3=1,6
       Do INH4=1,6
         if (Mah2(INH2)+Mah2(INH3)+Mah2(INH4) .gt.1D-15)then
        aC0TNH(INH2, INH3, INH4)=C0T(Mah2(INH2),Mah2(INH3),Mah2(INH4),Q2)
        aCEpTNH(INH2, INH3, INH4)=CEpT(Mah2(INH2),Mah2(INH3),Mah2(INH4),Q2)
         else
        aC0TNH(INH2, INH3, INH4)=C0T(regMass2,regMass2,regMass2,Q2)
        aCEpTNH(INH2, INH3, INH4)=CEpT(regMass2,regMass2,regMass2,Q2)
         endif
       enddo 
       enddo 
       enddo 

       Do ICH2=1,2
       Do ICH3=1,2
       Do ICH4=1,2
        if (MassHpm(ICH2)+MassHpm(ICH3)+MassHpm(ICH4) .gt.1D-15)then
        aC0TCH(ICH2,ICH3,ICH4)=C0T(MassHpm(ICH2)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,Q2)
        aCEpTCH(ICH2,ICH3,ICH4)=CEPT(MassHpm(ICH2)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,Q2)
        else
        aC0TCH(ICH2,ICH3,ICH4)=C0T(regMass2,regMass2,regMass2,Q2)
        aCEpTCH(ICH2,ICH3,ICH4)=CEpT(regMass2,regMass2,regMass2,Q2)
        endif
       enddo 
       enddo 
       enddo 

       Do INH2=1,6
       Do INH3=1,6
       Do ICH2=1,2
        aC0THHCH(INH2, INH3, ICH2)=C0T(Mah2(INH2),Mah2(INH3),MassHpm(ICH2)**2,Q2)
        aCEpTHHCH(INH2, INH3, ICH2)=CEpT(Mah2(INH2),Mah2(INH3),MassHpm(ICH2)**2,Q2)
       enddo 
       enddo 
       enddo 
       Do INH2=1,6
       Do ICH3=1,2
       Do ICH2=1,2
        aC0THCHCH(INH2, ICH3, ICH2)=C0T(Mah2(INH2),MassHpm(ICH3)**2,MassHpm(ICH2)**2,Q2)
        aCEpTHCHCH(INH2, ICH3, ICH2)=CEpT(Mah2(INH2),MassHpm(ICH3)**2,MassHpm(ICH2)**2,Q2)
       enddo 
       enddo 
       enddo 

       aB0Tcha= B0T(MCha2,MCha2,Q2)
       aBEpTcha=BEpT(MCha2,MCha2,Q2)
       Do INe3=1,3
       Do INe2=1,3
       aB0TNeu(INe2,INe3)= B0T(Mneu2(INe2),Mneu2(INe3),Q2)
       aBEpTNeu(INe2,INe3)=BEpT(Mneu2(INe2),Mneu2(INe3),Q2)
       aC0TChaChi2(INe2,INe3)= C0T(MCha2,Mneu2(INe2),Mneu2(INe3),Q2)
       aCEpTChaChi2(INe2,INe3)= CEpT(MCha2,Mneu2(INe2),Mneu2(INe3),Q2)
       enddo
       aC0TCha2Chi(INe3)= C0T(MCha2,MCha2,Mneu2(INe3),Q2)
       aCEpTCha2Chi(INe3)= CEpT(MCha2,MCha2,Mneu2(INe3),Q2)
       aB0TChaChi(INe3)=B0T(MCha2,Mneu2(INe3),Q2)
       aBEpTChaChi(INe3)=BEpT(MCha2,Mneu2(INe3),Q2)
       enddo

       aC0TCha3= C0T(MCha2,MCha2,MCha2,Q2)
       aCepTCha3= CEpT(MCha2,MCha2,MCha2,Q2)
       aC0TT3= C0T(M12,M12,M12,Q2)
       aCepTT3= CEpT(M12,M12,M12,Q2)

       Do INH2=1,3
       Do INH3=1,3
       Do INH4=1,3
        aC0TChi3(INH2,INH3,INH4)=C0T(MNeu2(INH2),MNeu2(INH3),MNeu2(INH4),Q2)
        aCEpTChi3(INH2,INH3,INH4)=CEpT(MNeu2(INH2),MNeu2(INH3),MNeu2(INH4),Q2)
       enddo 
       enddo 
       enddo 


       Do INe3=1,3
       Do INe4=1,3
       Do INH3=1,6
         call FD221C(Mneu(INe3)**2,Mneu(INe4)**2,Mah(INH3)**2,a0tNeu(INe3),a0tNeu(INe4),
     &   a0tNH(INH3),aetNeu(INe3),aetNeu(INe4),aetNH(INH3),Q2,FD221ChiChiH(1:10,INe3, INe4, INH3),0)
       enddo 
       enddo 
       enddo 
       Do INH3=1,6
         call FD221C(MCha2,MCha2,Mah(INH3)**2,a0tcha,a0tcha,a0tNH(INH3),aetcha,
     &   aetcha,aetNH(INH3),Q2,FD221ChaChaH(1:10,INH3),0)
       enddo 
       Do INe3=1,3
       Do ICH3=1,2
         call FD221C(MCha2,Mneu(INe3)**2,MassHpm(ICH3)**2,a0tcha,a0tNeu(INe3),a0tHpN(ICH3),
     &   aetcha,aetNeu(INe3),aetHpN(ICH3),Q2,FD221ChaChiHp(1:10,INe3, ICH3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD211C(M12,Mneu(INe3)**2,MassSu(ISt3)**2,a0t1,a0tNeu(INe3),aA0tN(ISt3),
     &   aet1,aetNeu(INe3),aAetN(ISt3),Q2,FD211TChiSt(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD221C(M12,Mneu(INe3)**2,MassSu(ISt3)**2,a0t1,a0tNeu(INe3),aA0tN(ISt3),
     &        aet1,aetNeu(INe3),aAetN(ISt3),Q2,FD221TChiSt(1:10,INe3, ISt3),0)
       enddo 
       enddo 
         call FD221C(M12,MCha2,MSQ2,a0t1,a0tcha,a0t5,aet1,aetcha,aet5,Q2,
     &   FD221TChaSB(1:10),0)
       Do INH3=1,6
         call FD211C(M12,Mah(INH3)**2,M12,a0t1,a0tNH(INH3),a0t1,aet1,
     &   aetNH(INH3),aet1,Q2,FD211THT(1:10,INH3),0)
       enddo 

       Do INH3=1,5
         call FD221C(M12,Mah(INH3)**2,M12,a0t1,a0tNH(INH3),a0t1,aet1,   
     &   aetNH(INH3),aet1,Q2,FD221THT(1:10,INH3),0)
       enddo 

       INH3=6
         call FD221X(M12,Mah(INH3)**2,M12,a0t1,a0tNH(INH3),a0t1,aet1,    
     &   aetNH(INH3),aet1,Q2,FD221THT(1:10,INH3),0)

       Do INH3=1,6
         call FD211C(MCha2,Mah(INH3)**2,MCha2,a0tcha,a0tNH(INH3),a0tcha,
     &   aetcha,aetNH(INH3),aetcha,Q2,FD211ChaHCha(1:10,INH3),0)
       enddo 

       Do INH3=1,5
         call FD221C(MCha2,Mah(INH3)**2,MCha2,a0tcha,a0tNH(INH3),a0tcha,   
     &   aetcha,aetNH(INH3),aetcha,Q2,FD221ChaHCha(1:10,INH3),0)
       enddo 
         INH3=6
         call FD221X(MCha2,Mah(INH3)**2,MCha2,a0tcha,a0tNH(INH3),a0tcha,  
     &   aetcha,aetNH(INH3),aetcha,Q2,FD221ChaHCha(1:10,INH3),0)

       Do INe3=1,3
       Do INH3=1,6
       Do INe4=1,3
         call FD211C(Mneu(INe3)**2,Mah(INH3)**2,Mneu(INe4)**2,a0tNeu(INe3),a0tNH(INH3),
     &   a0tNeu(INe4),aetNeu(INe3),aetNH(INH3),aetNeu(INe4),Q2,
     &   FD211ChiHChi(1:10,INe3, INH3, INe4),0)
       enddo 
       enddo 
       enddo 
       Do INe3=1,3
       Do INe4=1,3
       Do INH3=1,5
         call FD221C(Mneu(INe3)**2,Mah(INH3)**2,Mneu(INe4)**2,a0tNeu(INe3),a0tNH(INH3),  
     &   a0tNeu(INe4),aetNeu(INe3),aetNH(INH3),aetNeu(INe4),Q2,
     &   FD221ChiHChi(1:10,INe3, INH3, INe4),0)
       enddo 

       INH3=6
         call FD221X(Mneu(INe3)**2,Mah(INH3)**2,Mneu(INe4)**2,a0tNeu(INe3),a0tNH(INH3),   
     &   a0tNeu(INe4),aetNeu(INe3),aetNH(INH3),aetNeu(INe4),Q2,
     &   FD221ChiHChi(1:10,INe3, INH3, INe4),0)

       enddo 
       enddo 
       Do ICH3=1,2
         call FD211C(M12,MassHpm(ICH3)**2,0D0,a0t1,a0tHpN(ICH3),0D0,aet1,
     &   aetHpN(ICH3),0D0,Q2,FD211THpB(1:10,ICH3),0)
       enddo 
       Do INe3=1,3
       Do ICH3=1,2
         call FD211C(Mneu(INe3)**2,MassHpm(ICH3)**2,MCha2,a0tNeu(INe3),a0tHpN(ICH3),
     &   a0tcha,aetNeu(INe3),aetHpN(ICH3),aetcha,Q2,FD211ChiHpCha(1:10,INe3, ICH3),0)
       enddo 
       enddo 
       Do INe3=1,3
       ICH3=2
         call FD221C(Mneu(INe3)**2,MassHpm(ICH3)**2,MCha2,a0tNeu(INe3),a0tHpN(ICH3),   
     &   a0tcha,aetNeu(INe3),aetHpN(ICH3),aetcha,Q2,FD221ChiHpCha(1:10,INe3, ICH3),0)
       ICH3=1
         call FD221X(Mneu(INe3)**2,MassHpm(ICH3)**2,MCha2,a0tNeu(INe3),a0tHpN(ICH3),    
     &   a0tcha,aetNeu(INe3),aetHpN(ICH3),aetcha,Q2,FD221ChiHpCha(1:10,INe3, ICH3),0)       
       enddo 
       Do ICH3=1,2
       Do INe3=1,3
         call FD211C(MCha2,MassHpm(ICH3)**2,Mneu(INe3)**2,a0tcha,a0tHpN(ICH3),
     &   a0tNeu(INe3),aetcha,aetHpN(ICH3),aetNeu(INe3),Q2,FD211ChaHpChi(1:10,ICH3, INe3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD111C(Mneu(INe3)**2,MassSu(ISt3)**2,M12,a0tNeu(INe3),aA0tN(ISt3),
     &   a0t1,aetNeu(INe3),aAetN(ISt3),aet1,Q2,FD111ChiStT(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD211C(Mneu(INe3)**2,MassSu(ISt3)**2,M12,a0tNeu(INe3),aA0tN(ISt3),
     &   a0t1,aetNeu(INe3),aAetN(ISt3),aet1,Q2,FD211ChiSTT(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do ISt3=1,2
       Do INe3=1,3
         call FD211C(MassSu(ISt3)**2,Mneu(INe3)**2,M12,aA0tN(ISt3),a0tNeu(INe3),
     &   a0t1,aAetN(ISt3),aetNeu(INe3),aet1,Q2,FD211StChiT(1:10,ISt3, INe3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD221C(Mneu(INe3)**2,MassSu(ISt3)**2,M12,a0tNeu(INe3),aA0tN(ISt3),
     &   a0t1,aetNeu(INe3),aAetN(ISt3),aet1,Q2,FD221ChiStT(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do ISt3=1,2
         call FD211C(MCha2,MassSu(ISt3)**2,0D0,a0tcha,aA0tN(ISt3),0D0,aetcha,
     &   aAetN(ISt3),0D0,Q2,FD211ChaStB(1:10,ISt3),0)
       enddo 

       Do INH5=1,6
       Do INH3=1,5
       Do INH4=1,5
         call FD221C(Mah(INH3)**2,Mah(INH4)**2,Mah(INH5)**2,a0tNH(INH3),a0tNH(INH4),   
     &   a0tNH(INH5),aetNH(INH3),aetNH(INH4),aetNH(INH5),Q2,
     &   FD221HHH(1:10,INH3, INH4, INH5),0)
       enddo
       INH4=6
         call FD221X(Mah(INH3)**2,Mah(INH4)**2,Mah(INH5)**2,a0tNH(INH3),a0tNH(INH4),  
     &   a0tNH(INH5),aetNH(INH3),aetNH(INH4),aetNH(INH5),Q2,
     &   FD221HHH(1:10,INH3, INH4, INH5),0)
       enddo 

       INH3=6
       Do INH4=1,6
         call FD221X(Mah(INH3)**2,Mah(INH4)**2,Mah(INH5)**2,a0tNH(INH3),a0tNH(INH4),   
     &   a0tNH(INH5),aetNH(INH3),aetNH(INH4),aetNH(INH5),Q2,
     &   FD221HHH(1:10,INH3, INH4, INH5),0)
       enddo
       enddo 

       Do ICH4=1,2
       Do INH3=1,5
       ICH3=2
         call FD221C(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),    
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD221HHpHp(1:10,INH3, ICH3, ICH4),0)
       enddo 

       INH3=6
       Do ICH3=1,2
         call FD221X(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),     
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD221HHpHp(1:10,INH3, ICH3, ICH4),0)
       enddo

       Do INH3=1,5
       ICH3=1
         call FD221X(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),     
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD221HHpHp(1:10,INH3, ICH3, ICH4),0)
       enddo 

       enddo 




       Do ICH3=1,2
       Do ICH4=1,2
       Do INH3=1,6
         call FD111C(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD111HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 
       enddo 
       enddo 

       Do ICH4=1,2
       Do INH3=1,6
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),    
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD211HpHpH(1:10,ICH3, ICH4, INH3),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),   
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD211HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 
       enddo 
       ICH3=2
       ICH4=2
       Do INH3=1,6
         call FD221C(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),     
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD221HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 
       ICH3=1
       ICH4=2
       Do INH3=1,6
         call FD221X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),    
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD221HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 

       ICH3=2
       ICH4=1
       Do INH3=1,6
         call FD221X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),     
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD221HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 

       ICH3=1
       ICH4=1
       Do INH3=1,6
         call FD221X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,Mah(INH3)**2,a0tHpN(ICH3),    
     &   a0tHpN(ICH4),a0tNH(INH3),aetHpN(ICH3),aetHpN(ICH4),aetNH(INH3),Q2,
     &   FD221HpHpH(1:10,ICH3, ICH4, INH3),0)
       enddo 


       Do ISt3=1,2
       Do INH3=1,6
       Do ISt4=1,2
         call FD211C(MassSu(ISt3)**2,Mah(INH3)**2,MassSu(ISt4)**2,aA0tN(ISt3),
     &   a0tNH(INH3),aA0tN(ISt4),aAetN(ISt3),aetNH(INH3),aAetN(ISt4),Q2,
     &   FD211StHSt(1:10,ISt3, INH3, ISt4),0)
       enddo 
       enddo 
       enddo 


       Do ISt3=1,2
       Do ISt4=1,2
       Do INH3=1,5
         call FD221C(Mah(INH3)**2,MassSu(ISt3)**2,MassSu(ISt4)**2,a0tNH(INH3),             
     &   aA0tN(ISt3),aA0tN(ISt4),aetNH(INH3),aAetN(ISt3),aAetN(ISt4),Q2,
     &   FD221HStSt(1:10,INH3, ISt3, ISt4),0)
       enddo 
         INH3=6
         call FD221X(Mah(INH3)**2,MassSu(ISt3)**2,MassSu(ISt4)**2,a0tNH(INH3),             
     &   aA0tN(ISt3),aA0tN(ISt4),aetNH(INH3),aAetN(ISt3),aAetN(ISt4),Q2,
     &   FD221HStSt(1:10,INH3, ISt3, ISt4),0)
       enddo 
       enddo 

       Do ISt3=1,2
       Do ICH3=1,2
         call FD211C(MassSu(ISt3)**2,MassHpm(ICH3)**2,MSQ2,aA0tN(ISt3),
     &   a0tHpN(ICH3),a0t5,aAetN(ISt3),aetHpN(ICH3),aet5,Q2,
     &   FD211StHpSb(1:10,ISt3, ICH3),0)
       enddo 
       enddo 

       Do ISt3=1,2
       ICH3=2
         call FD221C(MassHpm(ICH3)**2,MassSu(ISt3)**2,MSQ2,a0tHpN(ICH3),          
     &   aA0tN(ISt3),a0t5,aetHpN(ICH3),aAetN(ISt3),aet5,Q2,
     &   FD221HpStSb(1:10,ICH3, ISt3),0)

       ICH3=1
         call FD221X(MassHpm(ICH3)**2,MassSu(ISt3)**2,MSQ2,a0tHpN(ICH3),          
     &   aA0tN(ISt3),a0t5,aetHpN(ICH3),aAetN(ISt3),aet5,Q2,
     &   FD221HpStSb(1:10,ICH3, ISt3),0)

       enddo 

       ICH3=2
         call FD221C(M12,MassHpm(ICH3)**2,0D0,a0t1,a0tHpN(ICH3),0D0,aet1,         
     & aetHpN(ICH3),0D0,Q2,FD221THpB(1:10,ICH3),0)

       ICH3=1
         call FD221X(M12,MassHpm(ICH3)**2,0D0,a0t1,a0tHpN(ICH3),0D0,aet1,         
     & aetHpN(ICH3),0D0,Q2,FD221THpB(1:10,ICH3),0)


       Do INe3=1,3
       ICH3=2
         call FD221C(MCha**2,MassHpm(ICH3)**2,Mneu(INe3)**2,a0tcha,a0tHpN(ICH3),               
     & a0tNeu(INe3),aetcha,aetHpN(ICH3),aetNeu(INe3),Q2,FD221ChaHpChi(1:10,ICH3, INe3),0)
       ICH3=1
         call FD221X(MCha**2,MassHpm(ICH3)**2,Mneu(INe3)**2,a0tcha,a0tHpN(ICH3),              
     & a0tNeu(INe3),aetcha,aetHpN(ICH3),aetNeu(INe3),Q2,FD221ChaHpChi(1:10,ICH3, INe3),0)

       enddo 
       Do ISt3=1,2
         call FD221C(MCha**2,MassSu(ISt3)**2,0D0,a0tcha,aA0tN(ISt3),0D0,
     & aetcha,aAetN(ISt3),0D0,Q2,FD221ChaStSb(1:10,ISt3),0)
       enddo 

c for charged Higgs self
       Do INH4=1,6
       Do ICH3=1,2
       Do INH5=1,6
         call FD111C(Mah(INH4)**2,MassHpm(ICH3)**2,Mah(INH5)**2,a0tNH(INH4),a0tHpN(ICH3),
     &  a0tNH(INH5),aetNH(INH4),aetHpN(ICH3),aetNH(INH5),Q2,FD111HHpH(1:10,INH4, ICH3, INH5),0)
       enddo 
       enddo 
       enddo 

       Do INH3=1,6
       Do ICH4=1,2   
         Do INH4=1,5
         call FD211C(Mah(INH4)**2,Mah(INH3)**2,MassHpm(ICH4)**2,a0tNH(INH4),a0tNH(INH3),
     &  a0tHpN(ICH4),aetNH(INH4),aetNH(INH3),aetHpN(ICH4),Q2,FD211HHHp(1:10,INH4, INH3, ICH4),0)
         enddo 
         INH4=6
         call FD211X(Mah(INH4)**2,Mah(INH3)**2,MassHpm(ICH4)**2,a0tNH(INH4),a0tNH(INH3),
     &  a0tHpN(ICH4),aetNH(INH4),aetNH(INH3),aetHpN(ICH4),Q2,FD211HHHp(1:10,INH4, INH3, ICH4),0)
       enddo 
       enddo 
       Do ICH3=1,2
       Do ICH5=1,2
       Do ICH4=1,2
         call FD111C(MassHpm(ICH3)**2,MassHpm(ICH5)**2,MassHpm(ICH4)**2,a0tHpN(ICH3),
     &  a0tHpN(ICH5),a0tHpN(ICH4),aetHpN(ICH3),aetHpN(ICH5),aetHpN(ICH4),Q2,FD111HpHpHp(1:10,ICH3, ICH5, ICH4),0)
       enddo 
       enddo 
       enddo 
       Do ICH3=1,2
       Do ISt3=1,2
       Do ISt4=1,2
         call FD111C(MassHpm(ICH3)**2,MassSu(ISt3)**2,MassSu(ISt4)**2,a0tHpN(ICH3),
     &  aA0tN(ISt3),aA0tN(ISt4),aetHpN(ICH3),aAetN(ISt3),aAetN(ISt4),Q2,FD111HpSTST(1:10,ICH3, ISt3, ISt4),0)
       enddo 
       enddo 
       enddo 
       Do INH3=1,6
       Do ISt3=1,2
         call FD111C(Mah(INH3)**2,MassSu(ISt3)**2,MSQ2,a0tNH(INH3),aA0tN(ISt3),
     &   a0t5,aetNH(INH3),aAetN(ISt3),aet5,Q2,FD111HSTSB(1:10,INH3, ISt3),0)
       enddo 
       enddo 
       Do ISt3=1,2
       Do INH3=1,5
         call FD211C(Mah(INH3)**2,MassSu(ISt3)**2,MSQ2,a0tNH(INH3),aA0tN(ISt3),
     &   a0t5,aetNH(INH3),aAetN(ISt3),aet5,Q2,FD211HSTSB(1:10,INH3, ISt3),0)
       enddo
         call FD211X(Mah(INH3)**2,MassSu(ISt3)**2,MSQ2,a0tNH(INH3),aA0tN(ISt3),
     &   a0t5,aetNH(INH3),aAetN(ISt3),aet5,Q2,FD211HSTSB(1:10,INH3, ISt3),0) 
       enddo 

         call FD211C(M12,M12,M12,a0t1,a0t1,a0t1,aet1,aet1,aet1,Q2,FD211TTT(1:10),0)
         call FD211C(MCha2,MCha2,MCha2,a0tcha,a0tcha,a0tcha,aetcha,aetcha,aetcha,
     &   Q2,FD211ChaChaCha(1:10),0)
       Do INe3=1,3
       Do INe4=1,3
       Do INe5=1,3
         call FD211C(Mneu2(INe3),Mneu2(INe4),Mneu2(INe5),a0tNeu(INe3),a0tNeu(INe4),
     &  a0tNeu(INe5),aetNeu(INe3),aetNeu(INe4),aetNeu(INe5),Q2,FD211ChiChiChi(1:10,INe3, INe4, INe3),0)
       enddo 
       enddo 
       enddo 
       Do INe3=1,3
         call FD211C(MCha2,Mneu2(INe3),MCha2,a0tcha,a0tNeu(INe3),a0tcha,aetcha,
     &   aetNeu(INe3),aetcha,Q2,FD211ChaChiCha(1:10,INe3),0)
       enddo 
       do ICH4=1,2
       aK0TTH(ICH4)=K0T0(0D0,M12,MassHpm(ICH4)**2,Q2)
       enddo
       aK0TT=K0T0(0D0,M12,0D0,Q2)
       aI0AT=I0A(0D0,0D0,M12,Q2)

       Do ICH3=1,2
         call FD111C(M12,MassHpm(ICH3)**2,M12,a0t1,a0tHpN(ICH3),
     &    a0t1,aet1,aetHpN(ICH3),aet1,Q2,FD111THpT(1:10,ICH3),0)
       enddo 
       Do ICH3=1,2
         call FD111C(MCha2,MassHpm(ICH3)**2,MCha2,a0tcha,a0tHpN(ICH3),a0tcha,
     &    aetcha,aetHpN(ICH3),aetcha,Q2,FD111ChaHpCha(1:10,ICH3),0)
       enddo 
       Do INe3=1,3
       Do ICH3=1,2
       Do INe4=1,3
         call FD111C(Mneu(INe3)**2,MassHpm(ICH3)**2,Mneu(INe4)**2,a0tNeu(INe3)
     &   ,a0tHpN(ICH3),a0tNeu(INe4),aetNeu(INe3),aetHpN(ICH3),aetNeu(INe4),Q2,
     &    FD111ChiHpChi(1:10,INe3, ICH3, INe4),0)
       enddo 
       enddo 
       enddo 
       Do INH3=1,6
         call FD111C(M12,Mah(INH3)**2,0D0,a0t1,a0tNH(INH3),0D0,
     &    aet1,aetNH(INH3),0D0,Q2,FD111THB(1:10,INH3),0)
       enddo 
       Do INe3=1,3
       Do INH3=1,6
         call FD111C(Mneu(INe3)**2,Mah(INH3)**2,MCha2,a0tNeu(INe3),a0tNH(INH3),
     &   a0tcha,aetNeu(INe3),aetNH(INH3),aetcha,Q2,FD111ChiHCha(1:10,INe3, INH3),0)
       enddo 
       enddo 


       Do ICH4=1,2
       Do ICH5=1,2
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,MassHpm(ICH4)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),
     &   a0tHpN(ICH4),a0tHpN(ICH5),aetHpN(ICH3),aetHpN(ICH4),aetHpN(ICH5),Q2,
     &   FD211HpHpHp(1:10,ICH3, ICH4, ICH5),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),
     &   a0tHpN(ICH4),a0tHpN(ICH5),aetHpN(ICH3),aetHpN(ICH4),aetHpN(ICH5),Q2,
     &   FD211HpHpHp(1:10,ICH3, ICH4, ICH5),0)
       enddo 
       enddo


       Do INe4=1,3
       Do INH3=1,6
         call FD111C(MCha2,Mneu(INe4)**2,Mah(INH3)**2,a0tcha,a0tNeu(INe4),a0tNH(INH3),
     &      aetcha,aetNeu(INe4),aetNH(INH3),Q2,FD111ChaChiH(1:10,INe4, INH3),0)
       enddo 
       enddo 
       Do INH3=1,6
         call FD111C(MCha2,MCha2,Mah(INH3)**2,a0tcha,a0tcha,a0tNH(INH3),aetcha,
     &    aetcha,aetNH(INH3),Q2,FD111ChaChaH(1:10,INH3),0)
       enddo 
       Do INe3=1,3
       Do INH3=1,6
         call FD111C(Mneu(INe3)**2,MCha2,Mah(INH3)**2,a0tNeu(INe3),a0tcha,a0tNH(INH3),
     &    aetNeu(INe3),aetcha,aetNH(INH3),Q2,FD111ChiChaH(1:10,INe3, INH3),0)
       enddo 
       enddo 
       Do ICH3=1,2
         call FD111C(MCha2,MCha2,MassHpm(ICH3)**2,a0tcha,a0tcha,a0tHpN(ICH3),aetcha,
     &    aetcha,aetHpN(ICH3),Q2,FD111ChaChaHp(1:10,ICH3),0)
       enddo 
       Do INe3=1,3
       Do INe4=1,3
       Do ICH3=1,2
         call FD111C(Mneu(INe3)**2,Mneu(INe4)**2,MassHpm(ICH3)**2,a0tNeu(INe3),a0tNeu(INe4),
     &   a0tHpN(ICH3),aetNeu(INe3),aetNeu(INe4),aetHpN(ICH3),Q2,FD111ChiChiHp(1:10,INe3, INe4, ICH3),0)
       enddo 
       enddo 
       enddo 
       Do ISt3=1,2
         call FD111C(M12,MCha2,MassSu(ISt3)**2,a0t1,a0tcha,aA0tN(ISt3),aet1,
     &   aetcha,aAetN(ISt3),Q2,FD111TChaST(1:10,ISt3),0)
       enddo 
       Do ISt3=1,2
         call FD111C(0D0,MCha2,MassSu(ISt3)**2,0D0,a0tcha,aA0tN(ISt3),0D0,aetcha,
     &   aAetN(ISt3),Q2,FD111BChaST(1:10,ISt3),0)
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD111C(0D0,Mneu(INe3)**2,MassSu(ISt3)**2,0D0,a0tNeu(INe3),
     &   aA0tN(ISt3),0D0,aetNeu(INe3),aAetN(ISt3),Q2,FD111BChiST(1:10,INe3, ISt3),0)
       enddo 
       enddo 
         call FD111C(MCha2,0D0,MSQ2,a0tcha,0D0,a0t5,aetcha,0D0,aet5,Q2,
     &   FD111ChaBSB(1:10),0)
       Do INe3=1,3
         call FD211C(Mneu(INe3)**2,0D0,MSQ2,a0tNeu(INe3),0D0,a0t5,aetNeu(INe3),
     &    0D0,aet5,Q2,FD211ChiBSB(1:10,INe3),0)
       enddo 
         call FD111C(M12,MCha2,MSQ2,a0t1,a0tcha,a0t5,aet1,aetcha,
     &    aet5,Q2,FD111TChaSB(1:10),0)
       Do INe3=1,3
         call FD111C(Mneu(INe3)**2,M12,MSQ2,a0tNeu(INe3),a0t1,a0t5,
     &   aetNeu(INe3),aet1,aet5,Q2,FD111ChiTSB(1:10,INe3),0)
       enddo 


       Do INH3=1,6
         call FD111C(0D0,Mah(INH3)**2,M12,0D0,a0tNH(INH3),a0t1,0D0,aetNH(INH3),
     &   aet1,Q2,FD111BHT(1:10,INH3),0)
       enddo 
       Do ICH3=1,2
         call FD111C(0D0,MassHpm(ICH3)**2,M12,0D0,a0tHpN(ICH3),a0t1,0D0,
     &   aetHpN(ICH3),aet1,Q2,FD111BHpT(1:10,ICH3),0)
       enddo 
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,0D0,M12,a0tHpN(ICH3),0D0,a0t1,
     &   aetHpN(ICH3),0D0,aet1,Q2,FD211HpBT(1:10,ICH3),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,0D0,M12,a0tHpN(ICH3),0D0,a0t1,
     &   aetHpN(ICH3),0D0,aet1,Q2,FD211HpBT(1:10,ICH3),0)
       
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,M12,M12,a0tHpN(ICH3),a0t1,a0t1,
     &   aetHpN(ICH3),aet1,aet1,Q2,FD211HpTT(1:10,ICH3),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,M12,M12,a0tHpN(ICH3),a0t1,a0t1,
     &   aetHpN(ICH3),aet1,aet1,Q2,FD211HpTT(1:10,ICH3),0)

       Do INH3=1,6
       Do INe4=1,3
         call FD111C(MCha2,Mah(INH3)**2,Mneu(INe4)**2,a0tcha,a0tNH(INH3),a0tNeu(INe4),
     &   aetcha,aetNH(INH3),aetNeu(INe4),Q2,FD111ChaHChi(1:10,INH3, INe4),0)
       enddo 
       enddo 
       Do ICH3=1,2
       Do INe4=1,3
         call FD111C(MCha**2,MassHpm(ICH3)**2,Mneu(INe4)**2,a0tcha,a0tHpN(ICH3),
     &  a0tNeu(INe4),aetcha,aetHpN(ICH3),aetNeu(INe4),Q2,FD111ChaHpChi(1:10,ICH3, INe4),0)
       enddo 
       enddo 

       Do INe4=1,3
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,MCha2,Mneu2(INe4),a0tHpN(ICH3),a0tcha,
     &  a0tNeu(INe4),aetHpN(ICH3),aetcha,aetNeu(INe4),Q2,FD211HpChaChi(1:10,ICH3, INe4),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,MCha2,Mneu2(INe4),a0tHpN(ICH3),a0tcha,
     &  a0tNeu(INe4),aetHpN(ICH3),aetcha,aetNeu(INe4),Q2,FD211HpChaChi(1:10,ICH3, INe4),0)
       enddo 


       Do INe3=1,3
       Do INe4=1,3
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,Mneu(INe3)**2,Mneu(INe4)**2,a0tHpN(ICH3),
     &  a0tNeu(INe3),a0tNeu(INe4),aetHpN(ICH3),aetNeu(INe3),aetNeu(INe4),Q2,
     &  FD211HpChiChi(1:10,ICH3, INe3, INe4),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,Mneu(INe3)**2,Mneu(INe4)**2,a0tHpN(ICH3),
     &  a0tNeu(INe3),a0tNeu(INe4),aetHpN(ICH3),aetNeu(INe3),aetNeu(INe4),Q2,
     &  FD211HpChiChi(1:10,ICH3, INe3, INe4),0)
       enddo 
       enddo 
       ICH3=2
         call FD211C(MassHpm(ICH3)**2,MCha2,MCha2,a0tHpN(ICH3),a0tcha,a0tcha,
     &   aetHpN(ICH3),aetcha,aetcha,Q2,FD211HpChaCha(1:10,ICH3),0)
       ICH3=1
         call FD211X(MassHpm(ICH3)**2,MCha2,MCha2,a0tHpN(ICH3),a0tcha,a0tcha,
     &   aetHpN(ICH3),aetcha,aetcha,Q2,FD211HpChaCha(1:10,ICH3),0)

         call FD111C(MSQ2,MCha2,M12,a0t5,a0tcha,a0t1,aet5,aetcha,aet1,
     &   Q2,FD111SBChaT(1:10),0)
       Do INe3=1,3
         call FD111C(MSQ2,Mneu(INe3)**2,M12,a0t5,a0tNeu(INe3),a0t1,aet5,
     &   aetNeu(INe3),aet1,Q2,FD111SBChiT(1:10,INe3),0)
       enddo 
       Do ISt3=1,2
         call FD111C(MCha2,MassSu(ISt3)**2,M12,a0tcha,aA0tN(ISt3),a0t1,
     &   aetcha,aAetN(ISt3),aet1,Q2,FD111ChaSTT(1:10,ISt3),0)
       enddo 
         call FD111C(MSQ2,MCha2,0D0,a0t5,a0tcha,0D0,aet5,aetcha,
     &   0D0,Q2,FD111SBChaB(1:10),0)
       Do INe3=1,3
         call FD111C(MSQ2,Mneu(INe3)**2,0D0,a0t5,a0tNeu(INe3),0D0,aet5,
     &   aetNeu(INe3),0D0,Q2,FD111SBChiB(1:10,INe3),0)
       enddo 
       Do ISt3=1,2
         call FD111C(MCha**2,MassSu(ISt3)**2,0D0,a0tcha,aA0tN(ISt3),0D0,aetcha,
     &   aAetN(ISt3),0D0,Q2,FD111ChaSTB(1:10,ISt3),0)
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD111C(Mneu(INe3)**2,MassSu(ISt3)**2,0D0,a0tNeu(INe3),aA0tN(ISt3),
     &   0D0,aetNeu(INe3),aAetN(ISt3),0D0,Q2,FD111ChiSTB(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do INe3=1,3
         call FD111C(Mneu(INe3)**2,MSQ2,0D0,a0tNeu(INe3),a0t5,0D0,aetNeu(INe3),
     &   aet5,0D0,Q2,FD111ChiSBB(1:10,INe3),0)
       enddo 
       Do INe3=1,3
         call FD111C(Mneu(INe3)**2,MSQ2,M12,a0tNeu(INe3),a0t5,a0t1,aetNeu(INe3),
     &   aet5,aet1,Q2,FD111ChiSBT(1:10,INe3),0)
       enddo 

       Do INe3=1,3
       Do INe5=1,3
       Do INH3=1,6
         call FD311C(Mneu(INe3)**2,Mneu(INe5)**2,Mah(INH3)**2,a0tNeu(INe3),
     &  a0tNeu(INe5),a0tNH(INH3),aetNeu(INe3),aetNeu(INe5),aetNH(INH3),Q2,
     &  FD311ChiChiH(1:10,INe3, INe5, INH3),0)
       enddo 
       enddo 
       enddo 
       Do INH3=1,6
         call FD311C(MCha**2,MCha**2,Mah(INH3)**2,a0tcha,a0tcha,a0tNH(INH3),
     &   aetcha,aetcha,aetNH(INH3),Q2,FD311ChaChaH(1:10,INH3),0)
       enddo 
       Do INe3=1,3
       Do ICH3=1,2
         call FD311C(Mneu(INe3)**2,MCha**2,MassHpm(ICH3)**2,a0tNeu(INe3),a0tcha,
     &   a0tHpN(ICH3),aetNeu(INe3),aetcha,aetHpN(ICH3),Q2,FD311ChiChaHp(1:10,INe3, ICH3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ICH3=1,2
         call FD311C(MCha**2,Mneu(INe3)**2,MassHpm(ICH3)**2,a0tcha,a0tNeu(INe3),a0tHpN(ICH3),
     &   aetcha,aetNeu(INe3),aetHpN(ICH3),Q2,FD311ChaChiHp(1:10,INe3, ICH3),0)
       enddo 
       enddo 
       Do INe3=1,3
       Do ISt3=1,2
         call FD311C(Mneu(INe3)**2,M1**2,MassSu(ISt3)**2,a0tNeu(INe3),a0t1,aA0tN(ISt3),
     &   aetNeu(INe3),aet1,aAetN(ISt3),Q2,FD311ChiTST(1:10,INe3, ISt3),0)
       enddo 
       enddo 
       Do ISt3=1,2
         call FD311C(MCha**2,0D0,MassSu(ISt3)**2,a0tcha,0D0,aA0tN(ISt3),
     &   aetcha,0D0,aAetN(ISt3),Q2,FD311ChaBST(1:10,ISt3),0)
       enddo 
       Do INe3=1,3
         call FD311C(Mneu(INe3)**2,0D0,MSQ2,a0tNeu(INe3),0D0,a0t5,
     &   aetNeu(INe3),0D0,aet5,Q2,FD311ChiBSB(1:10,INe3),0)
       enddo 
         call FD311C(MCha**2,M1**2,MSQ2,a0tcha,a0t1,a0t5,aetcha,
     &   aet1,aet5,Q2,FD311ChaTSB(1:10),0)

ccc new integrals

       Do INH3=1,5
         call FD311C(Mah2(INH3),M12,M12,a0tNH(INH3),a0t1,a0t1,
     &   aetNH(INH3),aet1,aet1,Q2,FD311HTT(1:10,INH3),0)
       enddo 
       INH3=6
         call FD311X(Mah2(INH3),M12,M12,a0tNH(INH3),a0t1,a0t1,
     &   aetNH(INH3),aet1,aet1,Q2,FD311HTT(1:10,INH3),0)


       Do INH3=1,5
         call FD311C(Mah2(INH3),MCha2,MCha2,a0tNH(INH3),a0tcha,
     &   a0tcha,aetNH(INH3),aetcha,aetcha,Q2,FD311HChaCha(1:10,INH3),0)
       enddo 

       INH3=6
         call FD311X(Mah2(INH3),MCha2,MCha2,a0tNH(INH3),a0tcha,
     &   a0tcha,aetNH(INH3),aetcha,aetcha,Q2,FD311HChaCha(1:10,INH3),0)


       Do INe3=1,3
       Do INe4=1,3
       Do INH3=1,5
         call FD311C(Mah(INH3)**2,Mneu(INe3)**2,Mneu(INe4)**2,a0tNH(INH3),
     &   a0tNeu(INe3),a0tNeu(INe4),aetNH(INH3),aetNeu(INe3),aetNeu(INe4),Q2,
     &   FD311HChiChi(1:10,INH3, INe3, INe4),0)
       enddo 

       INH3=6
         call FD311X(Mah(INH3)**2,Mneu(INe3)**2,Mneu(INe4)**2,a0tNH(INH3),
     &   a0tNeu(INe3),a0tNeu(INe4),aetNH(INH3),aetNeu(INe3),aetNeu(INe4),Q2,
     &   FD311HChiChi(1:10,INH3, INe3, INe4),0)

       enddo 
       enddo 


       ICH3=1
         call FD311Z(MassHpm(ICH3)**2,M1**2,0D0,a0tHpN(ICH3),a0t1,0D0,
     &   aetHpN(ICH3),aet1,0D0,Q2,FD311HpTB(1:10,ICH3),0)

       ICH3=2
         call FD311C(MassHpm(ICH3)**2,M1**2,0D0,a0tHpN(ICH3),a0t1,0D0,
     &   aetHpN(ICH3),aet1,0D0,Q2,FD311HpTB(1:10,ICH3),0)





       Do INe3=1,3
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,Mneu(INe3)**2,MCha**2,a0tHpN(ICH3),
     &   a0tNeu(INe3),a0tcha,aetHpN(ICH3),aetNeu(INe3),aetcha,Q2,FD311HpChiCha(1:10,ICH3, INe3),0)
       ICH3=1
         call FD311X(MassHpm(ICH3)**2,Mneu(INe3)**2,MCha**2,a0tHpN(ICH3),
     &   a0tNeu(INe3),a0tcha,aetHpN(ICH3),aetNeu(INe3),aetcha,Q2,FD311HpChiCha(1:10,ICH3, INe3),0)

       enddo 


       Do INH5=1,6
       Do INH6=1,6
       Do INH3=1,5
         call FD311C(Mah(INH3)**2,Mah(INH5)**2,Mah(INH6)**2,a0tNH(INH3),
     &   a0tNH(INH5),a0tNH(INH6),aetNH(INH3),aetNH(INH5),aetNH(INH6),Q2,
     &   FD311HHH(1:10,INH3, INH5, INH6),0)
       enddo 

       INH3=6
         if(Mah(INH5)*Mah(INH6) .gt. 0D0)then
         call FD311X(Mah(INH3)**2,Mah(INH5)**2,Mah(INH6)**2,a0tNH(INH3),
     &   a0tNH(INH5),a0tNH(INH6),aetNH(INH3),aetNH(INH5),aetNH(INH6),Q2,
     &   FD311HHH(1:10,INH3, INH5, INH6),0)
         else
         call FD311Z(Mah(INH3)**2,Mah(INH5)**2,Mah(INH6)**2,a0tNH(INH3),
     &   a0tNH(INH5),a0tNH(INH6),aetNH(INH3),aetNH(INH5),aetNH(INH6),Q2,
     &   FD311HHH(1:10,INH3, INH5, INH6),0)
         endif
       enddo 
       enddo 


       Do ICH3=1,2
       Do ICH4=1,2
       Do INH3=1,5
         call FD311C(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD311HHpHp(1:10,INH3, ICH3, ICH4),0)
       enddo 

       INH3=6
         if(MassHpm(ICH3)*MassHpm(ICH4) .gt. 0D0)then
         call FD311X(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD311HHpHp(1:10,INH3, ICH3, ICH4),0)
         else
         call FD311Z(Mah(INH3)**2,MassHpm(ICH3)**2,MassHpm(ICH4)**2,a0tNH(INH3),
     &   a0tHpN(ICH3),a0tHpN(ICH4),aetNH(INH3),aetHpN(ICH3),aetHpN(ICH4),Q2,
     &   FD311HHpHp(1:10,INH3, ICH3, ICH4),0)
         endif

       enddo 
       enddo 

       Do INH3=1,6
       Do ICH5=1,2
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,Mah(INH3)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),
     &   a0tNH(INH3),a0tHpN(ICH5),aetHpN(ICH3),aetNH(INH3),aetHpN(ICH5),Q2,
     &   FD311HpHHp(1:10,ICH3, INH3, ICH5),0)
 
       ICH3=1
         if(Mah(INH3)*MassHpm(ICH5).gt. 0D0)then
         call FD311X(MassHpm(ICH3)**2,Mah(INH3)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),
     &   a0tNH(INH3),a0tHpN(ICH5),aetHpN(ICH3),aetNH(INH3),aetHpN(ICH5),Q2,
     &   FD311HpHHp(1:10,ICH3, INH3, ICH5),0)  
         else
         call FD311Z(MassHpm(ICH3)**2,Mah(INH3)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),
     &   a0tNH(INH3),a0tHpN(ICH5),aetHpN(ICH3),aetNH(INH3),aetHpN(ICH5),Q2,
     &   FD311HpHHp(1:10,ICH3, INH3, ICH5),0)  
         endif    
       enddo 
       enddo 

       Do ISt3=1,2
       Do ISt4=1,2
       Do INH3=1,5
         call FD311C(Mah(INH3)**2,MassSu(ISt3)**2,MassSu(ISt4)**2,a0tNH(INH3),
     &   aA0tN(ISt3),aA0tN(ISt4),aetNH(INH3),aAetN(ISt3),aAetN(ISt4),Q2,
     &   FD311HSTST(1:10,INH3, ISt3, ISt4),0)
       enddo 
       INH3=6
         call FD311X(Mah(INH3)**2,MassSu(ISt3)**2,MassSu(ISt4)**2,a0tNH(INH3),
     &   aA0tN(ISt3),aA0tN(ISt4),aetNH(INH3),aAetN(ISt3),aAetN(ISt4),Q2,
     &   FD311HSTST(1:10,INH3, ISt3, ISt4),0)
       enddo 
       enddo 

       Do ISt3=1,2
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,MassSu(ISt3)**2,MSQ2,a0tHpN(ICH3),
     &   aA0tN(ISt3),a0t5,aetHpN(ICH3),aAetN(ISt3),aet5,Q2,FD311HpSTSB(1:10,ICH3, ISt3),0)
       ICH3=1
         call FD311X(MassHpm(ICH3)**2,MassSu(ISt3)**2,MSQ2,a0tHpN(ICH3),
     &   aA0tN(ISt3),a0t5,aetHpN(ICH3),aAetN(ISt3),aet5,Q2,FD311HpSTSB(1:10,ICH3, ISt3),0)
        
       enddo 


c for vev
       Do ICH3=2,2
       Do INH4=1,6
       Do INH5=1,6
       call FD211C(MassHpm(ICH3)**2,Mah2(INH4),Mah2(INH5),a0tHpN(ICH3),a0tNH(INH4),a0tNH(INH5),
     &  aetHpN(ICH3),aetNH(INH4),aetNH(INH5),Q2,FD211HpHH(1:10,ICH3, INH4, INH5),0)
       enddo 
       enddo 
       enddo 

       ICH3=1
       Do INH4=1,6
       Do INH5=1,6
         call FD211X(MassHpm(ICH3)**2,Mah2(INH4),Mah2(INH5),a0tHpN(ICH3),a0tNH(INH4),a0tNH(INH5),
     &  aetHpN(ICH3),aetNH(INH4),aetNH(INH5),Q2,FD211HpHH(1:10,ICH3, INH4, INH5),0)
       enddo 
       enddo 


 
       ICH3=2
       Do ISt4=1,2
         call FD211C(MassHpm(ICH3)**2,MSQ2,MassSu2(ISt4),a0tHpN(ICH3),a0t5,aA0tN(ISt4),
     &   aetHpN(ICH3),aet5,aAetN(ISt4),Q2,FD211HpSBST(1:10,ICH3, ISt4),0)
       enddo

       ICH3=1
       Do ISt4=1,2
         call FD211X(MassHpm(ICH3)**2,MSQ2,MassSu2(ISt4),a0tHpN(ICH3),a0t5,aA0tN(ISt4),
     &   aetHpN(ICH3),aet5,aAetN(ISt4),Q2,FD211HpSBST(1:10,ICH3, ISt4),0)
       enddo  
        
       ICH3=2
       Do ISt3=1,2
       Do ISt4=1,2
         call FD211C(MassHpm(ICH3)**2,MassSu2(ISt3),MassSu2(ISt4),a0tHpN(ICH3),
     &   aA0tN(ISt3),aA0tN(ISt4),aetHpN(ICH3),aAetN(ISt3),aAetN(ISt4),Q2,FD211HpSTST(1:10,ICH3, ISt3, ISt4),0)
       enddo 
       enddo 

       ICH3=1
       Do ISt3=1,2
       Do ISt4=1,2
         call FD211X(MassHpm(ICH3)**2,MassSu2(ISt3),MassSu2(ISt4),a0tHpN(ICH3),
     &   aA0tN(ISt3),aA0tN(ISt4),aetHpN(ICH3),aAetN(ISt3),aAetN(ISt4),Q2,FD211HpSTST(1:10,ICH3, ISt3, ISt4),0)
       enddo 
       enddo 


       ICH3=2
       Do ICH5=1,2
       Do ICH4=1,2
         call FD221C(MassHpm(ICH3)**2,MassHpm(ICH5)**2,MassHpm(ICH4)**2,a0tHpN(ICH3),a0tHpN(ICH5)
     &   ,a0tHpN(ICH4),aetHpN(ICH3),aetHpN(ICH5),aetHpN(ICH4),Q2,FD221HpHpHp(1:10,ICH3, ICH5, ICH4),0)
       enddo 
       enddo 


       ICH3=1
       Do ICH5=1,2
       Do ICH4=1,2
         call FD221X(MassHpm(ICH3)**2,MassHpm(ICH5)**2,MassHpm(ICH4)**2,a0tHpN(ICH3),a0tHpN(ICH5)
     &   ,a0tHpN(ICH4),aetHpN(ICH3),aetHpN(ICH5),aetHpN(ICH4),Q2,FD221HpHpHp(1:10,ICH3, ICH5, ICH4),0)
       enddo 
       enddo 
 

       Do ISt3=1,2
         call FD221C(0D0,MCha2,MassSu2(ISt3),0D0,a0tcha,aA0tN(ISt3),0D0,aetcha,aAetN(ISt3),Q2,FD221BChaST(1:10,ISt3),0)
       enddo 
       Do INe3=3,3
         call FD211C(0D0,Mneu2(INe3),MSQ2,0D0,a0tNeu(INe3),a0t5,0D0,aetNeu(INe3),aet5,Q2,FD211BChiSB(1:10,INe3),0)
       enddo 
       Do INe3=1,3
         call FD221C(0D0,Mneu2(INe3),MSQ2,0D0,a0tNeu(INe3),a0t5,0D0,aetNeu(INe3),aet5,Q2,FD221BChiSB(1:10,INe3),0)
       enddo 


       Do ICH3=1,2
         call FD211C(0D0,MassHpm(ICH3)**2,M12,0D0,a0tHpN(ICH3),a0t1,0D0,aetHpN(ICH3),aet1,Q2,FD211BHpT(1:10,ICH3),0)
       enddo 
       Do ICH3=2,2
         call FD221C(0D0,MassHpm(ICH3)**2,M12,0D0,a0tHpN(ICH3),a0t1,0D0,aetHpN(ICH3),aet1,Q2,FD221BHpT(1:10,ICH3),0)
       enddo 

         ICH3=1
         call FD221Z(0D0,MassHpm(ICH3)**2,M12,0D0,a0tHpN(ICH3),a0t1,0D0,aetHpN(ICH3),aet1,Q2,FD221BHpT(1:10,ICH3),0)

       Do INe3=1,3
         call FD211C(MSQ2,Mneu2(INe3),0D0,a0t5,a0tNeu(INe3),0D0,aet5,aetNeu(INe3),0D0,Q2,FD211SBChiB(1:10,INe3),0)
       enddo 
       Do INe3=1,3
         call FD221C(MSQ2,Mneu2(INe3),0D0,a0t5,a0tNeu(INe3),0D0,aet5,aetNeu(INe3),0D0,Q2,FD221SBChiB(1:10,INe3),0)
       enddo 
         call FD221C(MSQ2,MCha2,M12,a0t5,a0tcha,a0t1,aet5,aetcha,aet1,Q2,FD221SBChaT(1:10),0)

   
       Do ISt3=1,2
         ICH3=2
         call FD211C(MSQ2,MassHpm(ICH3)**2,MassSu2(ISt3),a0t5,a0tHpN(ICH3),aA0tN(ISt3),
     &      aet5,aetHpN(ICH3),aAetN(ISt3),Q2,FD211SBHpSt(1:10,ICH3, ISt3),0)

         call FD221C(MSQ2,MassHpm(ICH3)**2,MassSu2(ISt3),a0t5,a0tHpN(ICH3),aA0tN(ISt3),
     &       aet5,aetHpN(ICH3),aAetN(ISt3),Q2,FD221SBHpSt(1:10,ICH3, ISt3),0)


         ICH3=1
         call FD211C(MSQ2,MassHpm(ICH3)**2,MassSu2(ISt3),a0t5,a0tHpN(ICH3),aA0tN(ISt3),
     &      aet5,aetHpN(ICH3),aAetN(ISt3),Q2,FD211SBHpSt(1:10,ICH3, ISt3),0)

         call FD221X(MSQ2,MassHpm(ICH3)**2,MassSu2(ISt3),a0t5,a0tHpN(ICH3),aA0tN(ISt3),
     &       aet5,aetHpN(ICH3),aAetN(ISt3),Q2,FD221SBHpSt(1:10,ICH3, ISt3),0)
       enddo 


       ICH3=2
         call FD311C(MassHpm(ICH3)**2,M12,M12,a0tHpN(ICH3),a0t1,a0t1,
     &  aetHpN(ICH3),aet1,aet1,Q2,FD311HpTT(1:10,ICH3),0)
       ICH3=1
         call FD311X(MassHpm(ICH3)**2,M12,M12,a0tHpN(ICH3),a0t1,a0t1,
     &  aetHpN(ICH3),aet1,aet1,Q2,FD311HpTT(1:10,ICH3),0)
        
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,MCha2,MCha2,a0tHpN(ICH3),a0tcha,a0tcha,
     &   aetHpN(ICH3),aetcha,aetcha,Q2,FD311HpChaCha(1:10,ICH3),0)
       ICH3=1
         call FD311X(MassHpm(ICH3)**2,MCha2,MCha2,a0tHpN(ICH3),a0tcha,a0tcha,
     &   aetHpN(ICH3),aetcha,aetcha,Q2,FD311HpChaCha(1:10,ICH3),0)
       

       Do INe3=1,3
       Do INe4=1,3
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,Mneu2(INe3),Mneu2(INe4),a0tHpN(ICH3),a0tNeu(INe3),
     &    a0tNeu(INe4),aetHpN(ICH3),aetNeu(INe3),aetNeu(INe4),Q2,FD311HpChiChi(1:10,ICH3, INe3, INe4),0)
       ICH3=1
         call FD311X(MassHpm(ICH3)**2,Mneu2(INe3),Mneu2(INe4),a0tHpN(ICH3),a0tNeu(INe3),
     &    a0tNeu(INe4),aetHpN(ICH3),aetNeu(INe3),aetNeu(INe4),Q2,FD311HpChiChi(1:10,ICH3, INe3, INe4),0)
       enddo 
       enddo 
        


       Do INH4=1,6
       Do INH5=1,6
       ICH3=2
         call FD311C(MassHpm(ICH3)**2,Mah2(INH4),Mah2(INH5),a0tHpN(ICH3),a0tNH(INH4),a0tNH(INH5),
     &   aetHpN(ICH3),aetNH(INH4),aetNH(INH5),Q2,FD311HpHH(1:10,ICH3, INH4, INH5),0)

       ICH3=1
         call FD311X(MassHpm(ICH3)**2,Mah2(INH4),Mah2(INH5),a0tHpN(ICH3),a0tNH(INH4),a0tNH(INH5),
     &   aetHpN(ICH3),aetNH(INH4),aetNH(INH5),Q2,FD311HpHH(1:10,ICH3, INH4, INH5),0)
       enddo 
       enddo 
 

       Do INH4=1,6
       Do ICH4=1,2
       Do INH3=1,6
         call FD111C(Mah(INH3)**2,Mah(INH4)**2,MassHpm(ICH4)**2,a0tNH(INH3),a0tNH(INH4),
     &   a0tHpN(ICH4),aetNH(INH3),aetNH(INH4),aetHpN(ICH4),Q2,FD111HHHp(1:10,INH3, INH4, ICH4),0)
       enddo 
       enddo 
       enddo 

       Do ICH4=1,2
       Do ICH5=1,2
       ICH3= 2
         call FD311C(MassHpm(ICH3)**2,MassHpm(ICH4)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),a0tHpN(ICH4),
     &    a0tHpN(ICH5),aetHpN(ICH3),aetHpN(ICH4),aetHpN(ICH5),Q2,FD311HpHpHp(1:10,ICH3, ICH4, ICH5),0)

       ICH3= 1
         call FD311X(MassHpm(ICH3)**2,MassHpm(ICH4)**2,MassHpm(ICH5)**2,a0tHpN(ICH3),a0tHpN(ICH4),
     &    a0tHpN(ICH5),aetHpN(ICH3),aetHpN(ICH4),aetHpN(ICH5),Q2,FD311HpHpHp(1:10,ICH3, ICH4, ICH5),0)
       enddo 
       enddo 

       Do ISt3=1,2
       Do ISt4=1,2
       ICH3= 2
         call FD311C(MassHpm(ICH3)**2,MassSu2(ISt3),MassSu2(ISt4),a0tHpN(ICH3),aA0tN(ISt3)
     &      ,aA0tN(ISt4),aetHpN(ICH3),aAetN(ISt3),aAetN(ISt4),Q2,FD311HpSTST(1:10,ICH3, ISt3, ISt4),0)
       ICH3= 1
         call FD311X(MassHpm(ICH3)**2,MassSu2(ISt3),MassSu2(ISt4),a0tHpN(ICH3),aA0tN(ISt3)
     &      ,aA0tN(ISt4),aetHpN(ICH3),aAetN(ISt3),aAetN(ISt4),Q2,FD311HpSTST(1:10,ICH3, ISt3, ISt4),0)
       enddo 
       enddo 

c end Alphat_lambda 2019 

      end subroutine LIBLOOP_atlambda

c
c**********************************************************************
c
      subroutine CT1loop_atlambda(ischeme,dmsq4fin,dmsur4fin,dat4fin,MHpminput)
      implicit none
      integer i,j,k,l,ischeme
      integer I2G2,I5G2,I6G2,I2G3,I2G4,I3G3,I5G3,I5G4,I6G4,I6G3
      double precision A0T,AEpT,B0T,BEpT,fac,selfHH1MT4IN(6,6),
     .                 selfHH1MT4(6,6),dmsq4fin,dmsur4fin,facZ
      double complex B0P,B1P,BEpP,B1EpP
      double precision Pi2,CF,xF0,dPMW24,
     .     dPMZ24,dMW24,dMZ24,dMWeps,dMZeps,dMWoMW,dMZoMZ
      double complex tad1SP(6),tad1(6),tad1EP(6),tad1SPB(6),tad1EPB(6),
     .     tad1B(6),dat4fin,
     .     dMW24B(3),dMZ24B(3),dMW24S(3),dMZ24S(3),
     .     dMNeu1LIN(3,3,3),dMNeu1LBIN(3,3,3)
      logical MQrun,MHpminput
      common/vectorself/dMWoMW,dMZoMZ

#include "TLMT4.h"
#include "TLMT4lambda.h"
      pi2=pi**2
      fac= 1/16D0/pi2
      dReAkappa14 = 0D0
      dReAkappa14B = 0D0
      dReAkappa14S = 0D0
      dReAka14 = 0D0
      dabslambda14B=0D0
      dabslambda14=0D0
      dvS1B=0D0
      dabskappa14B=0D0
      dabskappa14=0D0
      dlambdaC4B=0D0

c atat contribution
      dZHu4=(-3*M12)/(8.*Pi2*SB2*vv2)
c new NMSSM contribution
      Zfactor1LIN(1:6,1:6)=0D0
      Zfactor1LIN(1,1)=-la2/16D0/pi2
      Zfactor1LIN(2,2)=-la2/16D0/pi2
      Zfactor1LIN(4,4)=-la2/16D0/pi2
      Zfactor1LIN(5,5)=-la2/16D0/pi2
      Zfactor1LIN(3,3)=-(la2+ka2)/8D0/pi2
      Zfactor1LIN(6,6)=-(la2+ka2)/8D0/pi2
      facZ=0D0 !to correct the setting
c
c one-loop tadpole
       do i=ihiggs,ihiggse
       tad1SP(i)=fac*(3*M22*CSSS(i,1,1)+3*M42*CSSS(i,2,2)-6*M1**3*(hy(i)+
     &  hyc(i)))
       tad1EP(i)=fac*((3*aet2*CSSS(i,1,1)+3*aet4*CSSS(i,2,2)-6*
     &  aet1*M1*(hy(i)+hyc(i))))
       tad1(i)=fac*(3*a0t2*CSSS(i,1,1)+3*a0t4*CSSS(i,2,2)-6*a0t1*
     &  M1*(hy(i)+hyc(i)))

       tad1SPB(i)=(-2*MCha**3*(CFF12S1L(i)+CFF12S1R(i)))
       tad1B(i)=(-2*a0tcha*MCha*(CFF12S1L(i)+CFF12S1R(i)))
       tad1EPB(i)=(-2*aetcha*MCha*(CFF12S1L(i)+CFF12S1R(i)))

       I2G2 =2
       tad1SPB(i)=CS1S3S3(i,I2G2,I2G2)*MassHpm(I2G2)**2+tad1SPB(i)
       tad1B(i)=A0THpN(I2G2)*CS1S3S3(i,I2G2,I2G2)+tad1B(i)
       tad1EPB(i)=AETHpN(I2G2)*CS1S3S3(i,I2G2,I2G2)+tad1EPB(i)

    	do I5G2 = ineu,3
       tad1SPB(i)=-((CFF11S1L(I5G2,I5G2,i)+CFF11S1R(I5G2,I5G2,i))*
     &  MNeu(I5G2)**3)+tad1SPB(i)
       tad1B(i)=-(A0TNeu(I5G2)*(CFF11S1L(I5G2,I5G2,i)+
     &  CFF11S1R(I5G2,I5G2,i))*MNeu(I5G2))+tad1B(i)
       tad1EPB(i)=-(AETNeu(I5G2)*(CFF11S1L(I5G2,I5G2,i)+
     &  CFF11S1R(I5G2,I5G2,i))*MNeu(I5G2))+tad1EPB(i)
    	enddo
    	do I6G2 = ihiggs,5
       tad1SPB(i)=(CS1S1S1(i,I6G2,I6G2)*Mah2(I6G2))/2.+tad1SPB(i)
       tad1B(i)=(A0TNH(I6G2)*CS1S1S1(i,I6G2,I6G2))/2.+tad1B(i)
       tad1EPB(i)=(AETNH(I6G2)*CS1S1S1(i,I6G2,I6G2))/2.+tad1EPB(i)
    	enddo

       tad1SPB(i)=tad1SPB(i)*fac
       tad1B(i)=tad1B(i)*fac 
       tad1EPB(i)=tad1EPB(i)*fac
       enddo

         dTad1MT4(1:6,1:3)=0D0
         dTad1MT4B(1:6,1:3)=0D0
         do i=1,6
          do j=1,6
         dTad1MT4(i,1) =   dTad1MT4(i,1) + ZH(j,i) * tad1SP(j)   
         dTad1MT4(i,2) =   dTad1MT4(i,2) + ZH(j,i) * tad1(j)   
         dTad1MT4(i,3) =   dTad1MT4(i,3) + ZH(j,i) * tad1EP(j) 
         dTad1MT4B(i,1) =   dTad1MT4B(i,1) + ZH(j,i) * tad1SPB(j)   
         dTad1MT4B(i,2) =   dTad1MT4B(i,2) + ZH(j,i) * tad1B(j)   
         dTad1MT4B(i,3) =   dTad1MT4B(i,3) + ZH(j,i) * tad1EPB(j) 
          enddo  
         enddo

         dTad1MT4S(1:6,1:3)=0D0 
c end one-loop tadpole
c one-loop charged Higgs
#ifndef CHARGEDHIGGSOS
       dMHp1MT4(1)=(3*(CS3SSS14*MSQ2+CS3S13S14(1)*CS3S14S13(1)+
     &  CS3S13S14(2)*CS3S14S13(2)+M22*CS3SSS13(1,1)+M42*CS3SSS13(2,2)-
     &  2*CTBS3R*M12*CTBS3C(2)))*fac-CB2*MHp2*dZHu4
       dMHp1MT4(2)=(3*(a0t5*(CS3SSS14*(M22-MSQ2)*(-M42+MSQ2)+M42*
     &  CS3S13S14(1)*CS3S14S13(1)-MSQ2*CS3S13S14(1)*CS3S14S13(1)+M22*
     &  CS3S13S14(2)*CS3S14S13(2)-MSQ2*CS3S13S14(2)*CS3S14S13(2))+a0t2*
     &  (M42-MSQ2)*(-(CS3S13S14(1)*CS3S14S13(1))+(-M22+MSQ2)*
     &  CS3SSS13(1,1))+(M22-MSQ2)*(-(a0t4*(CS3S13S14(2)*CS3S14S13(2)+
     &  (M42-MSQ2)*CS3SSS13(2,2)))+2*a0t1*CTBS3R*(M42-MSQ2)*CTBS3C(2)))
     &  )/((M22-MSQ2)*(-M42+MSQ2))*fac
       dMHp1MT4(3)=(3*(aet5*CS3SSS14+((aet2-aet5)*CS3S13S14(1)*
     &  CS3S14S13(1))/(M22-MSQ2)+((aet4-aet5)*CS3S13S14(2)*CS3S14S13(2)
     &  )/(M42-MSQ2)+aet2*CS3SSS13(1,1)+aet4*CS3SSS13(2,2)-2*aet1*
     &  CTBS3R*CTBS3C(2)))/(16.*Pi2)

        dMHp1MT4B(1:3)=0D0

        do I2G3 = 1,2
       dMHp1MT4B(1)=dMHp1MT4B(1)+CS3S3S3S3(2,I2G3,2,I2G3)*MassHpm(I2G3)**2
       dMHp1MT4B(2)=A0THpN(I2G3)*CS3S3S3S3(2,I2G3,2,I2G3)+dMHp1MT4B(2)
       dMHp1MT4B(3)=AETHpN(I2G3)*CS3S3S3S3(2,I2G3,2,I2G3)+dMHp1MT4B(3)
    	enddo

    	do I5G3 = ineu,3
       dMHp1MT4B(1)=dMHp1MT4B(1)+(-2*CF11F12S3L(I5G3,2)*(MCha*
     &  DCONJG(CF11F12S3R(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3L(I5G3,2))*(MCha2+MNeu2(I5G3)))-2*
     &  CF11F12S3R(I5G3,2)*(MCha*DCONJG(CF11F12S3L(I5G3,2))*
     &  MNeu(I5G3)+DCONJG(CF11F12S3R(I5G3,2))*(MCha2+MNeu2(I5G3)))
     &  )

       dMHp1MT4B(2)=dMHp1MT4B(2)+(-(CF11F12S3L(I5G3,2)*(2*MCha*
     &  B0P(0D0,MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12S3R(I5G3,2))*
     &  MNeu(I5G3)+DCONJG(CF11F12S3L(I5G3,2))*(a0tcha+A0T(MNeu2(I5G3)
     &  ,Q2)+B0P(0D0,MCha2,MNeu2(I5G3),Q2)*(MCha2+MNeu2(I5G3)))))-
     &  CF11F12S3R(I5G3,2)*(2*MCha*B0P(0D0,MCha2,MNeu2(I5G3),Q2)*
     &  DCONJG(CF11F12S3L(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3R(I5G3,2))*(a0tcha+A0T(MNeu2(I5G3),Q2)+
     &  B0P(0D0,MCha2,MNeu2(I5G3),Q2)*(MCha2+MNeu2(I5G3)))))

       dMHp1MT4B(3)=dMHp1MT4B(3)+(-2*CF11F12S3L(I5G3,2)*(2*MCha*
     &  BEpP(0D0,MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12S3R(I5G3,2))*
     &  MNeu(I5G3)+DCONJG(CF11F12S3L(I5G3,2))*(aetcha+AEpT(MNeu2(I5G3)
     &  ,Q2)+BEpP(0D0,MCha2,MNeu2(I5G3),Q2)*(MCha2+MNeu2(I5G3))))-2*
     &  CF11F12S3R(I5G3,2)*(2*MCha*BEpP(0D0,MCha2,MNeu2(I5G3),Q2)*
     &  DCONJG(CF11F12S3L(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3R(I5G3,2))*(aetcha+AEpT(MNeu2(I5G3),Q2)+
     &  BEpP(0D0,MCha2,MNeu2(I5G3),Q2)*(MCha2+MNeu2(I5G3)))))/2.
    	enddo

    	do I6G3 = ihiggs,6
       dMHp1MT4B(1)=dMHp1MT4B(1)+(CS1S1S3S3(I6G3,I6G3,2,2)*Mah2(I6G3))/2.
       dMHp1MT4B(2)=(A0TNH(I6G3)*CS1S1S3S3(I6G3,I6G3,2,2))/2.+dMHp1MT4B(2)
       dMHp1MT4B(3)=(AETNH(I6G3)*CS1S1S3S3(I6G3,I6G3,2,2))/2.+dMHp1MT4B(3)
    	enddo

    	I2G3 = 2
    	do I6G3 = ihiggs,6
       dMHp1MT4B(1)=CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+
     &  dMHp1MT4B(1)
       dMHp1MT4B(2)=aB0TNCH(I6G3,I2G3)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(2)
       dMHp1MT4B(3)=aBEpTNCH(I6G3,I2G3)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(3)
    	enddo

    	I2G3 = 1
    	do I6G3 = ihiggs,5
       dMHp1MT4B(1)=CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+
     &  dMHp1MT4B(1)
       dMHp1MT4B(2)= aB0TNCH(I6G3,I2G3)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(2)
       dMHp1MT4B(3)=aBEpTNCH(I6G3,I2G3)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(3)
    	enddo

       dMHp1MT4B(1:3)=dMHp1MT4B(1:3)*fac

       dMHp1MT4B(1) =dMHp1MT4B(1)-MHp2*(CB2*Zfactor1LIN(2,2)+SB2*Zfactor1LIN(1,1)) 

       dMHp1MT4S(1:3)=0D0

#else
      dMHp1MT4(1)=(3*(CS3SSS14*MSQ2+CS3S13S14(1)*CS3S14S13(1)+
     &  CS3S13S14(2)*CS3S14S13(2)+M22*CS3SSS13(1,1)+M42*CS3SSS13(2,2)-
     &  2*CTBS3R*M12*CTBS3C(2)+CTBS3R*MHp2*CTBS3C(2)))*fac
       dMHp1MT4(2)=(3*(a0t5*CS3SSS14+B0P(MHp2,M22,MSQ2,Q2)*CS3S13S14(1)
     &  *CS3S14S13(1)+B0P(MHp2,M42,MSQ2,Q2)*CS3S13S14(2)*CS3S14S13(2)+
     &  a0t2*CS3SSS13(1,1)+a0t4*CS3SSS13(2,2)-a0t1*CTBS3R*CTBS3C(2)-
     &  CTBS3R*M12*B0P(MHp2,0D0,M12,Q2)*CTBS3C(2)+CTBS3R*MHp2*
     &  B0P(MHp2,0D0,M12,Q2)*CTBS3C(2)))*fac
       dMHp1MT4(3)=(3*(aet5*CS3SSS14+BEpP(MHp2,M22,MSQ2,Q2)*
     &  CS3S13S14(1)*CS3S14S13(1)+BEpP(MHp2,M42,MSQ2,Q2)*CS3S13S14(2)*
     &  CS3S14S13(2)+aet2*CS3SSS13(1,1)+aet4*CS3SSS13(2,2)-aet1*CTBS3R*
     &  CTBS3C(2)-CTBS3R*M12*BEpP(MHp2,0D0,M12,Q2)*CTBS3C(2)+CTBS3R*
     &  MHp2*BEpP(MHp2,0D0,M12,Q2)*CTBS3C(2)))/(16.*Pi2)

        dMHp1MT4B(1:3)=0D0
   	    do I2G3 = 1,2
       dMHp1MT4B(1)=dMHp1MT4B(1)+CS3S3S3S3(2,I2G3,2,I2G3)*MassHpm(I2G3)**2
       dMHp1MT4B(2)=A0THpN(I2G3)*CS3S3S3S3(2,I2G3,2,I2G3)+dMHp1MT4B(2)
       dMHp1MT4B(3)=AETHpN(I2G3)*CS3S3S3S3(2,I2G3,2,I2G3)+dMHp1MT4B(3)
    	enddo
    	do I5G3 = ineu,3
       dMHp1MT4B(1)=dMHp1MT4B(1)+(CF11F12S3L(I5G3,2)*(-2*MCha*
     &  DCONJG(CF11F12S3R(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3L(I5G3,2))*(-2*MCha2+MHp2-2*MNeu2(I5G3)))+
     &  CF11F12S3R(I5G3,2)*(-2*MCha*DCONJG(CF11F12S3L(I5G3,2))*
     &  MNeu(I5G3)+DCONJG(CF11F12S3R(I5G3,2))*(-2*MCha2+MHp2-2*
     &  MNeu2(I5G3))))
       dMHp1MT4B(2)=dMHp1MT4B(2)+(-(CF11F12S3L(I5G3,2)*(2*MCha*
     &  B0P(MHp2,MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12S3R(I5G3,2))*
     &  MNeu(I5G3)+DCONJG(CF11F12S3L(I5G3,2))*(a0tcha+A0T(MNeu2(I5G3)
     &  ,Q2)+B0P(MHp2,MCha2,MNeu2(I5G3),Q2)*(MCha2-MHp2+MNeu2(I5G3)))))
     &  -CF11F12S3R(I5G3,2)*(2*MCha*B0P(MHp2,MCha2,MNeu2(I5G3),Q2)
     &  *DCONJG(CF11F12S3L(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3R(I5G3,2))*(a0tcha+A0T(MNeu2(I5G3),Q2)+
     &  B0P(MHp2,MCha2,MNeu2(I5G3),Q2)*(MCha2-MHp2+MNeu2(I5G3)))))
       dMHp1MT4B(3)=dMHp1MT4B(3)+(-2*CF11F12S3L(I5G3,2)*(2*MCha*
     &  BEpP(MHp2,MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12S3R(I5G3,2))
     &  *MNeu(I5G3)+DCONJG(CF11F12S3L(I5G3,2))*(aetcha+AEpT(MNeu2(I5G3)
     &  ,Q2)+BEpP(MHp2,MCha2,MNeu2(I5G3),Q2)*(MCha2-MHp2+MNeu2(I5G3))))
     &  -2*CF11F12S3R(I5G3,2)*(2*MCha*BEpP(MHp2,MCha2,MNeu2(I5G3)
     &  ,Q2)*DCONJG(CF11F12S3L(I5G3,2))*MNeu(I5G3)+
     &  DCONJG(CF11F12S3R(I5G3,2))*(aetcha+AEpT(MNeu2(I5G3),Q2)+
     &  BEpP(MHp2,MCha2,MNeu2(I5G3),Q2)*(MCha2-MHp2+MNeu2(I5G3)))))/2.
    	enddo
    	do I6G3 = ihiggs,6
       dMHp1MT4B(1)=dMHp1MT4B(1)+(CS1S1S3S3(I6G3,I6G3,2,2)*Mah2(I6G3))/2.
       dMHp1MT4B(2)=(A0TNH(I6G3)*CS1S1S3S3(I6G3,I6G3,2,2))/2.+dMHp1MT4B(2)
       dMHp1MT4B(3)=(AETNH(I6G3)*CS1S1S3S3(I6G3,I6G3,2,2))/2.+dMHp1MT4B(3)
    	enddo
    	do I2G3 = 1,2
    	do I6G3 = ihiggs,6
       dMHp1MT4B(1)=CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+
     &  dMHp1MT4B(1)
       dMHp1MT4B(2)=B0P(MHp2,Mah2(I6G3),MassHpm(I2G3)**2,Q2)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(2)
       dMHp1MT4B(3)=BEpP(MHp2,Mah2(I6G3),MassHpm(I2G3)**2,Q2)*
     &  CS1S3S3(I6G3,2,I2G3)*CS1S3S3(I6G3,I2G3,2)+dMHp1MT4B(3)
    	enddo
    	enddo
        dMHp1MT4B(1:3)=dMHp1MT4B(1:3)*fac
#endif

         dTanB4= dZHu4*TB/2D0
         dTanB4B= (Zfactor1LIN(2,2) -Zfactor1LIN(1,1))*TB/2D0
         dTanB4S=0D0 
         dPMW24 = -3/8.*M12/pi2/vv2
         dPMZ24 = -3/8.*M12/pi2/vv2 

         CF    = 3/16D0/pi2/vv2
         dMW24 = -CF*(-M12 + 2*A0T1+ZU(1,1)*DCONJG(ZU(1,1))*xF0(M22,MSQ2)+
     -      ZU(2,1)*DCONJG(ZU(2,1))*xF0(M42,MSQ2))
         dMZ24 = -(- 2*CF*M12*Log(M12/Q2) +CF*ZU(1,1)*DCONJG(ZU(1,1))
     -      *ZU(1,2)*DCONJG(ZU(1,2))* xF0(M22,M42))

         dMWeps=CF*( A0T1 - 2*AET1 - 
     -  (M22*A0T2*ZU(1,1)*ZUC(1,1))/(M22 - MSQ2) - 
     -  (2*MSQ2*AET2*ZU(1,1)*ZUC(1,1))/(M22 - MSQ2) - 
     -  (M42*A0T4*ZU(2,1)*ZUC(2,1))/(M42 - MSQ2) - 
     -  (2*MSQ2*AET4*ZU(2,1)*ZUC(2,1))/(M42 - MSQ2) + 
     -  MSQ2*A0T5*((ZU(1,1)*ZUC(1,1))/(M22 - MSQ2) + 
     -  (ZU(2,1)*ZUC(2,1))/(M42 - MSQ2)) + 
     -  (M12 - M22*ZU(1,1)*ZUC(1,1) - MSQ2*ZU(1,1)*ZUC(1,1) - 
     -  M42*ZU(2,1)*ZUC(2,1) -   MSQ2*ZU(2,1)*ZUC(2,1))/2. + 
     -  AET5*(2 + (2*MSQ2*ZU(1,1)*ZUC(1,1))/(M22 - MSQ2) + 
     -  (2*MSQ2*ZU(2,1)*ZUC(2,1))/(M42 - MSQ2)) )

          dMZeps=CF*(2*A0T1-2*AET1-((M22+M42)*(ZU(2,1)*ZUC(1,1)
     &  -2*ZU(2,2)*ZUC(1,2))*(ZU(1,1)*ZUC(2,1)-2*ZU(1,2)*ZUC(2,2)))
     &  /18.-(M22*A0T2*(ZU(2,1)*ZUC(1,1)-2*ZU(2,2)*ZUC(1,2))*
     &  (ZU(1,1)*ZUC(2,1)-2*ZU(1,2)*ZUC(2,2)))/(9.*(M22-M42))+(M42*
     &  A0T4*(ZU(2,1)*ZUC(1,1)-2*ZU(2,2)*ZUC(1,2))*(ZU(1,1)*
     &  ZUC(2,1)-2*ZU(1,2)*ZUC(2,2)))/(9.*(M22-M42))+(2*AET2*
     &  ((1+2*(CW2-SW2))**2*ZU(1,1)*ZUC(1,1)+16*SW2**2*ZU(1,2)*
     &  ZUC(1,2)-SW2**2*(ZU(1,1)*ZUC(1,1)+4*ZU(1,2)*ZUC(1,2))**2-3*CW
     &  2*ZU(1,1)*ZUC(1,1)*(3*CW2*ZU(1,1)*ZUC(1,1)-2*SW2*(ZU(1,1)*
     &  ZUC(1,1)+4*ZU(1,2)*ZUC(1,2)))-(M22*(ZU(2,1)*ZUC(1,1)-2*ZU(2,2)*
     &  ZUC(1,2))*(ZU(1,1)*ZUC(2,1)-2*ZU(1,2)*ZUC(2,2)))/(M22-M42)))
     &  /9.-(2*AET4*(-6*SW2*ZU(2,1)*ZUC(2,1)*(-4*M22+4*M42-
     &  CW2*(M42*ZU(1,1)*ZUC(1,1)-M22*ZU(2,1)*ZUC(2,1)+M42*ZU(2,1)*
     &  ZUC(2,1)))+3*(3*(-M22+M42)*ZU(2,1)*ZUC(2,1)-3*CW2**2*ZU(2,1)*
     &  ZUC(2,1)*(M42*ZU(1,1)*ZUC(1,1)-M22*ZU(2,1)*ZUC(2,1)+M42*ZU(2,1)
     &  *ZUC(2,1))+4*CW2*SW2*(-2*M22*ZU(2,1)*ZU(2,2)*ZUC(2,1)*
     &  ZUC(2,2)+M42*(ZU(1,1)*ZU(2,2)*ZUC(1,2)*ZUC(2,1)+ZU(2,1)*
     &  (ZU(1,2)*ZUC(1,1)+2*ZU(2,2)*ZUC(2,1))*ZUC(2,2))))+SW2**2*(M22*
     &  (ZU(2,1)**2*ZUC(2,1)**2+8*ZU(2,1)*ZUC(2,1)*(-2+ZU(2,2)*ZUC(2,2)
     &  )+16*ZU(2,2)*ZUC(2,2)*(-1+ZU(2,2)*ZUC(2,2)))-M42*(ZU(2,1)**2*
     &  ZUC(2,1)**2+4*ZU(2,2)*(ZU(1,1)*ZUC(1,2)*ZUC(2,1)+4*ZUC(2,2)*(-
     &  1+ZU(1,2)*ZUC(1,2)+ZU(2,2)*ZUC(2,2)))+ZU(2,1)*(4*ZU(1,2)*
     &  ZUC(1,1)*ZUC(2,2)+ZUC(2,1)*(-16+ZU(1,1)*ZUC(1,1)+8*ZU(2,2)*
     &  ZUC(2,2)))))))/(9.*(M22-M42)))

c new atlambda contribution
    	I2G3 = 2
       dMW24B(1)=(-4*SW**2*CS3S3V3V3(I2G3,I2G3)*MassHpm(I2G3)
     &  **2)/vv**2
       dMW24B(2)=(-4*SW**2*A0THpN(I2G3)*CS3S3V3V3(I2G3,I2G3))/vv**2
       dMW24B(3)=(-4*SW**2*AETHpN(I2G3)*CS3S3V3V3(I2G3,I2G3))/vv**2

    	do I6G3 = ihiggs,5
       dMW24B(1)=dMW24B(1)+(-2*SW**2*CS1S1V3V3(I6G3,I6G3)*Mah2(I6G3))/vv**2
       dMW24B(2)=(-2*SW**2*A0TNH(I6G3)*CS1S1V3V3(I6G3,I6G3))/vv**2+dMW24B(2)
       dMW24B(3)=(-2*SW**2*AETNH(I6G3)*CS1S1V3V3(I6G3,I6G3))/vv**2+dMW24B(3)
    	enddo

    	do I5G3 = ineu,3
       dMW24B(1)=dMW24B(1)+(-4*SW**2*(-2*MCha*(CF11F12V3R(I5G3)*
     &  DCONJG(CF11F12V3L(I5G3))+CF11F12V3L(I5G3)*
     &  DCONJG(CF11F12V3R(I5G3)))*MNeu(I5G3)+Abs(CF11F12V3L(I5G3))**2*
     &  (MCha2+MNeu2(I5G3))+Abs(CF11F12V3R(I5G3))**2*(MCha2+MNeu2(I5G3)
     &  )))/vv**2

       dMW24B(2)=dMW24B(2)+(2*SW**2*(CF11F12V3L(I5G3)*(4*MCha*
     &  B0T(MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12V3R(I5G3))*MNeu(I5G3)+
     &  DCONJG(CF11F12V3L(I5G3))*(MCha2-2*A0TNeu(I5G3)-2*MCha**2*
     &  B0T(MCha2,MNeu2(I5G3),Q2)+MNeu2(I5G3)))+CF11F12V3R(I5G3)*(4*
     &  MCha*B0T(MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12V3L(I5G3))*
     &  MNeu(I5G3)+DCONJG(CF11F12V3R(I5G3))*(MCha2-2*A0TNeu(I5G3)-2*
     &  MCha**2*B0T(MCha2,MNeu2(I5G3),Q2)+MNeu2(I5G3)))))/vv**2


       dMW24B(3)=dMW24B(3)+(SW**2*(CF11F12V3L(I5G3)*(8*MCha*
     &  BEpT(MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12V3R(I5G3))*MNeu(I5G3)+
     &  DCONJG(CF11F12V3L(I5G3))*(MCha2+2*A0TNeu(I5G3)-4*AETNeu(I5G3)+
     &  2*MCha2*B0T(MCha2,MNeu2(I5G3),Q2)-4*MCha2*
     &  BEpT(MCha2,MNeu2(I5G3),Q2)+MNeu2(I5G3)))+CF11F12V3R(I5G3)*(8*
     &  MCha*BEpT(MCha2,MNeu2(I5G3),Q2)*DCONJG(CF11F12V3L(I5G3))*
     &  MNeu(I5G3)+DCONJG(CF11F12V3R(I5G3))*(MCha2+2*A0TNeu(I5G3)-4*
     &  AETNeu(I5G3)+2*MCha2*B0T(MCha2,MNeu2(I5G3),Q2)-4*MCha2*
     &  BEpT(MCha2,MNeu2(I5G3),Q2)+MNeu2(I5G3)))))/vv**2
    	enddo

    	do I2G3 = 1,2
    	do I6G3 = ihiggs,6
        if ((MassHpm(I2G3)+Mah2(I6G3)) .gt.0) then
       dMW24B(1)=dMW24B(1)+(-4*SW**2*CS1S3V3(I6G3,I2G3)*
     &  DCONJG(CS1S3V3(I6G3,I2G3))*(Mah2(I6G3)+MassHpm(I2G3)**2))/vv**
     &  2
       dMW24B(2)=dMW24B(2)+(-2*SW**2*CS1S3V3(I6G3,I2G3)*
     &  DCONJG(CS1S3V3(I6G3,I2G3))*(2*A0THpN(I2G3)+Mah2(I6G3)+2*
     &   aB0TNCH(I6G3,I2G3)*Mah2(I6G3)+MassHpm(I2G3)**
     &  2))/vv**2
       dMW24B(3)=dMW24B(3)-((SW**2*CS1S3V3(I6G3,I2G3)*
     &  DCONJG(CS1S3V3(I6G3,I2G3))*(2*A0THpN(I2G3)+4*AETHpN(I2G3)+
     &  Mah2(I6G3)+2*aB0TNCH(I6G3,I2G3)*Mah2(I6G3)+4*
     &  aBEpTNCH(I6G3,I2G3)*Mah2(I6G3)+MassHpm(I2G3)**
     &  2))/vv**2)
        endif
    	enddo
    	enddo

       dMZ24B(1)=(-8*(CF12F12V2L-CF12F12V2R)**2*CW2*MCha2*SW2)/vv2
       dMZ24B(2)=(-4*CW2*SW2*((CF12F12V2L**2+CF12F12V2R**2)*(a0tcha-
     &  MCha2)+(CF12F12V2L**2-4*CF12F12V2L*CF12F12V2R+CF12F12V2R**2)*
     &  MCha2*B0T(MCha2,MCha2,Q2)))/vv2
       dMZ24B(3)=(2*CW2*SW2*((CF12F12V2L**2+CF12F12V2R**2)*(a0tcha-2*
     &  aetcha+MCha2)+(CF12F12V2L**2+CF12F12V2R**2)*MCha2*
     &  B0T(MCha2,MCha2,Q2)-2*(CF12F12V2L**2-4*CF12F12V2L*CF12F12V2R+
     &  CF12F12V2R**2)*MCha2*BEpT(MCha2,MCha2,Q2)))/vv2
    	I2G3 =2
       dMZ24B(1)=dMZ24B(1)+(-4*CW2*SW2*(2*CS3S3V2(2,2)**2+
     &  CS3S3V2V2(I2G3,I2G3))*MassHpm(I2G3)**2)/vv2
       dMZ24B(2)=dMZ24B(2)+(-4*CW2*SW2*(A0THpN(I2G3)*(CS3S3V2(2,2)**2+
     &  CS3S3V2V2(I2G3,I2G3))+CS3S3V2(2,2)**2*(1+aB0TCH(I2G3,I2G3)
     &  )*MassHpm(I2G3)**2))/vv2
       dMZ24B(3)=dMZ24B(3)+(-2*CW2*SW2*(CS3S3V2(2,2)**2*A0THpN(I2G3)+2*
     &  AETHpN(I2G3)*(CS3S3V2(2,2)**2+CS3S3V2V2(I2G3,I2G3))+CS3S3V2(2,2)**2*(1+
     &  aB0TCH(I2G3,I2G3)+2*aBEpTCH(I2G3,I2G3) )*MassHpm(I2G3)**2))/vv2

    	do I6G3 = ihiggs,5
       dMZ24B(1)=dMZ24B(1)+(-2*CW2*SW2*CS1S1V2V2(I6G3,I6G3)*Mah2(I6G3))/vv2
       dMZ24B(2)=(-2*CW2*SW2*A0TNH(I6G3)*CS1S1V2V2(I6G3,I6G3))/vv2+dMZ24B(2)
       dMZ24B(3)=(-2*CW2*SW2*AETNH(I6G3)*CS1S1V2V2(I6G3,I6G3))/vv2+dMZ24B(3)
    	enddo

    	do I5G3 = ineu,3
    	do I5G4 = ineu,3
       dMZ24B(1)=dMZ24B(1)+(-2*CW2*SW2*(CF11F11V2L(I5G4,I5G3)*(-2*
     &  CF11F11V2R(I5G3,I5G4)*MNeu(I5G3)*MNeu(I5G4)+
     &  CF11F11V2L(I5G3,I5G4)*(MNeu2(I5G3)+MNeu2(I5G4)))+
     &  CF11F11V2R(I5G4,I5G3)*(-2*CF11F11V2L(I5G3,I5G4)*MNeu(I5G3)*
     &  MNeu(I5G4)+CF11F11V2R(I5G3,I5G4)*(MNeu2(I5G3)+MNeu2(I5G4)))))
     &  /vv2
       dMZ24B(2)=dMZ24B(2)+(CW2*SW2*(-2*A0TNeu(I5G4)*
     &  (CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)+
     &  CF11F11V2R(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3))-2*B0T(MNeu2(I5G3)
     &  ,MNeu2(I5G4),Q2)*MNeu(I5G3)*(CF11F11V2R(I5G3,I5G4)*
     &  (CF11F11V2R(I5G4,I5G3)*MNeu(I5G3)-2*CF11F11V2L(I5G4,I5G3)*
     &  MNeu(I5G4))+CF11F11V2L(I5G3,I5G4)*(CF11F11V2L(I5G4,I5G3)*
     &  MNeu(I5G3)-2*CF11F11V2R(I5G4,I5G3)*MNeu(I5G4)))+
     &  (CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)+
     &  CF11F11V2R(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3))*(MNeu2(I5G3)+
     &  MNeu2(I5G4))))/vv2
       dMZ24B(3)=dMZ24B(3)+(CW2*SW2*(2*A0TNeu(I5G4)*
     &  (CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)+
     &  CF11F11V2R(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3))-4*AETNeu(I5G4)*
     &  (CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)+
     &  CF11F11V2R(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3))+8*BEpT(MNeu2(I5G3)
     &  ,MNeu2(I5G4),Q2)*CF11F11V2L(I5G4,I5G3)*CF11F11V2R(I5G3,I5G4)*
     &  MNeu(I5G3)*MNeu(I5G4)+8*BEpT(MNeu2(I5G3),MNeu2(I5G4),Q2)*
     &  CF11F11V2L(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3)*MNeu(I5G3)*
     &  MNeu(I5G4)+CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)*
     &  MNeu2(I5G3)+2*B0T(MNeu2(I5G3),MNeu2(I5G4),Q2)*
     &  CF11F11V2L(I5G3,I5G4)*CF11F11V2L(I5G4,I5G3)*MNeu2(I5G3)-4*
     &  BEpT(MNeu2(I5G3),MNeu2(I5G4),Q2)*CF11F11V2L(I5G3,I5G4)*
     &  CF11F11V2L(I5G4,I5G3)*MNeu2(I5G3)+CF11F11V2R(I5G3,I5G4)*
     &  CF11F11V2R(I5G4,I5G3)*MNeu2(I5G3)+2*B0T(MNeu2(I5G3),MNeu2(I5G4)
     &  ,Q2)*CF11F11V2R(I5G3,I5G4)*CF11F11V2R(I5G4,I5G3)*MNeu2(I5G3)-4*
     &  BEpT(MNeu2(I5G3),MNeu2(I5G4),Q2)*CF11F11V2R(I5G3,I5G4)*
     &  CF11F11V2R(I5G4,I5G3)*MNeu2(I5G3)+CF11F11V2L(I5G3,I5G4)*
     &  CF11F11V2L(I5G4,I5G3)*MNeu2(I5G4)+CF11F11V2R(I5G3,I5G4)*
     &  CF11F11V2R(I5G4,I5G3)*MNeu2(I5G4)))/(2.*vv2)
    	enddo
    	enddo

    	do I6G3 = ihiggs,6
    	do I6G4 = ihiggs,6
        if((Mah2(I6G3)+Mah2(I6G4)) .gt.0)then
       dMZ24B(1)=dMZ24B(1)+(-2*CW2*SW2*CS1S1V2(I6G3,I6G4)**2*
     &  (Mah2(I6G3)+Mah2(I6G4)))/vv2
       dMZ24B(2)=dMZ24B(2)-((CW2*SW2*CS1S1V2(I6G3,I6G4)**2*(2*
     &  A0TNH(I6G4)+Mah2(I6G3)+2*aB0TNH(I6G3,I6G4)*Mah2(I6G3)+
     &  Mah2(I6G4)))/vv2)
       dMZ24B(3)=dMZ24B(3)-(CW2*SW2*CS1S1V2(I6G3,I6G4)**2*(2*
     &  A0TNH(I6G4)+4*AETNH(I6G4)+Mah2(I6G3)+2*aB0TNH(I6G3,I6G4)*
     &  Mah2(I6G3)+4*aBEpTNH(I6G3,I6G4)*Mah2(I6G3)+Mah2(I6G4)))/(2.*
     &  vv2)
        endif
    	enddo
    	enddo

        do i=1,3
       dMZ24B(i)=dMZ24B(i)*fac
       dMZ24S(i)=0D0

       dMW24B(i)=dMW24B(i)*fac
       dMW24S(i)=0D0

       dv4B(i)= (-CW2/SW2/2.*(dMW24B(i) - dMZ24B(i))+1/2.*dMW24B(i))*vv
       dv4S(i)= (-CW2/SW2/2.*(dMW24S(i) - dMZ24S(i))+1/2.*dMW24S(i))*vv
       dSW4B(i)=- CW2/SW/2D0 *(dMW24B(i) -  dMZ24B(i))
       dSW4S(i)=- CW2/SW/2D0 *(dMW24S(i) -  dMZ24S(i))
       dCW4B(i) = -((dSW4B(i)*SW)/CW)
       dCW4S(i) = -((dSW4S(i)*SW)/CW)
        enddo     

         dv4(1)  =(-CW2/SW2/2.*(dPMW24 - dPMZ24)+1/2.*dPMW24)*vv
         dSW4(1)=- CW**2/SW/2D0 *(dPMW24 - dPMZ24)
         dabslambda14= -abslambda*(dv4(1)/vv + 1/2D0*CB2*dZHu4)
         dlambdaC4= -1/2D0* dZHu4* DCONJG(la)
         dMUE4= ( -0.5D0* dZHu4 *MUE)

         dv4(2)= (-CW2/SW2/2.*(dMW24 - dMZ24)+1/2.*dMW24)*vv
         dSW4(2)= -CW2/SW/2D0 *(dMW24 - dMZ24)
         dv4(3)= (-CW2/SW2/2.*(dMWeps - dMZeps)+1/2.*dMWeps)*vv
         dSW4(3)=- CW2/SW/2D0 *(dMWeps - dMZeps)
         dCW4(1:3) = -((dSW4(1:3)*SW)/CW)

         dMZoMZ = dMZ24B(2) + dMZ24 
         dMWoMW = dMW24B(2) + dMW24 

c neutral Higgs selfenergies at one-loop

       if(MHpmInput)then
       else
        do i=2,3
        dMHp1MT4(i)=(SB**4*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*dtad1MT4(1,i) + 
     -    CB**3*SB*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*
     -     dtad1MT4(2,i) - 
     -    (Cosphiy*Sink3s + Cosk3s*Sinphiy)*dtad1MT4(4,i) + 
     -    abslambda**2*CB*SB**2*
     -     (-(Cosk3s*Cosphiy) + Sink3s*Sinphiy)*vv2*dv4(i))/
     -  (CB*SB**2*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*vv)
        dMHp1MT4B(i)=(SB**4*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*dtad1MT4B(1,i) + 
     -    CB**3*SB*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*
     -     dtad1MT4B(2,i) - 
     -    (Cosphiy*Sink3s + Cosk3s*Sinphiy)*dtad1MT4B(4,i) + 
     -    abslambda**2*CB*SB**2*
     -     (-(Cosk3s*Cosphiy) + Sink3s*Sinphiy)*vv2*dv4B(i))/
     -  (CB*SB**2*(Cosk3s*Cosphiy - Sink3s*Sinphiy)*vv)
        enddo
       endif


        do i=1,6
         do j=1,6
       selfHH1MT4(i,j)=-2*MCha2*(CFF12S1R(i)*(2*CFF12S1L(j)+CFF12S1R(j)
     &  )+CFF12S1L(i)*(CFF12S1L(j)+2*CFF12S1R(j)))
    	I2G3 =2
       selfHH1MT4(i,j)=CS1S1S3S3(i,j,I2G3,I2G3)*MassHpm(I2G3)**2+
     &  selfHH1MT4(i,j)
    	
    	do I2G3 = 1,2
    	do I2G4 = 1,2
       selfHH1MT4(i,j)=CS1S3S3(i,I2G3,I2G4)*CS1S3S3(j,I2G4,I2G3)+
     &  selfHH1MT4(i,j)
    	enddo
    	enddo
    	do I5G3 = ineu,3
    	do I5G4 = ineu,3
       selfHH1MT4(i,j)=(-(CFF11S1R(I5G3,I5G4,j)*(CFF11S1R(I5G4,I5G3,i)*
     &  MNeu(I5G3)*MNeu(I5G4)+CFF11S1L(I5G4,I5G3,i)*(MNeu2(I5G3)+
     &  MNeu2(I5G4))))-CFF11S1L(I5G3,I5G4,j)*(CFF11S1L(I5G4,I5G3,i)*
     &  MNeu(I5G3)*MNeu(I5G4)+CFF11S1R(I5G4,I5G3,i)*(MNeu2(I5G3)+
     &  MNeu2(I5G4))))+selfHH1MT4(i,j)
    	enddo
    	enddo
    	do I6G3 = ihiggs,6
       selfHH1MT4(i,j)=(CS1S1S1S1(i,j,I6G3,I6G3)*Mah2(I6G3))/2.+selfHH1MT4(i,j)
    	enddo
    	do I6G3 = ihiggs,6
    	do I6G4 = ihiggs,6
       selfHH1MT4(i,j)=(CS1S1S1(i,I6G3,I6G4)*CS1S1S1(j,I6G3,I6G4))/2.+
     &  selfHH1MT4(i,j)
    	enddo
    	enddo
         selfHH1MT4(i,j)=selfHH1MT4(i,j)*fac
         enddo
       enddo

        do i=1,6
         do j=1,6
          selfHH1MT4IN(i,j)=0D0
          do I6G3=1,6
          do I6G4=1,6
          selfHH1MT4IN(i,j)=selfHH1MT4IN(i,j)+ZH(I6G3,i)*ZH(I6G4,j)*selfHH1MT4(I6G3,I6G4)
          enddo
          enddo
         enddo
       enddo


       dabslambda14B=-(CB*dTanB4B*(CB**4-SB**4)*(-2*MHp2+la2*vv2))/(2.*
     &  abslambda*SB*vv2)+dMHp1MT4B(1)/(abslambda*vv2)-(SB2*
     &  dtad1MT4B(1,1))/(abslambda*CB*vv**3)-(CB2*dtad1MT4B(2,1))
     &  /(abslambda*SB*vv**3)-(abslambda*dv4B(1))/vv+selfHH1MT4IN(1,2)
     &  /(abslambda*CB*SB*vv2)+(-abslambda/4.+MHp2/(2.*abslambda*vv2))*
     &  Zfactor1LIN(1,1)+(-abslambda/4.+MHp2/(2.*abslambda*vv2))*
     &  Zfactor1LIN(2,2)

       dlambdaC4B=dabslambda14B*DCONJG(la)/abs(la) 
 
       
       dvS1B=(CB2*dTanB4B*(abskappa*abslambda*CB**4*Cosphiy*vs2+
     &  abskappa*abslambda*Cosphiy*SB**4*vs2+2*abskappa*abslambda*CB2*
     &  Cosphiy*SB2*vs2+CB**5*la2*SB*vv2+2*CB**3*la2*SB*(2*vs2+SB2*vv2)
     &  +CB*(2*MHp2*SB+la2*SB**3*(4*vs2+SB2*vv2))))/(2.*abslambda**2*
     &  (CB2-SB2)*vS)+(2*abslambda*dabslambda14B*(-CB2+SB2)*vS*vv-la2*
     &  (CB2-SB2)*vS*dv4B(1)+CB*selfHH1MT4IN(1,3)-SB*selfHH1MT4IN(2,3))
     &  /(abslambda**2*(CB2-SB2)*vv)+(CB*(abskappa*abslambda*Cosphiy*
     &  SB*vs2+CB*(2*MHp2*SB2+la2*(-2*vs2+SB2*vv2)))*Zfactor1LIN(1,1))
     &  /(4.*abslambda**2*(CB2-SB2)*vS)+(SB*(abskappa*abslambda*CB*
     &  Cosphiy*vs2-2*la2*SB*vs2+CB2*SB*(2*MHp2+la2*vv2))*
     &  Zfactor1LIN(2,2))/(4.*abslambda**2*(-CB2+SB2)*vS)-(vS*
     &  Zfactor1LIN(3,3))/2.



       dabskappa14B=-((CB2*dTanB4B*(-2*MHp2*SB**3+abskappa*abslambda*
     &  CB**5*Cosphiy*vs2+abskappa*abslambda*CB*Cosphiy*SB**4*vs2+2*
     &  abskappa*abslambda*CB**3*Cosphiy*SB2*vs2+4*CB2*SB*(MHp2+la2*
     &  SB2*vs2)+2*CB**6*la2*SB*vv2+la2*SB**5*(2*vs2-SB2*vv2)+CB**4*
     &  la2*SB*(2*vs2+3*SB2*vv2)))/(abslambda*Cosphiy*SB*vS2))+(dvS1B*
     &  (-(abskappa*abslambda*CB**4*Cosphiy*SB*vs2)-2*abskappa*
     &  abslambda*CB2*Cosphiy*SB**3*vs2-abskappa*abslambda*Cosphiy*SB**
     &  5*vs2+CB**5*la2*(2*vs2+SB2*vv2)+2*CB**3*la2*SB2*(2*vs2+SB2*vv2)
     &  +CB*(2*MHp2*SB2+la2*SB**4*(2*vs2+SB2*vv2))))/(abslambda*
     &  Cosphiy*SB*vS**3)-(2*CB*SB*dMHp1MT4B(1))/(abslambda*Cosphiy*
     &  vS2)+(2*SB**3*dtad1MT4B(1,1))/(abslambda*Cosphiy*vS2*vv)+(2*CB*
     &  *3*dtad1MT4B(2,1))/(abslambda*Cosphiy*vS2*vv)-(abskappa*
     &  Cosphiy*dabslambda14B*SB*vs2*vv+2*CB*MHp2*SB2*dv4B(1)+CB*la2*(-
     &  2*vs2+3*SB2*vv2)*dv4B(1)+abslambda*(2*CB*dabslambda14B*vv*(-2*
     &  vs2+SB2*vv2)+abskappa*Cosphiy*SB*vs2*dv4B(1))+2*vS*
     &  selfHH1MT4IN(1,3))/(abslambda*Cosphiy*SB*vS2*vv)+(-abskappa/2.-
     &  (CB*(2*MHp2*SB2+la2*(-2*vs2+SB2*vv2)))/(2.*abslambda*Cosphiy*
     &  SB*vS2))*Zfactor1LIN(1,1)+(-abskappa/2.-(CB*(2*MHp2*SB2+la2*(-
     &  2*vs2+SB2*vv2)))/(2.*abslambda*Cosphiy*SB*vS2))*
     &  Zfactor1LIN(3,3)  



       dReAkappa14B=-((CB2*dTanB4B*(CB**4-SB**4)*vv2*(-2*abskappa*
     &  abslambda*CB2*SB2*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2+
     &  abskappa*abslambda*CB**4*(-(Cosk3s*Cosphiy)+3*Sink3s*Sinphiy)*
     &  vs2+abskappa*abslambda*SB**4*(-(Cosk3s*Cosphiy)+3*Sink3s*
     &  Sinphiy)*vs2+2*CB**5*Cosk3s*la2*SB*vv2+4*CB**3*Cosk3s*la2*SB**
     &  3*vv2+2*CB*Cosk3s*SB*(2*MHp2+la2*SB**4*vv2)))/(abskappa*sqrt2*
     &  vS**3))+(dvS1B*(2*Cosk3s*(-4*ka2*SB**4*vS**4+CB**6*la2*SB2*vv**
     &  4+2*CB**4*(-2*ka2*vS**4+la2*SB**4*vv**4)+CB2*SB2*(-8*ka2*vS**4+
     &  la2*SB**4*vv**4+2*MHp2*vv2))-abskappa*sqrt2*vS**3*DBLE(Aka)))
     &  /(abskappa*sqrt2*vS**4)-(CB2*Cosk3s*SB2*sqrt2*vv2*dMHp1MT4B(1))
     &  /(abskappa*vS**3)+(CB*Cosk3s*SB**4*sqrt2*vv*dtad1MT4B(1,1))
     &  /(abskappa*vS**3)+(CB**4*Cosk3s*SB*sqrt2*vv*dtad1MT4B(2,1))
     &  /(abskappa*vS**3)-(Cosk3s*sqrt2*dtad1MT4B(3,1))/(abskappa*vS2)+
     &  (CB*Sink3s*sqrt2*vv*dtad1MT4B(4,1))/(abskappa*vS**3)-(Sink3s*
     &  sqrt2*dtad1MT4B(6,1))/(abskappa*vS2)+(-8*abskappa*Cosk3s*
     &  dabskappa14B*vS**4+abskappa*CB*Cosk3s*Cosphiy*dabslambda14B*SB*
     &  vs2*vv2-3*abskappa*CB*dabslambda14B*SB*Sink3s*Sinphiy*vs2*vv2-
     &  dabskappa14B*sqrt2*vS**3*DBLE(Aka)-4*CB2*Cosk3s*MHp2*SB2*vv*
     &  dv4B(1)-4*CB2*Cosk3s*la2*SB2*vv**3*dv4B(1)+abslambda*CB*SB*vv*
     &  (-2*CB*Cosk3s*dabslambda14B*SB*vv**3+(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vs2*(dabskappa14B*vv+2*abskappa*dv4B(1)))+2*Cosk3s*
     &  vs2*selfHH1MT4IN(3,3))/(abskappa*sqrt2*vS**3)+((-4*CB**4*
     &  Cosk3s*ka2*vS**4-4*Cosk3s*ka2*SB**4*vS**4+abskappa*abslambda*
     &  CB*SB*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2*vv2-CB2*Cosk3s*SB2*
     &  (8*ka2*vS**4+la2*vv**4+2*MHp2*vv2)-abskappa*sqrt2*vS**3*
     &  DBLE(Aka))*Zfactor1LIN(3,3))/(abskappa*sqrt2*vS**3)

        if (ihiggs.gt.1)then
           dabslambda14B=0D0
           dvS1B=0D0
           dabskappa14B=0D0
           dReAkappa14B=0D0
           dlambdaC4B=0D0
        endif


  
       dMH1MT4BIN(1,1,1)=abslambda*dabslambda14B*SB2*vv2+SB2*
     &  dMHp1MT4B(1)+((CB**3+2*CB*SB2)*dtad1MT4B(1,1))/vv-(CB2*SB*
     &  dtad1MT4B(2,1))/vv+la2*SB2*vv*dv4B(1)+(SB2*(2*MHp2+la2*vv2)*
     &  Zfactor1LIN(1,1)*facZ)/2.
       dMH1MT4BIN(1,2,1)=abslambda*CB*dabslambda14B*SB*vv2-CB*SB*
     &  dMHp1MT4B(1)+(SB**3*dtad1MT4B(1,1))/vv+(CB**3*dtad1MT4B(2,1))
     &  /vv+CB*la2*SB*vv*dv4B(1)-(CB*SB*(2*MHp2-la2*vv2)*
     &  (Zfactor1LIN(1,1)+Zfactor1LIN(2,2))*facZ)/4.
       dMH1MT4BIN(1,3,1)=-(abslambda*Cosphiy*dabskappa14B*SB*vS*vv)/2.+
     &  dabslambda14B*(-(abskappa*Cosphiy*SB*vS*vv)/2.+abslambda*CB*(2*
     &  vS*vv-(SB2*vv**3)/vS))+(dvS1B*vv*(-(abskappa*abslambda*CB**4*
     &  Cosphiy*SB*vs2)-2*abskappa*abslambda*CB2*Cosphiy*SB**3*vs2-
     &  abskappa*abslambda*Cosphiy*SB**5*vs2+CB**5*la2*(2*vs2+SB2*vv2)+
     &  2*CB**3*la2*SB2*(2*vs2+SB2*vv2)+CB*(2*MHp2*SB2+la2*SB**4*(2*
     &  vs2+SB2*vv2))))/(2.*vS2)-(CB*SB2*vv*dMHp1MT4B(1))/vS+(SB**4*
     &  dtad1MT4B(1,1))/vS+(CB**3*SB*dtad1MT4B(2,1))/vS+((2*CB**5*la2*
     &  vs2-abskappa*abslambda*Cosphiy*SB*vs2+4*CB**3*la2*SB2*vs2-6*CB*
     &  *3*la2*SB**4*vv2-3*CB*la2*SB**6*vv2+SB2*(-2*CB*MHp2+2*CB*la2*
     &  SB2*vs2-3*CB**5*la2*vv2))*dv4B(1))/(2.*vS)-(vv*(abskappa*
     &  abslambda*Cosphiy*SB*vs2+CB*(2*MHp2*SB2+la2*(-2*vs2+SB2*vv2)))*
     &  (Zfactor1LIN(1,1)+Zfactor1LIN(3,3))*facZ)/(4.*vS)
       dMH1MT4BIN(1,4,1)=0
       dMH1MT4BIN(1,5,1)=dtad1MT4B(4,1)/(SB*vv)
       dMH1MT4BIN(1,6,1)=(-3*abskappa*abslambda*dvS1B*SB*Sinphiy*vv)
     &  /2.-(3*abslambda*dabskappa14B*SB*Sinphiy*vS*vv)/2.-(3*abskappa*
     &  dabslambda14B*SB*Sinphiy*vS*vv)/2.+dtad1MT4B(4,1)/vS-(3*
     &  abskappa*abslambda*SB*Sinphiy*vS*dv4B(1))/2.-(3*abskappa*
     &  abslambda*SB*Sinphiy*vS*vv*(Zfactor1LIN(1,1)+Zfactor1LIN(3,3))*facZ)
     &  /4.
       dMH1MT4BIN(2,2,1)=abslambda*CB2*dabslambda14B*vv2+CB2*
     &  dMHp1MT4B(1)-(CB*SB2*dtad1MT4B(1,1))/vv+((2*CB2*SB+SB**3)*
     &  dtad1MT4B(2,1))/vv+CB2*la2*vv*dv4B(1)+(CB2*(2*MHp2+la2*vv2)*
     &  Zfactor1LIN(2,2)*facZ)/2.
       dMH1MT4BIN(2,3,1)=-(abslambda*CB*Cosphiy*dabskappa14B*vS*vv)/2.+
     &  dabslambda14B*(-(abskappa*CB*Cosphiy*vS*vv)/2.+2*abslambda*SB*
     &  vS*vv-(abslambda*CB2*SB*vv**3)/vS)+(dvS1B*vv*(-(abskappa*
     &  abslambda*CB**5*Cosphiy*vs2)-abskappa*abslambda*CB*Cosphiy*SB**
     &  4*vs2+2*la2*SB**5*vs2-2*abskappa*abslambda*CB**3*Cosphiy*SB2*
     &  vs2+CB**6*la2*SB*vv2+2*CB**4*la2*SB*(vs2+SB2*vv2)+CB2*(2*MHp2*
     &  SB+la2*SB**3*(4*vs2+SB2*vv2))))/(2.*vS2)-(CB2*SB*vv*
     &  dMHp1MT4B(1))/vS+(CB*SB**3*dtad1MT4B(1,1))/vS+(CB**4*
     &  dtad1MT4B(2,1))/vS-((abskappa*abslambda*CB*Cosphiy*vs2-4*CB2*
     &  la2*SB**3*vs2-2*la2*SB**5*vs2+3*CB**6*la2*SB*vv2+6*CB**4*la2*
     &  SB**3*vv2+CB2*SB*(2*MHp2+la2*(-2*CB2*vs2+3*SB**4*vv2)))*dv4B(1)
     &  )/(2.*vS)-(vv*(abskappa*abslambda*CB*Cosphiy*vs2-2*la2*SB*vs2+
     &  CB2*SB*(2*MHp2+la2*vv2))*(Zfactor1LIN(2,2)+Zfactor1LIN(3,3))*facZ)
     &  /(4.*vS)
       dMH1MT4BIN(2,4,1)=dtad1MT4B(4,1)/(SB*vv)
       dMH1MT4BIN(2,5,1)=0
       dMH1MT4BIN(2,6,1)=(-3*abskappa*abslambda*CB*dvS1B*Sinphiy*vv)
     &  /2.-(3*abslambda*CB*dabskappa14B*Sinphiy*vS*vv)/2.-(3*abskappa*
     &  CB*dabslambda14B*Sinphiy*vS*vv)/2.+(CB*dtad1MT4B(4,1))/(SB*vS)-
     &  (3*abskappa*abslambda*CB*Sinphiy*vS*dv4B(1))/2.-(3*abskappa*
     &  abslambda*CB*Sinphiy*vS*vv*(Zfactor1LIN(2,2)+Zfactor1LIN(3,3))*facZ)
     &  /4.
       dMH1MT4BIN(3,3,1)=(abskappa*dReAkappa14B*vS)/(Cosk3s*sqrt2)+(CB*
     &  dabslambda14B*SB*vv2*(abskappa*(-(Cosk3s*Cosphiy)+3*Sink3s*
     &  Sinphiy)*vs2+2*abslambda*CB*Cosk3s*SB*vv2))/(2.*Cosk3s*vS2)+
     &  (dabskappa14B*(abslambda*CB*SB*(-(Cosk3s*Cosphiy)+3*Sink3s*
     &  Sinphiy)*vv2+vS*(8*abskappa*Cosk3s*vS+sqrt2*DBLE(Aka))))/(2.*
     &  Cosk3s)+(dvS1B*(-2*CB**6*Cosk3s*la2*SB2*vv**4+abskappa*SB**4*
     &  vS**3*(8*abskappa*Cosk3s*vS+sqrt2*DBLE(Aka))-2*CB2*SB2*(Cosk3s*
     &  (-8*ka2*vS**4+la2*SB**4*vv**4+2*MHp2*vv2)-abskappa*sqrt2*vS**3*
     &  DBLE(Aka))+CB**4*(-4*Cosk3s*la2*SB**4*vv**4+abskappa*vS**3*(8*
     &  abskappa*Cosk3s*vS+sqrt2*DBLE(Aka)))))/(2.*Cosk3s*vS**3)+(CB2*
     &  SB2*vv2*dMHp1MT4B(1))/vS2-(CB*SB**4*vv*dtad1MT4B(1,1))/vS2-(CB*
     &  *4*SB*vv*dtad1MT4B(2,1))/vS2+dtad1MT4B(3,1)/vS-(CB*Sink3s*vv*
     &  dtad1MT4B(4,1))/(Cosk3s*vS2)+(Sink3s*dtad1MT4B(6,1))/(Cosk3s*
     &  vS)+(CB*SB*vv*(2*CB*MHp2*SB-(abskappa*abslambda*(Cosk3s*
     &  Cosphiy-3*Sink3s*Sinphiy)*vs2)/Cosk3s+2*CB**5*la2*SB*vv2+4*CB**
     &  3*la2*SB**3*vv2+2*CB*la2*SB**5*vv2)*dv4B(1))/vS2+((4*CB**4*
     &  Cosk3s*ka2*vS**4+abskappa*abslambda*CB*SB*(-(Cosk3s*Cosphiy)+3*
     &  Sink3s*Sinphiy)*vs2*vv2+CB2*Cosk3s*SB2*(8*ka2*vS**4+la2*vv**4+
     &  2*MHp2*vv2)+abskappa*vS**3*(4*abskappa*Cosk3s*SB**4*vS+sqrt2*
     &  DBLE(Aka)))*Zfactor1LIN(3,3)*facZ)/(2.*Cosk3s*vS2)



       dMH1MT4BIN(3,4,1)=(abskappa*abslambda*dvS1B*SB*Sinphiy*vv)/2.+
     &  (abslambda*dabskappa14B*SB*Sinphiy*vS*vv)/2.+(abskappa*
     &  dabslambda14B*SB*Sinphiy*vS*vv)/2.+dtad1MT4B(4,1)/vS+(abskappa*
     &  abslambda*SB*Sinphiy*vS*dv4B(1))/2.+(abskappa*abslambda*SB*
     &  Sinphiy*vS*vv*(Zfactor1LIN(1,1)+Zfactor1LIN(3,3))*facZ)/4.
       dMH1MT4BIN(3,5,1)=(abskappa*abslambda*CB*dvS1B*Sinphiy*vv)/2.+
     &  (abslambda*CB*dabskappa14B*Sinphiy*vS*vv)/2.+(abskappa*CB*
     &  dabslambda14B*Sinphiy*vS*vv)/2.+(CB*dtad1MT4B(4,1))/(SB*vS)+
     &  (abskappa*abslambda*CB*Sinphiy*vS*dv4B(1))/2.+(abskappa*
     &  abslambda*CB*Sinphiy*vS*vv*(Zfactor1LIN(2,2)+Zfactor1LIN(3,3))*facZ)
     &  /4.
       dMH1MT4BIN(3,6,1)=2*abslambda*CB*dabskappa14B*SB*Sinphiy*vv2+2*
     &  abskappa*CB*dabslambda14B*SB*Sinphiy*vv2-(2*CB*vv*
     &  dtad1MT4B(4,1))/vS2+(2*dtad1MT4B(6,1))/vS+4*abskappa*abslambda*
     &  CB*SB*Sinphiy*vv*dv4B(1)+2*abskappa*abslambda*CB*SB*Sinphiy*
     &  vv2*Zfactor1LIN(3,3)*facZ
       dMH1MT4BIN(4,4,1)=abslambda*dabslambda14B*SB2*vv2+SB2*
     &  dMHp1MT4B(1)+((CB**3+2*CB*SB2)*dtad1MT4B(1,1))/vv-(CB2*SB*
     &  dtad1MT4B(2,1))/vv+la2*SB2*vv*dv4B(1)+(SB2*(2*MHp2+la2*vv2)*
     &  Zfactor1LIN(1,1)*facZ)/2.
       dMH1MT4BIN(4,5,1)=abslambda*CB*dabslambda14B*SB*vv2+CB*SB*
     &  dMHp1MT4B(1)-(SB**3*dtad1MT4B(1,1))/vv-(CB**3*dtad1MT4B(2,1))
     &  /vv+CB*la2*SB*vv*dv4B(1)+(CB*SB*(2*MHp2+la2*vv2)*
     &  (Zfactor1LIN(1,1)+Zfactor1LIN(2,2))*facZ)/4.
       dMH1MT4BIN(4,6,1)=(-3*abslambda*Cosphiy*dabskappa14B*SB*vS*vv)
     &  /2.+dabslambda14B*((-3*abskappa*Cosphiy*SB*vS*vv)/2.+
     &  (abslambda*CB*SB2*vv**3)/vS)+dvS1B*(-3*abskappa*abslambda*
     &  Cosphiy*SB*vv-(SB*(-3*abskappa*abslambda*Cosphiy*vs2*vv+2*CB**
     &  3*la2*SB**3*vv**3+CB*la2*SB**5*vv**3+CB*SB*vv*(2*MHp2+CB**4*
     &  la2*vv2)))/(2.*vS2))+(CB*SB2*vv*dMHp1MT4B(1))/vS-(SB**4*
     &  dtad1MT4B(1,1))/vS-(CB**3*SB*dtad1MT4B(2,1))/vS+(SB*(-3*
     &  abskappa*abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*
     &  SB**5*vv2+SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(1))/(2.*vS)+(SB*
     &  vv*(-3*abskappa*abslambda*Cosphiy*vs2+CB*SB*(2*MHp2+la2*vv2))*
     &  (Zfactor1LIN(1,1)+Zfactor1LIN(3,3))*facZ)/(4.*vS)
       dMH1MT4BIN(5,5,1)=abslambda*CB2*dabslambda14B*vv2+CB2*
     &  dMHp1MT4B(1)-(CB*SB2*dtad1MT4B(1,1))/vv+((2*CB2*SB+SB**3)*
     &  dtad1MT4B(2,1))/vv+CB2*la2*vv*dv4B(1)+(CB2*(2*MHp2+la2*vv2)*
     &  Zfactor1LIN(2,2)*facZ)/2.
       dMH1MT4BIN(5,6,1)=(-3*abslambda*CB*Cosphiy*dabskappa14B*vS*vv)
     &  /2.+dabslambda14B*((-3*abskappa*CB*Cosphiy*vS*vv)/2.+
     &  (abslambda*CB2*SB*vv**3)/vS)+dvS1B*(-3*abskappa*abslambda*CB*
     &  Cosphiy*vv-(CB*(-3*abskappa*abslambda*Cosphiy*vs2*vv+2*CB**3*
     &  la2*SB**3*vv**3+CB*la2*SB**5*vv**3+CB*SB*vv*(2*MHp2+CB**4*la2*
     &  vv2)))/(2.*vS2))+(CB2*SB*vv*dMHp1MT4B(1))/vS-(CB*SB**3*
     &  dtad1MT4B(1,1))/vS-(CB**4*dtad1MT4B(2,1))/vS+(CB*(-3*abskappa*
     &  abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*SB**5*vv2+
     &  SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(1))/(2.*vS)+(CB*vv*(-3*
     &  abskappa*abslambda*Cosphiy*vs2+CB*SB*(2*MHp2+la2*vv2))*
     &  (Zfactor1LIN(2,2)+Zfactor1LIN(3,3))*facZ)/(4.*vS)


       dMH1MT4BIN(6,6,1)=(-3*abskappa*dReAkappa14B*vS)/(Cosk3s*sqrt2)+
     &  (CB*dabslambda14B*SB*vv2*(3*abskappa*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vs2+2*abslambda*CB*Cosk3s*SB*vv2))/(2.*Cosk3s*vS2)+(3*
     &  dabskappa14B*(abslambda*CB*SB*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)
     &  *vv2-sqrt2*vS*DBLE(Aka)))/(2.*Cosk3s)-(dvS1B*(2*CB**6*Cosk3s*
     &  la2*SB2*vv**4+3*abskappa*SB**4*sqrt2*vS**3*DBLE(Aka)+CB**4*(4*
     &  Cosk3s*la2*SB**4*vv**4+3*abskappa*sqrt2*vS**3*DBLE(Aka))+2*CB2*
     &  SB2*(Cosk3s*la2*SB**4*vv**4+2*Cosk3s*MHp2*vv2+3*abskappa*sqrt2*
     &  vS**3*DBLE(Aka))))/(2.*Cosk3s*vS**3)+(CB2*SB2*vv2*dMHp1MT4B(1))
     &  /vS2-(CB*SB**4*vv*dtad1MT4B(1,1))/vS2-(CB**4*SB*vv*
     &  dtad1MT4B(2,1))/vS2+dtad1MT4B(3,1)/vS+(3*CB*Sink3s*vv*
     &  dtad1MT4B(4,1))/(Cosk3s*vS2)-(3*Sink3s*dtad1MT4B(6,1))/(Cosk3s*
     &  vS)+(CB*SB*vv*(2*CB*MHp2*SB+(3*abskappa*abslambda*(Cosk3s*
     &  Cosphiy-3*Sink3s*Sinphiy)*vs2)/Cosk3s+2*CB**5*la2*SB*vv2+4*CB**
     &  3*la2*SB**3*vv2+2*CB*la2*SB**5*vv2)*dv4B(1))/vS2+((3*abskappa*
     &  abslambda*CB*SB*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2*vv2+CB2*
     &  Cosk3s*SB2*vv2*(2*MHp2+la2*vv2)-3*abskappa*sqrt2*vS**3*
     &  DBLE(Aka))*Zfactor1LIN(3,3)*facZ)/(2.*Cosk3s*vS2)




       dMH1MT4BIN(1,1,2)=SB2*dMHp1MT4B(2)+((CB**3+2*CB*SB2)*
     &  dtad1MT4B(1,2))/vv-(CB2*SB*dtad1MT4B(2,2))/vv+la2*SB2*vv*
     &  dv4B(2)
       dMH1MT4BIN(1,2,2)=-(CB*SB*dMHp1MT4B(2))+(SB**3*dtad1MT4B(1,2))
     &  /vv+(CB**3*dtad1MT4B(2,2))/vv+CB*la2*SB*vv*dv4B(2)
       dMH1MT4BIN(1,3,2)=-((CB*SB2*vv*dMHp1MT4B(2))/vS)+(SB**4*
     &  dtad1MT4B(1,2))/vS+(CB**3*SB*dtad1MT4B(2,2))/vS+((2*CB**5*la2*
     &  vs2-abskappa*abslambda*Cosphiy*SB*vs2+4*CB**3*la2*SB2*vs2-6*CB*
     &  *3*la2*SB**4*vv2-3*CB*la2*SB**6*vv2+SB2*(-2*CB*MHp2+2*CB*la2*
     &  SB2*vs2-3*CB**5*la2*vv2))*dv4B(2))/(2.*vS)
       dMH1MT4BIN(1,4,2)=0
       dMH1MT4BIN(1,5,2)=dtad1MT4B(4,2)/(SB*vv)
       dMH1MT4BIN(1,6,2)=dtad1MT4B(4,2)/vS-(3*abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4B(2))/2.
       dMH1MT4BIN(2,2,2)=CB2*dMHp1MT4B(2)-(CB*SB2*dtad1MT4B(1,2))/vv+
     &  ((2*CB2*SB+SB**3)*dtad1MT4B(2,2))/vv+CB2*la2*vv*dv4B(2)
       dMH1MT4BIN(2,3,2)=-((CB2*SB*vv*dMHp1MT4B(2))/vS)+(CB*SB**3*
     &  dtad1MT4B(1,2))/vS+(CB**4*dtad1MT4B(2,2))/vS-((abskappa*
     &  abslambda*CB*Cosphiy*vs2-4*CB2*la2*SB**3*vs2-2*la2*SB**5*vs2+3*
     &  CB**6*la2*SB*vv2+6*CB**4*la2*SB**3*vv2+CB2*SB*(2*MHp2+la2*(-2*
     &  CB2*vs2+3*SB**4*vv2)))*dv4B(2))/(2.*vS)
       dMH1MT4BIN(2,4,2)=dtad1MT4B(4,2)/(SB*vv)
       dMH1MT4BIN(2,5,2)=0
       dMH1MT4BIN(2,6,2)=(CB*dtad1MT4B(4,2))/(SB*vS)-(3*abskappa*
     &  abslambda*CB*Sinphiy*vS*dv4B(2))/2.
       dMH1MT4BIN(3,3,2)=(CB2*SB2*vv2*dMHp1MT4B(2))/vS2-(CB*SB**4*vv*
     &  dtad1MT4B(1,2))/vS2-(CB**4*SB*vv*dtad1MT4B(2,2))/vS2+
     &  dtad1MT4B(3,2)/vS-(CB*Sink3s*vv*dtad1MT4B(4,2))/(Cosk3s*vS2)+
     &  (Sink3s*dtad1MT4B(6,2))/(Cosk3s*vS)+(CB*SB*vv*(2*CB*MHp2*SB-
     &  (abskappa*abslambda*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2)
     &  /Cosk3s+2*CB**5*la2*SB*vv2+4*CB**3*la2*SB**3*vv2+2*CB*la2*SB**
     &  5*vv2)*dv4B(2))/vS2
       dMH1MT4BIN(3,4,2)=dtad1MT4B(4,2)/vS+(abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4B(2))/2.
       dMH1MT4BIN(3,5,2)=(CB*dtad1MT4B(4,2))/(SB*vS)+(abskappa*
     &  abslambda*CB*Sinphiy*vS*dv4B(2))/2.
       dMH1MT4BIN(3,6,2)=(-2*CB*vv*dtad1MT4B(4,2))/vS2+(2*
     &  dtad1MT4B(6,2))/vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*
     &  dv4B(2)
       dMH1MT4BIN(4,4,2)=SB2*dMHp1MT4B(2)+((CB**3+2*CB*SB2)*
     &  dtad1MT4B(1,2))/vv-(CB2*SB*dtad1MT4B(2,2))/vv+la2*SB2*vv*
     &  dv4B(2)
       dMH1MT4BIN(4,5,2)=CB*SB*dMHp1MT4B(2)-(SB**3*dtad1MT4B(1,2))/vv-
     &  (CB**3*dtad1MT4B(2,2))/vv+CB*la2*SB*vv*dv4B(2)
       dMH1MT4BIN(4,6,2)=(CB*SB2*vv*dMHp1MT4B(2))/vS-(SB**4*
     &  dtad1MT4B(1,2))/vS-(CB**3*SB*dtad1MT4B(2,2))/vS+(SB*(-3*
     &  abskappa*abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*
     &  SB**5*vv2+SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(2))/(2.*vS)
       dMH1MT4BIN(5,5,2)=CB2*dMHp1MT4B(2)-(CB*SB2*dtad1MT4B(1,2))/vv+
     &  ((2*CB2*SB+SB**3)*dtad1MT4B(2,2))/vv+CB2*la2*vv*dv4B(2)
       dMH1MT4BIN(5,6,2)=(CB2*SB*vv*dMHp1MT4B(2))/vS-(CB*SB**3*
     &  dtad1MT4B(1,2))/vS-(CB**4*dtad1MT4B(2,2))/vS+(CB*(-3*abskappa*
     &  abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*SB**5*vv2+
     &  SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(2))/(2.*vS)
       dMH1MT4BIN(6,6,2)=(CB2*SB2*vv2*dMHp1MT4B(2))/vS2-(CB*SB**4*vv*
     &  dtad1MT4B(1,2))/vS2-(CB**4*SB*vv*dtad1MT4B(2,2))/vS2+
     &  dtad1MT4B(3,2)/vS+(3*CB*Sink3s*vv*dtad1MT4B(4,2))/(Cosk3s*vS2)-
     &  (3*Sink3s*dtad1MT4B(6,2))/(Cosk3s*vS)+(CB*SB*vv*(2*CB*MHp2*SB+
     &  (3*abskappa*abslambda*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2)
     &  /Cosk3s+2*CB**5*la2*SB*vv2+4*CB**3*la2*SB**3*vv2+2*CB*la2*SB**
     &  5*vv2)*dv4B(2))/vS2

       dMH1MT4BIN(1,1,3)=SB2*dMHp1MT4B(3)+((CB**3+2*CB*SB2)*
     &  dtad1MT4B(1,3))/vv-(CB2*SB*dtad1MT4B(2,3))/vv+la2*SB2*vv*
     &  dv4B(3)
       dMH1MT4BIN(1,2,3)=-(CB*SB*dMHp1MT4B(3))+(SB**3*dtad1MT4B(1,3))
     &  /vv+(CB**3*dtad1MT4B(2,3))/vv+CB*la2*SB*vv*dv4B(3)
       dMH1MT4BIN(1,3,3)=-((CB*SB2*vv*dMHp1MT4B(3))/vS)+(SB**4*
     &  dtad1MT4B(1,3))/vS+(CB**3*SB*dtad1MT4B(2,3))/vS+((2*CB**5*la2*
     &  vs2-abskappa*abslambda*Cosphiy*SB*vs2+4*CB**3*la2*SB2*vs2-6*CB*
     &  *3*la2*SB**4*vv2-3*CB*la2*SB**6*vv2+SB2*(-2*CB*MHp2+2*CB*la2*
     &  SB2*vs2-3*CB**5*la2*vv2))*dv4B(3))/(2.*vS)
       dMH1MT4BIN(1,4,3)=0
       dMH1MT4BIN(1,5,3)=dtad1MT4B(4,3)/(SB*vv)
       dMH1MT4BIN(1,6,3)=dtad1MT4B(4,3)/vS-(3*abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4B(3))/2.
       dMH1MT4BIN(2,2,3)=CB2*dMHp1MT4B(3)-(CB*SB2*dtad1MT4B(1,3))/vv+
     &  ((2*CB2*SB+SB**3)*dtad1MT4B(2,3))/vv+CB2*la2*vv*dv4B(3)
       dMH1MT4BIN(2,3,3)=-((CB2*SB*vv*dMHp1MT4B(3))/vS)+(CB*SB**3*
     &  dtad1MT4B(1,3))/vS+(CB**4*dtad1MT4B(2,3))/vS-((abskappa*
     &  abslambda*CB*Cosphiy*vs2-4*CB2*la2*SB**3*vs2-2*la2*SB**5*vs2+3*
     &  CB**6*la2*SB*vv2+6*CB**4*la2*SB**3*vv2+CB2*SB*(2*MHp2+la2*(-2*
     &  CB2*vs2+3*SB**4*vv2)))*dv4B(3))/(2.*vS)
       dMH1MT4BIN(2,4,3)=dtad1MT4B(4,3)/(SB*vv)
       dMH1MT4BIN(2,5,3)=0
       dMH1MT4BIN(2,6,3)=(CB*dtad1MT4B(4,3))/(SB*vS)-(3*abskappa*
     &  abslambda*CB*Sinphiy*vS*dv4B(3))/2.
       dMH1MT4BIN(3,3,3)=(CB2*SB2*vv2*dMHp1MT4B(3))/vS2-(CB*SB**4*vv*
     &  dtad1MT4B(1,3))/vS2-(CB**4*SB*vv*dtad1MT4B(2,3))/vS2+
     &  dtad1MT4B(3,3)/vS-(CB*Sink3s*vv*dtad1MT4B(4,3))/(Cosk3s*vS2)+
     &  (Sink3s*dtad1MT4B(6,3))/(Cosk3s*vS)+(CB*SB*vv*(2*CB*MHp2*SB-
     &  (abskappa*abslambda*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2)
     &  /Cosk3s+2*CB**5*la2*SB*vv2+4*CB**3*la2*SB**3*vv2+2*CB*la2*SB**
     &  5*vv2)*dv4B(3))/vS2
       dMH1MT4BIN(3,4,3)=dtad1MT4B(4,3)/vS+(abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4B(3))/2.
       dMH1MT4BIN(3,5,3)=(CB*dtad1MT4B(4,3))/(SB*vS)+(abskappa*
     &  abslambda*CB*Sinphiy*vS*dv4B(3))/2.
       dMH1MT4BIN(3,6,3)=(-2*CB*vv*dtad1MT4B(4,3))/vS2+(2*
     &  dtad1MT4B(6,3))/vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*
     &  dv4B(3)
       dMH1MT4BIN(4,4,3)=SB2*dMHp1MT4B(3)+((CB**3+2*CB*SB2)*
     &  dtad1MT4B(1,3))/vv-(CB2*SB*dtad1MT4B(2,3))/vv+la2*SB2*vv*
     &  dv4B(3)
       dMH1MT4BIN(4,5,3)=CB*SB*dMHp1MT4B(3)-(SB**3*dtad1MT4B(1,3))/vv-
     &  (CB**3*dtad1MT4B(2,3))/vv+CB*la2*SB*vv*dv4B(3)
       dMH1MT4BIN(4,6,3)=(CB*SB2*vv*dMHp1MT4B(3))/vS-(SB**4*
     &  dtad1MT4B(1,3))/vS-(CB**3*SB*dtad1MT4B(2,3))/vS+(SB*(-3*
     &  abskappa*abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*
     &  SB**5*vv2+SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(3))/(2.*vS)
       dMH1MT4BIN(5,5,3)=CB2*dMHp1MT4B(3)-(CB*SB2*dtad1MT4B(1,3))/vv+
     &  ((2*CB2*SB+SB**3)*dtad1MT4B(2,3))/vv+CB2*la2*vv*dv4B(3)
       dMH1MT4BIN(5,6,3)=(CB2*SB*vv*dMHp1MT4B(3))/vS-(CB*SB**3*
     &  dtad1MT4B(1,3))/vS-(CB**4*dtad1MT4B(2,3))/vS+(CB*(-3*abskappa*
     &  abslambda*Cosphiy*vs2+6*CB**3*la2*SB**3*vv2+3*CB*la2*SB**5*vv2+
     &  SB*(2*CB*MHp2+3*CB**5*la2*vv2))*dv4B(3))/(2.*vS)
       dMH1MT4BIN(6,6,3)=(CB2*SB2*vv2*dMHp1MT4B(3))/vS2-(CB*SB**4*vv*
     &  dtad1MT4B(1,3))/vS2-(CB**4*SB*vv*dtad1MT4B(2,3))/vS2+
     &  dtad1MT4B(3,3)/vS+(3*CB*Sink3s*vv*dtad1MT4B(4,3))/(Cosk3s*vS2)-
     &  (3*Sink3s*dtad1MT4B(6,3))/(Cosk3s*vS)+(CB*SB*vv*(2*CB*MHp2*SB+
     &  (3*abskappa*abslambda*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2)
     &  /Cosk3s+2*CB**5*la2*SB*vv2+4*CB**3*la2*SB**3*vv2+2*CB*la2*SB**
     &  5*vv2)*dv4B(3))/vS2

       do i=1,6
        do j=i,6
         dMH1MT4BIN(j,i,1:3)=dMH1MT4BIN(i,j,1:3)
        enddo
       enddo

       do i=1,6
        do j=1,6
         dMH1MT4B(i,j,1:3)=0D0
         do I2G3=1,6
         do I2G4=1,6         
       dMH1MT4B(i,j,1)=dMH1MT4B(i,j,1)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4BIN(I2G3,I2G4,1)
       dMH1MT4B(i,j,2)=dMH1MT4B(i,j,2)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4BIN(I2G3,I2G4,2)
       dMH1MT4B(i,j,3)=dMH1MT4B(i,j,3)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4BIN(I2G3,I2G4,3)
         enddo
         enddo
        enddo
       enddo


c 
       dMH1MT4IN(1,1,1)=abslambda*dabslambda14*SB2*vv2+CB**3*dTanB4*SB*
     &  (2*MHp2*SB2+CB**6*la2*vv2+la2*SB**6*vv2+3*CB**4*la2*SB2*vv2+CB*
     &  *2*(2*MHp2+3*la2*SB**4*vv2))+SB2*dMHp1MT4(1)+((CB**3+2*CB*SB2)*
     &  dtad1MT4(1,1))/vv-(CB2*SB*dtad1MT4(2,1))/vv+la2*SB2*vv*dv4(1)
       dMH1MT4IN(1,2,1)=(-4*CB**6*dTanB4*MHp2*vv+2*CB**10*dTanB4*la2*
     &  vv*vv2+4*CB**8*dTanB4*la2*SB2*vv*vv2-4*CB**4*dTanB4*la2*SB**4*
     &  SB2*vv*vv2+2*SB**3*(CB2*dTanB4*SB*vv*(2*MHp2-la2*SB**4*vv2)+2*
     &  dtad1MT4(1,1))+4*CB**3*dtad1MT4(2,1)+CB*SB*vv*(-2*dZHu4*facZ*MHp2+4*
     &  abslambda*dabslambda14*vv2+dZHu4*facZ*la2*vv2-4*dMHp1MT4(1)+4*la2*
     &  vv*dv4(1)))/(4.*vv)
       dMH1MT4IN(1,3,1)=(-(abskappa*abslambda*CB**9*Cosphiy*dTanB4*vs2*
     &  vv)-3*abskappa*abslambda*CB**7*Cosphiy*dTanB4*SB2*vs2*vv-2*CB**
     &  10*dTanB4*la2*SB*vv*vv2-6*CB**8*dTanB4*la2*SB**3*vv*vv2+CB**6*
     &  dTanB4*SB*vv*(-4*MHp2-6*la2*SB**4*vv2+abslambda**2*CB2*(-2*vs2+
     &  SB2*vv2))+CB**4*dTanB4*SB*vv*(-2*la2*SB**6*vv2+3*abslambda**2*
     &  CB2*SB2*(-2*vs2+SB2*vv2))+CB**5*la2*(2*vs2-3*SB2*vv2)*dv4(1)-
     &  CB**3*(abskappa*abslambda*Cosphiy*dTanB4*SB**4*(3*CB2+SB**2)*
     &  vs2*vv-2*SB*dtad1MT4(2,1)-4*la2*SB2*vs2*dv4(1)+6*la2*SB**4*vv2*
     &  dv4(1))+CB*(abslambda*dabslambda14*(4*vS**2*vv-2*SB2*vv**3)+2*
     &  la2*SB2**2*vs2*dv4(1)-3*la2*SB**6*vv2*dv4(1)-2*SB2*(vv*
     &  dMHp1MT4(1)+MHp2*dv4(1)))+SB*(CB2*dTanB4*SB**4*vv*(2*MHp2+la2*
     &  SB**2*(-2*vs2+SB2*vv2))+CB2**2*dTanB4*vv*(-2*MHp2*SB2+3*la2*SB*
     &  *4*(-2*vs2+SB2*vv2))+2*SB**3*dtad1MT4(1,1)-abskappa*Cosphiy*
     &  (dabslambda14*vS**2*vv+abslambda*vs2*dv4(1))))/(2.*vS)
       dMH1MT4IN(1,4,1)=0
       dMH1MT4IN(1,5,1)=dtad1MT4(4,1)/(SB*vv)
       dMH1MT4IN(1,6,1)=(2*dtad1MT4(4,1)-3*abskappa*Sinphiy*vS**2*
     &  (abslambda*CB**3*dTanB4*vv+dabslambda14*SB*vv+abslambda*SB*
     &  dv4(1)))/(2.*vS)
       dMH1MT4IN(2,2,1)=abslambda*CB2*dabslambda14*vv2+(CB2*dZHu4*facZ*(2*
     &  MHp2+la2*vv2))/2.-CB*dTanB4*SB*(CB**8*la2*vv2+3*CB**6*la2*SB**
     &  2*vv2+CB2*SB**2*(2*MHp2+la2*SB**4*vv2)+CB**4*(2*MHp2+3*la2*SB**
     &  4*vv2))+CB2*dMHp1MT4(1)-(CB*SB2*dtad1MT4(1,1))/vv+((2*CB2*SB+
     &  SB**3)*dtad1MT4(2,1))/vv+CB2*la2*vv*dv4(1)
       dMH1MT4IN(2,3,1)=(-2*CB**11*dTanB4*la2*vv*vv2+12*CB**5*dTanB4*
     &  la2*SB**6*vv*vv2+2*CB**9*dTanB4*vv*(2*la2*SB2*vv2+abslambda**2*
     &  (2*vs2-3*SB2*vv2))+2*CB**3*dTanB4*vv*(2*MHp2*(2*SB**4+CB2*SB2)+
     &  SB**4*(6*CB2*la2*vs2+2*abslambda**2*SB**2*vs2+2*la2*SB**4*vv2-
     &  CB2*la2*SB2*vv2))-2*CB**7*dTanB4*vv*(2*MHp2+3*la2*(-2*SB2*vs2-
     &  2*SB**4*vv2+SB2**2*vv2))+2*CB**6*SB*(abskappa*abslambda*CB2*
     &  Cosphiy*dTanB4*vs2*vv-3*la2*vv2*dv4(1))+2*CB**4*(3*abskappa*
     &  abslambda*CB2*Cosphiy*dTanB4*SB**3*vs2*vv+2*dtad1MT4(2,1)-6*
     &  la2*SB**3*vv2*dv4(1))-CB*(-4*SB**3*dtad1MT4(1,1)+abskappa*
     &  Cosphiy*(2*dabslambda14*vS**2*vv+abslambda*dZHu4*facZ*vs2*vv+2*
     &  abslambda*vs2*dv4(1)))+SB*(8*abslambda*dabslambda14*vS**2*vv+2*
     &  dZHu4*facZ*la2*vs2*vv+4*la2*SB**4*vs2*dv4(1)+CB2**2*(6*abskappa*
     &  abslambda*Cosphiy*dTanB4*SB**4*vs2*vv+4*la2*vs2*dv4(1))-CB2*(2*
     &  dZHu4*facZ*MHp2*vv-2*abskappa*abslambda*Cosphiy*dTanB4*SB**6*vs2*vv+
     &  4*abslambda*dabslambda14*vv**3+dZHu4*facZ*la2*vv*vv2+4*vv*
     &  dMHp1MT4(1)+4*MHp2*dv4(1)-8*la2*SB**2*vs2*dv4(1)+6*la2*SB**4*
     &  vv2*dv4(1))))/(4.*vS)
       dMH1MT4IN(2,4,1)=dtad1MT4(4,1)/(SB*vv)
       dMH1MT4IN(2,5,1)=0
       dMH1MT4IN(2,6,1)=(4*CB*dtad1MT4(4,1)-3*abskappa*SB*Sinphiy*vS**
     &  2*(-2*abslambda*CB2*dTanB4*SB*vv+CB*(2*dabslambda14*vv+
     &  abslambda*dZHu4*facZ*vv+2*abslambda*dv4(1))))/(4.*SB*vS)
       dMH1MT4IN(3,3,1)=(abskappa*abslambda*CB2*dTanB4*SB**6*(3*CB2+SB*
     &  *2)*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vS*vs2*vv2-3*abskappa*
     &  abslambda*CB**8*dTanB4*SB2*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*
     &  vS*vs2*vv2+abskappa*abslambda*CB**10*dTanB4*(-(Cosk3s*Cosphiy)+
     &  3*Sink3s*Sinphiy)*vS*vs2*vv2+2*CB**11*Cosk3s*dTanB4*la2*SB*vS*
     &  vv2**2+6*abslambda**2*CB**9*Cosk3s*dTanB4*SB*SB2*vS*vv2**2-6*
     &  abslambda**2*CB**5*CB2*Cosk3s*dTanB4*SB**3*SB2*vS*vv2**2+2*
     &  abslambda**2*CB**3*CB2*Cosk3s*dTanB4*SB**5*(-3*SB**2+SB2)*vS*
     &  vv2**2+2*CB**7*Cosk3s*dTanB4*SB*vS*vv2*(2*MHp2-abslambda**2*
     &  CB2*SB**2*vv2+3*la2*SB**4*vv2)+2*(CB2*Cosk3s*SB2*vS*vv2*
     &  dMHp1MT4(1)+Cosk3s*vS2*dtad1MT4(3,1)+Sink3s*vS2*dtad1MT4(6,1))+
     &  CB**6*SB**2*vS*vv2*(abskappa*abslambda*dTanB4*(CB2-3*SB**2)*
     &  (Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2+4*Cosk3s*la2*vv*dv4(1))+
     &  2*CB**2*Cosk3s*SB**2*vS*(abslambda*dabslambda14*vv2**2+2*vv*
     &  (MHp2+la2*SB**4*vv2)*dv4(1))-CB*vS*(2*CB2*Cosk3s*dTanB4*SB**5*
     &  vv2*(2*MHp2+la2*SB**4*vv2)+2*vv*(Cosk3s*SB**4*dtad1MT4(1,1)+
     &  Sink3s*dtad1MT4(4,1))+abskappa*SB*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vs2*(dabslambda14*vv2+2*abslambda*vv*dv4(1)))+CB**4*
     &  SB*vS*(-(abskappa*abslambda*dTanB4*SB**3*(-3*CB2+SB**2)*
     &  (Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2*vv2)-2*Cosk3s*vv*
     &  (dtad1MT4(2,1)-4*la2*SB**3*vv2*dv4(1))))/(2.*Cosk3s*vS*vS2)
       dMH1MT4IN(3,4,1)=(2*dtad1MT4(4,1)+abskappa*Sinphiy*vS**2*
     &  (abslambda*CB**3*dTanB4*vv+dabslambda14*SB*vv+abslambda*SB*
     &  dv4(1)))/(2.*vS)
       dMH1MT4IN(3,5,1)=(4*CB*dtad1MT4(4,1)+abskappa*SB*Sinphiy*vS**2*
     &  (-2*abslambda*CB2*dTanB4*SB*vv+CB*(2*dabslambda14*vv+abslambda*
     &  dZHu4*facZ*vv+2*abslambda*dv4(1))))/(4.*SB*vS)
       dMH1MT4IN(3,6,1)=2*(-((CB*vv*dtad1MT4(4,1))/vS2)+dtad1MT4(6,1)
     &  /vS+abskappa*Sinphiy*(CB*dabslambda14*SB*vv2+abslambda*(CB**4*
     &  dTanB4*vv2-CB2*dTanB4*SB2*vv2+2*CB*SB*vv*dv4(1))))
       dMH1MT4IN(4,4,1)=abslambda*dabslambda14*SB2*vv2+CB**3*dTanB4*SB*
     &  (2*MHp2*SB2+CB**6*la2*vv2+la2*SB**6*vv2+3*CB**4*la2*SB2*vv2+CB*
     &  *2*(2*MHp2+3*la2*SB**4*vv2))+SB2*dMHp1MT4(1)+((CB**3+2*CB*SB2)*
     &  dtad1MT4(1,1))/vv-(CB2*SB*dtad1MT4(2,1))/vv+la2*SB2*vv*dv4(1)
       dMH1MT4IN(4,5,1)=(4*CB**6*dTanB4*MHp2*vv+2*CB**10*dTanB4*la2*vv*
     &  vv2+4*CB**8*dTanB4*la2*SB2*vv*vv2+4*CB**4*dTanB4*vv*(MHp2*(SB**
     &  2-SB2)-la2*SB**6*vv2)-2*SB**3*(CB2*dTanB4*SB*vv*(2*MHp2+la2*SB*
     &  *4*vv2)+2*dtad1MT4(1,1))-4*CB**3*dtad1MT4(2,1)+CB*SB*vv*(dZHu4*facZ*
     &  (2*MHp2+la2*vv2)+4*(abslambda*dabslambda14*vv2+dMHp1MT4(1)+la2*
     &  vv*dv4(1))))/(4.*vv)
       dMH1MT4IN(4,6,1)=(-3*abskappa*abslambda*CB**9*Cosphiy*dTanB4*
     &  vs2*vv-9*abskappa*abslambda*CB**7*Cosphiy*dTanB4*SB2*vs2*vv+2*
     &  CB**10*dTanB4*la2*SB*vv*vv2+6*CB**8*dTanB4*la2*SB**3*vv*vv2+CB*
     &  *4*dTanB4*SB**3*(2*la2*SB**4-3*abslambda**2*CB2*SB2)*vv*vv2+CB*
     &  *6*dTanB4*SB*vv*(4*MHp2-abslambda**2*CB2*SB**2*vv2+6*la2*SB**4*
     &  vv2)+3*CB**5*la2*SB**2*vv2*dv4(1)-CB**3*(3*abskappa*abslambda*
     &  Cosphiy*dTanB4*SB**4*(3*CB2+SB**2)*vs2*vv+2*SB*dtad1MT4(2,1)-6*
     &  la2*SB**4*vv2*dv4(1))+CB*(2*abslambda*dabslambda14*SB2*vv**3+2*
     &  SB2*vv*dMHp1MT4(1)+SB**2*(2*MHp2+3*la2*SB**4*vv2)*dv4(1))+SB*
     &  (CB2**2*dTanB4*SB**2*vv*(2*MHp2-3*la2*SB**4*vv2)-CB2*dTanB4*SB*
     &  *4*vv*(2*MHp2+la2*SB**4*vv2)-2*SB**3*dtad1MT4(1,1)-3*abskappa*
     &  Cosphiy*(dabslambda14*vS**2*vv+abslambda*vs2*dv4(1))))/(2.*vS)
       dMH1MT4IN(5,5,1)=abslambda*CB2*dabslambda14*vv2+(CB2*dZHu4*facZ*(2*
     &  MHp2+la2*vv2))/2.-CB*dTanB4*SB*(CB**8*la2*vv2+3*CB**6*la2*SB**
     &  2*vv2+CB2*SB**2*(2*MHp2+la2*SB**4*vv2)+CB**4*(2*MHp2+3*la2*SB**
     &  4*vv2))+CB2*dMHp1MT4(1)-(CB*SB2*dtad1MT4(1,1))/vv+((2*CB2*SB+
     &  SB**3)*dtad1MT4(2,1))/vv+CB2*la2*vv*dv4(1)
       dMH1MT4IN(5,6,1)=(2*CB**11*dTanB4*la2*vv*vv2-12*CB**5*dTanB4*
     &  la2*SB**6*vv*vv2+2*CB**9*dTanB4*(3*abslambda**2-2*la2)*SB2*vv*
     &  vv2+2*CB**7*dTanB4*vv*(2*MHp2-3*la2*SB**4*vv2)-2*CB**3*dTanB4*
     &  vv*(4*MHp2*SB**4+2*CB2*MHp2*SB2-CB2*la2*SB**6*vv2+2*la2*SB**8*
     &  vv2)+2*CB2*SB*vv*(3*abskappa*abslambda*Cosphiy*dTanB4*SB**4*(3*
     &  CB2+SB**2)*vs2+2*(abslambda*dabslambda14*vv**2+dMHp1MT4(1)))+6*
     &  CB**6*SB*(abskappa*abslambda*CB2*Cosphiy*dTanB4*vs2*vv+la2*vv2*
     &  dv4(1))+2*CB**4*(9*abskappa*abslambda*CB2*Cosphiy*dTanB4*SB**3*
     &  vs2*vv-2*dtad1MT4(2,1)+6*la2*SB**3*vv2*dv4(1))+CB**2*SB*(2*
     &  dZHu4*facZ*MHp2*vv+dZHu4*facZ*la2*vv*vv2+4*MHp2*dv4(1)+6*la2*SB**4*vv2*
     &  dv4(1))-CB*(4*SB**3*dtad1MT4(1,1)+3*abskappa*Cosphiy*(2*
     &  dabslambda14*vS**2*vv+abslambda*dZHu4*facZ*vs2*vv+2*abslambda*vs2*
     &  dv4(1))))/(4.*vS)
       dMH1MT4IN(6,6,1)=(3*abskappa*abslambda*CB**10*dTanB4*(Cosk3s*
     &  Cosphiy-3*Sink3s*Sinphiy)*vS*vs2*vv2-3*abskappa*abslambda*CB2*
     &  dTanB4*SB**6*(3*CB2+SB**2)*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*
     &  vS*vs2*vv2+9*abskappa*abslambda*CB**8*dTanB4*SB2*(Cosk3s*
     &  Cosphiy-3*Sink3s*Sinphiy)*vS*vs2*vv2+2*CB**11*Cosk3s*dTanB4*
     &  la2*SB*vS*vv2**2+6*abslambda**2*CB**9*Cosk3s*dTanB4*SB*SB2*vS*
     &  vv2**2-6*abslambda**2*CB**5*CB2*Cosk3s*dTanB4*SB**3*SB2*vS*vv2*
     &  *2+2*abslambda**2*CB**3*CB2*Cosk3s*dTanB4*SB**5*(-3*SB**2+SB2)*
     &  vS*vv2**2+2*CB**7*Cosk3s*dTanB4*SB*vS*vv2*(2*MHp2-abslambda**2*
     &  CB2*SB**2*vv2+3*la2*SB**4*vv2)+2*(CB2*Cosk3s*SB2*vS*vv2*
     &  dMHp1MT4(1)+Cosk3s*vS2*dtad1MT4(3,1)-3*Sink3s*vS2*dtad1MT4(6,1)
     &  )+CB**6*SB**2*vS*vv2*(-3*abskappa*abslambda*dTanB4*(CB2-3*SB**
     &  2)*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2+4*Cosk3s*la2*vv*dv4(1)
     &  )+2*CB**2*Cosk3s*SB**2*vS*(abslambda*dabslambda14*vv2**2+2*vv*
     &  (MHp2+la2*SB**4*vv2)*dv4(1))-CB*vS*(2*CB2*Cosk3s*dTanB4*SB**5*
     &  vv2*(2*MHp2+la2*SB**4*vv2)+2*vv*(Cosk3s*SB**4*dtad1MT4(1,1)-3*
     &  Sink3s*dtad1MT4(4,1))-3*abskappa*SB*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vs2*(dabslambda14*vv2+2*abslambda*vv*dv4(1)))+CB**4*
     &  SB*vS*(3*abskappa*abslambda*dTanB4*SB**3*(-3*CB2+SB**2)*
     &  (Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vs2*vv2-2*Cosk3s*vv*
     &  (dtad1MT4(2,1)-4*la2*SB**3*vv2*dv4(1))))/(2.*Cosk3s*vS*vS2)


       dMH1MT4IN(1,1,2)=SB2*dMHp1MT4(2)+((CB**3+2*CB*SB2)*dtad1MT4(1,2)
     &  )/vv-(CB2*SB*dtad1MT4(2,2))/vv+la2*SB2*vv*dv4(2)
       dMH1MT4IN(1,2,2)=(SB**3*dtad1MT4(1,2)+CB**3*dtad1MT4(2,2)+CB*SB*
     &  vv*(-dMHp1MT4(2)+la2*vv*dv4(2)))/vv
       dMH1MT4IN(1,3,2)=(2*SB**4*dtad1MT4(1,2)-abskappa*abslambda*
     &  Cosphiy*SB*vs2*dv4(2)+CB**5*la2*(2*vs2-3*SB2*vv2)*dv4(2)+CB**3*
     &  (2*SB*dtad1MT4(2,2)+4*la2*SB2*vs2*dv4(2)-6*la2*SB**4*vv2*dv4(2)
     &  )+CB*(2*la2*SB2**2*vs2*dv4(2)-3*la2*SB**6*vv2*dv4(2)-2*SB2*(vv*
     &  dMHp1MT4(2)+MHp2*dv4(2))))/(2.*vS)
       dMH1MT4IN(1,4,2)=0
       dMH1MT4IN(1,5,2)=dtad1MT4(4,2)/(SB*vv)
       dMH1MT4IN(1,6,2)=dtad1MT4(4,2)/vS-(3*abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4(2))/2.
       dMH1MT4IN(2,2,2)=CB2*dMHp1MT4(2)-(CB*SB2*dtad1MT4(1,2))/vv+((2*
     &  CB2*SB+SB**3)*dtad1MT4(2,2))/vv+CB2*la2*vv*dv4(2)
       dMH1MT4IN(2,3,2)=(-2*CB2*SB*vv*dMHp1MT4(2)+2*CB*SB**3*
     &  dtad1MT4(1,2)+2*CB**4*dtad1MT4(2,2)-(abskappa*abslambda*CB*
     &  Cosphiy*vs2-4*CB2*la2*SB**3*vs2-2*la2*SB**5*vs2+3*CB**6*la2*SB*
     &  vv2+6*CB**4*la2*SB**3*vv2+CB2*SB*(2*MHp2-2*CB2*la2*vs2+3*la2*
     &  SB**4*vv2))*dv4(2))/(2.*vS)
       dMH1MT4IN(2,4,2)=dtad1MT4(4,2)/(SB*vv)
       dMH1MT4IN(2,5,2)=0
       dMH1MT4IN(2,6,2)=CB*(dtad1MT4(4,2)/(SB*vS)-(3*abskappa*
     &  abslambda*Sinphiy*vS*dv4(2))/2.)
       dMH1MT4IN(3,3,2)=(CB2*Cosk3s*SB2*vS*vv2*dMHp1MT4(2)+vS2*(Cosk3s*
     &  dtad1MT4(3,2)+Sink3s*dtad1MT4(6,2))+2*CB**6*Cosk3s*la2*SB**2*
     &  vS*vv*vv2*dv4(2)+2*CB**2*Cosk3s*SB**2*vS*vv*(MHp2+la2*SB**4*
     &  vv2)*dv4(2)-CB*vS*vv*(Cosk3s*SB**4*dtad1MT4(1,2)+Sink3s*
     &  dtad1MT4(4,2)+abskappa*abslambda*Cosk3s*Cosphiy*SB*vs2*dv4(2)-
     &  3*abskappa*abslambda*SB*Sink3s*Sinphiy*vs2*dv4(2))+CB**4*
     &  Cosk3s*SB*vS*vv*(-dtad1MT4(2,2)+4*la2*SB**3*vv2*dv4(2)))
     &  /(Cosk3s*vS*vS2)
       dMH1MT4IN(3,4,2)=dtad1MT4(4,2)/vS+(abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4(2))/2.
       dMH1MT4IN(3,5,2)=CB*(dtad1MT4(4,2)/(SB*vS)+(abskappa*abslambda*
     &  Sinphiy*vS*dv4(2))/2.)
       dMH1MT4IN(3,6,2)=(-2*CB*vv*dtad1MT4(4,2))/vS2+(2*dtad1MT4(6,2))
     &  /vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*dv4(2)
       dMH1MT4IN(4,4,2)=SB2*dMHp1MT4(2)+((CB**3+2*CB*SB2)*dtad1MT4(1,2)
     &  )/vv-(CB2*SB*dtad1MT4(2,2))/vv+la2*SB2*vv*dv4(2)
       dMH1MT4IN(4,5,2)=(-(SB**3*dtad1MT4(1,2))-CB**3*dtad1MT4(2,2)+CB*
     &  SB*vv*(dMHp1MT4(2)+la2*vv*dv4(2)))/vv
       dMH1MT4IN(4,6,2)=(-2*SB**4*dtad1MT4(1,2)-3*abskappa*abslambda*
     &  Cosphiy*SB*vs2*dv4(2)+3*CB**5*la2*SB**2*vv2*dv4(2)+CB**3*(-2*
     &  SB*dtad1MT4(2,2)+6*la2*SB**4*vv2*dv4(2))+CB*(2*SB2*vv*
     &  dMHp1MT4(2)+SB**2*(2*MHp2+3*la2*SB**4*vv2)*dv4(2)))/(2.*vS)
       dMH1MT4IN(5,5,2)=CB2*dMHp1MT4(2)-(CB*SB2*dtad1MT4(1,2))/vv+((2*
     &  CB2*SB+SB**3)*dtad1MT4(2,2))/vv+CB2*la2*vv*dv4(2)
       dMH1MT4IN(5,6,2)=(2*CB2*SB*vv*dMHp1MT4(2)+CB*(-2*CB**3*
     &  dtad1MT4(2,2)-3*abskappa*abslambda*Cosphiy*vs2*dv4(2)+3*CB*la2*
     &  SB**5*vv2*dv4(2)+CB*SB*(2*MHp2+3*CB**4*la2*vv2)*dv4(2)-2*SB**3*
     &  (dtad1MT4(1,2)-3*CB**3*la2*vv2*dv4(2))))/(2.*vS)
       dMH1MT4IN(6,6,2)=(CB2*Cosk3s*SB2*vS*vv2*dMHp1MT4(2)+vS2*(Cosk3s*
     &  dtad1MT4(3,2)-3*Sink3s*dtad1MT4(6,2))+2*CB**6*Cosk3s*la2*SB**2*
     &  vS*vv*vv2*dv4(2)+2*CB**2*Cosk3s*SB**2*vS*vv*(MHp2+la2*SB**4*
     &  vv2)*dv4(2)+CB*vS*vv*(-(Cosk3s*SB**4*dtad1MT4(1,2))+3*Sink3s*
     &  dtad1MT4(4,2)+3*abskappa*abslambda*Cosk3s*Cosphiy*SB*vs2*dv4(2)
     &  -9*abskappa*abslambda*SB*Sink3s*Sinphiy*vs2*dv4(2))+CB**4*
     &  Cosk3s*SB*vS*vv*(-dtad1MT4(2,2)+4*la2*SB**3*vv2*dv4(2)))
     &  /(Cosk3s*vS*vS2)


       dMH1MT4IN(1,1,3)=SB2*dMHp1MT4(3)+((CB**3+2*CB*SB2)*dtad1MT4(1,3)
     &  )/vv-(CB2*SB*dtad1MT4(2,3))/vv+la2*SB2*vv*dv4(3)
       dMH1MT4IN(1,2,3)=(SB**3*dtad1MT4(1,3)+CB**3*dtad1MT4(2,3)+CB*SB*
     &  vv*(-dMHp1MT4(3)+la2*vv*dv4(3)))/vv
       dMH1MT4IN(1,3,3)=(2*SB**4*dtad1MT4(1,3)-abskappa*abslambda*
     &  Cosphiy*SB*vs2*dv4(3)+CB**5*la2*(2*vs2-3*SB2*vv2)*dv4(3)+CB**3*
     &  (2*SB*dtad1MT4(2,3)+4*la2*SB2*vs2*dv4(3)-6*la2*SB**4*vv2*dv4(3)
     &  )+CB*(2*la2*SB2**2*vs2*dv4(3)-3*la2*SB**6*vv2*dv4(3)-2*SB2*(vv*
     &  dMHp1MT4(3)+MHp2*dv4(3))))/(2.*vS)
       dMH1MT4IN(1,4,3)=0
       dMH1MT4IN(1,5,3)=dtad1MT4(4,3)/(SB*vv)
       dMH1MT4IN(1,6,3)=dtad1MT4(4,3)/vS-(3*abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4(3))/2.
       dMH1MT4IN(2,2,3)=CB2*dMHp1MT4(3)-(CB*SB2*dtad1MT4(1,3))/vv+((2*
     &  CB2*SB+SB**3)*dtad1MT4(2,3))/vv+CB2*la2*vv*dv4(3)
       dMH1MT4IN(2,3,3)=(-2*CB2*SB*vv*dMHp1MT4(3)+2*CB*SB**3*
     &  dtad1MT4(1,3)+2*CB**4*dtad1MT4(2,3)-(abskappa*abslambda*CB*
     &  Cosphiy*vs2-4*CB2*la2*SB**3*vs2-2*la2*SB**5*vs2+3*CB**6*la2*SB*
     &  vv2+6*CB**4*la2*SB**3*vv2+CB2*SB*(2*MHp2-2*CB2*la2*vs2+3*la2*
     &  SB**4*vv2))*dv4(3))/(2.*vS)
       dMH1MT4IN(2,4,3)=dtad1MT4(4,3)/(SB*vv)
       dMH1MT4IN(2,5,3)=0
       dMH1MT4IN(2,6,3)=CB*(dtad1MT4(4,3)/(SB*vS)-(3*abskappa*
     &  abslambda*Sinphiy*vS*dv4(3))/2.)
       dMH1MT4IN(3,3,3)=(CB2*Cosk3s*SB2*vS*vv2*dMHp1MT4(3)+vS2*(Cosk3s*
     &  dtad1MT4(3,3)+Sink3s*dtad1MT4(6,3))+2*CB**6*Cosk3s*la2*SB**2*
     &  vS*vv*vv2*dv4(3)+2*CB**2*Cosk3s*SB**2*vS*vv*(MHp2+la2*SB**4*
     &  vv2)*dv4(3)-CB*vS*vv*(Cosk3s*SB**4*dtad1MT4(1,3)+Sink3s*
     &  dtad1MT4(4,3)+abskappa*abslambda*Cosk3s*Cosphiy*SB*vs2*dv4(3)-
     &  3*abskappa*abslambda*SB*Sink3s*Sinphiy*vs2*dv4(3))+CB**4*
     &  Cosk3s*SB*vS*vv*(-dtad1MT4(2,3)+4*la2*SB**3*vv2*dv4(3)))
     &  /(Cosk3s*vS*vS2)
       dMH1MT4IN(3,4,3)=dtad1MT4(4,3)/vS+(abskappa*abslambda*SB*
     &  Sinphiy*vS*dv4(3))/2.
       dMH1MT4IN(3,5,3)=CB*(dtad1MT4(4,3)/(SB*vS)+(abskappa*abslambda*
     &  Sinphiy*vS*dv4(3))/2.)
       dMH1MT4IN(3,6,3)=(-2*CB*vv*dtad1MT4(4,3))/vS2+(2*dtad1MT4(6,3))
     &  /vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*dv4(3)
       dMH1MT4IN(4,4,3)=SB2*dMHp1MT4(3)+((CB**3+2*CB*SB2)*dtad1MT4(1,3)
     &  )/vv-(CB2*SB*dtad1MT4(2,3))/vv+la2*SB2*vv*dv4(3)
       dMH1MT4IN(4,5,3)=(-(SB**3*dtad1MT4(1,3))-CB**3*dtad1MT4(2,3)+CB*
     &  SB*vv*(dMHp1MT4(3)+la2*vv*dv4(3)))/vv
       dMH1MT4IN(4,6,3)=(-2*SB**4*dtad1MT4(1,3)-3*abskappa*abslambda*
     &  Cosphiy*SB*vs2*dv4(3)+3*CB**5*la2*SB**2*vv2*dv4(3)+CB**3*(-2*
     &  SB*dtad1MT4(2,3)+6*la2*SB**4*vv2*dv4(3))+CB*(2*SB2*vv*
     &  dMHp1MT4(3)+SB**2*(2*MHp2+3*la2*SB**4*vv2)*dv4(3)))/(2.*vS)
       dMH1MT4IN(5,5,3)=CB2*dMHp1MT4(3)-(CB*SB2*dtad1MT4(1,3))/vv+((2*
     &  CB2*SB+SB**3)*dtad1MT4(2,3))/vv+CB2*la2*vv*dv4(3)


       dMH1MT4IN(5,6,3)=(2*CB2*SB*vv*dMHp1MT4(3)+CB*(-2*CB**3*
     &  dtad1MT4(2,3)-3*abskappa*abslambda*Cosphiy*vs2*dv4(3)+3*CB*la2*
     &  SB**5*vv2*dv4(3)+CB*SB*(2*MHp2+3*CB**4*la2*vv2)*dv4(3)-2*SB**3*
     &  (dtad1MT4(1,3)-3*CB**3*la2*vv2*dv4(3))))/(2.*vS)

       dMH1MT4IN(6,6,3)=(CB2*Cosk3s*SB2*vS*vv2*dMHp1MT4(3)+vS2*(Cosk3s*
     &  dtad1MT4(3,3)-3*Sink3s*dtad1MT4(6,3))+2*CB**6*Cosk3s*la2*SB**2*
     &  vS*vv*vv2*dv4(3)+2*CB**2*Cosk3s*SB**2*vS*vv*(MHp2+la2*SB**4*
     &  vv2)*dv4(3)+CB*vS*vv*(-(Cosk3s*SB**4*dtad1MT4(1,3))+3*Sink3s*
     &  dtad1MT4(4,3)+3*abskappa*abslambda*Cosk3s*Cosphiy*SB*vs2*dv4(3)
     &  -9*abskappa*abslambda*SB*Sink3s*Sinphiy*vs2*dv4(3))+CB**4*
     &  Cosk3s*SB*vS*vv*(-dtad1MT4(2,3)+4*la2*SB**3*vv2*dv4(3)))
     &  /(Cosk3s*vS*vS2)

       do i=1,6
        do j=i,6
         dMH1MT4IN(j,i,1:3)=dMH1MT4IN(i,j,1:3)
        enddo
       enddo

       do i=1,6
        do j=1,6
         dMH1MT4(i,j,1:3)=0D0
         do I2G3=1,6
         do I2G4=1,6         
       dMH1MT4(i,j,1)=dMH1MT4(i,j,1)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4IN(I2G3,I2G4,1)
       dMH1MT4(i,j,2)=dMH1MT4(i,j,2)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4IN(I2G3,I2G4,2)
       dMH1MT4(i,j,3)=dMH1MT4(i,j,3)+ZH(i,I2G3)*ZH(j,I2G4)*
     &                dMH1MT4IN(I2G3,I2G4,3)
         enddo
         enddo
        enddo
       enddo
c end neutral Higgs mass CT at one-loop

c CT charged Higgs mass

       dMCH1L(1,1,1)=(CB*dtad1MT4(1,1)+SB*dtad1MT4(2,1))/vv
       dMCH1L(1,2,1)=(abskappa*abslambda*Cosk3s*(-2*CB**4*dTanB4+CB*
     &  dZHu4*facZ*SB-2*CB2*dTanB4*SB2)*vS**2*vv+abslambda*(-2*CB**4*dTanB4+
     &  CB*dZHu4*facZ*SB-2*CB2*dTanB4*SB2)*sqrt2*vS*vv*DBLE(Ala)+CB*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*(abslambda**2*SB*(2*CB**4*dTanB4-CB*
     &  dZHu4*facZ*SB+2*CB2*dTanB4*SB2)*vv*vv2+4*(-(SB**2*dtad1MT4(1,1))+CB*
     &  SB*dtad1MT4(2,1)+Ic*dtad1MT4(4,1))))/(4.*CB*SB*(Cosk3s*Cosphiy-
     &  Sink3s*Sinphiy)*vv)
       dMCH1L(2,1,1)=(abskappa*abslambda*Cosk3s*(-2*CB**4*dTanB4+CB*
     &  dZHu4*facZ*SB-2*CB2*dTanB4*SB2)*vS**2*vv+abslambda*(-2*CB**4*dTanB4+
     &  CB*dZHu4*facZ*SB-2*CB2*dTanB4*SB2)*sqrt2*vS*vv*DBLE(Ala)+CB*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*(abslambda**2*SB*(2*CB**4*dTanB4-CB*
     &  dZHu4*facZ*SB+2*CB2*dTanB4*SB2)*vv*vv2-4*(SB**2*dtad1MT4(1,1)-CB*SB*
     &  dtad1MT4(2,1)+Ic*dtad1MT4(4,1))))/(4.*CB*SB*(Cosk3s*Cosphiy-
     &  Sink3s*Sinphiy)*vv)

       dMCH1L(1,1,2)=(CB*dtad1MT4(1,2)+SB*dtad1MT4(2,2))/vv
       dMCH1L(1,2,2)=(-(SB**2*dtad1MT4(1,2))+CB*SB*dtad1MT4(2,2)+Ic*
     &  dtad1MT4(4,2))/(SB*vv)
       dMCH1L(2,1,2)=-((SB**2*dtad1MT4(1,2)-CB*SB*dtad1MT4(2,2)+Ic*
     &  dtad1MT4(4,2))/(SB*vv))

       dMCH1L(1,1,3)=(CB*dtad1MT4(1,3)+SB*dtad1MT4(2,3))/vv
       dMCH1L(1,2,3)=(-(SB**2*dtad1MT4(1,3))+CB*SB*dtad1MT4(2,3)+Ic*
     &  dtad1MT4(4,3))/(SB*vv)
       dMCH1L(2,1,3)=-((SB**2*dtad1MT4(1,3)-CB*SB*dtad1MT4(2,3)+Ic*
     &  dtad1MT4(4,3))/(SB*vv))

       dMCH1L(2,2,2:3)= dMHp1MT4(2:3)
       dMCH1L(2,2,1)= dMHp1MT4(1)+MHp2*CB2*dZHu4 *facz !toTest

       dMCH1LB(1,1,1)=(CB*dtad1MT4B(1,1)+SB*dtad1MT4B(2,1))/vv
       dMCH1LB(1,2,1)=(abslambda*dTanB4B*(CB**4+CB2*SB2)*(abslambda*CB*
     &  SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv2-vS*(abskappa*Cosk3s*vS+
     &  sqrt2*DBLE(Ala))))/(2.*CB*SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy))-
     &  (SB*dtad1MT4B(1,1))/vv+(CB*dtad1MT4B(2,1))/vv+(Ic*
     &  dtad1MT4B(4,1))/(SB*vv)+(abslambda*(abslambda*CB*SB*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*vv2-vS*(abskappa*Cosk3s*vS+sqrt2*
     &  DBLE(Ala)))*(Zfactor1LIN(1,1)-Zfactor1LIN(2,2))*facZ)/(4*Cosk3s*
     &  Cosphiy-4*Sink3s*Sinphiy)
       dMCH1LB(2,1,1)=(abslambda*dTanB4B*(CB**4+CB2*SB2)*(abslambda*CB*
     &  SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv2-vS*(abskappa*Cosk3s*vS+
     &  sqrt2*DBLE(Ala))))/(2.*CB*SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy))-
     &  (SB*dtad1MT4B(1,1))/vv+(CB*dtad1MT4B(2,1))/vv-(Ic*
     &  dtad1MT4B(4,1))/(SB*vv)+(abslambda*(abslambda*CB*SB*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*vv2-vS*(abskappa*Cosk3s*vS+sqrt2*
     &  DBLE(Ala)))*(Zfactor1LIN(1,1)-Zfactor1LIN(2,2))*facZ)/(4*Cosk3s*
     &  Cosphiy-4*Sink3s*Sinphiy)

       dMCH1LB(1,1,2)=(CB*dtad1MT4B(1,2)+SB*dtad1MT4B(2,2))/vv
       dMCH1LB(1,2,2)=(-(SB**2*dtad1MT4B(1,2))+CB*SB*dtad1MT4B(2,2)+Ic*
     &  dtad1MT4B(4,2))/(SB*vv)
       dMCH1LB(2,1,2)=-((SB**2*dtad1MT4B(1,2)-CB*SB*dtad1MT4B(2,2)+Ic*
     &  dtad1MT4B(4,2))/(SB*vv))

       dMCH1LB(1,1,3)=(CB*dtad1MT4B(1,3)+SB*dtad1MT4B(2,3))/vv
       dMCH1LB(1,2,3)=(-(SB**2*dtad1MT4B(1,3))+CB*SB*dtad1MT4B(2,3)+Ic*
     &  dtad1MT4B(4,3))/(SB*vv)
       dMCH1LB(2,1,3)=-((SB**2*dtad1MT4B(1,3)-CB*SB*dtad1MT4B(2,3)+Ic*
     &  dtad1MT4B(4,3))/(SB*vv))
       dMCH1LB(2,2,2:3)= dMHp1MT4B(2:3)

       dMCH1LB(2,2,1)= dMHp1MT4B(1)+MHp2*(CB2*Zfactor1LIN(2,2)+SB2*Zfactor1LIN(1,1))*facz!toTest
c end CT charged Higgs mass

c chargino and neutralino CT mass matrix 

       dMNeu1LIN(1,1,1)=0
       dMNeu1LIN(1,2,1)=-((dabslambda14*explam*expvS*vS)/sqrt2)
       dMNeu1LIN(1,3,1)=-((abslambda*CB**3*dTanB4*expETA*explam*vv)/sqrt2)
     &  -(dabslambda14*expETA*explam*SB*vv)/sqrt2-(abslambda*expETA*
     &  explam*SB*dv4(1))/sqrt2
       dMNeu1LIN(2,2,1)=0
       dMNeu1LIN(2,3,1)=-((CB*dabslambda14*explam*vv)/sqrt2)+(abslambda*
     &  CB2*dTanB4*explam*SB*vv)/sqrt2-(abslambda*CB*explam*dv4(1))
     &  /sqrt2
       dMNeu1LIN(3,3,1)=0

       dMNeu1LIN(1,1,2:3)=0
       dMNeu1LIN(1,2,2:3)=0
       dMNeu1LIN(1,3,2:3)=-((abslambda*expETA*explam*SB*dv4(2:3))/sqrt2)
       dMNeu1LIN(2,2,2:3)=0
       dMNeu1LIN(2,3,2:3)=-((abslambda*CB*explam*dv4(2:3))/sqrt2)
       dMNeu1LIN(3,3,2:3)=0


       dMNeu1LBIN(1,1,1)=0
       dMNeu1LBIN(1,2,1)=-((abslambda*dvS1B*explam*expvS)/sqrt2)-
     &  (dabslambda14B*explam*expvS*vS)/sqrt2
       dMNeu1LBIN(1,3,1)=-((abslambda*CB**3*dTanB4B*expETA*explam*vv)
     &  /sqrt2)-(dabslambda14B*expETA*explam*SB*vv)/sqrt2-(abslambda*
     &  expETA*explam*SB*dv4B(1))/sqrt2
       dMNeu1LBIN(2,2,1)=0
       dMNeu1LBIN(2,3,1)=-((CB*dabslambda14B*explam*vv)/sqrt2)+(abslambda*
     &  CB2*dTanB4B*explam*SB*vv)/sqrt2-(abslambda*CB*explam*dv4B(1))
     &  /sqrt2
       dMNeu1LBIN(3,3,1)=abskappa*dvS1B*expkap*expvS*sqrt2+dabskappa14B*
     &  expkap*expvS*sqrt2*vS
       dMNeu1LBIN(1,1,2:3)=0
       dMNeu1LBIN(1,2,2:3)=0
       dMNeu1LBIN(1,3,2:3)=-((abslambda*expETA*explam*SB*dv4B(2:3))/sqrt2)
       dMNeu1LBIN(2,2,2:3)=0
       dMNeu1LBIN(2,3,2:3)=-((abslambda*CB*explam*dv4B(2:3))/sqrt2)
       dMNeu1LBIN(3,3,2:3)=0

       do i=1,3
        do j=i,3
       dMNeu1LIN(j,i,1:3) =dMNeu1LIN(i,j,1:3)
       dMNeu1LBIN(j,i,1:3)=dMNeu1LBIN(i,j,1:3)
        enddo
       enddo

       do i=1,3
        do j=1,3
       dMNeu1L(i,j,1:3)=0D0
       dMNeu1LB(i,j,1:3)=0D0
       do I2G3=1,3
        do I2G4=1,3
       dMNeu1L(i,j,1)= dMNeu1L(i,j,1)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LIN(I2G3,I2G4,1)
       dMNeu1L(i,j,2)= dMNeu1L(i,j,2)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LIN(I2G3,I2G4,2)
       dMNeu1L(i,j,3)= dMNeu1L(i,j,3)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LIN(I2G3,I2G4,3)
       dMNeu1LB(i,j,1)=dMNeu1LB(i,j,1)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LBIN(I2G3,I2G4,1)
       dMNeu1LB(i,j,2)=dMNeu1LB(i,j,2)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LBIN(I2G3,I2G4,2)
       dMNeu1LB(i,j,3)=dMNeu1LB(i,j,3)+DCONJG(ZNeu(i,I2G3))*
     &                 DCONJG(ZNeu(j,I2G4))*dMNeu1LBIN(I2G3,I2G4,3)
        enddo
       enddo

        enddo
       enddo

       dMcha1L(1)=(dabslambda14*vS)/sqrt2
       dMcha1L(2:3)=0

       dMcha1LB(1)=(abslambda*dvS1B)/sqrt2+(dabslambda14B*
     &  vS)/sqrt2

       dMcha1LB(2:3)=0


c chargino and neutralino CT mass matrix 

c CTs for Alambda, Akappa
       dReAla(1)=((Cosphiy*Sink3s+Cosk3s*Sinphiy)*sqrt2*dtad1MT4(4,1))
     &  /(abs(MUe)*sqrt2*SB*vv)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*((CB*
     &  dabslambda14*SB*(-2*MHp2+la2*vv2))/(abs(MUe)**2*2*sqrt2/vS)+(CB**2*
     &  dTanB4*(CB**4-SB**4)*(2*MHp2+la2*vv2))/(abs(MUe)*sqrt2*sqrt2)+
     &  (sqrt2*(CB*SB*vv*dMHp1MT4(1)-SB**3*dtad1MT4(1,1)-CB**3*
     &  dtad1MT4(2,1)+CB*la2*SB*vv2*dv4(1)))/(abs(MUe)*sqrt2*vv))
       
       dReAlaB(1)=(-(abslambda*Cosk3s*SB*vS*(abskappa*dvS1B+
     &  dabskappa14B*vS)*vv)+2*(Cosphiy*Sink3s+Cosk3s*Sinphiy)*
     &  dtad1MT4B(4,1))/(abs(MUe)*sqrt2*SB*sqrt2*vv)+(Cosk3s*Cosphiy-
     &  Sink3s*Sinphiy)*((CB*dabslambda14B*SB*(-2*MHp2+la2*vv2))/(
     &  sqrt2*abs(MUe)**2*2/vS)+(CB**2*dTanB4B*(CB**4-SB**4)*(2*MHp2+la2*vv2))
     &  /(abs(MUe)*sqrt2*sqrt2)+(2*CB*SB*vS*vv*dMHp1MT4B(1)-2*SB**3*vS*
     &  dtad1MT4B(1,1)-CB*(2*CB**2*vS*dtad1MT4B(2,1)+SB*vv*(2*dvS1B*
     &  MHp2+dvS1B*la2*vv2-2*la2*vS*vv*dv4B(1))))/(abs(MUe)*sqrt2*sqrt2*vS*
     &  vv))

       dImAla(1)=-(((Cosphiy*Sink3s+Cosk3s*Sinphiy)*dReAla(1))/(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy))+(sqrt2*dtad1MT4(4,1))/(abs(MUe)*sqrt2*SB*
     &  (Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv)
       dImAlaB(1)=(-(Sinphiy*sqrt2*(abskappa*dvS1B+dabskappa14B*vS))-2*
     &  (Cosphiy*Sink3s+Cosk3s*Sinphiy)*dReAlaB(1)+(2*sqrt2*
     &  dtad1MT4B(4,1))/(abs(MUe)*sqrt2*SB*vv))/(2.*(Cosk3s*Cosphiy-
     &  Sink3s*Sinphiy))




       if(abskappa.gt.1D-10)then
       dImAka(1)=(abslambda*CB**4*dTanB4*vv**2*(-(abskappa*Sinphiy*
     &  sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala)))/(abskappa*Cosk3s*vs2)+(CB*
     &  dabslambda14*SB*vv**2*(-(abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*
     &  Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*
     &  Sinphiy)*DIMAG(Ala)))/(abskappa*Cosk3s*vs2)-(abslambda*CB2*
     &  dTanB4*SB2*vv**2*(-(abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+
     &  Cosk3s*Sinphiy)*DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*
     &  DIMAG(Ala)))/(abskappa*Cosk3s*vs2)+(abslambda*CB*SB*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*vv**2*dImAla(1))/(abskappa*Cosk3s*vs2)+
     &  (abslambda*CB*SB*(Cosphiy*Sink3s+Cosk3s*Sinphiy)*vv**2*
     &  dReAla(1))/(abskappa*Cosk3s*vs2)-(sqrt2*dtad1MT4(6,1))
     &  /(abskappa*Cosk3s*vs2)+(2*abslambda*CB*SB*vv*(-(abskappa*
     &  Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+
     &  (Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala))*dv4(1))/(abskappa*
     &  Cosk3s*vs2)
       dImAkaB(1)=-((dReAkappa14B*Sink3s)/Cosk3s)-(abslambda*CB*
     &  dabskappa14B*SB*vv**2*((Cosphiy*Sink3s+Cosk3s*Sinphiy)*
     &  DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala)))
     &  /(abskappa**2*Cosk3s*vs2)+(abslambda*CB**4*dTanB4B*vv**2*(-
     &  (abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*Sinphiy)*
     &  DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala)))
     &  /(abskappa*Cosk3s*vs2)+(CB*dabslambda14B*SB*vv**2*(-(abskappa*
     &  Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+
     &  (Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala)))/(abskappa*Cosk3s*
     &  vs2)-(abslambda*CB2*dTanB4B*SB2*vv**2*(-(abskappa*Sinphiy*
     &  sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala)))/(abskappa*Cosk3s*vs2)-
     &  (abslambda*CB*dvS1B*SB*vv**2*(-(abskappa*Sinphiy*sqrt2*vS)+2*
     &  (Cosphiy*Sink3s+Cosk3s*Sinphiy)*DBLE(Ala)+2*(Cosk3s*Cosphiy-
     &  Sink3s*Sinphiy)*DIMAG(Ala)))/(abskappa*Cosk3s*vS**3)+
     &  (abslambda*CB*SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv**2*
     &  dImAlaB(1))/(abskappa*Cosk3s*vs2)+(abslambda*CB*SB*(Cosphiy*
     &  Sink3s+Cosk3s*Sinphiy)*vv**2*dReAlaB(1))/(abskappa*Cosk3s*vs2)-
     &  (sqrt2*dtad1MT4B(6,1))/(abskappa*Cosk3s*vs2)+(2*abslambda*CB*
     &  SB*vv*(-(abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*
     &  Sinphiy)*DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala))*
     &  dv4B(1))/(abskappa*Cosk3s*vs2)
        else
       dImAka(1)=0D0
       dImAkaB(1)=0D0
        endif

       dTlam(1)=(Ala*dabslambda14)*explam+la*(IC*dImAla(1)+dReAla(1))
       dTlamB(1)=(Ala*dabslambda14B)*explam+la*(IC*dImAlaB(1)+dReAlaB(1))
       dTk(1)=(Aka*dabskappa14)*expkap+kappa*(IC*dImAka(1)+dReAkappa14) 
       dTkB(1)=(Aka*dabskappa14B)*expkap+kappa*(dReAkappa14B+
     &  IC*dImAkaB(1))

       do i=2,3
       dReAla(i)=((Cosphiy*Sink3s+Cosk3s*Sinphiy)*sqrt2*dtad1MT4(4,i))
     &  /( abs(MUe)*sqrt2*SB*vv)+((Cosk3s*Cosphiy-Sink3s*Sinphiy)*sqrt2*
     &  (CB*SB*vv*dMHp1MT4(i)-SB**3*dtad1MT4(1,i)-CB**3*dtad1MT4(2,i)+
     &  CB*la2*SB*vv2*dv4(i)))/( abs(MUe)*sqrt2*vv)
       dReAlaB(i)=(2*(Cosphiy*Sink3s+Cosk3s*Sinphiy)*dtad1MT4B(4,i))
     &  /( abs(MUe)*sqrt2*SB*sqrt2*vv)+((Cosk3s*Cosphiy-Sink3s*Sinphiy)*
     &  (2*CB*SB*vS*vv*dMHp1MT4B(i)-2*SB**3*vS*dtad1MT4B(1,i)-CB*(2*CB*
     &  *2*vS*dtad1MT4B(2,i)-2*la2*SB*vS*vv**2*dv4B(i))))/( abs(MUe)*sqrt2*
     &  sqrt2*vS*vv)
       dImAla(i)=-(((Cosphiy*Sink3s+Cosk3s*Sinphiy)*dReAla(i))/(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy))+(sqrt2*dtad1MT4(4,i))/( abs(MUe)*sqrt2*SB*
     &  (Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv)
       dImAlaB(i)=(-2*(Cosphiy*Sink3s+Cosk3s*Sinphiy)*dReAlaB(i)+(2*
     &  sqrt2*dtad1MT4B(4,i))/( abs(MUe)*sqrt2*SB*vv))/(2.*(Cosk3s*
     &  Cosphiy-Sink3s*Sinphiy))
       if(abskappa.gt.1D-10)then
       dImAka(i)=(abslambda*CB*SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv**
     &  2*dImAla(i))/(abskappa*Cosk3s*vs2)+(abslambda*CB*SB*(Cosphiy*
     &  Sink3s+Cosk3s*Sinphiy)*vv**2*dReAla(i))/(abskappa*Cosk3s*vs2)-
     &  (sqrt2*dtad1MT4(6,i))/(abskappa*Cosk3s*vs2)+(2*abslambda*CB*SB*
     &  vv*(-(abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*
     &  Sinphiy)*DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala))*
     &  dv4(i))/(abskappa*Cosk3s*vs2)

       dImAkaB(i)=(abslambda*CB*SB*(Cosk3s*Cosphiy-Sink3s*Sinphiy)*vv**
     &  2*dImAlaB(i))/(abskappa*Cosk3s*vs2)+(abslambda*CB*SB*(Cosphiy*
     &  Sink3s+Cosk3s*Sinphiy)*vv**2*dReAlaB(i))/(abskappa*Cosk3s*vs2)-
     &  (sqrt2*dtad1MT4B(6,i))/(abskappa*Cosk3s*vs2)+(2*abslambda*CB*
     &  SB*vv*(-(abskappa*Sinphiy*sqrt2*vS)+(Cosphiy*Sink3s+Cosk3s*
     &  Sinphiy)*DBLE(Ala)+(Cosk3s*Cosphiy-Sink3s*Sinphiy)*DIMAG(Ala))*
     &  dv4B(i))/(abskappa*Cosk3s*vs2)

        else
       dImAka(i)=0D0
       dImAkaB(i)=0D0
        endif

       dTlam(i)=la*(IC*dImAla(i)+dReAla(i))
       dTlamB(i)=la*(IC*dImAlaB(i)+dReAlaB(i))
       dTkB(i)=IC*kappa*dImAkaB(i)
       dTk(i)=IC*kappa*dImAka(i)


       enddo
c end CTs for Alambda, Akappa

c CTs for top/stop sector

         dmt4(1:3) =0D0    
         dMSf4(1:2,1:2,1:3)=0D0

       dmt4(1)=(CF12F3S*CF12F3SC*M1)/4.+dmt4(1)
       dMSf4(1,1,1)=dMSf4(1,1,1)+(MSQ2*CS14S13S14S13MT4(1,1))
       dMSf4(1,1,1)=(M22-2*MCha2)*CF12F4S(1)*CF12F4SC(1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=((M22+M42-4*MCha2)*CF12F4S(1)*CF12F4SC(2))/2.+dMSf4(1,2,1)
       dMSf4(1,2,1)=dMSf4(1,2,1)+(MSQ2*CS14S13S14S13MT4(2,1))
       dMSf4(2,2,1)=(M42-2*MCha2)*CF12F4S(2)*CF12F4SC(2)+dMSf4(2,2,1)
       dMSf4(2,2,1)=dMSf4(2,2,1)+(MSQ2*CS14S13S14S13MT4(2,2))
       Do  I6G3= ihiggs,ihiggse
       dmt4(1)=dmt4(1)+(M1*(hy(I6G3)**2+hy(I6G3)*hyc(I6G3)+hyc(I6G3)**2))/2.
       dMSf4(1,1,1)=dMSf4(1,1,1)+((CSSSS(I6G3,I6G3,1,1)*Mah2(I6G3))/2.)
       dMSf4(1,2,1)=dMSf4(1,2,1)+((CSSSS(I6G3,I6G3,2,1)*Mah2(I6G3))/2.)
       dMSf4(2,2,1)=dMSf4(2,2,1)+(CSSSS(I6G3,I6G3,2,2)*Mah2(I6G3))/2.
       enddo
 
       Do  I2G3= 1,2
       dmt4(1)=(M1*CTBS3(I2G3)*CTBS3C(I2G3))/4.+dmt4(1)
       dMSf4(1,1,1)=dMSf4(1,1,1)+CS3S3S13S13(I2G3,I2G3,1,1)*MassHpm(I2G3)**2
       dMSf4(1,1,1)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)+dMSf4(1,2,1)
       dMSf4(1,2,1)=dMSf4(1,2,1)+CS3S3S13S13(I2G3,I2G3,2,1)*MassHpm(I2G3)**2
       dMSf4(2,2,1)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)+dMSf4(2,2,1)
       dMSf4(2,2,1)=dMSf4(2,2,1)+CS3S3S13S13(I2G3,I2G3,2,2)*MassHpm(I2G3)**2
       dMSf4(1,1,1)=dMSf4(1,1,1)+(
     &  CS13S13S13S13MT4(1,I2G3,1,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(1,I2G3,I2G3,1)*MassSu2(I2G3))
       dMSf4(1,2,1)=dMSf4(1,2,1)+(
     &  CS13S13S13S13MT4(2,I2G3,1,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,1)*MassSu2(I2G3))
       dMSf4(2,2,1)=dMSf4(2,2,1)+(
     &  CS13S13S13S13MT4(2,I2G3,2,I2G3)*MassSu2(I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,2)*MassSu2(I2G3))
       enddo

       Do  I5G3= ineu,ineue
       Do  I6G3= 1,2
       dmt4(1)=dmt4(1)+(CF11FSR(I5G3,I6G3)*(M1*CF11FSRC(I5G3,I6G3)+2*
     &  CF11FSLC(I5G3,I6G3)*MNeu(I5G3))+CF11FSL(I5G3,I6G3)*(M1*
     &  CF11FSLC(I5G3,I6G3)+2*CF11FSRC(I5G3,I6G3)*MNeu(I5G3)))/4d0
       enddo
       enddo

       Do  I5G3= ineu,ineue
       dMSf4(1,1,1)=dMSf4(1,1,1)+(CF11FSL(I5G3,1)*(-2*M1*
     &  CF11FSRC(I5G3,1)*MNeu(I5G3)+CF11FSLC(I5G3,1)*(-2*M12+M22-2*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,1)*(-2*M1*CF11FSLC(I5G3,1)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,1)*(-2*M12+M22-2*MNeu2(I5G3))))
       dMSf4(1,2,1)=dMSf4(1,2,1)+(CF11FSL(I5G3,1)*(-4*M1*
     &  CF11FSRC(I5G3,2)*MNeu(I5G3)+CF11FSLC(I5G3,2)*(-4*M12+M22+M42-4*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,1)*(-4*M1*CF11FSLC(I5G3,2)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,2)*(-4*M12+M22+M42-4*MNeu2(I5G3))))/2.
       dMSf4(2,2,1)=dMSf4(2,2,1)+(CF11FSL(I5G3,2)*(-2*M1*
     &  CF11FSRC(I5G3,2)*MNeu(I5G3)+CF11FSLC(I5G3,2)*(-2*M12+M42-2*
     &  MNeu2(I5G3)))+CF11FSR(I5G3,2)*(-2*M1*CF11FSLC(I5G3,2)*
     &  MNeu(I5G3)+CF11FSRC(I5G3,2)*(-2*M12+M42-2*MNeu2(I5G3))))
       enddo

       Do  I6G3= ihiggs,ihiggse
       Do  I6G4= 1,2
       dMSf4(1,1,1)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)+dMSf4(1,1,1)
       dMSf4(1,2,1)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)+dMSf4(1,2,1)
       dMSf4(2,2,1)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)+dMSf4(2,2,1)
       enddo
       enddo
c**********************
       dmt4(2)=-(CF12F3S*CF12F3SC*M1*DBLE(B1P(M12,MCha2,MSQ2,Q2)))/2d0+dmt4(2)

       dMSf4(1,1,2)=(a0t5*CS14S13S14S13MT4(1,1))+dMSf4(1,1,2)
       dMSf4(1,1,2)=CF12F4S(1)*CF12F4SC(1)*(-a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2)))+dMSf4(1,1,2)

       dMSf4(1,2,2)=(CF12F4S(1)*CF12F4SC(2)*(-2*a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2))+(M42-MCha2)*DBLE(B0P(M42,0D0,MCha2,Q2))
     &  ))/2.+dMSf4(1,2,2)
       dMSf4(1,2,2)=(a0t5*CS14S13S14S13MT4(2,1))+dMSf4(1,2,2)

       dMSf4(2,2,2)=CF12F4S(2)*CF12F4SC(2)*(-A0T(MCha2,Q2)+(M42-MCha2)*
     &  DBLE(B0P(M42,0D0,MCha2,Q2)))+dMSf4(2,2,2)
       dMSf4(2,2,2)=(a0t5*CS14S13S14S13MT4(2,2))+dMSf4(2,2,2)

       Do  I2G3= 1,2
       dmt4(2)=-(M1*DBLE(B1P(M12,0D0,MassHpm(I2G3)**2,Q2))*CTBS3(I2G3)*
     &  CTBS3C(I2G3))/2d0+dmt4(2)
       dMSf4(1,1,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,1,1)+dMSf4(1,1,2)
       dMSf4(1,1,2)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)*
     &  DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)*
     &  (DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))))/2.+dMSf4(1,2,2)
       dMSf4(1,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,1)
     &              +dMSf4(1,2,2)
       dMSf4(2,2,2)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)*
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(2,2,2)
       dMSf4(2,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,2)
     &  +dMSf4(2,2,2)
       dMSf4(1,1,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(1,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(1,I2G3,I2G3,1)))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,1)))+dMSf4(1,2,2)
       dMSf4(2,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,2,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,2)))+dMSf4(2,2,2)
       enddo

       Do  I6G3= ihiggs,ihiggse
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,M12,Mah2(I6G3),Q2))*hy(I6G3)*
     &  hyc(I6G3))+(M1*DBLE(B0P(M12,M12,Mah2(I6G3),Q2))*(hy(I6G3)**2+
     &  hyc(I6G3)**2))/2.)
       dMSf4(1,1,2)=((A0TNH(I6G3)*CSSSS(I6G3,I6G3,1,1))/2.)+dMSf4(1,1,2)
       dMSf4(1,2,2)=A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,1)/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=(A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,2))/2.+dMSf4(2,2,2)
       enddo

       Do  I6G3= ihiggs,ihiggse
       Do  I6G4= 1,2
       dMSf4(1,1,2)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)*
     &  DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)*
     &  (DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))))/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)*
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(2,2,2)
       enddo
       enddo


       Do  I5G3= ineu,ineue
       Do  I6G3= 1,2
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSL(I5G3,I6G3)*CF11FSLC(I5G3,I6G3)+CF11FSR(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3)))+DBLE(B0P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSLC(I5G3,I6G3)*CF11FSR(I5G3,I6G3)+CF11FSL(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3))*MNeu(I5G3))/2d0
       enddo
       enddo

       Do  I5G3= ineu,ineue
       dMSf4(1,1,2)=dMSf4(1,1,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,1))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,1)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,1)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,1)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,1)*MNeu(I5G3)+CF11FSRC(I5G3,1)*(M12-M22+
     &  MNeu2(I5G3)))))
       dMSf4(1,2,2)=dMSf4(1,2,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,2))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,2)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-(DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M22+
     &  MNeu2(I5G3)))))/2.-(DBLE(B0P(M42,M12,MNeu2(I5G3),Q2))*
     &  (CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))/2.)
       dMSf4(2,2,2)=dMSf4(2,2,2)+(-(A0T(M12,Q2)*(CF11FSL(I5G3,2)*
     &  CF11FSLC(I5G3,2)+CF11FSR(I5G3,2)*CF11FSRC(I5G3,2)))-
     &  A0T(MNeu2(I5G3),Q2)*(CF11FSL(I5G3,2)*CF11FSLC(I5G3,2)+
     &  CF11FSR(I5G3,2)*CF11FSRC(I5G3,2))-DBLE(B0P(M42,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,2)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,2)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))
       enddo

c eps part
       Do  I6G3= ihiggs,6
       dmt4(3)=dmt4(3)+(-(M1*DBLE(B1EpP(M12,M12,Mah2(I6G3),Q2))*hy(I6G3)*
     &  hyc(I6G3))+(M1*DBLE(BEpP(M12,M12,Mah2(I6G3),Q2))*(hy(I6G3)**2+
     &  hyc(I6G3)**2))/2.)
       dMSf4(1,1,3)=((AETNH(I6G3)*CSSSS(I6G3,I6G3,1,1))/2.)+dMSf4(1,1,3)
       dMSf4(2,2,3)=(AETNH(I6G3)*CSSSS(I6G3,I6G3,2,2))/2.+dMSf4(2,2,3)
       dMSf4(1,2,3)=AETNH(I6G3)*CSSSS(I6G3,I6G3,2,1)/2.+dMSf4(1,2,3)
       enddo

       Do  I2G3= 1,2
       dmt4(3)=-(M1*DBLE(B1EpP(M12,0D0,MassHpm(I2G3)**2,Q2))*CTBS3(I2G3)*
     &  CTBS3C(I2G3))/2.+dmt4(3)
       dMSf4(1,1,3)=AEpT(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,1,1)+dMSf4(1,1,3)
       dMSf4(1,1,3)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)*
     &  DBLE(BEpP(M22,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(1,1,3)

       dMSf4(2,2,3)=AEpT(MassHpm(I2G3)**2,Q2)*
     &  CS3S3S13S13(I2G3,I2G3,2,2)+dMSf4(2,2,3)
       dMSf4(2,2,3)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)*
     &  DBLE(BEpP(M42,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(2,2,3)
       dMSf4(1,2,3)=AEpT(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,1)
     &              +dMSf4(1,2,3)
       dMSf4(1,2,3)=(CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)*
     &  (DBLE(BEpP(M22,MSQ2,MassHpm(I2G3)**2,Q2))+
     &  DBLE(BEpP(M42,MSQ2,MassHpm(I2G3)**2,Q2))))/2.+dMSf4(1,2,3)
       enddo

       dmt4(3)=-(CF12F3S*CF12F3SC*M1*DBLE(B1EpP(M12,MCha2,MSQ2,Q2)))/2.+dmt4(3)
       dMSf4(1,1,3)=(aet5*CS14S13S14S13MT4(1,1))+dMSf4(1,1,3)
       dMSf4(1,1,3)=CF12F4S(1)*CF12F4SC(1)*(-aetcha+(M22-MCha2)*
     &  DBLE(BEpP(M22,0D0,MCha2,Q2)))+dMSf4(1,1,3)
       dMSf4(2,2,3)=CF12F4S(2)*CF12F4SC(2)*(-AEpT(MCha2,Q2)+(M42-MCha2)
     &  *DBLE(BEpP(M42,0D0,MCha2,Q2)))+dMSf4(2,2,3)
       dMSf4(2,2,3)=(aet5*CS14S13S14S13MT4(2,2))+dMSf4(2,2,3)
       dMSf4(1,2,3)=(CF12F4S(1)*CF12F4SC(2)*(-2*aetcha+(M22-MCha2)*
     &  DBLE(BEpP(M22,0D0,MCha2,Q2))+(M42-MCha2)*
     &  DBLE(BEpP(M42,0D0,MCha2,Q2))))/2.+dMSf4(1,2,3)
       dMSf4(1,2,3)=(aet5*CS14S13S14S13MT4(2,1))+dMSf4(1,2,3)

       Do  I5G3= ineu,3
       Do  I6G3= 1,2
       dmt4(3)=dmt4(3)+(-(M1*DBLE(B1EpP(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSL(I5G3,I6G3)*CF11FSLC(I5G3,I6G3)+CF11FSR(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3)))+DBLE(BEpP(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSLC(I5G3,I6G3)*CF11FSR(I5G3,I6G3)+CF11FSL(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3))*MNeu(I5G3))/2.
       enddo
       enddo


       Do  I6G3= 1,2
       dMSf4(1,1,3)=(aAETN(I6G3)*(CS13S13S13S13MT4(1,I6G3,1,I6G3)+3*
     &  CS13S13S13S13MT4(1,I6G3,I6G3,1)))+dMSf4(1,1,3)
       dMSf4(2,2,3)=(aAETN(I6G3)*(CS13S13S13S13MT4(2,I6G3,2,I6G3)+3*
     &  CS13S13S13S13MT4(2,I6G3,I6G3,2)))+dMSf4(2,2,3)
       dMSf4(1,2,3)=(aAETN(I6G3)*(CS13S13S13S13MT4(2,I6G3,1,I6G3)+3*
     &  CS13S13S13S13MT4(2,I6G3,I6G3,1)))+dMSf4(1,2,3)
       enddo
       Do  I5G3= ineu,3
       dMSf4(1,1,3)=dMSf4(1,1,3)+(-(aet1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,1))-AETNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,1)
     &  -aet1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-AETNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-DBLE(BEpP(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,1)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,1)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,1)*MNeu(I5G3)+CF11FSRC(I5G3,1)*(M12-M22+
     &  MNeu2(I5G3)))))
       dMSf4(2,2,3)=dMSf4(2,2,3)+(-(AEpT(M12,Q2)*(CF11FSL(I5G3,2)*
     &  CF11FSLC(I5G3,2)+CF11FSR(I5G3,2)*CF11FSRC(I5G3,2)))-
     &  AEpT(MNeu2(I5G3),Q2)*(CF11FSL(I5G3,2)*CF11FSLC(I5G3,2)+
     &  CF11FSR(I5G3,2)*CF11FSRC(I5G3,2))-DBLE(BEpP(M42,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,2)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,2)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))
       dMSf4(1,2,3)=dMSf4(1,2,3)+(-(aet1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,2))-AETNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,2)
     &  -aet1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-AETNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-(DBLE(BEpP(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M22+
     &  MNeu2(I5G3)))))/2.-(DBLE(BEpP(M42,M12,MNeu2(I5G3),Q2))*
     &  (CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))/2.)
       enddo

       Do  I6G3= ihiggs,6
       Do  I6G4= 1,2
       dMSf4(1,1,3)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)*
     &  DBLE(BEpP(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(1,1,3)
       dMSf4(2,2,3)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)*
     &  DBLE(BEpP(M42,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(2,2,3)
       dMSf4(1,2,3)=(CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)*
     &  (DBLE(BEpP(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+
     &  DBLE(BEpP(M42,Mah2(I6G3),MassSu2(I6G4),Q2))))/2.+dMSf4(1,2,3)
       enddo
       enddo
c end eps part

       dmt4(1:3)=dmt4(1:3)* fac 

       dMSf4(2,1,1:3)=DCONJG(dMSf4(1,2,1:3))
       dMSf4(1:2,1:2,1:3)=dMSf4(1:2,1:2,1:3)*fac

       do i=1, 3
        dMSQ4(i)=-2*M1*dMT4(i)+DCONJG(ZU(1,1))*dMSf4(1,1,i)*ZU(1,1)+
     &  DCONJG(ZU(2,1))*dMSf4(2,1,i)*ZU(1,1)+DCONJG(ZU(1,1))*
     &  dMSf4(1,2,i)*ZU(2,1)+DCONJG(ZU(2,1))*dMSf4(2,2,i)*ZU(2,1)
       enddo
 
       dAT4(1)=(-(dTanB4*M1)*(DCONJG(MUE))+M1*TB*DCONJG(dMUE4)
     &  -ATT*expETA*TB**2*dMT4(1)+(TB*DCONJG(MUE)*dMT4(1))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(2,2))*
     &  dMSf4(2,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*dMSf4(1,2,1)*ZU(2,1)
     &  +TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,1)*ZU(2,1))/(expETA*M1*TB**2)
       do i=2,3
       dAT4(i)=(-(ATT*expETA*TB**2*dMT4(i))+(TB*DCONJG(MUE)*dMT4(i))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,i)*ZU(1,1)+TB*
     &  *2*DCONJG(ZU(2,2))*dMSf4(2,1,i)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*
     &  dMSf4(1,2,i)*ZU(2,1)+TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,i)*ZU(2,1)
     &  )/(expETA*M1*TB**2)
       enddo 

#ifdef printMT4
       write(16,*)'##############################################################'
       write(16,*)' COUNTERTERM One_loop MT4'
       write(16,*)'Zfactor 1L~ al_t^2, dZHd, dHZu,dZhS=',0,dZHu4,0
       write(16,*)'Zfactor 1L~ al_lambda^2, al_kappa^2, dZHdB, dHZuB,dZhSB=', Zfactor1LIN(1,1),
     &              Zfactor1LIN(2,2), Zfactor1LIN(3,3)
       write(16,*)'dTanB1=',dTanB4
       write(16,*)'dTanB1B=',dTanB4B
       write(16,*)"+++++++++++++++++"
       write(16,*)'dMW24=',dPMW24,dMW24,dMWeps
       write(16,*)'dMW24B=',dMW24B(1:3)
       write(16,*)'dMW24S=',dMW24S(1:3)
       write(16,*)'dSW4=',dSW4
       write(16,*)'dSW4B=',dSW4B
       write(16,*)'dSW4S=',dSW4S
       write(16,*)'dMZ24=',dPMZ24,dMZ24,dMZeps
       write(16,*)'dMZ24B=',dMZ24B(1:3)
       write(16,*)'dMZ24S=',dMZ24S(1:3)
       write(16,*)'dMHp1MT4=',dMHp1MT4
       write(16,*)'dMHp1MT4B=',dMHp1MT4B
       write(16,*)'dMHp1MT4S=',dMHp1MT4S
       write(16,*)'dMAA1MT4(1)',dMAA1MT4(1:3)
       write(16,*)"+++++++++++++++++"
       write(16,*)'dMUE1=',dMUE4    
       write(16,*)'dv4=', dv4
       write(16,*)'dv4B=', dv4B
       write(16,*)'dv4S=', dv4S
       write(16,*)"+++++++++++++++++"
       write(16,*)'dlambdaC4=',dlambdaC4  
       write(16,*)'dlambdaC4B=',dlambdaC4B 
       write(16,*)'dlambdaC4S=',dlambdaC4S 
       write(16,*)'dabslambda14',dabslambda14 
       write(16,*)'dabslambda14B',dabslambda14B 
       write(16,*)'dabslambda14S',dabslambda14S
       write(16,*)'dvS1=0'
       write(16,*)'dvS1B=',dvS1B
       write(16,*)'dvS1S=',dvS1S
       write(16,*)'dabskappa14',dabskappa14 
       write(16,*)'dabskappa14B',dabskappa14B 
       write(16,*)'dabskappa14S',dabskappa14S
       write(16,*)'dReAkappa14',dReAkappa14
       write(16,*)'dReAkappa14B',dReAkappa14B
       write(16,*)'dReAkappa14S',dReAkappa14S
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole single pole mass basis p1", tad1SP(1:6)
       write(16,*)"one-loop tadpole finite mass basis  p1", tad1(1:6)
       write(16,*)"one-loop tadpole eps mass basis p1", tad1EP(1:6)
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole single pole mass basis p2", tad1SPB(1:6)
       write(16,*)"one-loop tadpole finite mass basis p2", tad1B(1:6)
       write(16,*)"one-loop tadpole eps mass basis p2", tad1EPB(1:6)
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole P3 =0"
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole single pole interaction basis p1", dTad1MT4(1:6,1) 
       write(16,*)"one-loop tadpole finite p1", dTad1MT4(1:6,2) 
       write(16,*)"one-loop tadpole eps p1", dTad1MT4(1:6,3) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole single pole interaction basis p2", dTad1MT4B(1:6,1) 
       write(16,*)"one-loop tadpole finite p2", dTad1MT4B(1:6,2) 
       write(16,*)"one-loop tadpole eps p2", dTad1MT4B(1:6,3) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"one-loop tadpole P3 =0"
       write(16,*)"+++++++++++++++++"


       write(16,*)"Single pole of neutral Higgs mass matrix one-loop in interaction basis P2"

       write(16,*)"Single pole of neutral Higgs selfenergies P3=0"
 
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P1, single pole"  
       write(16,*)"dMH1MT4IN(1,1:6,1)",dMH1MT4IN(1,1:6,1)     
       write(16,*)"dMH1MT4IN(2,2:6,1)",dMH1MT4IN(2,2:6,1)   
       write(16,*)"dMH1MT4IN(3,3:6,1)",dMH1MT4IN(3,3:6,1)  
       write(16,*)"dMH1MT4IN(4,4:6,1)",dMH1MT4IN(4,4:6,1)  
       write(16,*)"dMH1MT4IN(5,5:6,1)",dMH1MT4IN(5,5:6,1)  
       write(16,*)"dMH1MT4IN(6,6:6,1)",dMH1MT4IN(6,6:6,1) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P1, finite"  
       write(16,*)"dMH1MT4IN(1,1:6,2)",dMH1MT4IN(1,1:6,2)     
       write(16,*)"dMH1MT4IN(2,2:6,2)",dMH1MT4IN(2,2:6,2)   
       write(16,*)"dMH1MT4IN(3,3:6,2)",dMH1MT4IN(3,3:6,2)  
       write(16,*)"dMH1MT4IN(4,4:6,2)",dMH1MT4IN(4,4:6,2)  
       write(16,*)"dMH1MT4IN(5,5:6,2)",dMH1MT4IN(5,5:6,2)  
       write(16,*)"dMH1MT4IN(6,6:6,2)",dMH1MT4IN(6,6:6,2)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P1, eps"  
       write(16,*)"dMH1MT4IN(1,1:6,3)",dMH1MT4IN(1,1:6,3)     
       write(16,*)"dMH1MT4IN(2,2:6,3)",dMH1MT4IN(2,2:6,3)   
       write(16,*)"dMH1MT4IN(3,3:6,3)",dMH1MT4IN(3,3:6,3)  
       write(16,*)"dMH1MT4IN(4,4:6,3)",dMH1MT4IN(4,4:6,3)  
       write(16,*)"dMH1MT4IN(5,5:6,3)",dMH1MT4IN(5,5:6,3)  
       write(16,*)"dMH1MT4IN(6,6:6,3)",dMH1MT4IN(6,6:6,3)   
       write(16,*)"+++++++++++++++++"

       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P1, single pole"  
       write(16,*)"dMH1MT4(1,1:6,1)",dMH1MT4(1,1:6,1)     
       write(16,*)"dMH1MT4(2,2:6,1)",dMH1MT4(2,2:6,1)   
       write(16,*)"dMH1MT4(3,3:6,1)",dMH1MT4(3,3:6,1)  
       write(16,*)"dMH1MT4(4,4:6,1)",dMH1MT4(4,4:6,1)  
       write(16,*)"dMH1MT4(5,5:6,1)",dMH1MT4(5,5:6,1)  
       write(16,*)"dMH1MT4(6,6:6,1)",dMH1MT4(6,6:6,1) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P1, finite"  
       write(16,*)"dMH1MT4(1,1:6,2)",dMH1MT4(1,1:6,2)     
       write(16,*)"dMH1MT4(2,2:6,2)",dMH1MT4(2,2:6,2)   
       write(16,*)"dMH1MT4(3,3:6,2)",dMH1MT4(3,3:6,2)  
       write(16,*)"dMH1MT4(4,4:6,2)",dMH1MT4(4,4:6,2)  
       write(16,*)"dMH1MT4(5,5:6,2)",dMH1MT4(5,5:6,2)  
       write(16,*)"dMH1MT4(6,6:6,2)",dMH1MT4(6,6:6,2)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P1, eps"  
       write(16,*)"dMH1MT4(1,1:6,3)",dMH1MT4(1,1:6,3)     
       write(16,*)"dMH1MT4(2,2:6,3)",dMH1MT4(2,2:6,3)   
       write(16,*)"dMH1MT4(3,3:6,3)",dMH1MT4(3,3:6,3)  
       write(16,*)"dMH1MT4(4,4:6,3)",dMH1MT4(4,4:6,3)  
       write(16,*)"dMH1MT4(5,5:6,3)",dMH1MT4(5,5:6,3)  
       write(16,*)"dMH1MT4(6,6:6,3)",dMH1MT4(6,6:6,3)   
       write(16,*)"+++++++++++++++++"

       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P2, single pole"  
       write(16,*)"dMH1MT4BIN(1,1:6,1)",dMH1MT4BIN(1,1:6,1)     
       write(16,*)"dMH1MT4BIN(2,2:6,1)",dMH1MT4BIN(2,2:6,1)   
       write(16,*)"dMH1MT4BIN(3,3:6,1)",dMH1MT4BIN(3,3:6,1)  
       write(16,*)"dMH1MT4BIN(4,4:6,1)",dMH1MT4BIN(4,4:6,1)  
       write(16,*)"dMH1MT4BIN(5,5:6,1)",dMH1MT4BIN(5,5:6,1)  
       write(16,*)"dMH1MT4BIN(6,6:6,1)",dMH1MT4BIN(6,6:6,1) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P2, finite"  
       write(16,*)"dMH1MT4BIN(1,1:6,2)",dMH1MT4BIN(1,1:6,2)     
       write(16,*)"dMH1MT4BIN(2,2:6,2)",dMH1MT4BIN(2,2:6,2)   
       write(16,*)"dMH1MT4BIN(3,3:6,2)",dMH1MT4BIN(3,3:6,2)  
       write(16,*)"dMH1MT4BIN(4,4:6,2)",dMH1MT4BIN(4,4:6,2)  
       write(16,*)"dMH1MT4BIN(5,5:6,2)",dMH1MT4BIN(5,5:6,2)  
       write(16,*)"dMH1MT4BIN(6,6:6,2)",dMH1MT4BIN(6,6:6,2)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in interaction basis P2, eps"  
       write(16,*)"dMH1MT4BIN(1,1:6,3)",dMH1MT4BIN(1,1:6,3)     
       write(16,*)"dMH1MT4BIN(2,2:6,3)",dMH1MT4BIN(2,2:6,3)   
       write(16,*)"dMH1MT4BIN(3,3:6,3)",dMH1MT4BIN(3,3:6,3)  
       write(16,*)"dMH1MT4BIN(4,4:6,3)",dMH1MT4BIN(4,4:6,3)  
       write(16,*)"dMH1MT4BIN(5,5:6,3)",dMH1MT4BIN(5,5:6,3)  
       write(16,*)"dMH1MT4BIN(6,6:6,3)",dMH1MT4BIN(6,6:6,3)   
       write(16,*)"+++++++++++++++++"

       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P2, single pole"  
       write(16,*)"dMH1MT4B(1,1:6,1)",dMH1MT4B(1,1:6,1)     
       write(16,*)"dMH1MT4B(2,2:6,1)",dMH1MT4B(2,2:6,1)   
       write(16,*)"dMH1MT4B(3,3:6,1)",dMH1MT4B(3,3:6,1)  
       write(16,*)"dMH1MT4B(4,4:6,1)",dMH1MT4B(4,4:6,1)  
       write(16,*)"dMH1MT4B(5,5:6,1)",dMH1MT4B(5,5:6,1)  
       write(16,*)"dMH1MT4B(6,6:6,1)",dMH1MT4B(6,6:6,1) 
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P2, finite"  
       write(16,*)"dMH1MT4B(1,1:6,2)",dMH1MT4B(1,1:6,2)     
       write(16,*)"dMH1MT4B(2,2:6,2)",dMH1MT4B(2,2:6,2)   
       write(16,*)"dMH1MT4B(3,3:6,2)",dMH1MT4B(3,3:6,2)  
       write(16,*)"dMH1MT4B(4,4:6,2)",dMH1MT4B(4,4:6,2)  
       write(16,*)"dMH1MT4B(5,5:6,2)",dMH1MT4B(5,5:6,2)  
       write(16,*)"dMH1MT4B(6,6:6,2)",dMH1MT4B(6,6:6,2)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutral Higgs couterterm matrix in mass basis P2, eps"  
       write(16,*)"dMH1MT4B(1,1:6,3)",dMH1MT4B(1,1:6,3)     
       write(16,*)"dMH1MT4B(2,2:6,3)",dMH1MT4B(2,2:6,3)   
       write(16,*)"dMH1MT4B(3,3:6,3)",dMH1MT4B(3,3:6,3)  
       write(16,*)"dMH1MT4B(4,4:6,3)",dMH1MT4B(4,4:6,3)  
       write(16,*)"dMH1MT4B(5,5:6,3)",dMH1MT4B(5,5:6,3)  
       write(16,*)"dMH1MT4B(6,6:6,3)",dMH1MT4B(6,6:6,3)   
       write(16,*)"+++++++++++++++++"

       write(16,*)"charged Higgs couterterm matrix in mass basis P1, single pole"  
       write(16,*)"dMCH1L(1,1:2,1)",dMCH1L(1,1:2,1)   
       write(16,*)"dMCH1L(2,1:2,1)",dMCH1L(2,1:2,1) 
       write(16,*)"charged Higgs couterterm matrix in mass basis P1, finite"  
       write(16,*)"dMCH1L(1,1:2,2)",dMCH1L(1,1:2,2)   
       write(16,*)"dMCH1L(2,1:2,2)",dMCH1L(2,1:2,2)
       write(16,*)"charged Higgs couterterm matrix in mass basis P1, eps"
       write(16,*)"dMCH1L(1,1:2,3)",dMCH1L(1,1:2,3)   
       write(16,*)"dMCH1L(2,1:2,3)",dMCH1L(2,1:2,3)
       write(16,*)"+++++++++++++++++"
       write(16,*)"charged Higgs couterterm matrix in mass basis P2, single pole"  
       write(16,*)"dMCH1LB(1,1:2,1)",dMCH1LB(1,1:2,1)   
       write(16,*)"dMCH1LB(2,1:2,1)",dMCH1LB(2,1:2,1) 
       write(16,*)"charged Higgs couterterm matrix in mass basis P2, finite"  
       write(16,*)"dMCH1LB(1,1:2,2)",dMCH1LB(1,1:2,2)   
       write(16,*)"dMCH1LB(2,1:2,2)",dMCH1LB(2,1:2,2)
       write(16,*)"charged Higgs couterterm matrix in interaction basis P2, eps"
       write(16,*)"dMCH1LB(1,1:2,3)",dMCH1LB(1,1:2,3)   
       write(16,*)"dMCH1LB(2,1:2,3)",dMCH1LB(2,1:2,3)
       write(16,*)"+++++++++++++++++"
       write(16,*)"charged Higgs couterterm matrix in mass basis P3=0"  

       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutralino couterterm matrix in Interaction basis P1, single pole" 
       write(16,*)"dMNeu1LIN(1,1:3,1)",dMNeu1LIN(1,1:3,1)  
       write(16,*)"dMNeu1LIN(2,2:3,1)",dMNeu1LIN(2,2:3,1)    
       write(16,*)"dMNeu1LIN(3,3:3,1)",dMNeu1LIN(3,3:3,1)  
       write(16,*)"Neutralino couterterm matrix in Interaction basis P1, finite" 
       write(16,*)"dMNeu1LIN(1,1:3,2)",dMNeu1LIN(1,1:3,2)  
       write(16,*)"dMNeu1LIN(2,2:3,2)",dMNeu1LIN(2,2:3,2)    
       write(16,*)"dMNeu1LIN(3,3:3,2)",dMNeu1LIN(3,3:3,2)   
       write(16,*)"Neutralino couterterm matrix in Interaction basis P1, eps" 
       write(16,*)"dMNeu1LIN(1,1:3,3)",dMNeu1LIN(1,1:3,3)  
       write(16,*)"dMNeu1LIN(2,2:3,3)",dMNeu1LIN(2,2:3,3)    
       write(16,*)"dMNeu1LIN(3,3:3,3)",dMNeu1LIN(3,3:3,3)   
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutralino couterterm matrix in mass basis P1, single pole" 
       write(16,*)"dMNeu1L(1,1:3,1)",dMNeu1L(1,1:3,1)  
       write(16,*)"dMNeu1L(2,2:3,1)",dMNeu1L(2,2:3,1)    
       write(16,*)"dMNeu1L(3,3:3,1)",dMNeu1L(3,3:3,1)  
       write(16,*)"Neutralino couterterm matrix in mass basis P1, finite" 
       write(16,*)"dMNeu1L(1,1:3,2)",dMNeu1L(1,1:3,2)  
       write(16,*)"dMNeu1L(2,2:3,2)",dMNeu1L(2,2:3,2)    
       write(16,*)"dMNeu1L(3,3:3,2)",dMNeu1L(3,3:3,2)   
       write(16,*)"Neutralino couterterm matrix in mass basis P1, eps" 
       write(16,*)"dMNeu1L(1,1:3,3)",dMNeu1L(1,1:3,3)  
       write(16,*)"dMNeu1L(2,2:3,3)",dMNeu1L(2,2:3,3)    
       write(16,*)"dMNeu1L(3,3:3,3)",dMNeu1L(3,3:3,3)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutralino couterterm matrix in Interaction basis P2, single pole" 
       write(16,*)"dMNeu1LBIN(1,1:3,1)",dMNeu1LBIN(1,1:3,1)  
       write(16,*)"dMNeu1LBIN(2,2:3,1)",dMNeu1LBIN(2,2:3,1)    
       write(16,*)"dMNeu1LBIN(3,3:3,1)",dMNeu1LBIN(3,3:3,1)  
       write(16,*)"Neutralino couterterm matrix in Interaction basis P2, finite" 
       write(16,*)"dMNeu1LBIN(1,1:3,2)",dMNeu1LBIN(1,1:3,2)  
       write(16,*)"dMNeu1LBIN(2,2:3,2)",dMNeu1LBIN(2,2:3,2)    
       write(16,*)"dMNeu1LBIN(3,3:3,2)",dMNeu1LBIN(3,3:3,2)   
       write(16,*)"Neutralino couterterm matrix in Interaction basis P2, eps" 
       write(16,*)"dMNeu1LBIN(1,1:3,3)",dMNeu1LBIN(1,1:3,3)  
       write(16,*)"dMNeu1LBIN(2,2:3,3)",dMNeu1LBIN(2,2:3,3)    
       write(16,*)"dMNeu1LBIN(3,3:3,3)",dMNeu1LBIN(3,3:3,3)   
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutralino couterterm matrix in mass basis P2, single pole" 
       write(16,*)"dMNeu1LB(1,1:3,1)",dMNeu1LB(1,1:3,1)  
       write(16,*)"dMNeu1LB(2,2:3,1)",dMNeu1LB(2,2:3,1)    
       write(16,*)"dMNeu1LB(3,3:3,1)",dMNeu1LB(3,3:3,1)  
       write(16,*)"Neutralino couterterm matrix in mass basis P2, finite" 
       write(16,*)"dMNeu1LB(1,1:3,2)",dMNeu1LB(1,1:3,2)  
       write(16,*)"dMNeu1LB(2,2:3,2)",dMNeu1LB(2,2:3,2)    
       write(16,*)"dMNeu1LB(3,3:3,2)",dMNeu1LB(3,3:3,2)   
       write(16,*)"Neutralino couterterm matrix in mass basis P2, eps" 
       write(16,*)"dMNeu1LB(1,1:3,3)",dMNeu1LB(1,1:3,3)  
       write(16,*)"dMNeu1LB(2,2:3,3)",dMNeu1LB(2,2:3,3)    
       write(16,*)"dMNeu1LB(3,3:3,3)",dMNeu1LB(3,3:3,3)  
       write(16,*)"+++++++++++++++++"
       write(16,*)"Neutralino couterterm matrix in Interaction basis P3 =0" 

       write(16,*)"+++++++++++++++++"
       write(16,*)"Chagino couterterm matrix in mass basis P1, single pole, finite, eps" 
       write(16,*)"dMCha1L(1:3)",dMCha1L(1:3)  
       write(16,*)"Chagino couterterm matrix in mass basis P2, single pole, finite, eps" 
       write(16,*)"dMCha1LB(1:3)",dMCha1LB(1:3)  
       write(16,*)"Chagino couterterm matrix in mass basis P3=0" 

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ReAlambda P1, single pole,finite, eps'
       write(16,*)'dReAla(1:3)',dReAla(1:3) 
       write(16,*)'CTs of ReAlambda P2, single pole,finite, eps'
       write(16,*)'dReAlaB(1:3)',dReAlaB(1:3) 
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ImAlambda P1, single pole,finite, eps'
       write(16,*)'dImAla(1:3)',dImAla(1:3) 
       write(16,*)'CTs of ImAlambda P2, single pole,finite, eps'
       write(16,*)'dImAlaB(1:3)',dImAlaB(1:3) 
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ImAkappa P1, single pole,finite, eps'
       write(16,*)'dImAka(1:3)',dImAka(1:3)  
       write(16,*)'CTs of ImAkappa P2, single pole,finite, eps'
       write(16,*)'dImAkaB(1:3)',dImAkaB(1:3)  
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of Tlam P1, single pole,finite, eps'
       write(16,*)'dTlam(1:3)',dTlam(1:3) 
       write(16,*)'CTs of Tlam P2, single pole,finite, eps'
       write(16,*)'dTlamB(1:3)',dTlamB(1:3)  
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of Tkappa P1, single pole,finite, eps'
       write(16,*)'dTk(1:3)',dTk(1:3) 
       write(16,*)'CTs of Tlam P2, single pole,finite, eps'
       write(16,*)'dTkB(1:3)',dTkB(1:3) 

       write(16,*)"+++++++++++++++++"
       write(16,*)'dMT4 =',dmt4(1),dmt4(2),dmt4(3)       
       write(16,*)'dMSf4 11=',dMSf4(1,1,1),dMSf4(1,1,2),dMSf4(1,1,3)   
       write(16,*)'dMSf4 22=',dMSf4(2,2,1),dMSf4(2,2,2),dMSf4(2,2,3)   
       write(16,*)'dMSf4 12=',dMSf4(1,2,1),dMSf4(1,2,2),dMSf4(1,2,3)  
       write(16,*)'dMSf4 21=',dMSf4(2,1,1),dMSf4(2,1,2),dMSf4(2,1,3)  
       write(16,*)'dMSQ4(i)=',dMSQ4(1),dMSQ4(2),dMSQ4(3)
       write(16,*)'dAT4(i)=',dAT4
#endif


        do i=1,6
         do j=1,6
         dZHiggs4(i,j) = ZH(i,2)*ZH(j,2)*(dZHu4+Zfactor1LIN(2,2))
     &       + ZH(i,5)*ZH(j,5)*(dZHu4+Zfactor1LIN(5,5))  
     &       + (ZH(i,1)*ZH(j,1)+ZH(i,4)*ZH(j,4))*(Zfactor1LIN(1,1))  
     &       + (ZH(i,3)*ZH(j,3)+ZH(i,6)*ZH(j,6))*(Zfactor1LIN(3,3))          
         enddo
        enddo
         dZHiggs1(1:6,1:6,1)=  dZHiggs4(1:6,1:6) 
         dZHiggs1(1:6,1:6,2:3)= 0
 
         dMCha1L(1:3)= dMCha1L(1:3) +dMCha1LB(1:3)
         do i=1,3
            do j=1,3
              do k=1,3
         dMNeu1L(i,j,k)=dMNeu1L(i,j,k)+dMNeu1LB(i,j,k)
              enddo
            enddo
         enddo



         dMHp1MT4(1:3)=dMHp1MT4(1:3)+dMHp1MT4B(1:3)
         dMH1MT4(1:6,1:6,1:3)=dMH1MT4(1:6,1:6,1:3)+dMH1MT4B(1:6,1:6,1:3)
         dMCH1L(1:2,1:2,1:3)=dMCH1L(1:2,1:2,1:3)+dMCH1LB(1:2,1:2,1:3)

         dv4(1:3)= dv4(1:3)+dv4B(1:3)
         dSW4(1:3)=dSW4(1:3)+dSW4B(1:3)
	     dCW4(1:3)= -dSW4(1:3)*SW/CW

         dTanB4=dTanB4+dTanB4B
         dlambdaC4=dlambdaC4+dlambdaC4B+dlambdaC4S
         dabslambda14= dabslambda14+ dabslambda14B+ dabslambda14S
         dvS1=dvS1B+dvS1S
         dabskappa14=dabskappa14+dabskappa14B+dabskappa14S
         dReAkappa14 =dReAkappa14+dReAkappa14B+dReAkappa14S

         dMUE4B= la*dvS1B*expvS/sqrt2 + DCONJG(dlambdaC4B)*vS*expvS/sqrt2
         dMUE4=dMUE4+dMUE4B
      
         dTB14(1)   =dTanB4; dTB14(2:3)=0D0
         dabsla14(1)=dabslambda14; dabsla14(2:3)=0D0
         dabska14(1)=dabskappa14;  dabska14(2:3)=0
         dvs14(1)   = dvs1; dvs14(2:3) =0D0
         dReAka14(1)= dReAkappa14; dReAka14(2:3) =0D0
         dMUE14(1)  = dMUE4;   dMUE14(2:3)= 0D0
         dReAla(1:3)=dReAla(1:3)+dReAlaB(1:3)
         dImAla(1:3)=dImAla(1:3)+dImAlaB(1:3)
         dImAka(1:3)=dImAka(1:3)+dImAkaB(1:3)
         dTlam(1:3) =dTlam(1:3)+dTlamB(1:3)
         dTk(1:3)   = dTk(1:3) + dTkB(1:3)



         do i=1,6
         dTad1MT4(i,1) =   dTad1MT4(i,1) +dTad1MT4B(i,1)
         dTad1MT4(i,2) =   dTad1MT4(i,2) +dTad1MT4B(i,2) 
         dTad1MT4(i,3) =   dTad1MT4(i,3) +dTad1MT4B(i,3) 
         enddo

       dAT4(1)=(-(dTanB4*M1)*(DCONJG(MUE))+M1*TB*DCONJG(dMUE4)
     &  -ATT*expETA*TB**2*dMT4(1)+(TB*DCONJG(MUE)*dMT4(1))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(2,2))*
     &  dMSf4(2,1,1)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*dMSf4(1,2,1)*ZU(2,1)
     &  +TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,1)*ZU(2,1))/(expETA*M1*TB**2)

       dat4fin=dAT4(2)
       dmsq4fin=dMSQ4(2)
       dmsur4fin=-2*M1*dmt4(2) + dMsf4(1,1,2)*ZU(1,2)*ZUC(1,2) + 
     &  dMsf4(1,2,2)*ZU(2,2)*ZUC(1,2) + dMsf4(2,1,2)*ZU(1,2)*ZUC(2,2) +
     &  dMsf4(2,2,2)*ZU(2,2)*ZUC(2,2)

       if(ischeme.le.3)then
         dMCha1L(3)=0D0
         dMNeu1L(1:3,1:3,3)=0D0
         dMHp1MT4(3)=0D0
         dMH1MT4(1:6,1:6,3)=0D0
         dMCH1L(1:2,1:2,3)=0D0
         dv4(3)=0D0
         dSW4(3)=0D0
         dCW4(3)=0D0
         dReAla(3)=0D0
         dImAla(3)=0D0
         dImAka(3)=0D0
         dTlam(3)=0D0
         dTk(3)=0D0
         dTad1MT4(1:6,3)=0D0
         dAT4(3)=0D0
         dmt4(3)=0D0
         dMSQ4(3)=0D0
         dMSf4(1:2,1:2,3)=0D0
        if(ischeme.le.2)then
           dAT4(2)=0D0
           dmt4(2)=0D0
           dMSQ4(2)=0D0
           dMSf4(1:2,1:2,2)=0D0
        endif
      endif

      end subroutine CT1loop_atlambda
c*
c**********************************************************************
c*

      subroutine CT1L_TB(dmsq4fin,dmsur4fin,dat4fin)
      implicit none
      integer i,I2G2,I5G2,I6G2,I2G3,I2G4,I3G3,I5G3,I5G4,I6G4,I6G3
      double precision A0T,AEpT,B0T,BEpT,fac,
     .                 selfHH1MT4(6,6),dmsq4fin,dmsur4fin
      double complex B0P,B1P,BEpP,B1EpP,dat4fin

#include "TLMT4.h"
#include "TLMT4lambda.h"
      fac= 1/16D0/pi**2

      A0T1=A0T(M12,Q2)
      A0T2=A0T(M22,Q2)
      A0T4=A0T(M42,Q2)
      aA0TN(1)=A0T2
      aA0TN(2)=A0T4
      A0T5= A0T(MSQ2,Q2)
      do i=ihiggs,ihiggse
        A0TNH(i) =A0T(Mah2(i),Q2)
      enddo  
      do i=ineu,ineue
      A0TNeu(i)= A0T(MNeu2(i),Q2)
      enddo
      a0tcha=A0T(MCha2,Q2)
c CTs for top/stop sector

       dmt4(2)=-(CF12F3S*CF12F3SC*M1*DBLE(B1P(M12,MCha2,MSQ2,Q2)))/2d0

       dMSf4(1,1,2)=(a0t5*CS14S13S14S13MT4(1,1))
       dMSf4(1,1,2)=CF12F4S(1)*CF12F4SC(1)*(-a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2)))+dMSf4(1,1,2)

       dMSf4(1,2,2)=(CF12F4S(1)*CF12F4SC(2)*(-2*a0tcha+(M22-MCha2)*
     &  DBLE(B0P(M22,0D0,MCha2,Q2))+(M42-MCha2)*DBLE(B0P(M42,0D0,MCha2,Q2))
     &  ))/2.
       dMSf4(1,2,2)=(a0t5*CS14S13S14S13MT4(2,1))+dMSf4(1,2,2)

       dMSf4(2,2,2)=CF12F4S(2)*CF12F4SC(2)*(-a0tcha+(M42-MCha2)*
     &  DBLE(B0P(M42,0D0,MCha2,Q2)))
       dMSf4(2,2,2)=(a0t5*CS14S13S14S13MT4(2,2))+dMSf4(2,2,2)

       Do  I2G3= 1,2
       dmt4(2)=-(M1*DBLE(B1P(M12,0D0,MassHpm(I2G3)**2,Q2))*CTBS3(I2G3)*
     &  CTBS3C(I2G3))/2d0+dmt4(2)
       dMSf4(1,1,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,1,1)+dMSf4(1,1,2)
       dMSf4(1,1,2)=CS3S13S(I2G3,1)*CS3S13SC(I2G3,1)*
     &  DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CS3S13S(I2G3,2)*CS3S13SC(I2G3,1)*
     &  (DBLE(B0P(M22,MSQ2,MassHpm(I2G3)**2,Q2))+
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))))/2.+dMSf4(1,2,2)
       dMSf4(1,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,1)
     &              +dMSf4(1,2,2)
       dMSf4(2,2,2)=CS3S13S(I2G3,2)*CS3S13SC(I2G3,2)*
     &  DBLE(B0P(M42,MSQ2,MassHpm(I2G3)**2,Q2))+dMSf4(2,2,2)
       dMSf4(2,2,2)=A0T(MassHpm(I2G3)**2,Q2)*CS3S3S13S13(I2G3,I2G3,2,2)
     &  +dMSf4(2,2,2)
       dMSf4(1,1,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(1,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(1,I2G3,I2G3,1)))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,1,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,1)))+dMSf4(1,2,2)
       dMSf4(2,2,2)=(aA0TN(I2G3)*(CS13S13S13S13MT4(2,I2G3,2,I2G3)+3*
     &  CS13S13S13S13MT4(2,I2G3,I2G3,2)))+dMSf4(2,2,2)
       enddo

       Do  I6G3= ihiggs,ihiggse
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,M12,Mah2(I6G3),Q2))*hy(I6G3)*
     &  hyc(I6G3))+(M1*DBLE(B0P(M12,M12,Mah2(I6G3),Q2))*(hy(I6G3)**2+
     &  hyc(I6G3)**2))/2.)
       dMSf4(1,1,2)=((A0TNH(I6G3)*CSSSS(I6G3,I6G3,1,1))/2.)+dMSf4(1,1,2)
       dMSf4(1,2,2)=A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,1)/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=(A0TNH(I6G3)*CSSSS(I6G3,I6G3,2,2))/2.+dMSf4(2,2,2)
       enddo


       Do  I6G3= ihiggs,ihiggse
       Do  I6G4= 1,2
       dMSf4(1,1,2)=CSSS(I6G3,1,I6G4)*CSSS(I6G3,I6G4,1)*
     &  DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(1,1,2)
       dMSf4(1,2,2)=(CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,1)*
     &  (DBLE(B0P(M22,Mah2(I6G3),MassSu2(I6G4),Q2))+
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))))/2.+dMSf4(1,2,2)
       dMSf4(2,2,2)=CSSS(I6G3,2,I6G4)*CSSS(I6G3,I6G4,2)*
     &  DBLE(B0P(M42,Mah2(I6G3),MassSu2(I6G4),Q2))+dMSf4(2,2,2)
       enddo
       enddo


       Do  I5G3= ineu,ineue
       Do  I6G3= 1,2
       dmt4(2)=dmt4(2)+(-(M1*DBLE(B1P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSL(I5G3,I6G3)*CF11FSLC(I5G3,I6G3)+CF11FSR(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3)))+DBLE(B0P(M12,MNeu2(I5G3),MassSu2(I6G3),Q2))*
     &  (CF11FSLC(I5G3,I6G3)*CF11FSR(I5G3,I6G3)+CF11FSL(I5G3,I6G3)*
     &  CF11FSRC(I5G3,I6G3))*MNeu(I5G3))/2d0
       enddo
       enddo

       Do  I5G3= ineu,ineue
       dMSf4(1,1,2)=dMSf4(1,1,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,1))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,1)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,1)-DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,1)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,1)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,1)*MNeu(I5G3)+CF11FSRC(I5G3,1)*(M12-M22+
     &  MNeu2(I5G3)))))
       dMSf4(1,2,2)=dMSf4(1,2,2)+(-(a0t1*CF11FSL(I5G3,1)*
     &  CF11FSLC(I5G3,2))-A0TNeu(I5G3)*CF11FSL(I5G3,1)*CF11FSLC(I5G3,2)
     &  -a0t1*CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-A0TNeu(I5G3)*
     &  CF11FSR(I5G3,1)*CF11FSRC(I5G3,2)-(DBLE(B0P(M22,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M22+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M22+
     &  MNeu2(I5G3)))))/2.-(DBLE(B0P(M42,M12,MNeu2(I5G3),Q2))*
     &  (CF11FSL(I5G3,1)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,1)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))/2.)
       dMSf4(2,2,2)=dMSf4(2,2,2)+(-(a0t1*(CF11FSL(I5G3,2)*
     &  CF11FSLC(I5G3,2)+CF11FSR(I5G3,2)*CF11FSRC(I5G3,2)))-
     &  A0TNeu(I5G3)*(CF11FSL(I5G3,2)*CF11FSLC(I5G3,2)+
     &  CF11FSR(I5G3,2)*CF11FSRC(I5G3,2))-DBLE(B0P(M42,M12,MNeu2(I5G3)
     &  ,Q2))*(CF11FSL(I5G3,2)*(2*M1*CF11FSRC(I5G3,2)*MNeu(I5G3)+
     &  CF11FSLC(I5G3,2)*(M12-M42+MNeu2(I5G3)))+CF11FSR(I5G3,2)*(2*M1*
     &  CF11FSLC(I5G3,2)*MNeu(I5G3)+CF11FSRC(I5G3,2)*(M12-M42+
     &  MNeu2(I5G3)))))
       enddo


       dmt4(2)=dmt4(2)* fac 


       dMSf4(2,1,2)=DCONJG(dMSf4(1,2,2))
       dMSf4(1:2,1:2,2)=dMSf4(1:2,1:2,2)*fac

       i=2
        dMSQ4(i)=-2*M1*dMT4(i)+DCONJG(ZU(1,1))*dMSf4(1,1,i)*ZU(1,1)+
     &  DCONJG(ZU(2,1))*dMSf4(2,1,i)*ZU(1,1)+DCONJG(ZU(1,1))*
     &  dMSf4(1,2,i)*ZU(2,1)+DCONJG(ZU(2,1))*dMSf4(2,2,i)*ZU(2,1)
   
       dAT4(i)=(-(ATT*expETA*TB**2*dMT4(i))+(TB*DCONJG(MUE)*dMT4(i))
     &  +TB**2*DCONJG(ZU(1,2))*dMSf4(1,1,i)*ZU(1,1)+TB*
     &  *2*DCONJG(ZU(2,2))*dMSf4(2,1,i)*ZU(1,1)+TB**2*DCONJG(ZU(1,2))*
     &  dMSf4(1,2,i)*ZU(2,1)+TB**2*DCONJG(ZU(2,2))*dMSf4(2,2,i)*ZU(2,1)
     &  )/(expETA*M1*TB**2)


       dat4fin=dAT4(2)
       dmsq4fin=dMSQ4(2)
       dmsur4fin=-2*M1*dmt4(2) + dMsf4(1,1,2)*ZU(1,2)*ZUC(1,2) + 
     &  dMsf4(1,2,2)*ZU(2,2)*ZUC(1,2) + dMsf4(2,1,2)*ZU(1,2)*ZUC(2,2) +
     &  dMsf4(2,2,2)*ZU(2,2)*ZUC(2,2)
      end 
c*
c**********************************************************************
c*
      subroutine CTCLs_atlambda()
      implicit none
      integer ii,i,j,k,l,i1,i2,i3,I6G3,I6G4,I6G5,I6G2,gen1,gen2,
     .        gen3,gen4,ik,facZ
      common/ZfacInclude/facZ
      double precision pi2,fac
      double precision logM1,ZetaN2,diff1,diff2,diff3,CHselfCT,CHselfCTDP,CHselfCTSP
      double complex selfCT(6,6),selfCTDP(6,6),selfCTSP(6,6),
     .               TadCT(6),TadCTDP(6),TadCTSP(6)
      character*4 char1(6),char2(6)

      integer modselval(10)
      character spinfo1*100,spinfo2*100
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval


#include "TLMT4.h"
#include "TLMT4lambda.h"
         facZ=1
         pi2=pi**2
         fac= 1/16D0/pi2

       dZCHiggs4(1,1) =SB**2*(dZHu4+Zfactor1LIN(2,2))+CB**2*Zfactor1LIN(1,1)
       dZCHiggs4(1,2) =SB*CB*(-Zfactor1LIN(1,1)+dZHu4+Zfactor1LIN(2,2))
       dZCHiggs4(2,1) =dZCHiggs4(1,2)
       dZCHiggs4(2,2) =CB**2*(dZHu4+Zfactor1LIN(2,2))+SB**2*Zfactor1LIN(1,1)
       dZCHiggs1(1:2,1:2,1)=  dZCHiggs4(1:2,1:2)

       dS3TB(1)=(CB**4*dTanB4*M1*sqrt2)/(Sqrt(expETA)*SB**2*vv)-(CB*
     &  sqrt2*dMT4(1))/(Sqrt(expETA)*SB*vv)+(CB*M1*sqrt2*dv4(1))
     &  /(Sqrt(expETA)*SB*vv2) + CTBS3(2)*dZCHiggs4(2,2)/2D0+
     & CTBS3(1)*dZCHiggs4(2,1)/2D0

       dS3TB(2)=-((CB*sqrt2*dMT4(2))/(Sqrt(expETA)*SB*vv))+(CB*M1*
     &  sqrt2*dv4(2))/(Sqrt(expETA)*SB*vv2)
       dS3TB(3)=-((CB*sqrt2*dMT4(3))/(Sqrt(expETA)*SB*vv))+(CB*M1*
     &  sqrt2*dv4(3))/(Sqrt(expETA)*SB*vv2)

       dCS3SSS14(1)=(-4*CB**5*dTanB4*M12)/(SB**3*vv2)+(4*CB**2*M1*
     &  dMT4(1))/(SB**2*vv2)-(4*CB**2*M12*dv4(1))/(SB**2*vv**3)+
     &  CS3S3S14S14(2,2)*dZCHiggs4(2,2)+CS3S3S14S14(2,1)*dZCHiggs4(2,1)

       dCS3SSS14(2)=(4*CB**2*M1*dMT4(2))/(SB**2*vv2)-(4*CB**2*M12*
     &  dv4(2))/(SB**2*vv**3)
       dCS3SSS14(3)=(4*CB**2*M1*dMT4(3))/(SB**2*vv2)-(4*CB**2*M12*
     &  dv4(3))/(SB**2*vv**3)

         do j=1,2
       dCS3S13S14(j,1)=-((dMUE4*M1*sqrt2*ZUC(j,2))/vv)-(CB*M1*sqrt2*
     &  DCONJG(dAT4(1))*ZUC(j,2))/(expETA*SB*vv)+(CB**3*dTanB4*M1*(CB*
     &  expETA*M1*sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*
     &  sqrt2*ZUC(j,2)))/(expETA*SB**2*vv)+(M1*dv4(1)*(CB*expETA*M1*
     &  sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*
     &  ZUC(j,2)))/(expETA*SB*vv2)+dMT4(1)*(-((CB*M1*sqrt2*ZUC(j,1))
     &  /(SB*vv))-(CB*expETA*M1*sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+
     &  expETA*MUE*SB*sqrt2*ZUC(j,2))/(expETA*SB*vv))+
     &  CS3S13S14(j)*dZCHiggs4(2,2)/2D0+ CS3S13S(1,j)*dZChiggs4(1,2)/2D0

            do i=2,3
       dCS3S13S14(j,i)=-((CB*M1*sqrt2*DCONJG(dAT4(i))*ZUC(j,2))
     &  /(expETA*SB*vv))+(M1*dv4(i)*(CB*expETA*M1*sqrt2*ZUC(j,1)+ATC*
     &  CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*ZUC(j,2)))/(expETA*SB*
     &  vv2)+dMT4(i)*(-((CB*M1*sqrt2*ZUC(j,1))/(SB*vv))-(CB*expETA*M1*
     &  sqrt2*ZUC(j,1)+ATC*CB*sqrt2*ZUC(j,2)+expETA*MUE*SB*sqrt2*
     &  ZUC(j,2))/(expETA*SB*vv))
            enddo

        dCS3S14S13(j,1:3)=DCONJG(dCS3S13S14(j,1:3))


           do k=1,2
       dCS3S13S(j,k,1)=(-4*CB**5*dTanB4*M12*ZU(k,2)*ZUC(j,2))/(SB**3*
     &  vv2)+(4*CB**2*M1*dMT4(1)*ZU(k,2)*ZUC(j,2))/(SB**2*vv2)-(4*CB**
     &  2*M12*dv4(1)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
     &  +CS3S3S13S13(2,2,j,k)*dZChiggs4(2,2)+CS3S3S13S13(2,1,j,k)*dZChiggs4(2,1)
  

       dCS3S13S(j,k,2)=(4*CB**2*M1*dMT4(2)*ZU(k,2)*ZUC(j,2))/(SB**2*
     &  vv2)-(4*CB**2*M12*dv4(2)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
       dCS3S13S(j,k,3)=(4*CB**2*M1*dMT4(3)*ZU(k,2)*ZUC(j,2))/(SB**2*
     &  vv2)-(4*CB**2*M12*dv4(3)*ZU(k,2)*ZUC(j,2))/(SB**2*vv**3)
       dCS3S13SC(j,k,1:3)=      DCONJG( dCS3S13S(j,k,1:3))
           
            do l=1,6
              i=1
       dCSSS(l,k,j,i)=DCONJG(dMUE4)*(-((M1*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(SB*vv))+
     &  (IC*M1*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv))+dAT4(i)*((expETA*M1*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(expETA*IC*M1*ZH(l,5)*ZU(j,2)
     &  *ZUC(k,1))/(SB*vv))+dlambdaC4*(-((CB*M1*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2))+(CB*IC*M1*ZH(l,6)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2))+dMUE4*(-((M1*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  vv))-(IC*M1*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv))+DCONJG(dAT4(i))*
     &  ((M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-(IC*M1*ZH(l,5)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB*vv))+DCONJG(dlambdaC4)*(-((CB*
     &  expvS*M1*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))-(CB*expvS*IC*M1*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))-CB**2*dTanB4*SB*(-((M1*
     &  DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2))+(IC*M1*
     &  DCONJG(la)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)-(expvS*
     &  la*M1*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(expvS*IC*la*M1*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2))+CB**3*dTanB4*((-2*M12*
     &  ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv)+(M1*MUEC*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(SB**2*vv)-(ATT*expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB**2*vv)+(CB*M1*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB**2*sqrt2)-(IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB**2*vv)-
     &  (ATT*expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB**2*vv)-(CB*IC*
     &  M1*DCONJG(la)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2)+
     &  (M1*MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*vv)-(ATC*M1*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB**2*vv)+(CB*expvS*la*M1*ZH(l,3)*
     &  ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB**2*vv)+(ATC*IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))
     &  /(expETA*SB**2*vv)+(CB*expvS*IC*la*M1*ZH(l,6)*ZU(j,1)*ZUC(k,2))
     &  /(SB**2*sqrt2)-(2*M12*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv))+
     &  dMT4(i)*((4*M1*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv)-(MUEC*ZH(l,1)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv)-(CB*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*
     &  sqrt2)+(IC*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*
     &  IC*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(CB*IC*DCONJG(la)*ZH(l,6)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)-(MUE*ZH(l,1)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv)+(ATC*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)
     &  -(CB*expvS*la*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(IC*MUE*
     &  ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv)-(ATC*IC*ZH(l,5)*ZU(j,1)*
     &  ZUC(k,2))/(expETA*SB*vv)-(CB*expvS*IC*la*ZH(l,6)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2)+(4*M1*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv))+
     &  dv4(i)*((-2*M12*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv2)+(M1*MUEC*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-(ATT*expETA*M1*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv2)-(IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1)
     &  )/(SB*vv2)-(ATT*expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv2)
     &  +(M1*MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*vv2)-(ATC*M1*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,2))/(expETA*SB*vv2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv2)+(ATC*IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))
     &  /(expETA*SB*vv2)-(2*M12*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv2))
            do i=1,6
               dCSSS(l,k,j,1)=dCSSS(l,k,j,1)+dZHiggs4(l,i)*CSSS(i,k,j)/2D0
            enddo            

            do i=2,3
       dCSSS(l,k,j,i)=dAT4(i)*((expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv)+(expETA*IC*M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv))+
     &  DCONJG(dAT4(i))*((M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-
     &  (IC*M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv))+dMT4(i)*((4*
     &  M1*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB*vv)-(MUEC*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(SB*vv)+(ATT*expETA*ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv)
     &  -(CB*DCONJG(la)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2)+(IC*
     &  MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv)+(ATT*expETA*IC*ZH(l,5)*
     &  ZU(j,2)*ZUC(k,1))/(SB*vv)+(CB*IC*DCONJG(la)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2)-(MUE*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  vv)+(ATC*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv)-(CB*expvS*la*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)-(IC*MUE*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv)-(ATC*IC*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*
     &  vv)-(CB*expvS*IC*la*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2)+(4*M1*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB*vv))+dv4(i)*((-2*M12*ZH(l,2)*
     &  ZU(j,1)*ZUC(k,1))/(SB*vv2)+(M1*MUEC*ZH(l,1)*ZU(j,2)*ZUC(k,1))
     &  /(SB*vv2)-(ATT*expETA*M1*ZH(l,2)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-
     &  (IC*M1*MUEC*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(SB*vv2)-(ATT*expETA*IC*
     &  M1*ZH(l,5)*ZU(j,2)*ZUC(k,1))/(SB*vv2)+(M1*MUE*ZH(l,1)*ZU(j,1)*
     &  ZUC(k,2))/(SB*vv2)-(ATC*M1*ZH(l,2)*ZU(j,1)*ZUC(k,2))/(expETA*
     &  SB*vv2)+(IC*M1*MUE*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*vv2)+(ATC*IC*
     &  M1*ZH(l,5)*ZU(j,1)*ZUC(k,2))/(expETA*SB*vv2)-(2*M12*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,2))/(SB*vv2))

            enddo
            do i1=1,6

       dCSSSS(l,i1,k,j,1)=dlambdaC4*IC*((IC*M1*ZH(i1,3)*ZH(l,1)*ZU(j,2)
     &  *ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,6)*ZH(l,1)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(IC*M1*ZH(i1,1)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,4)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,3)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*M1*ZH(i1,6)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(M1*ZH(i1,1)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*M1*ZH(i1,4)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv))+IC*DCONJG(dlambdaC4)*((expvS*IC*
     &  M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*M1*
     &  ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*M1*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*M1*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*M1*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv))+IC*dv4(1)*
     &  ((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv**3)+(4*
     &  IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv**3)-(IC*M1*
     &  DCONJG(la)*ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)-(IC*M1*DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,4)*ZH(l,3)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,3)*
     &  ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)+(IC*M1*
     &  DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,1)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)+(IC*M1*DCONJG(la)*ZH(i1,4)*ZH(l,6)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,3)*ZH(l,1)
     &  *ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,6)*ZH(l,1)
     &  *ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,1)*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,4)*
     &  ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,3)*
     &  ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*M1*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*
     &  M1*ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(4*IC*M12*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv**3)+(4*IC*M12*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv**3))+CB**3*dTanB4*
     &  IC*((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**3*vv2)+
     &  (4*IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**3*vv2)-(IC*
     &  M1*DCONJG(la)*ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*
     &  sqrt2*vv)-(M1*DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB**2*sqrt2*vv)-(IC*M1*DCONJG(la)*ZH(i1,1)*ZH(l,3)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(M1*DCONJG(la)*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(M1*
     &  DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*
     &  sqrt2*vv)+(IC*M1*DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB**2*sqrt2*vv)-(M1*DCONJG(la)*ZH(i1,1)*ZH(l,6)*ZU(j,2)
     &  *ZUC(k,1))/(expvS*SB**2*sqrt2*vv)+(IC*M1*DCONJG(la)*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB**2*sqrt2*vv)-(expvS*IC*la*
     &  M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*
     &  la*M1*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)-
     &  (expvS*IC*la*M1*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB**2*
     &  sqrt2*vv)+(expvS*la*M1*ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB**
     &  2*sqrt2*vv)+(expvS*la*M1*ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))
     &  /(SB**2*sqrt2*vv)+(expvS*IC*la*M1*ZH(i1,6)*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*la*M1*ZH(i1,1)*ZH(l,6)*
     &  ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(expvS*IC*la*M1*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB**2*sqrt2*vv)+(4*IC*M12*ZH(i1,2)*
     &  ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**3*vv2)+(4*IC*M12*ZH(i1,5)*
     &  ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**3*vv2))+IC*dMT4(1)*((-4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)+(IC*DCONJG(la)*
     &  ZH(i1,3)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+
     &  (DCONJG(la)*ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv)+(IC*DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,6)*ZH(l,4)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,1)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(expvS*
     &  IC*la*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*
     &  la*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*
     &  la*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2))

           do i=1,6
       dCSSSS(l,i1,k,j,1)=dCSSSS(l,i1,k,j,1)+dZHiggs4(l,i)*CSSSS(i,i1,k,j)/2D0
     &         +dZHiggs4(i1,i)*CSSSS(l,i,k,j)/2D0 
           enddo       

           do i=2,3
       dCSSSS(l,i1,k,j,i)=IC*dv4(i)*((4*IC*M12*ZH(i1,2)*ZH(l,2)*ZU(j,1)
     &  *ZUC(k,1))/(SB**2*vv**3)+(4*IC*M12*ZH(i1,5)*ZH(l,5)*ZU(j,1)*
     &  ZUC(k,1))/(SB**2*vv**3)-(IC*M1*DCONJG(la)*ZH(i1,3)*ZH(l,1)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,6)*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(IC*M1*
     &  DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(M1*DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*
     &  SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv2)+(IC*M1*DCONJG(la)*ZH(i1,6)*ZH(l,4)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)-(M1*DCONJG(la)*ZH(i1,1)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv2)+(IC*M1*
     &  DCONJG(la)*ZH(i1,4)*ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv2)-(expvS*IC*la*M1*ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  sqrt2*vv2)+(expvS*la*M1*ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*
     &  sqrt2*vv2)-(expvS*IC*la*M1*ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))
     &  /(SB*sqrt2*vv2)+(expvS*IC*la*M1*ZH(i1,6)*ZH(l,4)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*la*M1*ZH(i1,1)*ZH(l,6)*ZU(j,1)*
     &  ZUC(k,2))/(SB*sqrt2*vv2)+(expvS*IC*la*M1*ZH(i1,4)*ZH(l,6)*
     &  ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv2)+(4*IC*M12*ZH(i1,2)*ZH(l,2)*
     &  ZU(j,2)*ZUC(k,2))/(SB**2*vv**3)+(4*IC*M12*ZH(i1,5)*ZH(l,5)*
     &  ZU(j,2)*ZUC(k,2))/(SB**2*vv**3))+IC*dMT4(i)*((-4*IC*M1*ZH(i1,2)
     &  *ZH(l,2)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)-(4*IC*M1*ZH(i1,5)*
     &  ZH(l,5)*ZU(j,1)*ZUC(k,1))/(SB**2*vv2)+(IC*DCONJG(la)*ZH(i1,3)*
     &  ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(IC*
     &  DCONJG(la)*ZH(i1,1)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*
     &  vv)+(DCONJG(la)*ZH(i1,4)*ZH(l,3)*ZU(j,2)*ZUC(k,1))/(expvS*SB*
     &  sqrt2*vv)+(DCONJG(la)*ZH(i1,3)*ZH(l,4)*ZU(j,2)*ZUC(k,1))
     &  /(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,6)*ZH(l,4)*ZU(j,2)*
     &  ZUC(k,1))/(expvS*SB*sqrt2*vv)+(DCONJG(la)*ZH(i1,1)*ZH(l,6)*
     &  ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)-(IC*DCONJG(la)*ZH(i1,4)*
     &  ZH(l,6)*ZU(j,2)*ZUC(k,1))/(expvS*SB*sqrt2*vv)+(expvS*IC*la*
     &  ZH(i1,3)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,6)*ZH(l,1)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)+(expvS*IC*la*
     &  ZH(i1,1)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,4)*ZH(l,3)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,3)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,6)*ZH(l,4)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*la*
     &  ZH(i1,1)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(expvS*IC*la*
     &  ZH(i1,4)*ZH(l,6)*ZU(j,1)*ZUC(k,2))/(SB*sqrt2*vv)-(4*IC*M1*
     &  ZH(i1,2)*ZH(l,2)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2)-(4*IC*M1*
     &  ZH(i1,5)*ZH(l,5)*ZU(j,2)*ZUC(k,2))/(SB**2*vv2))
            enddo
            enddo
           enddo
         enddo
        enddo

 
       do l=1,6
       dhy(l,1)=-((CB**3*dTanB4*M1*(ZH(l,2)+IC*ZH(l,5)))/(SB**2*vv))+
     &  (dMT4(1)*(ZH(l,2)+IC*ZH(l,5)))/(SB*vv)-(M1*dv4(1)*(ZH(l,2)+IC*
     &  ZH(l,5)))/(SB*vv2) 
       do i=1,6
       dhy(l,1)= dhy(l,1)+dZHiggs4(l,i)* hy(i)/2D0
       enddo
     
       dhyc(l,1)=DCONJG(dhy(l,1))
       do i=2,3
       dhy(l,i)=(dMT4(i)*(ZH(l,2)+IC*ZH(l,5)))/(SB*vv)-(M1*dv4(i)*
     &  (ZH(l,2)+IC*ZH(l,5)))/(SB*vv2)
       dhyc(l,i)=DCONJG(dhy(l,i))
       enddo
       enddo

c new couplings
      do i=1,3
       CS1S1S1IN(1:6,1:6,1:6)=0D0

       CS1S1S1IN(1,1,1)=0
       CS1S1S1IN(1,1,2)=2*abslambda*SB*vv*dabsla14(i)+abslambda**2*CB**
     &  3*vv*dTB14(i)+abslambda**2*SB*dv4(i)
       CS1S1S1IN(1,1,3)=2*abslambda*vS*dabsla14(i)+abslambda**2*dvS14(i)
       CS1S1S1IN(1,1,4)=0
       CS1S1S1IN(1,1,5)=0
       CS1S1S1IN(1,1,6)=0
       CS1S1S1IN(1,2,2)=2*abslambda*CB*vv*dabsla14(i)-abslambda**2*CB**
     &  2*SB*vv*dTB14(i)+abslambda**2*CB*dv4(i)
       CS1S1S1IN(1,2,3)=(((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(((-2*abskappa*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)-DCONJG(dTlam(i))/(2.*expETA*expvS*sqrt2)-
     &  (expETA*expvS*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*abslambda*
     &  expETA**2*explam*expvS)/expkap-(2*abskappa*abslambda*expkap*
     &  expvS**5)/explam)*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,2,4)=0
       CS1S1S1IN(1,2,5)=0
       CS1S1S1IN(1,2,6)=(Ic*((2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(Ic*((2*abskappa*expETA**2*explam*expvS*
     &  vS)/expkap-(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+(Ic*DCONJG(dTlam(i)))/(2.*expETA*expvS*
     &  sqrt2)-(expETA*expvS*Ic*dTlam(i))/(2.*sqrt2)+(((2*abskappa*
     &  abslambda*expETA**2*explam*expvS)/expkap-(2*abskappa*abslambda*
     &  expkap*expvS**5)/explam)*Ic*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,3,3)=(((-2*abslambda*expETA**2*explam*expvS*SB*vv)
     &  /expkap-(2*abslambda*expkap*expvS**5*SB*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+((8*abslambda*CB*expETA*expvS**3*vv-(2*
     &  abskappa*expETA**2*explam*expvS*SB*vv)/expkap-(2*abskappa*
     &  expkap*expvS**5*SB*vv)/explam)*dabsla14(i))/(4.*expETA*expvS**
     &  3)-abslambda**2*CB**2*SB*vv*dTB14(i)+(CB**3*((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+((4*abslambda**2*CB*expETA*expvS**3-(2*abskappa*
     &  abslambda*expETA**2*explam*expvS*SB)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*SB)/explam)*dv4(i))/(4.*expETA*expvS*
     &  *3)
       CS1S1S1IN(1,3,4)=0
       CS1S1S1IN(1,3,5)=(Ic*((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap+(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(Ic*((-2*abskappa*expETA**2*explam*expvS*
     &  vS)/expkap+(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+(Ic*DCONJG(dTlam(i)))/(2.*expETA*expvS*
     &  sqrt2)-(expETA*expvS*Ic*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS)/expkap+(2*abskappa*abslambda*
     &  expkap*expvS**5)/explam)*Ic*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,3,6)=(Ic*((2*abslambda*expETA**2*explam*expvS*SB*vv)
     &  /expkap-(2*abslambda*expkap*expvS**5*SB*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(Ic*((2*abskappa*expETA**2*explam*expvS*
     &  SB*vv)/expkap-(2*abskappa*expkap*expvS**5*SB*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)+(CB**3*Ic*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(Ic*((2*abskappa*abslambda*expETA**2*explam*expvS*SB)
     &  /expkap-(2*abskappa*abslambda*expkap*expvS**5*SB)/explam)*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,4,4)=0
       CS1S1S1IN(1,4,5)=0
       CS1S1S1IN(1,4,6)=0
       CS1S1S1IN(1,5,5)=2*abslambda*CB*vv*dabsla14(i)-abslambda**2*CB**
     &  2*SB*vv*dTB14(i)+abslambda**2*CB*dv4(i)
       CS1S1S1IN(1,5,6)=(((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(((-2*abskappa*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+DCONJG(dTlam(i))/(2.*expETA*expvS*sqrt2)+
     &  (expETA*expvS*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*abslambda*
     &  expETA**2*explam*expvS)/expkap-(2*abskappa*abslambda*expkap*
     &  expvS**5)/explam)*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(1,6,6)=(((2*abslambda*expETA**2*explam*expvS*SB*vv)
     &  /expkap+(2*abslambda*expkap*expvS**5*SB*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+((8*abslambda*CB*expETA*expvS**3*vv+(2*
     &  abskappa*expETA**2*explam*expvS*SB*vv)/expkap+(2*abskappa*
     &  expkap*expvS**5*SB*vv)/explam)*dabsla14(i))/(4.*expETA*expvS**
     &  3)-abslambda**2*CB**2*SB*vv*dTB14(i)+(CB**3*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap+(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+((4*abslambda**2*CB*expETA*expvS**3+(2*abskappa*
     &  abslambda*expETA**2*explam*expvS*SB)/expkap+(2*abskappa*
     &  abslambda*expkap*expvS**5*SB)/explam)*dv4(i))/(4.*expETA*expvS*
     &  *3)
       CS1S1S1IN(2,2,2)=0
       CS1S1S1IN(2,2,3)=2*abslambda*vS*dabsla14(i)+abslambda**2*dvS14(i)
       CS1S1S1IN(2,2,4)=0
       CS1S1S1IN(2,2,5)=0
       CS1S1S1IN(2,2,6)=0
       CS1S1S1IN(2,3,3)=(((-2*abslambda*CB*expETA**2*explam*expvS*vv)
     &  /expkap-(2*abslambda*CB*expkap*expvS**5*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(((-2*abskappa*CB*expETA**2*explam*
     &  expvS*vv)/expkap-(2*abskappa*CB*expkap*expvS**5*vv)/explam+8*
     &  abslambda*expETA*expvS**3*SB*vv)*dabsla14(i))/(4.*expETA*expvS*
     &  *3)+abslambda**2*CB**3*vv*dTB14(i)-(CB**2*SB*((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((-2*abskappa*abslambda*CB*expETA**2*explam*expvS)
     &  /expkap-(2*abskappa*abslambda*CB*expkap*expvS**5)/explam+4*
     &  abslambda**2*expETA*expvS**3*SB)*dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,3,4)=(Ic*((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap+(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(Ic*((-2*abskappa*expETA**2*explam*expvS*
     &  vS)/expkap+(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+(Ic*DCONJG(dTlam(i)))/(2.*expETA*expvS*
     &  sqrt2)-(expETA*expvS*Ic*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS)/expkap+(2*abskappa*abslambda*
     &  expkap*expvS**5)/explam)*Ic*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,3,5)=0
       CS1S1S1IN(2,3,6)=(Ic*((2*abslambda*CB*expETA**2*explam*expvS*vv)
     &  /expkap-(2*abslambda*CB*expkap*expvS**5*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(Ic*((2*abskappa*CB*expETA**2*explam*
     &  expvS*vv)/expkap-(2*abskappa*CB*expkap*expvS**5*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)-(CB**2*Ic*SB*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((2*abskappa*abslambda*CB*expETA**2*explam*expvS)
     &  /expkap-(2*abskappa*abslambda*CB*expkap*expvS**5)/explam)*Ic*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,4,4)=2*abslambda*SB*vv*dabsla14(i)+abslambda**2*CB**
     &  3*vv*dTB14(i)+abslambda**2*SB*dv4(i)
       CS1S1S1IN(2,4,5)=0
       CS1S1S1IN(2,4,6)=(((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(((-2*abskappa*expETA**2*explam*expvS*vS)
     &  /expkap-(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+DCONJG(dTlam(i))/(2.*expETA*expvS*sqrt2)+
     &  (expETA*expvS*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*abslambda*
     &  expETA**2*explam*expvS)/expkap-(2*abskappa*abslambda*expkap*
     &  expvS**5)/explam)*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(2,5,5)=0
       CS1S1S1IN(2,5,6)=0
       CS1S1S1IN(2,6,6)=(((2*abslambda*CB*expETA**2*explam*expvS*vv)
     &  /expkap+(2*abslambda*CB*expkap*expvS**5*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(((2*abskappa*CB*expETA**2*explam*expvS*
     &  vv)/expkap+(2*abskappa*CB*expkap*expvS**5*vv)/explam+8*
     &  abslambda*expETA*expvS**3*SB*vv)*dabsla14(i))/(4.*expETA*expvS*
     &  *3)+abslambda**2*CB**3*vv*dTB14(i)-(CB**2*SB*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap+(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((2*abskappa*abslambda*CB*expETA**2*explam*expvS)
     &  /expkap+(2*abskappa*abslambda*CB*expkap*expvS**5)/explam+4*
     &  abslambda**2*expETA*expvS**3*SB)*dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,3)=12*abskappa*vS*dabska14(i)+DCONJG(dTk(i))
     &  /(expvS**3*sqrt2)+(expvS**3*dTk(i))/sqrt2+6*abskappa**2*
     &  dvS14(i)
       CS1S1S1IN(3,3,4)=(Ic*((-2*abslambda*expETA**2*explam*expvS*SB*
     &  vv)/expkap+(2*abslambda*expkap*expvS**5*SB*vv)/explam)*
     &  dabska14(i))/(4.*expETA*expvS**3)+(Ic*((-2*abskappa*expETA**2*
     &  explam*expvS*SB*vv)/expkap+(2*abskappa*expkap*expvS**5*SB*vv)
     &  /explam)*dabsla14(i))/(4.*expETA*expvS**3)+(CB**3*Ic*((-2*
     &  abskappa*abslambda*expETA**2*explam*expvS*vv)/expkap+(2*
     &  abskappa*abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*
     &  expETA*expvS**3)+(Ic*((-2*abskappa*abslambda*expETA**2*explam*
     &  expvS*SB)/expkap+(2*abskappa*abslambda*expkap*expvS**5*SB)
     &  /explam)*dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,5)=(Ic*((-2*abslambda*CB*expETA**2*explam*expvS*
     &  vv)/expkap+(2*abslambda*CB*expkap*expvS**5*vv)/explam)*
     &  dabska14(i))/(4.*expETA*expvS**3)+(Ic*((-2*abskappa*CB*expETA**
     &  2*explam*expvS*vv)/expkap+(2*abskappa*CB*expkap*expvS**5*vv)
     &  /explam)*dabsla14(i))/(4.*expETA*expvS**3)-(CB**2*Ic*SB*((-2*
     &  abskappa*abslambda*expETA**2*explam*expvS*vv)/expkap+(2*
     &  abskappa*abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*
     &  expETA*expvS**3)+(((-2*abskappa*abslambda*CB*expETA**2*explam*
     &  expvS)/expkap+(2*abskappa*abslambda*CB*expkap*expvS**5)/explam)
     &  *Ic*dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,3,6)=-((Ic*DCONJG(dTk(i)))/(expvS**3*sqrt2))+(expvS*
     &  *3*Ic*dTk(i))/sqrt2
       CS1S1S1IN(3,4,4)=2*abslambda*vS*dabsla14(i)+abslambda**2*dvS14(i)
       CS1S1S1IN(3,4,5)=(((2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap+(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(((2*abskappa*expETA**2*explam*expvS*vS)
     &  /expkap+(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)+DCONJG(dTlam(i))/(2.*expETA*expvS*sqrt2)+
     &  (expETA*expvS*dTlam(i))/(2.*sqrt2)+(((2*abskappa*abslambda*
     &  expETA**2*explam*expvS)/expkap+(2*abskappa*abslambda*expkap*
     &  expvS**5)/explam)*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,4,6)=(((-2*abslambda*expETA**2*explam*expvS*SB*vv)
     &  /expkap-(2*abslambda*expkap*expvS**5*SB*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(((-2*abskappa*expETA**2*explam*expvS*
     &  SB*vv)/expkap-(2*abskappa*expkap*expvS**5*SB*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)+(CB**3*((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((-2*abskappa*abslambda*expETA**2*explam*expvS*SB)
     &  /expkap-(2*abskappa*abslambda*expkap*expvS**5*SB)/explam)*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,5,5)=2*abslambda*vS*dabsla14(i)+abslambda**2*dvS14(i)
       CS1S1S1IN(3,5,6)=(((-2*abslambda*CB*expETA**2*explam*expvS*vv)
     &  /expkap-(2*abslambda*CB*expkap*expvS**5*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(((-2*abskappa*CB*expETA**2*explam*
     &  expvS*vv)/expkap-(2*abskappa*CB*expkap*expvS**5*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)-(CB**2*SB*((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((-2*abskappa*abslambda*CB*expETA**2*explam*expvS)
     &  /expkap-(2*abskappa*abslambda*CB*expkap*expvS**5)/explam)*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(3,6,6)=4*abskappa*vS*dabska14(i)-DCONJG(dTk(i))
     &  /(expvS**3*sqrt2)-(expvS**3*dTk(i))/sqrt2+2*abskappa**2*
     &  dvS14(i)
       CS1S1S1IN(4,4,4)=0
       CS1S1S1IN(4,4,5)=0
       CS1S1S1IN(4,4,6)=0
       CS1S1S1IN(4,5,5)=0
       CS1S1S1IN(4,5,6)=(Ic*((-2*abslambda*expETA**2*explam*expvS*vS)
     &  /expkap+(2*abslambda*expkap*expvS**5*vS)/explam)*dabska14(i))
     &  /(4.*expETA*expvS**3)+(Ic*((-2*abskappa*expETA**2*explam*expvS*
     &  vS)/expkap+(2*abskappa*expkap*expvS**5*vS)/explam)*dabsla14(i))
     &  /(4.*expETA*expvS**3)-(Ic*DCONJG(dTlam(i)))/(2.*expETA*expvS*
     &  sqrt2)+(expETA*expvS*Ic*dTlam(i))/(2.*sqrt2)+(((-2*abskappa*
     &  abslambda*expETA**2*explam*expvS)/expkap+(2*abskappa*abslambda*
     &  expkap*expvS**5)/explam)*Ic*dvS14(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(4,6,6)=(Ic*((2*abslambda*expETA**2*explam*expvS*SB*vv)
     &  /expkap-(2*abslambda*expkap*expvS**5*SB*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(Ic*((2*abskappa*expETA**2*explam*expvS*
     &  SB*vv)/expkap-(2*abskappa*expkap*expvS**5*SB*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)+(CB**3*Ic*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(Ic*((2*abskappa*abslambda*expETA**2*explam*expvS*SB)
     &  /expkap-(2*abskappa*abslambda*expkap*expvS**5*SB)/explam)*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(5,5,5)=0
       CS1S1S1IN(5,5,6)=0
       CS1S1S1IN(5,6,6)=(Ic*((2*abslambda*CB*expETA**2*explam*expvS*vv)
     &  /expkap-(2*abslambda*CB*expkap*expvS**5*vv)/explam)*dabska14(i)
     &  )/(4.*expETA*expvS**3)+(Ic*((2*abskappa*CB*expETA**2*explam*
     &  expvS*vv)/expkap-(2*abskappa*CB*expkap*expvS**5*vv)/explam)*
     &  dabsla14(i))/(4.*expETA*expvS**3)-(CB**2*Ic*SB*((2*abskappa*
     &  abslambda*expETA**2*explam*expvS*vv)/expkap-(2*abskappa*
     &  abslambda*expkap*expvS**5*vv)/explam)*dTB14(i))/(4.*expETA*
     &  expvS**3)+(((2*abskappa*abslambda*CB*expETA**2*explam*expvS)
     &  /expkap-(2*abskappa*abslambda*CB*expkap*expvS**5)/explam)*Ic*
     &  dv4(i))/(4.*expETA*expvS**3)
       CS1S1S1IN(6,6,6)=(Ic*DCONJG(dTk(i)))/(expvS**3*sqrt2)-(expvS**3*
     &  Ic*dTk(i))/sqrt2
       do ii=1,6
         do j=ii,6
          do k= j,6
            CS1S1S1IN(k,j,ii) = CS1S1S1IN(ii,j,k)
            CS1S1S1IN(k,ii,j) = CS1S1S1IN(ii,j,k)
            CS1S1S1IN(ii,k,j) = CS1S1S1IN(ii,j,k)
            CS1S1S1IN(j,k,ii) = CS1S1S1IN(ii,j,k)
            CS1S1S1IN(j,ii,k) = CS1S1S1IN(ii,j,k)
          enddo
         enddo
        enddo

       do ii=1,6
         do j=1,6
          do k= 1,6
            dCS1S1S1(ii,j,k,i) =0D0
            do i1=1,6
            do i2=1,6
            do i3=1,6
            dCS1S1S1(ii,j,k,i)=dCS1S1S1(ii,j,k,i)+ZH(ii,i1)*ZH(j,i2)*ZH(k,i3)*CS1S1S1IN(i1,i2,i3)
            enddo
            enddo
            enddo
            if (i.eq.1)then
            do l=1,6
               dCS1S1S1(ii,j,k,1)=dCS1S1S1(ii,j,k,1)+dZHiggs4(ii,l)*CS1S1S1(l,k,j)/2D0+
     &          dZHiggs4(j,l)*CS1S1S1(ii,l,k)/2D0+dZHiggs4(k,l)*CS1S1S1(ii,j,l)/2D0   
            enddo 
            endif
          enddo
         enddo
        enddo

       enddo


      do i=1,3
       CS1S1S1IN(1:6,1:6,1:6)=0D0
       CS1S1S1IN(1,1,2)=-((abslambda*expeta*SB*vv*dabsla14(i))/expETA)-
     &  (abslambda**2*CB**3*expeta*vv*dTB14(i))/(2.*expETA)-(abslambda*
     &  *2*expeta*SB*dv4(i))/(2.*expETA)
       CS1S1S1IN(1,2,1)=-(abslambda*SB*vv*dabsla14(i))-(abslambda**2*
     &  CB**3*vv*dTB14(i))/2.-(abslambda**2*SB*dv4(i))/2.
       CS1S1S1IN(2,1,2)=-((abslambda*CB*expeta*vv*dabsla14(i))/expETA)+
     &  (abslambda**2*CB**2*expeta*SB*vv*dTB14(i))/(2.*expETA)-
     &  (abslambda**2*CB*expeta*dv4(i))/(2.*expETA)
       CS1S1S1IN(2,2,1)=-(abslambda*CB*vv*dabsla14(i))+(abslambda**2*
     &  CB**2*SB*vv*dTB14(i))/2.-(abslambda**2*CB*dv4(i))/2.
       CS1S1S1IN(3,1,1)=(2*abslambda*expeta*vS*dabsla14(i))/expETA+
     &  (abslambda**2*expeta*dvS14(i))/expETA
       CS1S1S1IN(3,1,2)=(abslambda*expeta**2*explam*vS*dabska14(i))
     &  /(expETA*expkap*expvS**2)+(abskappa*expeta**2*explam*vS*
     &  dabsla14(i))/(expETA*expkap*expvS**2)+(expeta*expvS*dTlam(i))
     &  /sqrt2+(abskappa*abslambda*expeta**2*explam*dvS14(i))/(expETA*
     &  expkap*expvS**2)
       CS1S1S1IN(3,2,1)=(abslambda*expkap*expvS**2*vS*dabska14(i))
     &  /(expETA*explam)+(abskappa*expkap*expvS**2*vS*dabsla14(i))
     &  /(expETA*explam)+DCONJG(dTlam(i))/(expETA*expvS*sqrt2)+
     &  (abskappa*abslambda*expkap*expvS**2*dvS14(i))/(expETA*explam)
       CS1S1S1IN(3,2,2)=2*abslambda*vS*dabsla14(i)+abslambda**2*dvS14(i)
       CS1S1S1IN(4,1,2)=(abslambda*expeta*Ic*SB*vv*dabsla14(i))/expETA+
     &  (abslambda**2*CB**3*expeta*Ic*vv*dTB14(i))/(2.*expETA)+
     &  (abslambda**2*expeta*Ic*SB*dv4(i))/(2.*expETA)
       CS1S1S1IN(4,2,1)=-(abslambda*Ic*SB*vv*dabsla14(i))-(abslambda**
     &  2*CB**3*Ic*vv*dTB14(i))/2.-(abslambda**2*Ic*SB*dv4(i))/2.
       CS1S1S1IN(5,1,2)=(abslambda*CB*expeta*Ic*vv*dabsla14(i))/expETA-
     &  (abslambda**2*CB**2*expeta*Ic*SB*vv*dTB14(i))/(2.*expETA)+
     &  (abslambda**2*CB*expeta*Ic*dv4(i))/(2.*expETA)
       CS1S1S1IN(5,2,1)=-(abslambda*CB*Ic*vv*dabsla14(i))+(abslambda**
     &  2*CB**2*Ic*SB*vv*dTB14(i))/2.-(abslambda**2*CB*Ic*dv4(i))/2.
       CS1S1S1IN(6,1,2)=-((abslambda*expeta**2*explam*Ic*vS*dabska14(i)
     &  )/(expETA*expkap*expvS**2))-(abskappa*expeta**2*explam*Ic*vS*
     &  dabsla14(i))/(expETA*expkap*expvS**2)+(expeta*expvS*Ic*dTlam(i)
     &  )/sqrt2-(abskappa*abslambda*expeta**2*explam*Ic*dvS14(i))
     &  /(expETA*expkap*expvS**2)
       CS1S1S1IN(6,2,1)=(abslambda*expkap*expvS**2*Ic*vS*dabska14(i))
     &  /(expETA*explam)+(abskappa*expkap*expvS**2*Ic*vS*dabsla14(i))
     &  /(expETA*explam)-(Ic*DCONJG(dTlam(i)))/(expETA*expvS*sqrt2)+
     &  (abskappa*abslambda*expkap*expvS**2*Ic*dvS14(i))/(expETA*
     &  explam)

       do ii=1,6
         do j=1,2
          do k= 1,2
            dCS1S3S3(ii,j,k,i) =0D0
            do i1=1,6
            do i2=1,2
            do i3=1,2
            dCS1S3S3(ii,j,k,i)=dCS1S3S3(ii,j,k,i)+ZH(ii,i1)*ZP(j,i2)*ZP(k,i3)*CS1S1S1IN(i1,i2,i3)
            enddo
            enddo
            enddo

            if (i.eq.1)then
 
            do l=1,6
               dCS1S3S3(ii,j,k,1)=dCS1S3S3(ii,j,k,1)+dZHiggs4(ii,l)*CS1S3S3(l,j,k)/2D0                
            enddo 

            do l=1,2
               dCS1S3S3(ii,j,k,1)=dCS1S3S3(ii,j,k,1)+dZCHiggs4(j,l)*CS1S3S3(ii,l,k)/2D0  
     &          +dZCHiggs4(k,l)*CS1S3S3(ii,j,l)/2D0                
            enddo 


            endif

          enddo
         enddo
        enddo

       enddo

      do i=1,3
       CS1S1S1S1IN(1:6,1:6,1:2,1:2)=0D0
       CS1S1S1S1IN(1,2,1,2)=-(abslambda*dabsla14(i))
       CS1S1S1S1IN(1,2,2,1)=-(abslambda*dabsla14(i))
       CS1S1S1S1IN(1,5,1,2)=abslambda*Ic*dabsla14(i)
       CS1S1S1S1IN(1,5,2,1)=-(abslambda*Ic*dabsla14(i))
       CS1S1S1S1IN(2,4,1,2)=abslambda*Ic*dabsla14(i)
       CS1S1S1S1IN(2,4,2,1)=-(abslambda*Ic*dabsla14(i))
       CS1S1S1S1IN(3,3,1,1)=2*abslambda*dabsla14(i)
       CS1S1S1S1IN(3,3,1,2)=(abslambda*expETA*explam*dabska14(i))
     &  /(expkap*expvS**2)+(abskappa*expETA*explam*dabsla14(i))
     &  /(expkap*expvS**2)
       CS1S1S1S1IN(3,3,2,1)=(abslambda*expkap*expvS**2*dabska14(i))
     &  /(expETA*explam)+(abskappa*expkap*expvS**2*dabsla14(i))
     &  /(expETA*explam)
       CS1S1S1S1IN(3,3,2,2)=2*abslambda*dabsla14(i)
       CS1S1S1S1IN(3,6,1,2)=-((abslambda*expETA*explam*Ic*dabska14(i))
     &  /(expkap*expvS**2))-(abskappa*expETA*explam*Ic*dabsla14(i))
     &  /(expkap*expvS**2)
       CS1S1S1S1IN(3,6,2,1)=(abslambda*expkap*expvS**2*Ic*dabska14(i))
     &  /(expETA*explam)+(abskappa*expkap*expvS**2*Ic*dabsla14(i))
     &  /(expETA*explam)
       CS1S1S1S1IN(4,5,1,2)=abslambda*dabsla14(i)
       CS1S1S1S1IN(4,5,2,1)=abslambda*dabsla14(i)
       CS1S1S1S1IN(6,6,1,1)=2*abslambda*dabsla14(i)
       CS1S1S1S1IN(6,6,1,2)=-((abslambda*expETA*explam*dabska14(i))
     &  /(expkap*expvS**2))-(abskappa*expETA*explam*dabsla14(i))
     &  /(expkap*expvS**2)
       CS1S1S1S1IN(6,6,2,1)=-((abslambda*expkap*expvS**2*dabska14(i))
     &  /(expETA*explam))-(abskappa*expkap*expvS**2*dabsla14(i))
     &  /(expETA*explam)
       CS1S1S1S1IN(6,6,2,2)=2*abslambda*dabsla14(i)

       do ii=1,6
         do j=ii,6
       CS1S1S1S1IN(j,ii,1:2,1:2)=CS1S1S1S1IN(ii,j,1:2,1:2)
	     enddo
       enddo

       do ii=1,6
        do i1=1,6
         do j=1,2
          do k= 1,2
            dCS1S1S3S3(ii,i1,j,k,i) =0D0
            do gen1=1,6
            do gen2=1,6
            do gen3=1,2
            do gen4=1,2
            dCS1S1S3S3(ii,i1,j,k,i)=dCS1S1S3S3(ii,i1,j,k,i)+ZH(ii,gen1)*ZH(i1,gen2)*
     &          ZP(j,gen3)*ZP(k,gen4)*CS1S1S1S1IN(gen1,gen2,gen3,gen4)
            enddo
            enddo
            enddo
            enddo

            if (i.eq.1)then
 
            do l=1,6
               dCS1S1S3S3(ii,i1,j,k,1)=dCS1S1S3S3(ii,i1,j,k,1)+dZHiggs4(ii,l)*CS1S1S3S3(l,i1,j,k)/2D0
     &         +dZHiggs4(i1,l)*CS1S1S3S3(ii,l,j,k)/2D0                
            enddo 

            do l=1,2
               dCS1S1S3S3(ii,i1,j,k,1)=dCS1S1S3S3(ii,i1,j,k,1)+dZCHiggs4(j,l)*CS1S1S3S3(ii,i1,l,k)/2D0  
     &          +dZCHiggs4(k,l)*CS1S1S3S3(ii,i1,j,l)/2D0                
            enddo 


            endif

           enddo
          enddo
         enddo
        enddo

       enddo

       do ik=1,3
       CS1S1S1S1IN(1:6,1:6,1:6,1:6)=0D0
       CS1S1S1S1IN(1,1,2,2)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(1,1,3,3)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(1,1,5,5)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(1,1,6,6)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(1,2,3,3)=(((-2*abslambda*expETA**2*explam)/expkap-
     &  (2*abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(1,2,3,6)=(((2*abslambda*expETA**2*explam)/expkap-(2*
     &  abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(1,2,6,6)=(((2*abslambda*expETA**2*explam)/expkap+(2*
     &  abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((2*abskappa*expETA**2*explam)/expkap+(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(1,3,3,5)=(((-2*abslambda*expETA**2*explam)/expkap+
     &  (2*abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*
     &  expETA*expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap+(2*
     &  abskappa*expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*
     &  expvS**2)
       CS1S1S1S1IN(1,3,5,6)=(((-2*abslambda*expETA**2*explam)/expkap-
     &  (2*abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(1,5,6,6)=(((2*abslambda*expETA**2*explam)/expkap-(2*
     &  abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(2,2,3,3)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(2,2,4,4)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(2,2,6,6)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(2,3,3,4)=(((-2*abslambda*expETA**2*explam)/expkap+
     &  (2*abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*
     &  expETA*expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap+(2*
     &  abskappa*expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*
     &  expvS**2)
       CS1S1S1S1IN(2,3,4,6)=(((-2*abslambda*expETA**2*explam)/expkap-
     &  (2*abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(2,4,6,6)=(((2*abslambda*expETA**2*explam)/expkap-(2*
     &  abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(3,3,3,3)=12*abskappa*dabska14(ik)
       CS1S1S1S1IN(3,3,4,4)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(3,3,4,5)=(((2*abslambda*expETA**2*explam)/expkap+(2*
     &  abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((2*abskappa*expETA**2*explam)/expkap+(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(3,3,5,5)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(3,3,6,6)=4*abskappa*dabska14(ik)
       CS1S1S1S1IN(3,4,5,6)=(((-2*abslambda*expETA**2*explam)/expkap+
     &  (2*abslambda*expkap*expvS**4)/explam)*Ic*dabska14(ik))/(4.*
     &  expETA*expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap+(2*
     &  abskappa*expkap*expvS**4)/explam)*Ic*dabsla14(ik))/(4.*expETA*
     &  expvS**2)
       CS1S1S1S1IN(4,4,5,5)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(4,4,6,6)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(4,5,6,6)=(((-2*abslambda*expETA**2*explam)/expkap-
     &  (2*abslambda*expkap*expvS**4)/explam)*dabska14(ik))/(4.*expETA*
     &  expvS**2)+(((-2*abskappa*expETA**2*explam)/expkap-(2*abskappa*
     &  expkap*expvS**4)/explam)*dabsla14(ik))/(4.*expETA*expvS**2)
       CS1S1S1S1IN(5,5,6,6)=2*abslambda*dabsla14(ik)
       CS1S1S1S1IN(6,6,6,6)=12*abskappa*dabska14(ik)


       do i=1,6
         do i1=i,6
         do j=i1,6
          do k= j,6
            CS1S1S1S1IN(k,i,i1,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i,j,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,j,i,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,j,i1,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i1,i,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(k,i1,j,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(j,i,i1,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i,k,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,k,i,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,k,i1,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i1,i,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(j,i1,k,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(i1,i,j,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,i,k,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,k,i,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,k,j,i) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,j,i,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i1,j,k,i) = CS1S1S1S1IN(i,i1,j,k)

            CS1S1S1S1IN(i,i1,k,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,k,i1,j) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,k,j,i1) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,j,i1,k) = CS1S1S1S1IN(i,i1,j,k)
            CS1S1S1S1IN(i,j,k,i1) = CS1S1S1S1IN(i,i1,j,k)
          enddo
          enddo
         enddo
        enddo

       do i=1,6
        do i1=1,6
         do j=1,6
          do k= 1,6
            dCS1S1S1S1(i,i1,j,k,ik) =0D0
            do gen1=1,6
            do gen2=1,6
            do gen3=1,6
            do gen4=1,6
            dCS1S1S1S1(i,i1,j,k,ik)=dCS1S1S1S1(i,i1,j,k,ik)+ZH(i,gen1)*ZH(i1,gen2)*
     &          ZH(j,gen3)*ZH(k,gen4)*CS1S1S1S1IN(gen1,gen2,gen3,gen4)
            enddo
            enddo
            enddo
            enddo

            if (ik.eq.1)then
 
            do l=1,6
               dCS1S1S1S1(i,i1,j,k,1)=dCS1S1S1S1(i,i1,j,k,1)+dZHiggs4(i,l)*CS1S1S1S1(l,i1,j,k)/2D0
     &        +dZHiggs4(i1,l)*CS1S1S1S1(i,l,j,k)/2D0+dZHiggs4(j,l)*CS1S1S1S1(i,i1,l,k)/2D0
     &        +dZHiggs4(k,l)*CS1S1S1S1(i,i1,j,l)/2D0              
            enddo 

            endif

           enddo
          enddo
         enddo
        enddo

       enddo

       do i=1,3
       do ik=1,2
        do i1=1,2
         do j=1,2
          do k= 1,2
       dCS3S3S3S3(ik,i1,j,k,i)=dabsla14(i)*(2*abslambda*ZP(i1,2)*
     &  ZP(ik,1)*ZP(j,2)*ZP(k,1)+2*abslambda*ZP(i1,1)*ZP(ik,2)*ZP(j,2)*
     &  ZP(k,1)+2*abslambda*ZP(i1,2)*ZP(ik,1)*ZP(j,1)*ZP(k,2)+2*
     &  abslambda*ZP(i1,1)*ZP(ik,2)*ZP(j,1)*ZP(k,2))

            if (i.eq.1)then
 
            do l=1,2
               dCS3S3S3S3(ik,i1,j,k,1)=dCS3S3S3S3(ik,i1,j,k,1)+dZCHiggs4(ik,l)*CS3S3S3S3(l,i1,j,k)/2D0
     &        +dZCHiggs4(i1,l)*CS3S3S3S3(ik,l,j,k)/2D0+dZCHiggs4(j,l)*CS3S3S3S3(ik,i1,l,k)/2D0
     &        +dZCHiggs4(k,l)*CS3S3S3S3(ik,i1,j,l)/2D0              
            enddo 

            endif

           enddo
          enddo
         enddo
        enddo

        do ik=1,3
         do k=1,2
       dCF11F12S3L(ik,k,i)=(expETA*dabsla14(i)*ZNeuC(ik,3)*ZP(k,2))/expvS
       dCF11F12S3R(ik,k,i)=(dabsla14(i)*ZNeu(ik,3)*ZP(k,1))/explam
            if (i.eq.1)then
            do l=1,2
               dCF11F12S3L(ik,k,1)=dCF11F12S3L(ik,k,1)+dZCHiggs4(k,l)*CF11F12S3L(ik,l)/2D0        
               dCF11F12S3R(ik,k,1)=dCF11F12S3R(ik,k,1)+dZCHiggs4(k,l)*CF11F12S3R(ik,l)/2D0        
            enddo 
            endif
         enddo
        enddo

        do ik=1,3
         do j=1,3
           do k=1,6
       dCFF11S1L(ik,j,k,i)=dabsla14(i)*(-((explam*expvS*ZH(k,3)*
     &  ZNeuC(ik,2)*ZNeuC(j,1))/sqrt2)-(explam*expvS*Ic*ZH(k,6)*
     &  ZNeuC(ik,2)*ZNeuC(j,1))/sqrt2-(expETA*explam*ZH(k,2)*
     &  ZNeuC(ik,3)*ZNeuC(j,1))/sqrt2-(expETA*explam*Ic*ZH(k,5)*
     &  ZNeuC(ik,3)*ZNeuC(j,1))/sqrt2-(explam*expvS*ZH(k,3)*ZNeuC(ik,1)
     &  *ZNeuC(j,2))/sqrt2-(explam*expvS*Ic*ZH(k,6)*ZNeuC(ik,1)*
     &  ZNeuC(j,2))/sqrt2-(explam*ZH(k,1)*ZNeuC(ik,3)*ZNeuC(j,2))
     &  /sqrt2-(explam*Ic*ZH(k,4)*ZNeuC(ik,3)*ZNeuC(j,2))/sqrt2-
     &  (expETA*explam*ZH(k,2)*ZNeuC(ik,1)*ZNeuC(j,3))/sqrt2-(expETA*
     &  explam*Ic*ZH(k,5)*ZNeuC(ik,1)*ZNeuC(j,3))/sqrt2-(explam*ZH(k,1)
     &  *ZNeuC(ik,2)*ZNeuC(j,3))/sqrt2-(explam*Ic*ZH(k,4)*ZNeuC(ik,2)*
     &  ZNeuC(j,3))/sqrt2)+dabska14(i)*(expkap*expvS*sqrt2*ZH(k,3)*
     &  ZNeuC(ik,3)*ZNeuC(j,3)+expkap*expvS*Ic*sqrt2*ZH(k,6)*
     &  ZNeuC(ik,3)*ZNeuC(j,3))
       dCFF11S1R(ik,j,k,i)=dabsla14(i)*(-((ZH(k,3)*ZNeu(ik,2)*ZNeu(j,1)
     &  )/(explam*expvS*sqrt2))+(Ic*ZH(k,6)*ZNeu(ik,2)*ZNeu(j,1))
     &  /(explam*expvS*sqrt2)-(ZH(k,2)*ZNeu(ik,3)*ZNeu(j,1))/(expETA*
     &  explam*sqrt2)+(Ic*ZH(k,5)*ZNeu(ik,3)*ZNeu(j,1))/(expETA*explam*
     &  sqrt2)-(ZH(k,3)*ZNeu(ik,1)*ZNeu(j,2))/(explam*expvS*sqrt2)+(Ic*
     &  ZH(k,6)*ZNeu(ik,1)*ZNeu(j,2))/(explam*expvS*sqrt2)-(ZH(k,1)*
     &  ZNeu(ik,3)*ZNeu(j,2))/(explam*sqrt2)+(Ic*ZH(k,4)*ZNeu(ik,3)*
     &  ZNeu(j,2))/(explam*sqrt2)-(ZH(k,2)*ZNeu(ik,1)*ZNeu(j,3))
     &  /(expETA*explam*sqrt2)+(Ic*ZH(k,5)*ZNeu(ik,1)*ZNeu(j,3))
     &  /(expETA*explam*sqrt2)-(ZH(k,1)*ZNeu(ik,2)*ZNeu(j,3))/(explam*
     &  sqrt2)+(Ic*ZH(k,4)*ZNeu(ik,2)*ZNeu(j,3))/(explam*sqrt2))+
     &  dabska14(i)*((sqrt2*ZH(k,3)*ZNeu(ik,3)*ZNeu(j,3))/(expkap*
     &  expvS)-(Ic*sqrt2*ZH(k,6)*ZNeu(ik,3)*ZNeu(j,3))/(expkap*expvS))

            if (i.eq.1)then
            do l=1,6
               dCFF11S1L(ik,j,k,1)=dCFF11S1L(ik,j,k,1)+dZHiggs4(k,l)*CFF11S1L(ik,j,l)/2D0
               dCFF11S1R(ik,j,k,1)=dCFF11S1R(ik,j,k,1)+dZHiggs4(k,l)*CFF11S1R(ik,j,l)/2D0
            enddo 
            endif

           enddo
          enddo
         enddo

        do k=1,6
       dCFF12S1L(k,i)=(dabsla14(i)*(ZH(k,3)+Ic*ZH(k,6)))/sqrt2
       dCFF12S1R(k,i)=(dabsla14(i)*(ZH(k,3)-Ic*ZH(k,6)))/sqrt2
            if (i.eq.1)then
            do l=1,6
               dCFF12S1L(k,1)=dCFF12S1L(k,1)+dZHiggs4(k,l)*CFF12S1L(l)/2D0
               dCFF12S1R(k,1)=dCFF12S1R(k,1)+dZHiggs4(k,l)*CFF12S1R(l)/2D0
            enddo 
            endif
        enddo
       enddo
c end new couplings
c Cterms for the couplings with gauge boson
        do i=1,3
        do j=1,3
        do gen3=1,3
       dCF11F11V2L(i,j,gen3)=-(dCW4(gen3)*(ZNeu(i,1)*ZNeuC(j,1)-
     &  ZNeu(i,2)*ZNeuC(j,2)))/(2.*CW**2*SW)-(dSW4(gen3)*(ZNeu(i,1)*
     &  ZNeuC(j,1)-ZNeu(i,2)*ZNeuC(j,2)))/(2.*CW*SW**2)
       dCF11F11V2R(i,j,gen3)=(dCW4(gen3)*(ZNeu(j,1)*ZNeuC(i,1)-
     &  ZNeu(j,2)*ZNeuC(i,2)))/(2.*CW**2*SW)+(dSW4(gen3)*(ZNeu(j,1)*
     &  ZNeuC(i,1)-ZNeu(j,2)*ZNeuC(i,2)))/(2.*CW*SW**2)
        enddo
        enddo
        enddo

        do i=1,3
        do gen3=1,3
       dCF11F12V3L(i,gen3)=-((VChaC*dSW4(gen3)*ZNeu(i,1))/(Sqrt2*SW**2))
       dCF11F12V3R(i,gen3)=(dSW4(gen3)*ZNeuC(i,2))/(Sqrt2*SW**2)
        enddo
        enddo

        do i=1,6
        do j=1,2
        do gen3=1,3

       dCS1S3V3(i,j,gen3)=-(dSW4(gen3)*(IC*ZH(i,1)*ZP(j,1)+ZH(i,4)*
     &  ZP(j,1)+(-(IC*ZH(i,2))+ZH(i,5))*ZP(j,2)))/(2.*SW**2)

            if(gen3.eq.1)then
            do l=1,6
       dCS1S3V3(i,j,gen3)=dCS1S3V3(i,j,gen3)+CS1S3V3(l,j)*facZ*dZHiggs4(l,i)/2D0
            enddo
            do l=1,2
       dCS1S3V3(i,j,gen3)=dCS1S3V3(i,j,gen3)+CS1S3V3(i,l)*facZ*dZCHiggs4(l,j)/2D0
            enddo
            endif
       dCS1S3V3C(i,j,gen3)=DCONJG(dCS1S3V3(i,j,gen3))
        enddo
        enddo
        enddo

        do i=1,2
        do j=1,2
        do gen3=1,3
       dCS3S3V2V2(i,j,gen3)=dSW4(gen3)*((2*(CW**2-SW**2)*(ZP(i,1)*
     &  ZP(j,1)+ZP(i,2)*ZP(j,2)))/(CW**2*SW)+((CW**2-SW**2)**2*(ZP(i,1)
     &  *ZP(j,1)+ZP(i,2)*ZP(j,2)))/(CW**2*SW**3))+dCW4(gen3)*((-2*(CW**
     &  2-SW**2)*(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2)))/(CW*SW**2)+((CW**2-
     &  SW**2)**2*(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2)))/(CW**3*SW**2))

            if(gen3.eq.1)then
            do l=1,2
            dCS3S3V2V2(i,j,gen3)=dCS3S3V2V2(i,j,gen3)+CS3S3V2V2(i,l)*facZ*dZCHiggs4(l,j)/2D0
     &           +CS3S3V2V2(l,j)*facZ*dZCHiggs4(l,i)/2D0
            enddo
            endif
        enddo
        enddo
        enddo


        do i=1,6
        do j=1,6
        do gen3=1,3
       dCS1S1V2V2(i,j,gen3)=(dCW4(gen3)*(ZH(i,1)*ZH(j,1)+ZH(i,2)*
     &  ZH(j,2)+ZH(i,4)*ZH(j,4)+ZH(i,5)*ZH(j,5)))/(CW**3*SW**2)+
     &  (dSW4(gen3)*(ZH(i,1)*ZH(j,1)+ZH(i,2)*ZH(j,2)+ZH(i,4)*ZH(j,4)+
     &  ZH(i,5)*ZH(j,5)))/(CW**2*SW**3)


       dCS1S1V2(i,j,gen3)=-(dCW4(gen3)*(ZH(i,4)*ZH(j,1)-ZH(i,5)*ZH(j,2)
     &  -ZH(i,1)*ZH(j,4)+ZH(i,2)*ZH(j,5)))/(2.*CW**2*SW)-(dSW4(gen3)*
     &  (ZH(i,4)*ZH(j,1)-ZH(i,5)*ZH(j,2)-ZH(i,1)*ZH(j,4)+ZH(i,2)*
     &  ZH(j,5)))/(2.*CW*SW**2)

            if(gen3.eq.1)then
            do l=1,6
            dCS1S1V2V2(i,j,gen3)=dCS1S1V2V2(i,j,gen3)+CS1S1V2V2(i,l)*facZ*dZHiggs4(l,j)/2D0
     &           +CS1S1V2V2(l,j)*facZ*dZHiggs4(l,i)/2D0
             dCS1S1V2(i,j,gen3)= dCS1S1V2(i,j,gen3)+CS1S1V2(i,l)*facZ*dZHiggs4(l,j)/2D0
     &           +CS1S1V2(l,j)*facZ*dZHiggs4(l,i)/2D0
            enddo
            endif
        enddo
        enddo
        enddo


        do gen3=1,3
       dCF12F12V2L(gen3)=(-(1/SW)-(-CW**2+SW**2)/(2.*CW**2*SW))*
     &  dCW4(gen3)+(1/CW-(-CW**2+SW**2)/(2.*CW*SW**2))*dSW4(gen3)
       dCF12F12V2R(gen3)=(-(1/SW)-(-CW**2+SW**2)/(2.*CW**2*SW))*
     &  dCW4(gen3)+(1/CW-(-CW**2+SW**2)/(2.*CW*SW**2))*dSW4(gen3)
        enddo

        do i=1,2
        do j=1,2
        If(i.eq.j) then
        do gen3=1,3
       dCS3S3V2(i,j,gen3)=(1/(2.*SW)+SW/(2.*CW**2))*dCW4(gen3)+(-1/(2.*
     &  CW)-CW/(2.*SW**2))*dSW4(gen3)
            if(gen3.eq.1)then
             do l=1,2
             dCS3S3V2(i,j,gen3)=dCS3S3V2(i,j,gen3)+CS3S3V2(i,l)*facZ*dZCHiggs4(l,j)/2D0
     &       +CS3S3V2(l,j)*facZ*dZCHiggs4(l,i)/2D0
             enddo
            endif
        enddo
        else
       dCS3S3V2(i,j,1:3)=0
        endif
        enddo
        enddo


        do i=1,2
        do gen3=1,3
       dCS14S13V3(i,gen3)=-((dSW4(gen3)*ZU(i,1))/(Sqrt2*SW**2))
        enddo
        enddo


       dF4F3V3(1:3)=-(dSW4(1:3)/(Sqrt(expETA)*Sqrt2*SW**2))

	   dCS14S14V3V3(1:3)=1/(SW**3)*dSW4(1:3)
        do i=1,2
        do j=1,2
        do gen3=1,3
       dCS13S13V3V3(i,j,gen3)=(dSW4(gen3)*ZU(j,1)*ZUC(i,1))/SW**3
        enddo
        enddo
        enddo

        do i=1,6
        do j=1,6
          do gen3=1,3
       dCS1S1V3V3(i,j,gen3)=(dSW4(gen3)*(ZH(i,1)*ZH(j,1)+ZH(i,2)*ZH(j,2)+
     &  ZH(i,4)*ZH(j,4)+ZH(i,5)*ZH(j,5)))/SW**3
            if(gen3.eq.1)then
             do l=1,6
       dCS1S1V3V3(i,j,gen3)=dCS1S1V3V3(i,j,gen3)+CS1S1V3V3(i,l)*facZ*dZHiggs4(l,j)/2D0
     &      +CS1S1V3V3(l,j)*facZ*dZHiggs4(l,i)/2D0
             enddo
            endif
          enddo
        enddo

        enddo

        do i=1,2
        do j=1,2
           do gen3=1,3
       dCS3S3V3V3(i,j,gen3)=(dSW4(gen3)*(ZP(i,1)*ZP(j,1)+ZP(i,2)*ZP(j,2)))/SW**3
            if(gen3.eq.1)then
             do l=1,2
       dCS3S3V3V3(i,j,gen3)=dCS3S3V3V3(i,j,gen3)+CS3S3V3V3(i,l)*facZ*dZCHiggs4(l,j)/2D0
     &           +CS3S3V3V3(l,j)*facZ*dZCHiggs4(l,i)/2D0
             enddo
            endif
           enddo
        enddo
        enddo

        do i=1,2
        do j=1,2
        do gen3=1,3
       dCS13S13V2(i,j,gen3)=dSW4(gen3)*((-2*SW*ZU(j,1)*ZUC(i,1)-8*SW*
     &  ZU(j,2)*ZUC(i,2))/(6.*CW*SW)-((3*CW**2-SW**2)*ZU(j,1)*ZUC(i,1)-
     &  4*SW**2*ZU(j,2)*ZUC(i,2))/(6.*CW*SW**2))+dCW4(gen3)*((ZU(j,1)*
     &  ZUC(i,1))/SW-((3*CW**2-SW**2)*ZU(j,1)*ZUC(i,1)-4*SW**2*ZU(j,2)*
     &  ZUC(i,2))/(6.*CW**2*SW))
        enddo
        enddo
        enddo

        do gen3=1,3
       dCS14S14V2(gen3)=(-1/(2.*SW)+SW/(6.*CW**2))*dCW4(gen3)+(-1/(6.*
     &  CW)+CW/(2.*SW**2))*dSW4(gen3)
        enddo
        do gen3=1,3
       dCS14S14V2V2(gen3)=((3-2*SW**2)**2*dCW4(gen3))/(9.*CW**3*SW**2)+
     &  ((4*(3-2*SW**2))/(9.*CW**2*SW)+(3-2*SW**2)**2/(9.*CW**2*SW**3))
     &  *dSW4(gen3)
        enddo

        do i=1,2
        do j=1,2
        do gen3=1,3
       dCS13S13V2V2(i,j,gen3)=-(dCW4(gen3)*(-((3-4*SW**2)**2*ZU(j,1)*
     &  ZUC(i,1))-16*SW**4*ZU(j,2)*ZUC(i,2)))/(9.*CW**3*SW**2)+
     &  dSW4(gen3)*((16*SW*(3-4*SW**2)*ZU(j,1)*ZUC(i,1)-64*SW**3*
     &  ZU(j,2)*ZUC(i,2))/(18.*CW**2*SW**2)-(-((3-4*SW**2)**2*ZU(j,1)*
     &  ZUC(i,1))-16*SW**4*ZU(j,2)*ZUC(i,2))/(9.*CW**2*SW**3))
        enddo
        enddo
        enddo

        do gen3=1,3
       dCF3F3V2L(gen3)=(-1/(2.*SW)-SW/(6.*CW**2))*dCW4(gen3)+(1/(6.*CW)
     &  +CW/(2.*SW**2))*dSW4(gen3)
       dCF3F3V2R(gen3)=(-2*SW*dCW4(gen3))/(3.*CW**2)+(2*dSW4(gen3))/(3.*CW)
        enddo
c end Cterms for the couplings with gauge boson
c Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021. 
       if(modselval(9).gt.0) then
       OPEN(17,FILE='out.m', STATUS= 'UNKNOWN', ACCESS='APPEND')
       call mathematica_list(17,'Res[dMassHpm2]',dMHp1MT4,3)
       call mathematica_nxmmatrix_c(17,'Res[dMassHpm1SP]',dMCH1L(:,:,1),2,2)
       call mathematica_nxmmatrix_c(17,'Res[dMassHpm1FIN]',dMCH1L(:,:,2),2,2)
       call mathematica_nxmmatrix_c(17,'Res[dMassHpm1EPS]',dMCH1L(:,:,3),2,2)
       call mathematica_nxmmatrix(17,'Res[dMasshh1SP]',dMH1MT4(:,:,1),6,6)
       call mathematica_nxmmatrix(17,'Res[dMasshh1FIN]',dMH1MT4(:,:,2),6,6)
       call mathematica_nxmmatrix(17,'Res[dMasshh1EPS]',dMH1MT4(:,:,3),6,6)
       call mathematica_list_c(17,'Res[dReAlam]',dReAla,3)
       call mathematica_list(17,'Res[dv]',dv4,3)
       call mathematica_list_c(17,'Res[dAt]',dAt4,3)
       call mathematica_nxmmatrix_c(17,'Res[dMSf1]',dMSf4(:,:,1),2,2)
       call mathematica_nxmmatrix_c(17,'Res[dMSf2]',dMSf4(:,:,2),2,2)
       call mathematica_nxmmatrix_c(17,'Res[dMSf3]',dMSf4(:,:,3),2,2)
       call mathematica_list_c(17,'Res[dMSQ4]',dMSQ4,3)
       call mathematica_list(17,'Res[dmt]',dmt4,3)
       call mathematica_nxmmatrix_c(17,'Res[dtad]',dTad1MT4,6,3)
       call mathematica_list(17,'Res[dCW]',dCW4,3)
       CLOSE(17)
       endif
c end Martin Gabelmann added connections to mathematica code for momentum dependent Higgs
c selfenergies 3/2021. 
#ifdef printMT4

       write(16,*)'###################new counterterm##########################'       
       write(16,*) "dZHu4=",dZHu4+Zfactor1LIN(2,2)
       write(16,*) "dZHd4=",Zfactor1LIN(1,1)
       write(16,*) "dZHS4=",Zfactor1LIN(3,3)
      
       write(16,*)"dTanB4=", dTanB4
       write(16,*)'dMUE1=',dMUE4    
       write(16,*)'dv4=', dv4
       write(16,*)'dlambdaC4=',dlambdaC4  
       write(16,*)'dabslambda14',dabslambda14 
       write(16,*)'dSW4=',dSW4
       write(16,*)'dabskappa14',dabskappa14 
       write(16,*)'dReAkappa14',dReAkappa14
       write(16,*)'dMHp1MT4=',dMHp1MT4 
       write(16,*)"Charged Higgs wave function renormalization",dZCHiggs4(1:2,1:2)
       write(16,*)"neutral Higgs wave function renormalization",dZHiggs4(1:6,1:6)
       write(16,*)'dAT4(i)=',dAT4

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ReAlambda P1, single pole,finite, eps'
       write(16,*)'dReAla(1:3)',dReAla(1:3) 

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ImAlambda P1, single pole,finite, eps'
       write(16,*)'dImAla(1:3)',dImAla(1:3) 

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of ImAkappa P1, single pole,finite, eps'
       write(16,*)'dImAka(1:3)',dImAka(1:3)  

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of Tlam P1, single pole,finite, eps'
       write(16,*)'dTlam(1:3)',dTlam(1:3) 
 
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'CTs of Tkappa P1, single pole,finite, eps'
       write(16,*)'dTk(1:3)',dTk(1:3) 


       write(16,*)'##############################################################'
       write(16,*)'counter term of the couplings'

       write(16,*)'              ', '       pole          ', '    finite          ' , '      epsilon        ' 
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 1  neutral  Higgs 2 top'    
       do i=1,6  
       write(16,*)'dhy(',i,')'  , dhy(i,1), dhy(i,2), dhy(i,3)
       enddo
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 1 neutral  Higgs 2 stop'  
       do i=1,6
       write(16,*)'dCSSS(',i,',1,1)=', dcSSS(i,1,1,1:3)
       write(16,*)'dcSSS(',i,',1,2)=', dcSSS(i,1,2,1:3)
       write(16,*)'dcSSS(',i,',2,2)=', dcSSS(i,2,2,1:3)
       enddo

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 2 neutral Higgs 2 stop'  

       write(16,*)'dCSSSS(1,1,1,1)=', dcSSSS(1,1,1,1,1:3)
       write(16,*)'dcSSSS(1,1,1,2)=', dcSSSS(1,1,1,2,1:3)
       write(16,*)'dcSSSS(1,1,2,2)=', dcSSSS(1,1,2,2,1:3)

       write(16,*)'dcSSSS(1,2,1,1)=', dcSSSS(1,2,1,1,1:3)
       write(16,*)'dcSSSS(1,2,1,2)=', dcSSSS(1,2,1,2,1:3)
       write(16,*)'dcSSSS(1,2,2,2)=', dcSSSS(1,2,2,2,1:3)

       write(16,*)'dCSSSS(3,3,1,1)=', dcSSSS(3,3,1,1,1:3)
       write(16,*)'dcSSSS(3,4,1,2)=', dcSSSS(3,4,1,2,1:3)
       write(16,*)'dcSSSS(4,4,2,2)=', dcSSSS(4,4,2,2,1:3)


       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 2 charged Higgs 2 stop' 
       write(16,*)'dCS3S13S(1,1)=', dCS3S13S(1,1,1:3)
       write(16,*)'dCS3S13S(1,2)=', dCS3S13S(1,2,1:3)
       write(16,*)'dCS3S13S(2,2)=', dCS3S13S(2,2,1:3)

       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 2 charged Higgs 2 sbottom' 
       write(16,*)'dCS3SSS14=', dCS3SSS14(1), dCS3SSS14(2), dCS3SSS14(3)
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 1 charged Higgs 1 sbottom, 1 stop' 
       write(16,*)'dCS3S13S14(1)' ,dCS3S13S14(1,1),dCS3S13S14(1,2),dCS3S13S14(1,3)
       write(16,*)'dCS3S13S14(2)' ,dCS3S13S14(2,1),dCS3S13S14(2,2),dCS3S13S14(2,3)
       write(16,*)'+++++++++++++++++++++++++'
       write(16,*)'couplings 1 charged Higgs 1 bottom, 1 top' 
       write(16,*)'dS3TB',dS3TB(1),dS3TB(2),dS3TB(3) 

      Write(16,*) '-----New couplings counterterms-----------------------------------------'

      Write(16,*) '1) Trilinear higgs self couplings'
      Write(16,*) "dCS1S1S1(1,1,1)",dCS1S1S1(1,1,1,1:3)
      Write(16,*) "dCS1S1S1(1,2,1)",dCS1S1S1(1,2,1,1:3)
      Write(16,*) "dCS1S1S1(1,3,1)",dCS1S1S1(1,3,1,1:3)
      Write(16,*) "dCS1S1S1(1,4,1)",dCS1S1S1(1,4,1,1:3)
      Write(16,*) "dCS1S1S1(1,5,1)",dCS1S1S1(1,5,1,1:3)
      Write(16,*) "dCS1S1S1(1,6,1)",dCS1S1S1(1,6,1,1:3)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '2) 1 neutral higgs - 2 charged Higgs'
      Write(16,*) "dCS1S3S3(1,1,1)",dCS1S3S3(1,1,1,1:3)
      Write(16,*) "dCS1S3S3(1,1,2)",dCS1S3S3(1,1,2,1:3)
      Write(16,*) "dCS1S3S3(1,2,2)",dCS1S3S3(1,2,2,1:3)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '3) 2 neutral higgs - 2 charged Higgs, single pole'
      Write(16,*) "dCS1S1S3S3(1,1,1,1)",dCS1S1S3S3(1,1,1,1,1)
      Write(16,*) "dCS1S1S3S3(1,1,1,2)",dCS1S1S3S3(1,1,1,2,1)
      Write(16,*) "dCS1S1S3S3(1,1,2,2)",dCS1S1S3S3(1,1,2,2,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '4) 4 neutral Higgs self couplings, single pole'
      Write(16,*) "dCS1S1S1S1(1,1,1,1)",dCS1S1S1S1(1,1,1,1,1)
      Write(16,*) "dCS1S1S1S1(1,2,1,2)",dCS1S1S1S1(1,2,1,2,1)
      Write(16,*) "dCS1S1S1S1(1,3,2,2)",dCS1S1S1S1(1,3,2,2,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '5) 4 charged Higgs self couplings, single pole'
      Write(16,*) "dCS3S3S3S3(1,1,1,1)",dCS3S3S3S3(1,1,1,1,1)
      Write(16,*) "dCS3S3S3S3(1,2,1,2)",dCS3S3S3S3(1,2,1,2,1)
      Write(16,*) "dCS3S3S3S3(1,2,2,2)",dCS3S3S3S3(1,2,2,2,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '6) 1 neutralino, chargino charged Higgs couplings, single pole'
      Write(16,*) "dCF11F12S3L(1,2)",dCF11F12S3L(1,2,1)
      Write(16,*) "dCF11F12S3R(1,2)",dCF11F12S3R(1,2,1)
      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '9) 2 neutralino i,j, Higgs k (left and right) dCFF11S1L(i,j,k), dCFF11S1R(i,j,k), single pole'
      Write(16,*) 'dCFF11S1L(1,1,1)',dCFF11S1L(1,1,1,1)
      Write(16,*) 'dCFF11S1R(1,1,1)',dCFF11S1R(1,1,1,1)
      Write(16,*) 'dCFF11S1L(1,1,2)',dCFF11S1L(1,1,2,1)
      Write(16,*) 'dCFF11S1R(1,1,2)',dCFF11S1R(1,1,2,1)

      Write(16,*) '---------------------------------------------------------'
      Write(16,*) '10) 2 chargino 1 neutral Higgs (left and right) dCFF12S1L(i), dCFF12S1R(i), single pole'
      Write(16,*) 'dCFF12S1L(1)',dCFF12S1L(1,1)
      Write(16,*) 'dCFF12S1L(2)',dCFF12S1R(2,1)
#endif 
      end subroutine CTCLs_atlambda


c*
c**********************************************************************
c*
      subroutine TadMT4_atlambda(atad)
      implicit none
      integer i,j,k,gen3,gen4,h1,gen2
      double precision mha,mha2,fac,fac1,M22m42I,mahs,aetx,a0tx,A0T,
     .   coef(10),I0ACha(2),I0A,MHpm2(2),M22m42,M24,M44,AEPT,K0T0
      double precision FD211MHM1(10,6),FD211CHM1(10,2)
      double precision atad(6),atadCT(6)
      double complex abrx(6)
#include "TLMT4.h"
#include "TLMT4lambda.h"

      fac=1D0/16D0/Pi**4
      fac1=1D0/16D0/Pi**2
      M22m42=(M22-M42)
      M22m42I= 1/M22m42
      M24=M22**2;  M44=M42**2
      MHpm2(1)=MassHpm(1)**2;   MHpm2(2)=MassHpm(2)**2 
      do h1=ihiggs,6
       abrx(h1)=0
       mahs=Mah2(h1);a0tx=a0tnh(h1);aetx=aetnh(h1)
       call FD211C(M12,M12,Mahs,a0t1,a0t1,a0tx,aet1,aet1,aetx,Q2,coef,0)
       do gen3=1,10
        FD211MHM1(gen3,h1)=coef(gen3)
       enddo
      enddo      
      mahs=MHp2
      call FD211C(M12,0D0,MHp2,a0t1,0D0,a0thp,aet1,0D0,aethp,Q2,coef,0)
      do gen3=1,10
        FD211CHM1(gen3,2)=coef(gen3)
      enddo   
      mahs=0D0
      call FD211C(M12,0D0,Mahs,a0t1,0D0,0D0,aet1,0D0,0D0,Q2,coef,0)
      do gen3=1,10
        FD211CHM1(gen3,1)=coef(gen3)
      enddo   

       I0Acha(1)=I0A(0D0,M22,MCha2,Q2)
       I0Acha(2)=I0A(0D0,M42,MCha2,Q2)

       abrx(1:6)=0

      do h1=ineu,3
       abrx(1)=abrx(1)+(M22m42I*(CF11FSR(h1,2)*(CF11FSRC(h1,1)*(F
     &  D111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1
     &  ,2))+M1*CF11FSLC(h1,1)*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNe
     &  u(h1))+CF11FSL(h1,2)*(CF11FSLC(h1,1)*(FD111Chi(2,h1,1)-FD111Chi
     &  (2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1,2))+M1*CF11FSRC(h1,1)*(
     &  -FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNeu(h1))))/6.
       abrx(2)=abrx(2)+(M22m42I*(CF11FSR(h1,1)*(CF11FSRC(h1,2)*(F
     &  D111Chi(2,h1,1)-FD111Chi(2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1
     &  ,2))+M1*CF11FSLC(h1,2)*(-FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNe
     &  u(h1))+CF11FSL(h1,1)*(CF11FSLC(h1,2)*(FD111Chi(2,h1,1)-FD111Chi
     &  (2,h1,2)-FD111Chi(3,h1,1)+FD111Chi(3,h1,2))+M1*CF11FSRC(h1,2)*(
     &  -FD111Chi(1,h1,1)+FD111Chi(1,h1,2))*MNeu(h1))))/6.
       abrx(3)=abrx(3)+(CF11FSR(h1,1)*(CF11FSRC(h1,1)*(FD211Chi(2
     &  ,h1,1)-FD211Chi(3,h1,1))-M1*CF11FSLC(h1,1)*FD211Chi(1,h1,1)*MNe
     &  u(h1))+CF11FSL(h1,1)*(CF11FSLC(h1,1)*(FD211Chi(2,h1,1)-FD211Chi
     &  (3,h1,1))-M1*CF11FSRC(h1,1)*FD211Chi(1,h1,1)*MNeu(h1)))/6.
       abrx(4)=abrx(4)+(CF11FSR(h1,2)*(CF11FSRC(h1,2)*(FD211Chi(2
     &  ,h1,2)-FD211Chi(3,h1,2))-M1*CF11FSLC(h1,2)*FD211Chi(1,h1,2)*MNe
     &  u(h1))+CF11FSL(h1,2)*(CF11FSLC(h1,2)*(FD211Chi(2,h1,2)-FD211Chi
     &  (3,h1,2))-M1*CF11FSRC(h1,2)*FD211Chi(1,h1,2)*MNeu(h1)))/6.
       abrx(5)=abrx(5)+(M1*CF11FSL(h1,1)*CF11FSLC(h1,1)*(FD211Chi
     &  M1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSR(h1,1)*CF11FSRC(h1,1)*
     &  (FD211ChiM1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSL(h1,2)*CF11FS
     &  LC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))+M1*CF11FSR(h1,
     &  2)*CF11FSRC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))-M12*C
     &  F11FSL(h1,1)*CF11FSRC(h1,1)*FD211ChiM1(1,h1,1)*MNeu(h1)-M12*CF1
     &  1FSL(h1,2)*CF11FSRC(h1,2)*FD211ChiM1(1,h1,2)*MNeu(h1)-CF11FSLC(
     &  h1,1)*CF11FSR(h1,1)*FD211ChiM1(4,h1,1)*MNeu(h1)-CF11FSLC(h1,2)*
     &  CF11FSR(h1,2)*FD211ChiM1(4,h1,2)*MNeu(h1))/6.
       abrx(6)=abrx(6)+(M1*CF11FSL(h1,1)*CF11FSLC(h1,1)*(FD211Chi
     &  M1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSR(h1,1)*CF11FSRC(h1,1)*
     &  (FD211ChiM1(2,h1,1)-FD211ChiM1(4,h1,1))+M1*CF11FSL(h1,2)*CF11FS
     &  LC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))+M1*CF11FSR(h1,
     &  2)*CF11FSRC(h1,2)*(FD211ChiM1(2,h1,2)-FD211ChiM1(4,h1,2))-M12*C
     &  F11FSLC(h1,1)*CF11FSR(h1,1)*FD211ChiM1(1,h1,1)*MNeu(h1)-M12*CF1
     &  1FSLC(h1,2)*CF11FSR(h1,2)*FD211ChiM1(1,h1,2)*MNeu(h1)-CF11FSL(h
     &  1,1)*CF11FSRC(h1,1)*FD211ChiM1(4,h1,1)*MNeu(h1)-CF11FSL(h1,2)*C
     &  F11FSRC(h1,2)*FD211ChiM1(4,h1,2)*MNeu(h1))/6.
      enddo


      do i=ihiggs,ihiggse
      atad(i)=0
        do gen2=1,2
         do gen3=1,2
          do h1=ihiggs,ihiggse
       atad(i)=atad(i)-(3*CSSS(i,gen2,gen3)*CSSSS(h1,h1,gen3,gen2)*
     &  (A0TNH(h1)*aB0T1(gen2,gen3)+AETNH(h1)+aBEpT1(gen2,gen3)*Mah2(
     &  h1)))/32d0
          enddo
       atad(i)=atad(i)-(3*(AETHp+A0THp*aB0T1(gen2,gen3)+MHp2*aBEpT1
     &  (gen2,gen3))*CS3S3S13S13(2,2,gen3,gen2)*CSSS(i,gen2,gen3))/16d0
          do gen4=1,2
       atad(i)=atad(i)-(3*(CS13S13S13S13MT4(gen3,gen4,gen2,gen4)+
     &  3*CS13S13S13S13MT4(gen3,gen4,gen4,gen2))*CSSS(i,gen2,gen3)*(aAE
     &  TN(gen4)+aA0TN(gen4)*aB0T1(gen2,gen3)+aBEpT1(gen2,gen3)*MassSu2
     &  (gen4)))/16d0
          enddo
       atad(i)=atad(i)-(3*(aet5+a0t5*aB0T1(gen2,gen3)+MSQ2*aBEpT1(gen2,
     &  gen3))*CS14S13S14S13MT4(gen3,gen2)*CSSS(i,gen2,gen3))/16d0
         enddo
        enddo

       do h1=ihiggs,ihiggse
       atad(i)=atad(i)+CSSS(h1,1,1)*((3*CSSSS(i,h1,1,1)*FD111MH(h
     &  1,1,1))/16.+(3*CSSS(h1,2,1)*CSSS(i,1,2)*(FD111MH(h1,1,1)-FD111M
     &  H(h1,1,2)))/(16.*(M22-M42))+(3*CSSS(h1,1,2)*CSSS(i,2,1)*(FD111M
     &  H(h1,1,1)-FD111MH(h1,1,2)))/(16.*(M22-M42)))+CSSS(h1,1,2)*((3*C
     &  SSSS(i,h1,2,1)*FD111MH(h1,1,2))/16.+(3*CSSS(h1,2,2)*CSSS(i,2,1)
     &  *(FD111MH(h1,1,2)-FD111MH(h1,2,2)))/(16.*(M22-M42)))+(3*CSSS(h1
     &  ,2,2)*CSSSS(i,h1,2,2)*FD111MH(h1,2,2))/16.+(3*CSSS(h1,1,1)**2*C
     &  SSS(i,1,1)*FD211MH(h1,1,1))/16.+CSSS(h1,2,1)*((3*CSSSS(i,h1,1,2
     &  )*FD111MH(h1,1,2))/16.+(3*CSSS(h1,2,2)*CSSS(i,1,2)*(FD111MH(h1,
     &  1,2)-FD111MH(h1,2,2)))/(16.*(M22-M42))+CSSS(h1,1,2)*((3*CSSS(i,
     &  1,1)*FD211MH(h1,1,2))/16.+(3*CSSS(i,2,2)*FD211MH(h1,2,1))/16.))
     &  +(3*CSSS(h1,2,2)**2*CSSS(i,2,2)*FD211MH(h1,2,2))/16.-(3*M1*hy(i
     &  )*(M12*FD211MHM1(1,h1)*hy(h1)**2+hyc(h1)*(2*FD211MHM1(2,h1)*hy(
     &  h1)+FD211MHM1(4,h1)*hyc(h1))))/8.-(3*M1*(FD211MHM1(4,h1)*hy(h1)
     &  **2+hyc(h1)*(2*FD211MHM1(2,h1)*hy(h1)+M12*FD211MHM1(1,h1)*hyc(h
     &  1)))*hyc(i))/8d0
       enddo

       do h1=1,2
       atad(i)=atad(i)+(3*M22m42I*CS3S13S(h1,2)*CS3S13SC(h1,1)*CS
     &  SS(i,1,2)*(FD111CH(h1,1)-FD111CH(h1,2)))/16.+(3*M22m42I*CS3S13S
     &  (h1,1)*CS3S13SC(h1,2)*CSSS(i,2,1)*(FD111CH(h1,1)-FD111CH(h1,2))
     &  )/16.+(3*(CS1S3SSC(i,h1,1)*CS3S13S(h1,1)*FD111CH(h1,1)+CS1S3SS(
     &  i,h1,1)*CS3S13SC(h1,1)*FD111CH(h1,1)+(CS1S3SSC(i,h1,2)*CS3S13S(
     &  h1,2)+CS1S3SS(i,h1,2)*CS3S13SC(h1,2))*FD111CH(h1,2)))/16.+(3*CS
     &  3S13S(h1,1)*CS3S13SC(h1,1)*CSSS(i,1,1)*FD211CH(h1,1))/16.+(3*CS
     &  3S13S(h1,2)*CS3S13SC(h1,2)*CSSS(i,2,2)*FD211CH(h1,2))/16.-(3*M1
     &  *CTBS3(h1)*CTBS3C(h1)*FD211CHM1(2,h1)*hy(i))/8.-(3*M1*CTBS3(h1)
     &  *CTBS3C(h1)*FD211CHM1(2,h1)*hyc(i))/8d0
       enddo


       atad(i)=atad(i)+((3*CF12F3S*CF12F3SC*M1*(FD211Cha(2)-FD211
     &  Cha(4))*(hy(i)+hyc(i)))/8.+(3*CF12F4S(1)*CF12F4SC(1)*CSSS(i,1,1
     &  )*(-2*aet2*M22+M22**2+2*a0t2*(a0tcha+M22-MCha2)+2*aet2*MCha2+M2
     &  2*MCha2+2*M22*I0Acha(1)))/(16.*M22)+(3*CF12F4S(2)*CF12F4SC(2)*C
     &  SSS(i,2,2)*(-2*aet4*M42+M42**2+2*a0t4*(a0tcha+M42-MCha2)+2*aet4
     &  *MCha2+M42*MCha2+2*M42*I0Acha(2)))/(16.*M42)+(3*M22m42I*CF12F4S
     &  (1)*CF12F4SC(2)*CSSS(i,1,2)*(a0t2*a0tcha-a0t4*a0tcha-aet2*M22+a
     &  et4*M42+2*aet2*MCha2-2*aet4*MCha2+(M22-MCha2)*I0Acha(1)+(-M42+M
     &  Cha2)*I0Acha(2)))/16.+(3*M22m42I*CF12F4S(2)*CF12F4SC(1)*CSSS(i,
     &  2,1)*(a0t2*a0tcha-a0t4*a0tcha-aet2*M22+aet4*M42+2*aet2*MCha2-2*
     &  aet4*MCha2+(M22-MCha2)*I0Acha(1)+(-M42+MCha2)*I0Acha(2)))/16d0)
 
       atad(i)=atad(i)+(9*(abrx(3)*CSSS(i,1,1)+abrx(2)*CSSS(i,1,2)
     &  +abrx(1)*CSSS(i,2,1)+abrx(4)*CSSS(i,2,2)+abrx(5)*hy(i)+abrx(6)
     &  *hyc(i)))/4d0


c
c----------------------------------------------------------------------------
c alphat_lambda -2019

    	do ICH2 = 1,2
    	do ICH3 = 1,2
    	do ICH4 = 1,2
          if ((ICH2+ICH3).ne.2) then
       atad(i)=atad(i)+-(CS1S3S3(i,ICH2,ICH3)*
     &  CS3S3S3S3(ICH3,ICH4,ICH2,ICH4)*(A0THpN(ICH4)*aB0TCH(ICH2,ICH3)+
     &  AETHpN(ICH4)+aBEpTCH(ICH2,ICH3)*MassHpm(ICH4)**2))/16.
          endif
    	enddo
    	enddo
    	enddo


    	do ICH2 = 1,2
    	do ICH3 = 1,2
    	do INH2 = ihiggs,6
          if ((ICH2+ICH3).ne.2) then
       atad(i)=atad(i)+-(CS1S1S3S3(INH2,INH2,ICH3,ICH2)*
     &  CS1S3S3(i,ICH2,ICH3)*(A0TNH(INH2)*aB0TCH(ICH2,ICH3)+AETNH(INH2)
     &  +aBEpTCH(ICH2,ICH3)*Mah2(INH2)))/32.
           endif
    	enddo
    	enddo
    	enddo

    	do INH2 = ihiggs,6
    	do INH3 = ihiggs,6
    	do INH4 = ihiggs,6
          if ((INH2+iNH3).ne.12) then
       atad(i)=atad(i)-(CS1S1S1(i,INH2,INH3)*
     &  CS1S1S1S1(INH2,INH3,INH4,INH4)*(A0TNH(INH4)*aB0TNH(INH2,INH3)+
     &  AETNH(INH4)+aBEpTNH(INH2,INH3)*Mah2(INH4)))/64.
          endif
    	enddo
    	enddo
    	enddo


    	do ICH2 = 1,2
    	do INH2 = ihiggs,6
    	do INH3 = ihiggs,6
          if ((INH2+iNH3).ne.12) then
       atad(i)=atad(i)-(CS1S1S1(i,INH2,INH3)*
     &  CS1S1S3S3(INH2,INH3,ICH2,ICH2)*(A0THpN(ICH2)*aB0TNH(INH2,INH3)+
     &  AETHpN(ICH2)+aBEpTNH(INH2,INH3)*MassHpm(ICH2)**2))/32.
           endif
    	enddo
    	enddo
    	enddo


    	do ICH2 = 1,2
    	do ICH3 = 1,2
          if ((ICH2+ICH3).ne.2) then
       atad(i)=atad(i)+(-3*(aet5+a0t5*aB0TCH(ICH2,ICH3)+MSQ2*
     &  aBEpTCH(ICH2,ICH3))*CS1S3S3(i,ICH2,ICH3)*CS3S3S14S14(ICH3,ICH2)
     &  )/16.
          endif
    	enddo
    	enddo


    	do ICH2 = 1,2
    	do ICH3 = 1,2
    	do ISt2 = 1,2
          if ((ICH2+ICH3).ne.2) then
       atad(i)=atad(i)+(-3*CS1S3S3(i,ICH2,ICH3)*
     &  CS3S3S13S13(ICH3,ICH2,ISt2,ISt2)*(aAETN(ISt2)+aA0TN(ISt2)*
     &  aB0TCH(ICH2,ICH3)+aBEpTCH(ICH2,ICH3)*MassSu2(ISt2)))/16.
          endif
    	enddo
    	enddo
    	enddo


    	do INH2 = ihiggs,6
    	do INH3 = ihiggs,6
    	do ISt2 = 1,2
          if ((INH2+iNH3).ne.12) then
       atad(i)=atad(i)+(-3*CS1S1S1(i,INH2,INH3)*
     &  CSSSS(INH2,INH3,ISt2,ISt2)*(aAETN(ISt2)+aA0TN(ISt2)*
     &  aB0TNH(INH2,INH3)+aBEpTNH(INH2,INH3)*MassSu2(ISt2)))/32.
          endif
    	enddo
    	enddo
    	enddo
c
       
       do INe2=ineu,3
       do INe3=ineu,3
       do INe4=ineu,3
       do INH2=ihiggs,6
              if( Abs(MNeu(INe2) - MNeu(INe4))/(MNeu(INe2) + MNeu(INe4)) .gt. epsi) then 
       atad(i)=atad(i)+(CFF11S1R(INe2,INe3,INH2)*(-
     &  (CFF11S1L(INe3,INe4,INH2)*(FD111ChiChiH(2,INe2,INe3,INH2)-
     &  FD111ChiChiH(2,INe3,INe4,INH2))*(CFF11S1R(INe4,INe2,i)*
     &  MNeu(INe2)+CFF11S1L(INe4,INe2,i)*MNeu(INe4)))+
     &  CFF11S1R(INe3,INe4,INH2)*MNeu(INe3)*(CFF11S1L(INe4,INe2,i)*(-
     &  FD111ChiChiH(4,INe2,INe3,INH2)+FD111ChiChiH(4,INe4,INe3,INH2))+
     &  CFF11S1R(INe4,INe2,i)*(-FD111ChiChiH(1,INe2,INe3,INH2)+
     &  FD111ChiChiH(1,INe3,INe4,INH2))*MNeu(INe2)*MNeu(INe4)))+
     &  CFF11S1L(INe2,INe3,INH2)*(CFF11S1R(INe4,INe2,i)*
     &  (CFF11S1L(INe3,INe4,INH2)*(-FD111ChiChiH(4,INe2,INe3,INH2)+
     &  FD111ChiChiH(4,INe4,INe3,INH2))*MNeu(INe3)+
     &  CFF11S1R(INe3,INe4,INH2)*(-FD111ChiChiH(2,INe2,INe3,INH2)+
     &  FD111ChiChiH(2,INe3,INe4,INH2))*MNeu(INe4))+
     &  CFF11S1L(INe4,INe2,i)*MNeu(INe2)*(CFF11S1R(INe3,INe4,INH2)*(-
     &  FD111ChiChiH(2,INe2,INe3,INH2)+FD111ChiChiH(2,INe3,INe4,INH2))+
     &  CFF11S1L(INe3,INe4,INH2)*(-FD111ChiChiH(1,INe2,INe3,INH2)+
     &  FD111ChiChiH(1,INe3,INe4,INH2))*MNeu(INe3)*MNeu(INe4))))/(16.*
     &  (MNeu(INe2)**2-MNeu(INe4)**2))
              else
               if( Abs(MNeu(INe2)) .gt. epsi) then 
       atad(i)=atad(i)+(-(CFF11S1R(INe3,INe4,INH2)*
     &  (CFF11S1L(INe2,INe3,INH2)*(CFF11S1L(INe4,INe2,i)+
     &  CFF11S1R(INe4,INe2,i))*FD211ChiChiH(2,INe2,INe3,INH2)*
     &  MNeu(INe2)+CFF11S1R(INe2,INe3,INH2)*MNeu(INe3)*
     &  (CFF11S1L(INe4,INe2,i)*FD211ChiChiH(4,INe2,INe3,INH2)+
     &  CFF11S1R(INe4,INe2,i)*FD211ChiChiH(1,INe2,INe3,INH2)*
     &  MNeu2(INe2))))-CFF11S1L(INe3,INe4,INH2)*(CFF11S1R(INe4,INe2,i)*
     &  (CFF11S1R(INe2,INe3,INH2)*FD211ChiChiH(2,INe2,INe3,INH2)*
     &  MNeu(INe2)+CFF11S1L(INe2,INe3,INH2)*
     &  FD211ChiChiH(4,INe2,INe3,INH2)*MNeu(INe3))+
     &  CFF11S1L(INe4,INe2,i)*(CFF11S1R(INe2,INe3,INH2)*
     &  FD211ChiChiH(2,INe2,INe3,INH2)*MNeu(INe2)+
     &  CFF11S1L(INe2,INe3,INH2)*FD211ChiChiH(1,INe2,INe3,INH2)*
     &  MNeu(INe3)*MNeu2(INe2))))/16.
               endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

        !finite
        
       do INH2=ihiggs,6
       atad(i)=atad(i)+-(MCha*(CFF12S1R(i)*(MCha**2*CFF12S1R(INH2)**2*
     &  FD211ChaChaH(1,INH2)+2*CFF12S1L(INH2)*CFF12S1R(INH2)*
     &  FD211ChaChaH(2,INH2)+CFF12S1L(INH2)**2*FD211ChaChaH(4,INH2))+
     &  CFF12S1L(i)*(MCha**2*CFF12S1L(INH2)**2*FD211ChaChaH(1,INH2)+2*
     &  CFF12S1L(INH2)*CFF12S1R(INH2)*FD211ChaChaH(2,INH2)+
     &  CFF12S1R(INH2)**2*FD211ChaChaH(4,INH2))))/8.
       enddo 


        
       do ICH2=1,2
       do INe2=ineu,3
       do INe3=ineu,3
              if( Abs(MNeu(INe2) - MNeu(INe3))/(MNeu(INe2) + MNeu(INe3)) .gt. epsi) then 
       atad(i)=atad(i)+(CF11F12S3R(INe3,ICH2)*(CFF11S1L(INe3,INe2,i)*
     &  (MCha*DCONJG(CF11F12S3L(INe2,ICH2))*(-
     &  FD111ChiChaHp(4,INe2,ICH2)+FD111ChiChaHp(4,INe3,ICH2))+
     &  DCONJG(CF11F12S3R(INe2,ICH2))*(-FD111ChaChiHp(2,INe2,ICH2)+
     &  FD111ChaChiHp(2,INe3,ICH2))*MNeu(INe2))+CFF11S1R(INe3,INe2,i)*
     &  (DCONJG(CF11F12S3R(INe2,ICH2))*(-FD111ChaChiHp(2,INe2,ICH2)+
     &  FD111ChaChiHp(2,INe3,ICH2))+MCha*DCONJG(CF11F12S3L(INe2,ICH2))*
     &  (-FD111ChaChiHp(1,INe2,ICH2)+FD111ChaChiHp(1,INe3,ICH2))*
     &  MNeu(INe2))*MNeu(INe3))+CF11F12S3L(INe3,ICH2)*
     &  (CFF11S1R(INe3,INe2,i)*(MCha*DCONJG(CF11F12S3R(INe2,ICH2))*(-
     &  FD111ChiChaHp(4,INe2,ICH2)+FD111ChiChaHp(4,INe3,ICH2))+
     &  DCONJG(CF11F12S3L(INe2,ICH2))*(-FD111ChaChiHp(2,INe2,ICH2)+
     &  FD111ChaChiHp(2,INe3,ICH2))*MNeu(INe2))+CFF11S1L(INe3,INe2,i)*
     &  (DCONJG(CF11F12S3L(INe2,ICH2))*(-FD111ChaChiHp(2,INe2,ICH2)+
     &  FD111ChaChiHp(2,INe3,ICH2))+MCha*DCONJG(CF11F12S3R(INe2,ICH2))*
     &  (-FD111ChaChiHp(1,INe2,ICH2)+FD111ChaChiHp(1,INe3,ICH2))*
     &  MNeu(INe2))*MNeu(INe3)))/(8.*(MNeu(INe2)**2-MNeu(INe3)**2))
              else
               if( Abs(MNeu(INe2)) .gt. epsi) then 
       atad(i)=atad(i)+(-(CF11F12S3R(INe3,ICH2)*(CFF11S1L(INe3,INe2,i)*
     &  (MCha*DCONJG(CF11F12S3L(INe2,ICH2))*FD211ChiChaHpm(4,INe2,ICH2)
     &  +DCONJG(CF11F12S3R(INe2,ICH2))*FD211ChiChaHpm(2,INe2,ICH2)*
     &  MNeu(INe2))+CFF11S1R(INe3,INe2,i)*(DCONJG(CF11F12S3R(INe2,ICH2)
     &  )*FD211ChiChaHpm(2,INe2,ICH2)*MNeu(INe2)+MCha*
     &  DCONJG(CF11F12S3L(INe2,ICH2))*FD211ChiChaHpm(1,INe2,ICH2)*
     &  MNeu2(INe2))))-CF11F12S3L(INe3,ICH2)*(CFF11S1R(INe3,INe2,i)*
     &  (MCha*DCONJG(CF11F12S3R(INe2,ICH2))*FD211ChiChaHpm(4,INe2,ICH2)
     &  +DCONJG(CF11F12S3L(INe2,ICH2))*FD211ChiChaHpm(2,INe2,ICH2)*
     &  MNeu(INe2))+CFF11S1L(INe3,INe2,i)*(DCONJG(CF11F12S3L(INe2,ICH2)
     &  )*FD211ChiChaHpm(2,INe2,ICH2)*MNeu(INe2)+MCha*
     &  DCONJG(CF11F12S3R(INe2,ICH2))*FD211ChiChaHpm(1,INe2,ICH2)*
     &  MNeu2(INe2))))/8.
               endif
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH2=1,2
       do INe2=ineu,3
       atad(i)=atad(i)+(-(MCha*(CFF12S1L(i)+CFF12S1R(i))*
     &  (CF11F12S3L(INe2,ICH2)*DCONJG(CF11F12S3L(INe2,ICH2))+
     &  CF11F12S3R(INe2,ICH2)*DCONJG(CF11F12S3R(INe2,ICH2)))*
     &  FD211ChaChiHp(2,INe2,ICH2))-(CF11F12S3R(INe2,ICH2)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))*(MCha2*CFF12S1R(i)*
     &  FD211ChaChiHp(1,INe2,ICH2)+CFF12S1L(i)*
     &  FD211ChaChiHp(4,INe2,ICH2))+CF11F12S3L(INe2,ICH2)*
     &  DCONJG(CF11F12S3R(INe2,ICH2))*(MCha2*CFF12S1L(i)*
     &  FD211ChaChiHp(1,INe2,ICH2)+CFF12S1R(i)*
     &  FD211ChaChiHp(4,INe2,ICH2)))*MNeu(INe2))/8.
       enddo 
       enddo 

        
       do INe2=ineu,3
       do INe3=ineu,3
       do ISt2=1,2
              if( Abs(MNeu(INe2) - MNeu(INe3))/(MNeu(INe2) + MNeu(INe3)) .gt. epsi) then 
       atad(i)=atad(i)+(-3*(CF11FSRC(INe2,ISt2)*(CF11FSR(INe3,ISt2)*
     &  (FD111TChiST(2,INe2,ISt2)-FD111TChiST(2,INe3,ISt2))*
     &  (CFF11S1L(INe3,INe2,i)*MNeu(INe2)+CFF11S1R(INe3,INe2,i)*
     &  MNeu(INe3))+M1*CF11FSL(INe3,ISt2)*(CFF11S1R(INe3,INe2,i)*
     &  (FD111ChiTST(4,INe2,ISt2)-FD111ChiTST(4,INe3,ISt2))+
     &  CFF11S1L(INe3,INe2,i)*(FD111TChiST(1,INe2,ISt2)-
     &  FD111TChiST(1,INe3,ISt2))*MNeu(INe2)*MNeu(INe3)))+
     &  CF11FSLC(INe2,ISt2)*(CF11FSL(INe3,ISt2)*
     &  (FD111TChiST(2,INe2,ISt2)-FD111TChiST(2,INe3,ISt2))*
     &  (CFF11S1R(INe3,INe2,i)*MNeu(INe2)+CFF11S1L(INe3,INe2,i)*
     &  MNeu(INe3))+M1*CF11FSR(INe3,ISt2)*(CFF11S1L(INe3,INe2,i)*
     &  (FD111ChiTST(4,INe2,ISt2)-FD111ChiTST(4,INe3,ISt2))+
     &  CFF11S1R(INe3,INe2,i)*(FD111TChiST(1,INe2,ISt2)-
     &  FD111TChiST(1,INe3,ISt2))*MNeu(INe2)*MNeu(INe3)))))/(8.*
     &  (MNeu(INe2)**2-MNeu(INe3)**2))
              else
               if( Abs(MNeu(INe2)) .gt. epsi) then 
       atad(i)=atad(i)+(-3*M1*(CF11FSLC(INe2,ISt2)*CF11FSR(INe3,ISt2)*
     &  CFF11S1L(INe3,INe2,i)+CF11FSL(INe3,ISt2)*CF11FSRC(INe2,ISt2)*
     &  CFF11S1R(INe3,INe2,i))*FD211ChiTST(4,INe2,ISt2)-3*
     &  (CF11FSL(INe3,ISt2)*CF11FSLC(INe2,ISt2)+CF11FSR(INe3,ISt2)*
     &  CF11FSRC(INe2,ISt2))*(CFF11S1L(INe3,INe2,i)+
     &  CFF11S1R(INe3,INe2,i))*FD211ChiTST(2,INe2,ISt2)*MNeu(INe2)-3*
     &  M1*(CF11FSL(INe3,ISt2)*CF11FSRC(INe2,ISt2)*
     &  CFF11S1L(INe3,INe2,i)+CF11FSLC(INe2,ISt2)*CF11FSR(INe3,ISt2)*
     &  CFF11S1R(INe3,INe2,i))*FD211ChiTST(1,INe2,ISt2)*MNeu2(INe2))/8.
               endif
              endif
       enddo 
       enddo 
       enddo 

        
       do ISt2=1,2
       atad(i)=atad(i)+(-3*MCha*CF12F4S(ISt2)*CF12F4SC(ISt2)*
     &  (CFF12S1L(i)+CFF12S1R(i))*FD211ChaBST(2,ISt2))/8.
       enddo 
        
       atad(i)=atad(i)+(-3*CF12F3S*CF12F3SC*MCha*(CFF12S1L(i)+
     &  CFF12S1R(i))*FD211ChaTSB(2))/8.

c
       
       do INH2=ihiggs,6
       do INH3=ihiggs,6
              if( Abs(Mah(INH2) - Mah(INH3)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH3)*(CFF12S1R(INH2)*(MCha2*
     &  CFF12S1R(INH3)*(-FD111ChaHCha(1,INH2)+FD111ChaHCha(1,INH3))+
     &  CFF12S1L(INH3)*(FD111ChaHCha(2,INH2)-FD111ChaHCha(2,INH3)-
     &  FD111ChaHCha(4,INH2)+FD111ChaHCha(4,INH3)))+CFF12S1L(INH2)*
     &  (MCha2*CFF12S1L(INH3)*(-FD111ChaHCha(1,INH2)+
     &  FD111ChaHCha(1,INH3))+CFF12S1R(INH3)*(FD111ChaHCha(2,INH2)-
     &  FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH2)+FD111ChaHCha(4,INH3))
     &  )+3*(FD111THT(2,INH2)-FD111THT(2,INH3)-FD111THT(4,INH2)+
     &  FD111THT(4,INH3))*(hy(INH3)*hyc(INH2)+hy(INH2)*hyc(INH3))-3*
     &  M12*FD111THT(1,INH2)*(hy(INH2)*hy(INH3)+hyc(INH2)*hyc(INH3))+3*
     &  M12*FD111THT(1,INH3)*(hy(INH2)*hy(INH3)+hyc(INH2)*hyc(INH3))))
     &  /(16.*(Mah(INH2)**2-Mah(INH3)**2))
              else
               if( Abs(Mah(INH2)) .gt.epsi) then 
       atad(i)=atad(i)+-(CS1S1S1(i,INH2,INH3)*(CFF12S1R(INH2)*(MCha2*
     &  CFF12S1R(INH3)*FD211HChaCha(1,INH2)+CFF12S1L(INH3)*(-
     &  FD211HChaCha(2,INH2)+FD211HChaCha(3,INH2)))+CFF12S1L(INH2)*
     &  (MCha2*CFF12S1L(INH3)*FD211HChaCha(1,INH2)+CFF12S1R(INH3)*(-
     &  FD211HChaCha(2,INH2)+FD211HChaCha(3,INH2)))-3*(FD211HTT(2,INH2)
     &  -FD211HTT(3,INH2))*(hy(INH3)*hyc(INH2)+hy(INH2)*hyc(INH3))+3*
     &  M12*FD211HTT(1,INH2)*(hy(INH2)*hy(INH3)+hyc(INH2)*hyc(INH3))))
     &  /16.
               endif
              endif
       enddo 
       enddo 
      
       do INe2=1,3
       do INe3=1,3
       do INH2=ihiggs,6
       do INH3=ihiggs,6
              if( Abs(Mah(INH2) - Mah(INH3)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH3)*((CFF11S1L(INe3,INe2,INH3)
     &  *CFF11S1R(INe2,INe3,INH2)+CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH3))*(FD111ChiHChi(2,INe2,INH2,INe3)-
     &  FD111ChiHChi(2,INe2,INH3,INe3)-FD111ChiHChi(4,INe2,INH2,INe3)+
     &  FD111ChiHChi(4,INe2,INH3,INe3))-(CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1L(INe3,INe2,INH3)+CFF11S1R(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH3))*(FD111ChiHChi(1,INe2,INH2,INe3)-
     &  FD111ChiHChi(1,INe2,INH3,INe3))*MNeu(INe2)*MNeu(INe3)))/(32.*
     &  (Mah(INH2)**2-Mah(INH3)**2))
              else
               if( Abs(Mah(INH2)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH3)*((CFF11S1L(INe3,INe2,INH3)
     &  *CFF11S1R(INe2,INe3,INH2)+CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH3))*(FD211HNeuNeu(2,INH2,INe2,INe3)-
     &  FD211HNeuNeu(3,INH2,INe2,INe3))-(CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1L(INe3,INe2,INH3)+CFF11S1R(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH3))*FD211HNeuNeu(1,INH2,INe2,INe3)*
     &  MNeu(INe2)*MNeu(INe3)))/32.
               endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
              if( Abs(MassHpm(ICH2) - MassHpm(ICH3)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH3,ICH2)*CTBS3(ICH2)*CTBS3C(ICH3)
     &  *(FD111THpB(2,ICH2)-FD111THpB(2,ICH3)-FD111THpB(4,ICH2)+
     &  FD111THpB(4,ICH3)))/(8.*(MassHpm(ICH2)**2-MassHpm(ICH3)**2))
              else
               if( Abs(MassHpm(ICH2)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH3,ICH2)*CTBS3(ICH2)*CTBS3C(ICH3)
     &  *(FD211HpTB(2,ICH2)-FD211HpTB(3,ICH2)))/8.
               endif
              endif
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
       do INe2=1,3
              if( Abs(MassHpm(ICH2) - MassHpm(ICH3)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S3S3(i,ICH3,ICH2)*((CF11F12S3L(INe2,ICH3)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3R(INe2,ICH3)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*(FD111ChiHpCha(2,INe2,ICH2)-
     &  FD111ChiHpCha(2,INe2,ICH3)-FD111ChiHpCha(4,INe2,ICH2)+
     &  FD111ChiHpCha(4,INe2,ICH3))-MCha*(CF11F12S3R(INe2,ICH3)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3L(INe2,ICH3)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*(FD111ChiHpCha(1,INe2,ICH2)-
     &  FD111ChiHpCha(1,INe2,ICH3))*MNeu(INe2)))/(8.*(MassHpm(ICH2)**2-
     &  MassHpm(ICH3)**2))
              else
               if( Abs(MassHpm(ICH2)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S3S3(i,ICH3,ICH2)*((CF11F12S3L(INe2,ICH3)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3R(INe2,ICH3)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*(FD211HpChiCha(2,ICH2,INe2)-
     &  FD211HpChiCha(3,ICH2,INe2))-MCha*(CF11F12S3R(INe2,ICH3)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3L(INe2,ICH3)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*FD211HpChiCha(1,ICH2,INe2)*
     &  MNeu(INe2)))/8.
               endif
              endif
       enddo 
       enddo 
       enddo 
c
        
       do INH2=1,6
       do INH3=1,6
       do INH4=1,6
       do INH5=1,6
              if( Abs(Mah(INH2) - Mah(INH5)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH5)*CS1S1S1(INH2,INH3,INH4)*
     &  CS1S1S1(INH3,INH4,INH5)*(FD111HHH(1,INH2,INH3,INH4)-
     &  FD111HHH(1,INH3,INH5,INH4)))/(64.*(Mah(INH2)**2-Mah(INH5)**2))
              else
               if( Abs(Mah(INH2)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH5)*CS1S1S1(INH2,INH3,INH4)*
     &  CS1S1S1(INH3,INH4,INH5)*FD211HHH(1,INH2,INH3,INH4))/64.
               endif
              endif
       enddo 

       enddo 
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
       do INH2=1,6
       do INH3=1,6
              if( Abs(Mah(INH2) - Mah(INH3)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH3)*CS1S3S3(INH2,ICH3,ICH2)*
     &  CS1S3S3(INH3,ICH2,ICH3)*(FD111HHpHp(1,INH2,ICH2,ICH3)-
     &  FD111HHpHp(1,INH3,ICH2,ICH3)))/(32.*(Mah(INH2)**2-Mah(INH3)**2)
     &  )
              else
               if( Abs(Mah(INH2)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH3)*CS1S3S3(INH2,ICH3,ICH2)*
     &  CS1S3S3(INH3,ICH2,ICH3)*FD211HHpHp(1,INH2,ICH2,ICH3))/32.
               endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
       do ICH4=1,2
       do INH2=1,6
              if( Abs(MassHpm(ICH2) - MassHpm(ICH4)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S3S3(i,ICH4,ICH2)*CS1S3S3(INH2,ICH2,ICH3)*
     &  CS1S3S3(INH2,ICH3,ICH4)*(FD111HHpHp(1,INH2,ICH2,ICH3)-
     &  FD111HHpHp(1,INH2,ICH4,ICH3)))/(16.*(MassHpm(ICH2)**2-
     &  MassHpm(ICH4)**2))
              else
               if( Abs(MassHpm(ICH2)) .gt.epsi) then 
       atad(i)=atad(i)+(CS1S3S3(i,ICH4,ICH2)*CS1S3S3(INH2,ICH2,ICH3)*
     &  CS1S3S3(INH2,ICH3,ICH4)*FD211HpHHp(1,ICH2,INH2,ICH3))/16.
               endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do INH2=1,6
       do INH3=1,6
       do ISt2=1,2
       do ISt3=1,2
              if( Abs(Mah(INH2) - Mah(INH3)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S1S1(i,INH2,INH3)*CSSS(INH2,ISt3,ISt2)*
     &  CSSS(INH3,ISt2,ISt3)*(FD111HSTST(1,INH2,ISt2,ISt3)-
     &  FD111HSTST(1,INH3,ISt2,ISt3)))/(32.*(Mah(INH2)**2-Mah(INH3)**2)
     &  )
              else
               if( Abs(Mah(INH2)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S1S1(i,INH2,INH3)*CSSS(INH2,ISt3,ISt2)*
     &  CSSS(INH3,ISt2,ISt3)*FD211HSTST(1,INH2,ISt2,ISt3))/32.
               endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
       do ISt2=1,2
              if( Abs(MassHpm(ICH2) - MassHpm(ICH3)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH3,ICH2)*CS3S13S(ICH2,ISt2)*
     &  CS3S13SC(ICH3,ISt2)*(FD111HpSTSB(1,ICH2,ISt2)-
     &  FD111HpSTSB(1,ICH3,ISt2)))/(16.*(MassHpm(ICH2)**2-MassHpm(ICH3)
     &  **2))
              else
               if( Abs(MassHpm(ICH2)) .gt.epsi) then 
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH3,ICH2)*CS3S13S(ICH2,ISt2)*
     &  CS3S13SC(ICH3,ISt2)*FD211HpSTSB(1,ICH2,ISt2))/16.
               endif
              endif
       enddo 
       enddo 
       enddo 
c

        INH2=6
        INH3=6      
       atad(i)=atad(i)+-(CS1S1S1(i,INH2,INH2)*(MCha2*CFF12S1L(INH2)**2*
     &  FD211HChaCha(1,INH2)+MCha2*CFF12S1R(INH2)**2*
     &  FD211HChaCha(1,INH2)+2*CFF12S1L(INH2)*CFF12S1R(INH2)*(-
     &  FD211HChaCha(2,INH2)+FD211HChaCha(3,INH2))+6*(-FD211HTT(2,INH2)
     &  +FD211HTT(3,INH2))*hy(INH2)*hyc(INH2)+3*M12*FD211HTT(1,INH2)*
     &  (hy(INH2)**2+hyc(INH2)**2)))/16.

        INH2=6
        INH3=6       
       do INe2=ineu,3
       do INe3=ineu,3
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH2)*((CFF11S1L(INe3,INe2,INH2)
     &  *CFF11S1R(INe2,INe3,INH2)+CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH2))*(FD211HNeuNeu(2,INH2,INe2,INe3)-
     &  FD211HNeuNeu(3,INH2,INe2,INe3))-(CFF11S1L(INe2,INe3,INH2)*
     &  CFF11S1L(INe3,INe2,INH2)+CFF11S1R(INe2,INe3,INH2)*
     &  CFF11S1R(INe3,INe2,INH2))*FD211HNeuNeu(1,INH2,INe2,INe3)*
     &  MNeu(INe2)*MNeu(INe3)))/32.
       enddo 
       enddo 



        ICH2=1
        ICH3=1        
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH2,ICH2)*CTBS3(ICH2)*CTBS3C(ICH2)
     &  *(FD211HpTB(2,ICH2)-FD211HpTB(3,ICH2)))/8.


        ICH2=1
        ICH3=1
       do INe2=ineu,3
       atad(i)=atad(i)+(CS1S3S3(i,ICH2,ICH2)*((CF11F12S3L(INe2,ICH2)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3R(INe2,ICH2)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*(FD211HpChiCha(2,ICH2,INe2)-
     &  FD211HpChiCha(3,ICH2,INe2))-MCha*(CF11F12S3R(INe2,ICH2)*
     &  DCONJG(CF11F12S3L(INe2,ICH2))+CF11F12S3L(INe2,ICH2)*
     &  DCONJG(CF11F12S3R(INe2,ICH2)))*FD211HpChiCha(1,ICH2,INe2)*
     &  MNeu(INe2)))/8.
       enddo 
c

        INH2=6
        INH5=6
       do INH3= ihiggs,6
       do INH4= ihiggs,6
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH2)*CS1S1S1(INH2,INH3,INH4)*
     &  CS1S1S1(INH3,INH4,INH2)*FD211HHH(1,INH2,INH3,INH4))/64.
       enddo 
       enddo 

        INH2=6
        INH3=6
       do ICH2=1,2
       do ICH3=1,2
       atad(i)=atad(i)+(CS1S1S1(i,INH2,INH2)*CS1S3S3(INH2,ICH2,ICH3)*
     &  CS1S3S3(INH2,ICH3,ICH2)*FD211HHpHp(1,INH2,ICH2,ICH3))/32.
       enddo 
       enddo 


        ICH2=1
        ICH4=1
       do ICH3=1,2
       do INH2= ihiggs,6
       atad(i)=atad(i)+(CS1S3S3(i,ICH2,ICH2)*CS1S3S3(INH2,ICH2,ICH3)*
     &  CS1S3S3(INH2,ICH3,ICH2)*FD211HpHHp(1,ICH2,INH2,ICH3))/16.
       enddo 
       enddo 

        INH2=6
        INH3=6
       do ISt2=1,2
       do ISt3=1,2
       atad(i)=atad(i)+(3*CS1S1S1(i,INH2,INH2)*CSSS(INH2,ISt2,ISt3)*
     &  CSSS(INH2,ISt3,ISt2)*FD211HSTST(1,INH2,ISt2,ISt3))/32.
       enddo 
       enddo 
        ICH2=1
        ICH3=1
       do ISt2=1,2
       atad(i)=atad(i)+(3*CS1S3S3(i,ICH2,ICH2)*CS3S13S(ICH2,ISt2)*
     &  CS3S13SC(ICH2,ISt2)*FD211HpSTSB(1,ICH2,ISt2))/16.
       enddo 
c       
       do INH2= ihiggs,6
       do INH3= ihiggs,6
       do INH4= ihiggs,6
       atad(i)=atad(i)+(CS1S1S1(INH2,INH3,INH4)*
     &  CS1S1S1S1(i,INH2,INH3,INH4)*FD111HHH(1,INH2,INH3,INH4))/96.

       enddo 
       enddo 
       enddo 


        
       do ICH2=1,2
       do ICH3=1,2
       do INH2= ihiggs,6
       atad(i)=atad(i)+(CS1S1S3S3(i,INH2,ICH3,ICH2)*
     &  CS1S3S3(INH2,ICH2,ICH3)*FD111HHpHp(1,INH2,ICH2,ICH3))/16.
       enddo 
       enddo 
       enddo 
c

       atad(i)=atad(i)+(-(CS1S1S1(i,6,6)*(A0TNH(1)*aB0TNH(6,6)*
     &  CS1S1S1S1(6,6,1,1)+AETNH(1)*CS1S1S1S1(6,6,1,1)+A0TNH(2)*
     &  aB0TNH(6,6)*CS1S1S1S1(6,6,2,2)+AETNH(2)*CS1S1S1S1(6,6,2,2)+
     &  A0TNH(3)*aB0TNH(6,6)*CS1S1S1S1(6,6,3,3)+AETNH(3)*
     &  CS1S1S1S1(6,6,3,3)+A0TNH(4)*aB0TNH(6,6)*CS1S1S1S1(6,6,4,4)+
     &  AETNH(4)*CS1S1S1S1(6,6,4,4)+A0TNH(5)*aB0TNH(6,6)*
     &  CS1S1S1S1(6,6,5,5)+AETNH(5)*CS1S1S1S1(6,6,5,5)+2*AETHp*
     &  CS1S1S3S3(6,6,2,2)+2*A0THp*aB0TNH(6,6)*CS1S1S3S3(6,6,2,2)+2*
     &  MHp2*aBEpTNH(6,6)*CS1S1S3S3(6,6,2,2)+6*aet2*CSSSS(6,6,1,1)+6*
     &  a0t2*aB0TNH(6,6)*CSSSS(6,6,1,1)+6*M22*aBEpTNH(6,6)*
     &  CSSSS(6,6,1,1)+6*aet4*CSSSS(6,6,2,2)+6*a0t4*aB0TNH(6,6)*
     &  CSSSS(6,6,2,2)+6*M42*aBEpTNH(6,6)*CSSSS(6,6,2,2)+aBEpTNH(6,6)*
     &  CS1S1S1S1(6,6,1,1)*Mah2(1)+aBEpTNH(6,6)*CS1S1S1S1(6,6,2,2)*
     &  Mah2(2)+aBEpTNH(6,6)*CS1S1S1S1(6,6,3,3)*Mah2(3)+aBEpTNH(6,6)*
     &  CS1S1S1S1(6,6,4,4)*Mah2(4)+aBEpTNH(6,6)*CS1S1S1S1(6,6,5,5)*
     &  Mah2(5)))-2*CS1S3S3(i,1,1)*(A0TNH(1)*aB0TCH(1,1)*
     &  CS1S1S3S3(1,1,1,1)+AETNH(1)*CS1S1S3S3(1,1,1,1)+A0TNH(2)*
     &  aB0TCH(1,1)*CS1S1S3S3(2,2,1,1)+AETNH(2)*CS1S1S3S3(2,2,1,1)+
     &  A0TNH(3)*aB0TCH(1,1)*CS1S1S3S3(3,3,1,1)+AETNH(3)*
     &  CS1S1S3S3(3,3,1,1)+A0TNH(4)*aB0TCH(1,1)*CS1S1S3S3(4,4,1,1)+
     &  AETNH(4)*CS1S1S3S3(4,4,1,1)+A0TNH(5)*aB0TCH(1,1)*
     &  CS1S1S3S3(5,5,1,1)+AETNH(5)*CS1S1S3S3(5,5,1,1)+6*aet2*
     &  CS3S3S13S13(1,1,1,1)+6*a0t2*aB0TCH(1,1)*CS3S3S13S13(1,1,1,1)+6*
     &  M22*aBEpTCH(1,1)*CS3S3S13S13(1,1,1,1)+6*aet4*
     &  CS3S3S13S13(1,1,2,2)+6*a0t4*aB0TCH(1,1)*CS3S3S13S13(1,1,2,2)+6*
     &  M42*aBEpTCH(1,1)*CS3S3S13S13(1,1,2,2)+6*aet5*CS3S3S14S14(1,1)+
     &  6*a0t5*aB0TCH(1,1)*CS3S3S14S14(1,1)+6*MSQ2*aBEpTCH(1,1)*
     &  CS3S3S14S14(1,1)+2*AETHp*CS3S3S3S3(1,2,1,2)+2*A0THp*aB0TCH(1,1)
     &  *CS3S3S3S3(1,2,1,2)+2*MHp2*aBEpTCH(1,1)*CS3S3S3S3(1,2,1,2)+
     &  aBEpTCH(1,1)*CS1S1S3S3(1,1,1,1)*Mah2(1)+aBEpTCH(1,1)*
     &  CS1S1S3S3(2,2,1,1)*Mah2(2)+aBEpTCH(1,1)*CS1S1S3S3(3,3,1,1)*
     &  Mah2(3)+aBEpTCH(1,1)*CS1S1S3S3(4,4,1,1)*Mah2(4)+aBEpTCH(1,1)*
     &  CS1S1S3S3(5,5,1,1)*Mah2(5)))/64.
c

       atadCT(i)=-2*(MCha*(aetcha*(dCFF12S1L(i,1)+
     &  dCFF12S1R(i,1))+a0tcha*(dCFF12S1L(i,2)+dCFF12S1R(i,2))+MCha2*
     &  (dCFF12S1L(i,3)+dCFF12S1R(i,3)))+3*M1*(aet1*(dhy(i,1)+dhyc(i,1)
     &  )+a0t1*(dhy(i,2)+dhyc(i,2))+M12*(dhy(i,3)+dhyc(i,3))))

       do INe2=ineu,3
       atadCT(i)=atadCT(i)+MNeu(INe2)*(-(AETNeu(INe2)*
     &  (dCFF11S1L(INe2,INe2,i,1)+dCFF11S1R(INe2,INe2,i,1)))-
     &  A0TNeu(INe2)*(dCFF11S1L(INe2,INe2,i,2)+dCFF11S1R(INe2,INe2,i,2)
     &  )-(dCFF11S1L(INe2,INe2,i,3)+dCFF11S1R(INe2,INe2,i,3))*
     &  MNeu2(INe2))
       enddo 
c

       do INH2= ihiggs,6
       atadCT(i)=atadCT(i)+(AETNH(INH2)*dCS1S1S1(i,INH2,INH2,1)+
     &  A0TNH(INH2)*dCS1S1S1(i,INH2,INH2,2)+dCS1S1S1(i,INH2,INH2,3)*
     &  Mah2(INH2))/2.
       enddo 
        
       do ICH2=1,2
       atadCT(i)=atadCT(i)+(AETHpN(ICH2)*dCS1S3S3(i,ICH2,ICH2,1)+
     &  A0THpN(ICH2)*dCS1S3S3(i,ICH2,ICH2,2)+dCS1S3S3(i,ICH2,ICH2,3)*
     &  MassHpm(ICH2)**2)
       enddo 
        
       do ISt2=1,2
       atadCT(i)=atadCT(i)+3*(aAETN(ISt2)*dCSSS(i,ISt2,ISt2,1)+
     &  aA0TN(ISt2)*dCSSS(i,ISt2,ISt2,2)+dCSSS(i,ISt2,ISt2,3)*
     &  MassSu2(ISt2))       
       enddo 
c
       
       atadCT(i)=atadCT(i)+-2*((CFF12S1L(i)+CFF12S1R(i))*(aetcha*
     &  dMCha1L(1)+a0tcha*dMCha1L(2)+MCha2*(2*aBEpTCha*dMCha1L(1)+2*
     &  aB0TCha*dMCha1L(2)+3*dMCha1L(3)))+3*(aet1*dmt4(1)+a0t1*dmt4(2)+
     &  M12*(2*aBEpTN1*dmt4(1)+2*aB0TN1*dmt4(2)+3*dmt4(3)))*(hy(i)+
     &  hyc(i)))

       
       do INe2=1,3
       do INe3=1,3
       atadCT(i)=atadCT(i)+(-(AETNeu(INe3)*(CFF11S1L(INe2,INe3,i)*
     &  DCONJG(dMNeu1L(INe2,INe3,1))+CFF11S1R(INe2,INe3,i)*
     &  dMNeu1L(INe3,INe2,1)))-A0TNeu(INe3)*(CFF11S1L(INe2,INe3,i)*
     &  DCONJG(dMNeu1L(INe2,INe3,2))+CFF11S1R(INe2,INe3,i)*
     &  dMNeu1L(INe3,INe2,2))-aBEpTNeu(INe2,INe3)*MNeu(INe2)*
     &  (CFF11S1R(INe2,INe3,i)*(dMNeu1L(INe3,INe2,1)*MNeu(INe2)+
     &  DCONJG(dMNeu1L(INe2,INe3,1))*MNeu(INe3))+CFF11S1L(INe2,INe3,i)*
     &  (DCONJG(dMNeu1L(INe2,INe3,1))*MNeu(INe2)+dMNeu1L(INe3,INe2,1)*
     &  MNeu(INe3)))-aB0TNeu(INe2,INe3)*MNeu(INe2)*
     &  (CFF11S1R(INe2,INe3,i)*(dMNeu1L(INe3,INe2,2)*MNeu(INe2)+
     &  DCONJG(dMNeu1L(INe2,INe3,2))*MNeu(INe3))+CFF11S1L(INe2,INe3,i)*
     &  (DCONJG(dMNeu1L(INe2,INe3,2))*MNeu(INe2)+dMNeu1L(INe3,INe2,2)*
     &  MNeu(INe3)))-MNeu(INe2)*(CFF11S1R(INe2,INe3,i)*
     &  (dMNeu1L(INe3,INe2,3)*MNeu(INe2)+DCONJG(dMNeu1L(INe2,INe3,3))*
     &  MNeu(INe3))+CFF11S1L(INe2,INe3,i)*(DCONJG(dMNeu1L(INe2,INe3,3))
     &  *MNeu(INe2)+dMNeu1L(INe3,INe2,3)*MNeu(INe3)))-
     &  CFF11S1L(INe2,INe3,i)*DCONJG(dMNeu1L(INe2,INe3,3))*MNeu2(INe3)-
     &  CFF11S1R(INe2,INe3,i)*dMNeu1L(INe3,INe2,3)*MNeu2(INe3))
       enddo 
       enddo 
c       
       do INH2=1,6
       do INH3=1,6
              if( Mah(INH2) + Mah(INH3) .gt.epsi) then 
       atadCT(i)=atadCT(i)+(CS1S1S1(i,INH2,INH3)*(2*
     &  dMH1MT4(INH2,INH3,3)-2*AETNH(INH3)*dZHiggs1(INH2,INH3,1)-2*
     &  A0TNH(INH3)*dZHiggs1(INH2,INH3,2)-dZHiggs1(INH2,INH3,3)*
     &  (Mah2(INH2)+Mah2(INH3))+aBEpTNH(INH2,INH3)*(2*
     &  dMH1MT4(INH2,INH3,1)+dZHiggs1(INH2,INH3,1)*(-Mah2(INH2)+
     &  Mah2(INH3)))+aB0TNH(INH2,INH3)*(2*dMH1MT4(INH2,INH3,2)+
     &  dZHiggs1(INH2,INH3,2)*(-Mah2(INH2)+Mah2(INH3)))))/4.
              endif
       enddo 
       enddo 
        
       do ICH2=1,2
       do ICH3=1,2
              if( MassHpm(ICH2) + MassHpm(ICH3) .gt.epsi) then 
       atadCT(i)=atadCT(i)+(CS1S3S3(i,ICH3,ICH2)*(2*dMCH1L(ICH3,ICH2,3)
     &  -2*AETHpN(ICH3)*dZCHiggs1(ICH3,ICH2,1)-2*A0THpN(ICH3)*
     &  dZCHiggs1(ICH3,ICH2,2)-dZCHiggs1(ICH3,ICH2,3)*(MassHpm(ICH2)**
     &  2+MassHpm(ICH3)**2)+aBEpTCH(ICH2,ICH3)*(2*dMCH1L(ICH3,ICH2,1)+
     &  dZCHiggs1(ICH3,ICH2,1)*(-MassHpm(ICH2)**2+MassHpm(ICH3)**2))+
     &  aB0TCH(ICH2,ICH3)*(2*dMCH1L(ICH3,ICH2,2)+dZCHiggs1(ICH3,ICH2,2)
     &  *(-MassHpm(ICH2)**2+MassHpm(ICH3)**2))))/2.
              endif
       enddo 
       enddo 
        
       do ISt2=1,2
       do ISt3=1,2
              if( MassSu(ISt2) + MassSu(ISt3) .gt.epsi) then 
       atadCT(i)=atadCT(i)+3*CSSS(i,ISt3,ISt2)*(aBEpT1(ISt2,ISt3)*
     &  dMSf4(ISt3,ISt2,1)+aB0T1(ISt2,ISt3)*dMSf4(ISt3,ISt2,2)+
     &  dMSf4(ISt3,ISt2,3))
              endif
       enddo 
       enddo 

c CTGBC

       INH2=6; INH3=6
       atadCT(i)=atadCT(i)+(CS1S1S1(i,INH2,INH2)*(aBEpTNH(INH2,INH2)*
     &  dMH1MT4(INH2,INH2,1)+aB0TNH(INH2,INH2)*dMH1MT4(INH2,INH2,2)+
     &  dMH1MT4(INH2,INH2,3)-AETNH(INH2)*dZHiggs1(INH2,INH2,1)-
     &  A0TNH(INH2)*dZHiggs1(INH2,INH2,2)-dZHiggs1(INH2,INH2,3)*
     &  Mah2(INH2)))/2.

       ICH2=1; ICH3=1
       atadCT(i)=atadCT(i)+CS1S3S3(i,ICH2,ICH2)*(aBEpTCH(ICH2,ICH2)*
     &  dMCH1L(ICH2,ICH2,1)+aB0TCH(ICH2,ICH2)*dMCH1L(ICH2,ICH2,2)+
     &  dMCH1L(ICH2,ICH2,3)-AETHpN(ICH2)*dZCHiggs1(ICH2,ICH2,1)-
     &  A0THpN(ICH2)*dZCHiggs1(ICH2,ICH2,2)-dZCHiggs1(ICH2,ICH2,3)*
     &  MassHpm(ICH2)**2)
      enddo

      do i=1,6
       atad(i)=atadCT(i)*fac1+atad(i)*fac
      enddo
#ifdef printMT4
      Write(16,110) "TwoLoopTadpole[SUM]={"
      do i=1,5
      write(16,111) (atad(i)),","
      enddo
      i=6
      write(16,111) (atad(i)),"};"

110   format(A)   
111   format(F24.14,A)
#endif
      end subroutine TadMT4_atlambda

c
c======================================================================
c
c**********************************************************************
      subroutine CHself_atlambda(SCH)
      implicit none
      integer i,j,gen2,gen3,gen4,gen5,h1
      double precision mha,mha2,fac,fac1,mahs,
     .  SCH,SCHSP,SCHDP,I0A,A0T,AEpTss
      double precision self,selfSP,selfDP,selfCT,selfCTSP,
     .                selfCTDP,MSQ
 
      double complex aSCH,aSCHSP,aSCHDP

#include "TLMT4.h"
#include "TLMT4lambda.h"

      fac=1D0/16D0/Pi**4
      fac1=1D0/16D0/Pi**2
      self=0D0;  selfSP =0D0;  selfDP=0D0
      selfCT=0D0;  selfCTSP =0D0;  selfCTDP=0D0
      MSQ=Sqrt(MSQ2)
c One-loop inserted Counterterms diagrams
       i=2;j=2
        
       selfCT=selfCT-12*M1*CTBS3(i)*CTBS3C(j)*(aBEpTN1*dmt4(1)+aB0TN1*
     &  dmt4(2)+dmt4(3))
       do INe3=ineu,3
       do INe4=ineu,3
              if( MNeu(INe3) + MNeu(INe4) .gt. 1D-15) then 
       selfCT=selfCT+(-2*(CF11F12S3L(INe4,j)*
     &  (DCONJG(dMNeu1L(INe4,INe3,3))*(MCha*DCONJG(CF11F12S3R(INe3,i))+
     &  DCONJG(CF11F12S3L(INe3,i))*MNeu(INe3))+
     &  DCONJG(CF11F12S3L(INe3,i))*dMNeu1L(INe3,INe4,3)*MNeu(INe4))+
     &  CF11F12S3R(INe4,j)*(MCha*DCONJG(CF11F12S3L(INe3,i))*
     &  dMNeu1L(INe3,INe4,3)+DCONJG(CF11F12S3R(INe3,i))*
     &  (dMNeu1L(INe3,INe4,3)*MNeu(INe3)+DCONJG(dMNeu1L(INe4,INe3,3))*
     &  MNeu(INe4))))-2*(MCha*aCEpTChaChi2(INe3,INe4)*
     &  (CF11F12S3R(INe4,j)*(MCha*dMNeu1L(INe3,INe4,1)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,i))+DCONJG(CF11F12S3R(INe3,i))*
     &  MNeu(INe3))+DCONJG(dMNeu1L(INe4,INe3,1))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,i))+DCONJG(CF11F12S3L(INe3,i))*
     &  MNeu(INe3))*MNeu(INe4))+CF11F12S3L(INe4,j)*(MCha*
     &  DCONJG(dMNeu1L(INe4,INe3,1))*(MCha*DCONJG(CF11F12S3R(INe3,i))+
     &  DCONJG(CF11F12S3L(INe3,i))*MNeu(INe3))+dMNeu1L(INe3,INe4,1)*
     &  (MCha*DCONJG(CF11F12S3L(INe3,i))+DCONJG(CF11F12S3R(INe3,i))*
     &  MNeu(INe3))*MNeu(INe4)))+aBEpTNeu(INe3,INe4)*
     &  (CF11F12S3L(INe4,j)*(DCONJG(dMNeu1L(INe4,INe3,1))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,i))+DCONJG(CF11F12S3L(INe3,i))*
     &  MNeu(INe3))+DCONJG(CF11F12S3L(INe3,i))*dMNeu1L(INe3,INe4,1)*
     &  MNeu(INe4))+CF11F12S3R(INe4,j)*(MCha*DCONJG(CF11F12S3L(INe3,i))
     &  *dMNeu1L(INe3,INe4,1)+DCONJG(CF11F12S3R(INe3,i))*
     &  (dMNeu1L(INe3,INe4,1)*MNeu(INe3)+DCONJG(dMNeu1L(INe4,INe3,1))*
     &  MNeu(INe4)))))-2*(MCha*aC0TChaChi2(INe3,INe4)*
     &  (CF11F12S3R(INe4,j)*(MCha*dMNeu1L(INe3,INe4,2)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,i))+DCONJG(CF11F12S3R(INe3,i))*
     &  MNeu(INe3))+DCONJG(dMNeu1L(INe4,INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,i))+DCONJG(CF11F12S3L(INe3,i))*
     &  MNeu(INe3))*MNeu(INe4))+CF11F12S3L(INe4,j)*(MCha*
     &  DCONJG(dMNeu1L(INe4,INe3,2))*(MCha*DCONJG(CF11F12S3R(INe3,i))+
     &  DCONJG(CF11F12S3L(INe3,i))*MNeu(INe3))+dMNeu1L(INe3,INe4,2)*
     &  (MCha*DCONJG(CF11F12S3L(INe3,i))+DCONJG(CF11F12S3R(INe3,i))*
     &  MNeu(INe3))*MNeu(INe4)))+aB0TNeu(INe3,INe4)*(CF11F12S3L(INe4,j)
     &  *(DCONJG(dMNeu1L(INe4,INe3,2))*(MCha*DCONJG(CF11F12S3R(INe3,i))
     &  +DCONJG(CF11F12S3L(INe3,i))*MNeu(INe3))+
     &  DCONJG(CF11F12S3L(INe3,i))*dMNeu1L(INe3,INe4,2)*MNeu(INe4))+
     &  CF11F12S3R(INe4,j)*(MCha*DCONJG(CF11F12S3L(INe3,i))*
     &  dMNeu1L(INe3,INe4,2)+DCONJG(CF11F12S3R(INe3,i))*
     &  (dMNeu1L(INe3,INe4,2)*MNeu(INe3)+DCONJG(dMNeu1L(INe4,INe3,2))*MNeu(INe4))))))
              endif
       enddo 
       enddo 
       do INe3=ineu,3
       selfCT=selfCT+(-4*MCha*(CF11F12S3L(INe3,j)*
     &  DCONJG(CF11F12S3L(INe3,i))+CF11F12S3R(INe3,j)*
     &  DCONJG(CF11F12S3R(INe3,i)))*(aBEpTChaChi(INe3)*dMCha1L(1)+
     &  MCha2*aCEpTCha2Chi(INe3)*dMCha1L(1)+(aB0TChaChi(INe3)+MCha2*
     &  aC0TCha2Chi(INe3))*dMCha1L(2)+dMCha1L(3))-2*(CF11F12S3R(INe3,j)
     &  *DCONJG(CF11F12S3L(INe3,i))+CF11F12S3L(INe3,j)*
     &  DCONJG(CF11F12S3R(INe3,i)))*(aBEpTChaChi(INe3)*dMCha1L(1)+
     &  aB0TChaChi(INe3)*dMCha1L(2)+2*MCha2*(aCEpTCha2Chi(INe3)*
     &  dMCha1L(1)+aC0TCha2Chi(INe3)*dMCha1L(2))+dMCha1L(3))*MNeu(INe3)
     &  )
       enddo 
c
c CTtopo1-NoGBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .gt. 1D-15) then 
       selfCT=selfCT+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*(-2*
     &  (aBEpTNCH(INH4,ICH3)*dZHiggs1(INH3,INH4,1)+aB0TNCH(INH4,ICH3)*
     &  dZHiggs1(INH3,INH4,2)+dZHiggs1(INH3,INH4,3))+
     &  aCepTHHCH(INH3,INH4,ICH3)*(2*dMH1MT4(INH3,INH4,1)+
     &  dZHiggs1(INH3,INH4,1)*(-Mah2(INH3)+Mah2(INH4)))+
     &  aC0THHCH(INH3,INH4,ICH3)*(2*dMH1MT4(INH3,INH4,2)+
     &  dZHiggs1(INH3,INH4,2)*(-Mah2(INH3)+Mah2(INH4)))))/2.
              endif
       enddo 
       enddo 
       enddo 
   
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .gt. 1D-15) then 
       selfCT=selfCT+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*(-2*
     &  (aBEpTCH(ICH3,ICH4)*dZCHiggs1(ICH4,ICH3,1)+aB0TCH(ICH3,ICH4)*
     &  dZCHiggs1(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,3))+
     &  aCepTHCHCH(INH3,ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,1)+
     &  dZCHiggs1(ICH4,ICH3,1)*(-2*Mah2(INH3)+MassHpm(ICH3)**2+
     &  MassHpm(ICH4)**2))+aC0THCHCH(INH3,ICH3,ICH4)*(2*
     &  dMCH1L(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,2)*(-2*Mah2(INH3)+
     &  MassHpm(ICH3)**2+MassHpm(ICH4)**2))))/2.
              endif
       enddo 
       enddo 
       enddo 
        
       do ISt3=1,2
       do ISt4=1,2
       selfCT=selfCT+3*CS3S13S14(ISt3)*CS3S14S13(ISt4)*
     &  (aCEpTH1(ISt3,ISt4)*dMSf4(ISt3,ISt4,1)+aC0TH1(ISt3,ISt4)*
     &  dMSf4(ISt3,ISt4,2))
       enddo 
       enddo 
        
       do ISt3=1,2
       selfCT=selfCT+3*CS3S13S14(ISt3)*CS3S14S13(ISt3)*(aCEpTH2(ISt3)*
     &  dMSQ4(1)+aC0TH2(ISt3)*dMSQ4(2))
       enddo 
c
c CTtopo1-GBC
c

        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .lt. 1D-15) then 
       selfCT=selfCT+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*(-2*
     &  (aBEpTNCH(INH4,ICH3)*dZHiggs1(INH3,INH4,1)+aB0TNCH(INH4,ICH3)*
     &  dZHiggs1(INH3,INH4,2)+dZHiggs1(INH3,INH4,3))+
     &  aCepTHHCH(INH3,INH4,ICH3)*(2*dMH1MT4(INH3,INH4,1)+
     &  dZHiggs1(INH3,INH4,1)*(-Mah2(INH3)+Mah2(INH4)))+
     &  aC0THHCH(INH3,INH4,ICH3)*(2*dMH1MT4(INH3,INH4,2)+
     &  dZHiggs1(INH3,INH4,2)*(-Mah2(INH3)+Mah2(INH4)))))/2.
              endif
       enddo 
       enddo 
       enddo      
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .lt. 1D-15) then 
       selfCT=selfCT+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*(-2*
     &  (aBEpTCH(ICH3,ICH4)*dZCHiggs1(ICH4,ICH3,1)+aB0TCH(ICH3,ICH4)*
     &  dZCHiggs1(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,3))+
     &  aCepTHCHCH(INH3,ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,1)+
     &  dZCHiggs1(ICH4,ICH3,1)*(-2*Mah2(INH3)+MassHpm(ICH3)**2+
     &  MassHpm(ICH4)**2))+aC0THCHCH(INH3,ICH3,ICH4)*(2*
     &  dMCH1L(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,2)*(-2*Mah2(INH3)+
     &  MassHpm(ICH3)**2+MassHpm(ICH4)**2))))/2.
              endif
       enddo 
       enddo 
       enddo 
c
c CTtopo2-GBC
c     
       do INH3=ihiggs,6
       selfCT=selfCT+(AETNH(INH3)*dCS1S1S3S3(INH3,INH3,2,2,1)+
     &  A0TNH(INH3)*dCS1S1S3S3(INH3,INH3,2,2,2)+
     &  dCS1S1S3S3(INH3,INH3,2,2,3)*Mah2(INH3))/2.
       enddo 

        
       do ICH3=1,2
       selfCT=selfCT+(AETHpN(ICH3)*dCS3S3S3S3(2,ICH3,2,ICH3,1)+
     &  A0THpN(ICH3)*dCS3S3S3S3(2,ICH3,2,ICH3,2)+
     &  dCS3S3S3S3(2,ICH3,2,ICH3,3)*MassHpm(ICH3)**2)
       enddo 

        
       do ISt3=1,2
       selfCT=selfCT+3*(aAETN(ISt3)*dCS3S13S(ISt3,ISt3,1)+aA0TN(ISt3)*
     &  dCS3S13S(ISt3,ISt3,2)+dCS3S13S(ISt3,ISt3,3)*MassSu2(ISt3))
       enddo 

       selfCT=selfCT+3*(aet5*dCS3SSS14(1)+a0t5*dCS3SSS14(2)+MSQ2*dCS3SSS14(3))
c
c CTtopo3-NoGBC
c       
       selfCT=selfCT+-6*(aet1*(CTBS3R*DCONJG(dS3TB(1))+CTBS3C(2)*
     &  dS3TB(1))+a0t1*(CTBS3R*DCONJG(dS3TB(2))+CTBS3C(2)*dS3TB(2))+
     &  M12*(CTBS3R*DCONJG(dS3TB(3))+CTBS3C(2)*dS3TB(3)))

        
       do INe3=ineu,3
       selfCT=selfCT+-2*(MCha2*dCF11F12S3L(INe3,2,3)*
     &  DCONJG(CF11F12S3L(INe3,2))+MCha2*dCF11F12S3R(INe3,2,3)*
     &  DCONJG(CF11F12S3R(INe3,2))+MCha2*CF11F12S3L(INe3,2)*
     &  DCONJG(dCF11F12S3L(INe3,2,3))+AETNeu(INe3)*
     &  (dCF11F12S3L(INe3,2,1)*DCONJG(CF11F12S3L(INe3,2))+
     &  dCF11F12S3R(INe3,2,1)*DCONJG(CF11F12S3R(INe3,2))+
     &  CF11F12S3L(INe3,2)*DCONJG(dCF11F12S3L(INe3,2,1))+
     &  CF11F12S3R(INe3,2)*DCONJG(dCF11F12S3R(INe3,2,1)))+A0TNeu(INe3)*
     &  (dCF11F12S3L(INe3,2,2)*DCONJG(CF11F12S3L(INe3,2))+
     &  dCF11F12S3R(INe3,2,2)*DCONJG(CF11F12S3R(INe3,2))+
     &  CF11F12S3L(INe3,2)*DCONJG(dCF11F12S3L(INe3,2,2))+
     &  CF11F12S3R(INe3,2)*DCONJG(dCF11F12S3R(INe3,2,2)))+MCha2*
     &  CF11F12S3R(INe3,2)*DCONJG(dCF11F12S3R(INe3,2,3))+MCha*
     &  dCF11F12S3R(INe3,2,3)*DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+
     &  MCha*dCF11F12S3L(INe3,2,3)*DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)+MCha*CF11F12S3R(INe3,2)*DCONJG(dCF11F12S3L(INe3,2,3)
     &  )*MNeu(INe3)+MCha*CF11F12S3L(INe3,2)*
     &  DCONJG(dCF11F12S3R(INe3,2,3))*MNeu(INe3)+MCha*aBEpTChaChi(INe3)
     &  *(DCONJG(CF11F12S3R(INe3,2))*(MCha*dCF11F12S3R(INe3,2,1)+
     &  dCF11F12S3L(INe3,2,1)*MNeu(INe3))+DCONJG(CF11F12S3L(INe3,2))*
     &  (MCha*dCF11F12S3L(INe3,2,1)+dCF11F12S3R(INe3,2,1)*MNeu(INe3))+
     &  CF11F12S3R(INe3,2)*(MCha*DCONJG(dCF11F12S3R(INe3,2,1))+
     &  DCONJG(dCF11F12S3L(INe3,2,1))*MNeu(INe3))+CF11F12S3L(INe3,2)*
     &  (MCha*DCONJG(dCF11F12S3L(INe3,2,1))+
     &  DCONJG(dCF11F12S3R(INe3,2,1))*MNeu(INe3)))+MCha*
     &  aB0TChaChi(INe3)*(DCONJG(CF11F12S3R(INe3,2))*(MCha*
     &  dCF11F12S3R(INe3,2,2)+dCF11F12S3L(INe3,2,2)*MNeu(INe3))+
     &  DCONJG(CF11F12S3L(INe3,2))*(MCha*dCF11F12S3L(INe3,2,2)+
     &  dCF11F12S3R(INe3,2,2)*MNeu(INe3))+CF11F12S3R(INe3,2)*(MCha*
     &  DCONJG(dCF11F12S3R(INe3,2,2))+DCONJG(dCF11F12S3L(INe3,2,2))*
     &  MNeu(INe3))+CF11F12S3L(INe3,2)*(MCha*
     &  DCONJG(dCF11F12S3L(INe3,2,2))+DCONJG(dCF11F12S3R(INe3,2,2))*
     &  MNeu(INe3)))+dCF11F12S3L(INe3,2,3)*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu2(INe3)+dCF11F12S3R(INe3,2,3)*DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu2(INe3)+CF11F12S3L(INe3,2)*DCONJG(dCF11F12S3L(INe3,2,3))*
     &  MNeu2(INe3)+CF11F12S3R(INe3,2)*DCONJG(dCF11F12S3R(INe3,2,3))*
     &  MNeu2(INe3))
       enddo 
c
c CTtopo4+7-NoGBC
c      
       do ICH3=1,2
       do INH3=ihiggs,6
              if( Mah(INH3) + MassHpm(ICH3) .gt. 1D-15) then 
       selfCT=selfCT+(CS1S3S3(INH3,ICH3,2)*dCS1S3S3(INH3,2,ICH3,3)+
     &  aBEpTNCH(INH3,ICH3)*(CS1S3S3(INH3,ICH3,2)*
     &  dCS1S3S3(INH3,2,ICH3,1)+CS1S3S3(INH3,2,ICH3)*
     &  dCS1S3S3(INH3,ICH3,2,1))+aB0TNCH(INH3,ICH3)*
     &  (CS1S3S3(INH3,ICH3,2)*dCS1S3S3(INH3,2,ICH3,2)+
     &  CS1S3S3(INH3,2,ICH3)*dCS1S3S3(INH3,ICH3,2,2))+
     &  CS1S3S3(INH3,2,ICH3)*dCS1S3S3(INH3,ICH3,2,3))
              endif
       enddo 
       enddo 
      
       do ISt3=1,2
       selfCT=selfCT+3*(CS3S14S13(ISt3)*dCS3S13S14(ISt3,3)+
     &  aBEpTH1(ISt3)*(CS3S14S13(ISt3)*dCS3S13S14(ISt3,1)+
     &  CS3S13S14(ISt3)*dCS3S14S13(ISt3,1))+aB0TH1(ISt3)*
     &  (CS3S14S13(ISt3)*dCS3S13S14(ISt3,2)+CS3S13S14(ISt3)*
     &  dCS3S14S13(ISt3,2))+CS3S13S14(ISt3)*dCS3S14S13(ISt3,3))
       enddo 
c
c CTtopo5+8-NoGBC
c

       ICH3=1
       INH3=6
        
       selfCT=selfCT+(CS1S3S3(INH3,ICH3,2)*dCS1S3S3(INH3,2,ICH3,3)+
     &  aBEpTNCH(INH3,ICH3)*(CS1S3S3(INH3,ICH3,2)*
     &  dCS1S3S3(INH3,2,ICH3,1)+CS1S3S3(INH3,2,ICH3)*
     &  dCS1S3S3(INH3,ICH3,2,1))+aB0TNCH(INH3,ICH3)*
     &  (CS1S3S3(INH3,ICH3,2)*dCS1S3S3(INH3,2,ICH3,2)+
     &  CS1S3S3(INH3,2,ICH3)*dCS1S3S3(INH3,ICH3,2,2))+
     &  CS1S3S3(INH3,2,ICH3)*dCS1S3S3(INH3,ICH3,2,3)) 

c
c CTtopo5+8-GBC
c

        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3) + Mah(INH4) .gt. 1D-15) then 
       selfCT=selfCT+(CS1S1S3S3(INH3,INH4,2,2)*(2*dMH1MT4(INH3,INH4,3)-
     &  2*AETNH(INH4)*dZHiggs1(INH3,INH4,1)-2*A0TNH(INH4)*
     &  dZHiggs1(INH3,INH4,2)-dZHiggs1(INH3,INH4,3)*(Mah2(INH3)+
     &  Mah2(INH4))+aBEpTNH(INH3,INH4)*(2*dMH1MT4(INH3,INH4,1)+
     &  dZHiggs1(INH3,INH4,1)*(-Mah2(INH3)+Mah2(INH4)))+
     &  aB0TNH(INH3,INH4)*(2*dMH1MT4(INH3,INH4,2)+dZHiggs1(INH3,INH4,2)
     &  *(-Mah2(INH3)+Mah2(INH4)))))/4.
              endif
       enddo 
       enddo 

        
       do ICH3=1,2
       do ICH4=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .gt. 1D-15) then 
       selfCT=selfCT+(CS3S3S3S3(2,ICH4,2,ICH3)*(2*dMCH1L(ICH4,ICH3,3)-
     &  2*AETHpN(ICH4)*dZCHiggs1(ICH4,ICH3,1)-2*A0THpN(ICH4)*
     &  dZCHiggs1(ICH4,ICH3,2)-dZCHiggs1(ICH4,ICH3,3)*(MassHpm(ICH3)**
     &  2+MassHpm(ICH4)**2)+aBEpTCH(ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,1)+
     &  dZCHiggs1(ICH4,ICH3,1)*(-MassHpm(ICH3)**2+MassHpm(ICH4)**2))+
     &  aB0TCH(ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,2)
     &  *(-MassHpm(ICH3)**2+MassHpm(ICH4)**2))))/2.
              endif
       enddo 
       enddo 
        
       do ISt3=1,2
       do ISt4=1,2
       selfCT=selfCT+3*CS3SSS13(ISt4,ISt3)*(aBEpT1(ISt3,ISt4)*
     &  dMSf4(ISt4,ISt3,1)+aB0T1(ISt3,ISt4)*dMSf4(ISt4,ISt3,2)+
     &  dMSf4(ISt4,ISt3,3))
       enddo 
       enddo 

        
       selfCT=selfCT+3*CS3SSS14*(aBEpT5*dMSQ4(1)+aB0T5*dMSQ4(2)+dMSQ4(3))
c
c CTtopo6-NoGBC
c

       INH3=6
       INH4=6
        

       selfCT=selfCT+(CS1S1S3S3(INH3,INH4,2,2)*(2*dMH1MT4(INH3,INH4,3)-
     &  2*AETNH(INH4)*dZHiggs1(INH3,INH4,1)-2*A0TNH(INH4)*
     &  dZHiggs1(INH3,INH4,2)-dZHiggs1(INH3,INH4,3)*(Mah2(INH3)+
     &  Mah2(INH4))+aBEpTNH(INH3,INH4)*(2*dMH1MT4(INH3,INH4,1)+
     &  dZHiggs1(INH3,INH4,1)*(-Mah2(INH3)+Mah2(INH4)))+
     &  aB0TNH(INH3,INH4)*(2*dMH1MT4(INH3,INH4,2)+dZHiggs1(INH3,INH4,2)
     &  *(-Mah2(INH3)+Mah2(INH4)))))/4.

       ICH3=1
       ICH4=1
       selfCT=selfCT+(CS3S3S3S3(2,ICH4,2,ICH3)*(2*dMCH1L(ICH4,ICH3,3)-
     &  2*AETHpN(ICH4)*dZCHiggs1(ICH4,ICH3,1)-2*A0THpN(ICH4)*
     &  dZCHiggs1(ICH4,ICH3,2)-dZCHiggs1(ICH4,ICH3,3)*(MassHpm(ICH3)**
     &  2+MassHpm(ICH4)**2)+aBEpTCH(ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,1)+
     &  dZCHiggs1(ICH4,ICH3,1)*(-MassHpm(ICH3)**2+MassHpm(ICH4)**2))+
     &  aB0TCH(ICH3,ICH4)*(2*dMCH1L(ICH4,ICH3,2)+dZCHiggs1(ICH4,ICH3,2)
     &  *(-MassHpm(ICH3)**2+MassHpm(ICH4)**2))))/2.
c
c CTtopo6-GBC
c

c end One-loop inserted Counterterms diagrams

c start Genuine one-loop diagram
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .gt. 1D-15) then 
       self=self-(CS1S1S1S1(INH3,INH4,INH5,INH5)*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH4,ICH3,2)*(A0TNH(INH5)*aC0THHCH(INH3,INH4,ICH3)+
     &  aCepTHHCH(INH3,INH4,ICH3)*Mah2(INH5)))/32.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .gt. 1D-15) then 
       self=self-(CS1S3S3(INH3,2,ICH3)*(CS1S1S3S3(INH4,INH4,ICH3,ICH4)
     &  *CS1S3S3(INH3,ICH4,2)*(A0TNH(INH4)*aC0THCHCH(INH3,ICH3,ICH4)+
     &  aCepTHCHCH(INH3,ICH3,ICH4)*Mah2(INH4))+2*
     &  CS1S1S3S3(INH3,INH4,ICH4,ICH4)*CS1S3S3(INH4,ICH3,2)*
     &  (A0THpN(ICH4)*aC0THHCH(INH3,INH4,ICH3)+
     &  aCepTHHCH(INH3,INH4,ICH3)*MassHpm(ICH4)**2)))/32.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .gt. 1D-15) then 
       self=self-(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S3S3S3(ICH3,ICH5,ICH4,ICH5)*(A0THpN(ICH5)*
     &  aC0THCHCH(INH3,ICH3,ICH4)+aCepTHCHCH(INH3,ICH3,ICH4)*
     &  MassHpm(ICH5)**2))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 

        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .gt. 1D-15) then 
       self=self+(-3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*
     &  CSSSS(INH3,INH4,ISt3,ISt3)*(aA0TN(ISt3)*
     &  aC0THHCH(INH3,INH4,ICH3)+aCepTHHCH(INH3,INH4,ICH3)*
     &  MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
    
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .gt. 1D-15) then 
       self=self+(-3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S3S13S13(ICH3,ICH4,ISt3,ISt3)*(aA0TN(ISt3)*
     &  aC0THCHCH(INH3,ICH3,ICH4)+aCepTHCHCH(INH3,ICH3,ICH4)*
     &  MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .gt. 1D-15) then 
       self=self+(-3*(a0t5*aC0THCHCH(INH3,ICH3,ICH4)+MSQ2*
     &  aCepTHCHCH(INH3,ICH3,ICH4))*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH3,ICH4,2)*CS3S3S14S14(ICH3,ICH4))/16.
              endif
       enddo 
       enddo 
       enddo 
c
c topo1-NoGBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .lt. 1D-15) then 
       self=self-(CS1S1S1S1(INH3,INH4,INH5,INH5)*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH4,ICH3,2)*(A0TNH(INH5)*aC0THHCH(INH3,INH4,ICH3)+
     &  aCepTHHCH(INH3,INH4,ICH3)*Mah2(INH5)))/32.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .lt. 1D-15) then 
       self=self-(CS1S3S3(INH3,2,ICH3)*(CS1S1S3S3(INH4,INH4,ICH3,ICH4)
     &  *CS1S3S3(INH3,ICH4,2)*(A0TNH(INH4)*aC0THCHCH(INH3,ICH3,ICH4)+
     &  aCepTHCHCH(INH3,ICH3,ICH4)*Mah2(INH4))+2*
     &  CS1S1S3S3(INH3,INH4,ICH4,ICH4)*CS1S3S3(INH4,ICH3,2)*
     &  (A0THpN(ICH4)*aC0THHCH(INH3,INH4,ICH3)+
     &  aCepTHHCH(INH3,INH4,ICH3)*MassHpm(ICH4)**2)))/32.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .lt. 1D-15) then 
       self=self-(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S3S3S3(ICH3,ICH5,ICH4,ICH5)*(A0THpN(ICH5)*
     &  aC0THCHCH(INH3,ICH3,ICH4)+aCepTHCHCH(INH3,ICH3,ICH4)*
     &  MassHpm(ICH5)**2))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3)*Mah(INH4) + Mah(INH3)*MassHpm(ICH3) + Mah(INH4)*MassHpm(ICH3) .lt. 1D-15) then 
       self=self+(-3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*
     &  CSSSS(INH3,INH4,ISt3,ISt3)*(aA0TN(ISt3)*
     &  aC0THHCH(INH3,INH4,ICH3)+aCepTHHCH(INH3,INH4,ICH3)*
     &  MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .lt. 1D-15) then 
       self=self+(-3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S3S13S13(ICH3,ICH4,ISt3,ISt3)*(aA0TN(ISt3)*
     &  aC0THCHCH(INH3,ICH3,ICH4)+aCepTHCHCH(INH3,ICH3,ICH4)*
     &  MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( Mah(INH3)*MassHpm(ICH3) + Mah(INH3)*MassHpm(ICH4) + MassHpm(ICH3)*MassHpm(ICH4) .lt. 1D-15) then 
       self=self+(-3*(a0t5*aC0THCHCH(INH3,ICH3,ICH4)+MSQ2*
     &  aCepTHCHCH(INH3,ICH3,ICH4))*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH3,ICH4,2)*CS3S3S14S14(ICH3,ICH4))/16.
              endif
       enddo 
       enddo 
       enddo 
c
c topo1-GBC
c

       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Mah(INH3) + Mah(INH4) .gt. 1D-15) then 
       self=self-(CS1S1S1S1(INH3,INH4,INH5,INH5)*
     &  CS1S1S3S3(INH3,INH4,2,2)*(A0TNH(INH5)*aB0TNH(INH3,INH4)+
     &  AETNH(INH5)+aBEpTNH(INH3,INH4)*Mah2(INH5)))/64.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3) + Mah(INH4) .gt. 1D-15) then 
       self=self-(CS1S1S3S3(INH3,INH4,2,2)*
     &  CS1S1S3S3(INH3,INH4,ICH3,ICH3)*(A0THpN(ICH3)*aB0TNH(INH3,INH4)+
     &  AETHpN(ICH3)+aBEpTNH(INH3,INH4)*MassHpm(ICH3)**2))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( MassHpm(ICH3) + MassHpm(ICH4) .gt. 1D-15) then 
       self=self-(CS1S1S3S3(INH3,INH3,ICH4,ICH3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4)*(A0TNH(INH3)*aB0TCH(ICH3,ICH4)+
     &  AETNH(INH3)+aBEpTCH(ICH3,ICH4)*Mah2(INH3)))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .gt. 1D-15) then 
       self=self-(CS3S3S3S3(2,ICH3,2,ICH4)*
     &  CS3S3S3S3(ICH4,ICH5,ICH3,ICH5)*(A0THpN(ICH5)*aB0TCH(ICH3,ICH4)+
     &  AETHpN(ICH5)+aBEpTCH(ICH3,ICH4)*MassHpm(ICH5)**2))/16.
              endif
       enddo 
       enddo 
       enddo 
        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3) + Mah(INH4) .gt. 1D-15) then 
       self=self+(-3*CS1S1S3S3(INH3,INH4,2,2)*
     &  CSSSS(INH3,INH4,ISt3,ISt3)*(aAETN(ISt3)+aA0TN(ISt3)*
     &  aB0TNH(INH3,INH4)+aBEpTNH(INH3,INH4)*MassSu2(ISt3)))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ISt3=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .gt. 1D-15) then 
       self=self+(-3*CS3S3S13S13(ICH4,ICH3,ISt3,ISt3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4)*(aAETN(ISt3)+aA0TN(ISt3)*
     &  aB0TCH(ICH3,ICH4)+aBEpTCH(ICH3,ICH4)*MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .gt. 1D-15) then 
       self=self+(-3*(aet5+a0t5*aB0TCH(ICH3,ICH4)+MSQ2*
     &  aBEpTCH(ICH3,ICH4))*CS3S3S14S14(ICH4,ICH3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4))/16.
              endif
       enddo 
       enddo 
c
c topo11-NoGBC
c
        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Mah(INH3) + Mah(INH4) .lt. 1D-15) then 
       self=self-(CS1S1S1S1(INH3,INH4,INH5,INH5)*
     &  CS1S1S3S3(INH3,INH4,2,2)*(A0TNH(INH5)*aB0TNH(INH3,INH4)+
     &  AETNH(INH5)+aBEpTNH(INH3,INH4)*Mah2(INH5)))/64.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3) + Mah(INH4) .lt. 1D-15) then 
       self=self-(CS1S1S3S3(INH3,INH4,2,2)*
     &  CS1S1S3S3(INH3,INH4,ICH3,ICH3)*(A0THpN(ICH3)*aB0TNH(INH3,INH4)+
     &  AETHpN(ICH3)+aBEpTNH(INH3,INH4)*MassHpm(ICH3)**2))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( MassHpm(ICH3) + MassHpm(ICH4) .lt. 1D-15) then 
       self=self-(CS1S1S3S3(INH3,INH3,ICH4,ICH3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4)*(A0TNH(INH3)*aB0TCH(ICH3,ICH4)+
     &  AETNH(INH3)+aBEpTCH(ICH3,ICH4)*Mah2(INH3)))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .lt. 1D-15) then 
       self=self-(CS3S3S3S3(2,ICH3,2,ICH4)*
     &  CS3S3S3S3(ICH4,ICH5,ICH3,ICH5)*(A0THpN(ICH5)*aB0TCH(ICH3,ICH4)+
     &  AETHpN(ICH5)+aBEpTCH(ICH3,ICH4)*MassHpm(ICH5)**2))/16.
              endif
       enddo 
       enddo 
       enddo 
        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3) + Mah(INH4) .lt. 1D-15) then 
       self=self+(-3*CS1S1S3S3(INH3,INH4,2,2)*
     &  CSSSS(INH3,INH4,ISt3,ISt3)*(aAETN(ISt3)+aA0TN(ISt3)*
     &  aB0TNH(INH3,INH4)+aBEpTNH(INH3,INH4)*MassSu2(ISt3)))/32.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ISt3=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .lt. 1D-15) then 
       self=self+(-3*CS3S3S13S13(ICH4,ICH3,ISt3,ISt3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4)*(aAETN(ISt3)+aA0TN(ISt3)*
     &  aB0TCH(ICH3,ICH4)+aBEpTCH(ICH3,ICH4)*MassSu2(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
              if( MassHpm(ICH3) + MassHpm(ICH4) .lt. 1D-15) then 
       self=self+(-3*(aet5+a0t5*aB0TCH(ICH3,ICH4)+MSQ2*
     &  aBEpTCH(ICH3,ICH4))*CS3S3S14S14(ICH4,ICH3)*
     &  CS3S3S3S3(2,ICH3,2,ICH4))/16.
              endif
       enddo 
       enddo 
c
c topo11-GBC
c
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3) + MassHpm(ICH3) .gt. 1D-15) then 
              if( Mah(INH4) + MassHpm(ICH4) .gt. 1D-15) then 
       self=self-((aB0TNCH(INH3,ICH3)*aB0TNCH(INH4,ICH4)+
     &  aBEpTNCH(INH3,ICH3)+aBEpTNCH(INH4,ICH4))*
     &  CS1S1S3S3(INH3,INH4,ICH3,ICH4)*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH4,ICH4,2))/16.
              endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3) + MassHpm(ICH3) .gt. 1D-15) then 
       self=self+(-3*(aB0TH1(ISt3)*aB0TNCH(INH3,ICH3)+aBEpTH1(ISt3)+
     &  aBEpTNCH(INH3,ICH3))*(CS1S3S3(INH3,ICH3,2)*
     &  CS1S3SSC(INH3,ICH3,ISt3)*CS3S13S14(ISt3)+CS1S3S3(INH3,2,ICH3)*
     &  CS1S3SS(INH3,ICH3,ISt3)*CS3S14S13(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 

c
c topo12-NoGBC
c
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Mah(INH3) + MassHpm(ICH3) .lt. 1D-15 .or. (Mah(INH4) + MassHpm(ICH4)) .lt. 1D-15) then 
       self=self-((aB0TNCH(INH3,ICH3)*aB0TNCH(INH4,ICH4)+
     &  aBEpTNCH(INH3,ICH3)+aBEpTNCH(INH4,ICH4))*
     &  CS1S1S3S3(INH3,INH4,ICH3,ICH4)*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH4,ICH4,2))/16.
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Mah(INH3) + MassHpm(ICH3) .lt. 1D-15) then 
       self=self+(-3*(aB0TH1(ISt3)*aB0TNCH(INH3,ICH3)+aBEpTH1(ISt3)+
     &  aBEpTNCH(INH3,ICH3))*(CS1S3S3(INH3,ICH3,2)*
     &  CS1S3SSC(INH3,ICH3,ISt3)*CS3S13S14(ISt3)+CS1S3S3(INH3,2,ICH3)*
     &  CS1S3SS(INH3,ICH3,ISt3)*CS3S14S13(ISt3)))/16.
              endif
       enddo 
       enddo 
       enddo 

c
c topo12-GBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       self=self+(CS1S1S3S3(INH3,INH4,2,ICH3)*
     &  CS1S1S3S3(INH3,INH4,ICH3,2)*FD111HHpH(1,INH3,ICH3,INH4))/32.
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       self=self+(CS3S3S3S3(2,ICH5,ICH3,ICH4)*
     &  CS3S3S3S3(ICH3,ICH4,2,ICH5)*FD111HpHpHp(1,ICH3,ICH4,ICH5))/32.
       enddo 
       enddo 
       enddo 

c
c topo15-NoGBC
c
c
c topo15-GBC
c
        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(Mah(INH3) - Mah(INH4)) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(2*MCha2*AETNH(INH3)*
     &  CFF12S1L(INH4)*CFF12S1R(INH3)-2*MCha2*AETNH(INH4)*
     &  CFF12S1L(INH4)*CFF12S1R(INH3)+2*MCha2*AETNH(INH3)*
     &  CFF12S1L(INH3)*CFF12S1R(INH4)-2*MCha2*AETNH(INH4)*
     &  CFF12S1L(INH3)*CFF12S1R(INH4)-2*MCha2*CFF12S1L(INH3)*
     &  CFF12S1L(INH4)*FD111ChaHCha(1,INH3)-2*MCha2*CFF12S1L(INH4)*
     &  CFF12S1R(INH3)*FD111ChaHCha(1,INH3)-2*MCha2*CFF12S1L(INH3)*
     &  CFF12S1R(INH4)*FD111ChaHCha(1,INH3)-2*MCha2*CFF12S1R(INH3)*
     &  CFF12S1R(INH4)*FD111ChaHCha(1,INH3)+2*MCha2*CFF12S1L(INH3)*
     &  CFF12S1L(INH4)*FD111ChaHCha(1,INH4)+2*MCha2*CFF12S1L(INH4)*
     &  CFF12S1R(INH3)*FD111ChaHCha(1,INH4)+2*MCha2*CFF12S1L(INH3)*
     &  CFF12S1R(INH4)*FD111ChaHCha(1,INH4)+2*MCha2*CFF12S1R(INH3)*
     &  CFF12S1R(INH4)*FD111ChaHCha(1,INH4)+6*M12*(-FD111THT(1,INH3)+
     &  FD111THT(1,INH4))*hy(INH3)*hy(INH4)+6*M12*AETNH(INH3)*hy(INH4)*
     &  hyc(INH3)-6*M12*AETNH(INH4)*hy(INH4)*hyc(INH3)-6*M12*
     &  FD111THT(1,INH3)*hy(INH4)*hyc(INH3)+6*M12*FD111THT(1,INH4)*
     &  hy(INH4)*hyc(INH3)+6*M12*AETNH(INH3)*hy(INH3)*hyc(INH4)-6*M12*
     &  AETNH(INH4)*hy(INH3)*hyc(INH4)-6*M12*FD111THT(1,INH3)*hy(INH3)*
     &  hyc(INH4)+6*M12*FD111THT(1,INH4)*hy(INH3)*hyc(INH4)-6*M12*
     &  FD111THT(1,INH3)*hyc(INH3)*hyc(INH4)+6*M12*FD111THT(1,INH4)*
     &  hyc(INH3)*hyc(INH4)+2*A0TNH(INH3)*(a0tcha*CFF12S1L(INH4)*
     &  CFF12S1R(INH3)+a0tcha*CFF12S1L(INH3)*CFF12S1R(INH4)+3*a0t1*
     &  hy(INH4)*hyc(INH3)+3*a0t1*hy(INH3)*hyc(INH4))-2*A0TNH(INH4)*
     &  (a0tcha*CFF12S1L(INH4)*CFF12S1R(INH3)+a0tcha*CFF12S1L(INH3)*
     &  CFF12S1R(INH4)+3*a0t1*hy(INH4)*hyc(INH3)+3*a0t1*hy(INH3)*
     &  hyc(INH4))+2*aetcha*CFF12S1L(INH4)*CFF12S1R(INH3)*Mah2(INH3)+2*
     &  aetcha*CFF12S1L(INH3)*CFF12S1R(INH4)*Mah2(INH3)+CFF12S1L(INH4)*
     &  CFF12S1R(INH3)*FD111ChaHCha(1,INH3)*Mah2(INH3)+CFF12S1L(INH3)*
     &  CFF12S1R(INH4)*FD111ChaHCha(1,INH3)*Mah2(INH3)+6*aet1*hy(INH4)*
     &  hyc(INH3)*Mah2(INH3)+3*FD111THT(1,INH3)*hy(INH4)*hyc(INH3)*
     &  Mah2(INH3)+6*aet1*hy(INH3)*hyc(INH4)*Mah2(INH3)+3*
     &  FD111THT(1,INH3)*hy(INH3)*hyc(INH4)*Mah2(INH3)-((CFF12S1L(INH4)
     &  *CFF12S1R(INH3)+CFF12S1L(INH3)*CFF12S1R(INH4))*(2*aetcha+
     &  FD111ChaHCha(1,INH4))+3*(2*aet1+FD111THT(1,INH4))*(hy(INH4)*
     &  hyc(INH3)+hy(INH3)*hyc(INH4)))*Mah2(INH4)))/(32.*(Mah(INH3)**2-
     &  Mah(INH4)**2))
              else
                if( Mah(INH3) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(-2*MCha2*(CFF12S1L(INH3)*
     &  CFF12S1L(INH4)+CFF12S1R(INH3)*CFF12S1R(INH4))*
     &  FD211HChaCha(1,INH4)-6*M12*FD211HTT(1,INH4)*(hy(INH3)*hy(INH4)+
     &  hyc(INH3)*hyc(INH4))+(3*(hy(INH4)*hyc(INH3)+hy(INH3)*hyc(INH4))
     &  *(2*(a0t1-M12)*A0TNH(INH4)+2*M12*AETNH(INH4)+Mah2(INH4)*(-2*
     &  a0t1+2*aet1+FD111THT(1,INH4)+FD211HTT(1,INH4)*(-2*M12+
     &  Mah2(INH4)))))/Mah2(INH4)+((CFF12S1L(INH4)*CFF12S1R(INH3)+
     &  CFF12S1L(INH3)*CFF12S1R(INH4))*(2*(a0tcha-MCha2)*A0TNH(INH4)+2*
     &  MCha2*AETNH(INH4)+Mah2(INH4)*(-2*a0tcha+2*aetcha+
     &  FD111ChaHCha(1,INH4)+FD211HChaCha(1,INH4)*(-2*MCha2+Mah2(INH4))
     &  )))/Mah2(INH4)))/32.
                endif
              endif
       enddo 
       enddo 
        
       do INe3=ineu,3
       do INe4=ineu,3
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(Mah(INH3) - Mah(INH4)) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(-((CFF11S1L(INe4,INe3,INH3)
     &  *CFF11S1R(INe3,INe4,INH4)+CFF11S1L(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*(FD111ChiChiH(2,INe3,INe4,INH3)-
     &  FD111ChiChiH(2,INe3,INe4,INH4)))-(CFF11S1L(INe3,INe4,INH4)*
     &  CFF11S1L(INe4,INe3,INH3)+CFF11S1R(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*(FD111ChiChiH(1,INe3,INe4,INH3)-
     &  FD111ChiChiH(1,INe3,INe4,INH4))*MNeu(INe3)*MNeu(INe4)))/(32.*
     &  (Mah(INH3)**2-Mah(INH4)**2))
              else
                if( Mah(INH3) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(-((CFF11S1L(INe3,INe4,INH4)
     &  *CFF11S1L(INe4,INe3,INH3)+CFF11S1R(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*FD211HNeuNeu(1,INH4,INe4,INe3)*
     &  MNeu(INe3)*MNeu(INe4))+((CFF11S1L(INe4,INe3,INH3)*
     &  CFF11S1R(INe3,INe4,INH4)+CFF11S1L(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*(A0TNeu(INe3)*(A0TNH(INH4)-Mah2(INH4)
     &  )+A0TNeu(INe4)*(A0TNH(INH4)-Mah2(INH4))+Mah2(INH4)*
     &  (AETNeu(INe3)+AETNeu(INe4)+FD111ChiChiH(1,INe3,INe4,INH4)+
     &  FD211HNeuNeu(1,INH4,INe4,INe3)*(Mah2(INH4)-MNeu2(INe3)-
     &  MNeu2(INe4)))-(A0TNH(INH4)-AETNH(INH4))*(MNeu2(INe3)+
     &  MNeu2(INe4))))/(2.*Mah2(INH4))))/32.
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
              if( Abs(MassHpm(ICH3) - MassHpm(ICH4)) .gt. 1D-15) then 
       self=self+(3*CS3S3S3S3(2,ICH4,2,ICH3)*CTBS3(ICH3)*CTBS3C(ICH4)*
     &  (a0t1*A0THpN(ICH3)-a0t1*A0THpN(ICH4)+M12*AETHpN(ICH3)-M12*
     &  AETHpN(ICH4)-M12*aK0TTH(ICH3)+M12*aK0TTH(ICH4)+aet1*
     &  MassHpm(ICH3)**2+aK0TTH(ICH3)*MassHpm(ICH3)**2-(aet1+
     &  aK0TTH(ICH4))*MassHpm(ICH4)**2))/(16.*(MassHpm(ICH3)**2-
     &  MassHpm(ICH4)**2))
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
       self=self+(3*CS3S3S3S3(2,ICH4,2,ICH3)*CTBS3(ICH3)*CTBS3C(ICH4)*
     &  (2*M12*AETHpN(ICH4)+2*A0THpN(ICH4)*(a0t1-M12+MassHpm(ICH4)**2)+
     &  MassHpm(ICH4)**2*(2*aet1+M12+2*aK0TTH(ICH4)+MassHpm(ICH4)**2)))
     &  /(16.*MassHpm(ICH4)**2)
                endif
              endif
       enddo 
       enddo 

       do ICH3=1,2
       do ICH4=1,2
       do INe3=ineu,3
              if( Abs(MassHpm(ICH3) - MassHpm(ICH4)) .gt. 1D-15) then 
       self=self+(CS3S3S3S3(2,ICH3,2,ICH4)*(-((CF11F12S3L(INe3,ICH3)*
     &  DCONJG(CF11F12S3L(INe3,ICH4))+CF11F12S3R(INe3,ICH3)*
     &  DCONJG(CF11F12S3R(INe3,ICH4)))*(FD111ChaChiHp(2,INe3,ICH3)-
     &  FD111ChaChiHp(2,INe3,ICH4)))-MCha*(CF11F12S3R(INe3,ICH3)*
     &  DCONJG(CF11F12S3L(INe3,ICH4))+CF11F12S3L(INe3,ICH3)*
     &  DCONJG(CF11F12S3R(INe3,ICH4)))*(FD111ChaChiHp(1,INe3,ICH3)-
     &  FD111ChaChiHp(1,INe3,ICH4))*MNeu(INe3)))/(8.*(MassHpm(ICH3)**2-
     &  MassHpm(ICH4)**2))
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
       self=self+(CS3S3S3S3(2,ICH3,2,ICH4)*(-(MCha*
     &  (CF11F12S3R(INe3,ICH3)*DCONJG(CF11F12S3L(INe3,ICH4))+
     &  CF11F12S3L(INe3,ICH3)*DCONJG(CF11F12S3R(INe3,ICH4)))*
     &  FD211HpChiCha(1,ICH4,INe3)*MNeu(INe3))+((CF11F12S3L(INe3,ICH3)*
     &  DCONJG(CF11F12S3L(INe3,ICH4))+CF11F12S3R(INe3,ICH3)*
     &  DCONJG(CF11F12S3R(INe3,ICH4)))*(A0THpN(ICH4)*(a0tcha-MCha2+
     &  A0TNeu(INe3)-MNeu2(INe3))+AETHpN(ICH4)*(MCha2+MNeu2(INe3))+
     &  MassHpm(ICH4)**2*(-a0tcha+aetcha-A0TNeu(INe3)+AETNeu(INe3)+
     &  FD111ChaChiHp(1,INe3,ICH4)-FD211HpChiCha(1,ICH4,INe3)*(MCha2-
     &  MassHpm(ICH4)**2+MNeu2(INe3)))))/(2.*MassHpm(ICH4)**2)))/8.
                endif
              endif
       enddo 
       enddo 
       enddo 

c
c topo13-NoGBC
c

        INH3=6
        INH4=6
        
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(-(MCha2*(CFF12S1L(INH3)*
     &  CFF12S1L(INH4)+CFF12S1R(INH3)*CFF12S1R(INH4))*
     &  FD211HChaCha(1,INH4))-3*M12*FD211HTT(1,INH4)*(hy(INH3)*hy(INH4)
     &  +hyc(INH3)*hyc(INH4))+((hy(INH4)*hyc(INH3)+hy(INH3)*hyc(INH4))*
     &  (12*aet1-12*a0t1*logx+6*logx**2*M12+M12*Pi**2+6*
     &  FD111THT(1,INH4)+6*FD211HTT(1,INH4)*(-2*M12+Mah2(INH4))))/4.+
     &  ((CFF12S1L(INH4)*CFF12S1R(INH3)+CFF12S1L(INH3)*CFF12S1R(INH4))*
     &  (12*aetcha-12*a0tcha*logx+6*logx**2*MCha2+MCha2*Pi**2+6*
     &  FD111ChaHCha(1,INH4)+6*FD211HChaCha(1,INH4)*(-2*MCha2+
     &  Mah2(INH4))))/12.))/16.
        INH3=6
        INH4=6     
       do INe3=ineu,3
       do INe4=ineu,3
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*(-((CFF11S1L(INe3,INe4,INH4)
     &  *CFF11S1L(INe4,INe3,INH3)+CFF11S1R(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*FD211HNeuNeu(1,INH4,INe4,INe3)*
     &  MNeu(INe3)*MNeu(INe4))+((CFF11S1L(INe4,INe3,INH3)*
     &  CFF11S1R(INe3,INe4,INH4)+CFF11S1L(INe3,INe4,INH4)*
     &  CFF11S1R(INe4,INe3,INH3))*(-12*logx*A0TNeu(INe3)-12*logx*
     &  A0TNeu(INe4)+12*(AETNeu(INe3)+AETNeu(INe4)+
     &  FD111ChiChiH(1,INe3,INe4,INH4)+FD211HNeuNeu(1,INH4,INe4,INe3)*
     &  Mah2(INH4))+(6*logx**2+Pi**2-12*FD211HNeuNeu(1,INH4,INe4,INe3))
     &  *(MNeu2(INe3)+MNeu2(INe4))))/24.))/32.
       enddo 
       enddo 
        ICH3=1
        ICH4=1
       
       self=self+((CS3S3S3S3(2,ICH4,2,ICH3)*CTBS3(ICH3)*
     &  CTBS3C(ICH4)*(2*M12*(-12*a0t1*(-1+logx)+6*(aet1+aI0AT+aK0TT+
     &  M12)+M12*(3*(logx**2+logx2)+Pi**2))+(-12*aI0AT+12*a0t1*(-2+
     &  logx)-M12*(12+6*logx2+Pi**2))*MassHpm(ICH4)**2))/(64.*M12))     

        ICH3=1
        ICH4=1
        
       do INe3=ineu,3
       self=self+(CS3S3S3S3(2,ICH3,2,ICH4)*(-(MCha*
     &  (CF11F12S3R(INe3,ICH3)*DCONJG(CF11F12S3L(INe3,ICH4))+
     &  CF11F12S3L(INe3,ICH3)*DCONJG(CF11F12S3R(INe3,ICH4)))*
     &  FD211HpChiCha(1,ICH4,INe3)*MNeu(INe3))+((CF11F12S3L(INe3,ICH3)*
     &  DCONJG(CF11F12S3L(INe3,ICH4))+CF11F12S3R(INe3,ICH3)*
     &  DCONJG(CF11F12S3R(INe3,ICH4)))*(12*aetcha-12*a0tcha*logx+6*
     &  logx**2*MCha2+MCha2*Pi**2-12*logx*A0TNeu(INe3)+12*AETNeu(INe3)+
     &  12*FD111ChaChiHp(1,INe3,ICH4)+(6*logx**2+Pi**2)*MNeu2(INe3)-12*
     &  FD211HpChiCha(1,ICH4,INe3)*(MCha2-MassHpm(ICH4)**2+MNeu2(INe3))
     &  ))/24.))/8.
       enddo 

c
c topo13-GBC
c
        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
       do INH6=ihiggs,6
              if( Abs(Mah(INH5) - Mah(INH6)) .gt. 1D-15) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S1(INH3,INH4,INH6)*
     &  CS1S1S3S3(INH5,INH6,2,2)*(FD111HHH(1,INH3,INH4,INH5)-
     &  FD111HHH(1,INH3,INH4,INH6)))/(64.*(Mah(INH5)**2-Mah(INH6)**2))
              else
                if( Mah(INH5) .gt. 1D-15) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S1(INH3,INH4,INH6)*
     &  CS1S1S3S3(INH5,INH6,2,2)*FD211HHH(1,INH6,INH4,INH3))/64.
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(Mah(INH3) - Mah(INH4)) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*CS1S3S3(INH3,ICH3,ICH4)*
     &  CS1S3S3(INH4,ICH4,ICH3)*(FD111HpHpH(1,ICH3,ICH4,INH3)-
     &  FD111HpHpH(1,ICH3,ICH4,INH4)))/(32.*(Mah(INH3)**2-Mah(INH4)**2)
     &  )
              else
                if( Mah(INH3) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*CS1S3S3(INH3,ICH3,ICH4)*
     &  CS1S3S3(INH4,ICH4,ICH3)*FD211HHpHp(1,INH4,ICH4,ICH3))/32.
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
              if( Abs(MassHpm(ICH4) - MassHpm(ICH5)) .gt. 1D-15) then 
       self=self+(CS1S3S3(INH3,ICH3,ICH4)*CS1S3S3(INH3,ICH5,ICH3)*
     &  CS3S3S3S3(2,ICH4,2,ICH5)*(FD111HHpHp(1,INH3,ICH3,ICH4)-
     &  FD111HHpHp(1,INH3,ICH3,ICH5)))/(16.*(MassHpm(ICH4)**2-
     &  MassHpm(ICH5)**2))
              else
                if( MassHpm(ICH4) .gt. 1D-15) then 
       self=self+(CS1S3S3(INH3,ICH3,ICH4)*CS1S3S3(INH3,ICH5,ICH3)*
     &  CS3S3S3S3(2,ICH4,2,ICH5)*FD211HpHpH(1,ICH5,ICH3,INH3))/16.
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

        
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
       do ISt4=1,2
              if( Abs(Mah(INH3) - Mah(INH4)) .gt. 1D-15) then 
       self=self+(3*CS1S1S3S3(INH3,INH4,2,2)*CSSS(INH3,ISt3,ISt4)*
     &  CSSS(INH4,ISt4,ISt3)*(FD111HSTST(1,INH3,ISt3,ISt4)-
     &  FD111HSTST(1,INH4,ISt3,ISt4)))/(32.*(Mah(INH3)**2-Mah(INH4)**2)
     &  )
              else
                if( Mah(INH3) .gt. 1D-15) then 
       self=self+(3*CS1S1S3S3(INH3,INH4,2,2)*CSSS(INH3,ISt3,ISt4)*
     &  CSSS(INH4,ISt4,ISt3)*FD211HSTST(1,INH4,ISt4,ISt3))/32.
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ISt3=1,2
              if( Abs(MassHpm(ICH3) - MassHpm(ICH4)) .gt. 1D-15) then 
       self=self+(3*CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*
     &  CS3S3S3S3(2,ICH4,2,ICH3)*(FD111HpSTSB(1,ICH3,ISt3)-
     &  FD111HpSTSB(1,ICH4,ISt3)))/(16.*(MassHpm(ICH3)**2-MassHpm(ICH4)
     &  **2))
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
       self=self+(3*CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*
     &  CS3S3S3S3(2,ICH4,2,ICH3)*FD211HpSTSB(1,ICH4,ISt3))/16.
                endif
              endif
       enddo 
       enddo 
       enddo
c
c topo14-NoGBC
c

        INH5=6
        INH6=6       
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S1(INH3,INH4,INH6)*
     &  CS1S1S3S3(INH5,INH6,2,2)*FD211HHH(1,INH6,INH4,INH3))/64.
       enddo 
       enddo 
        INH3=6
        INH4=6       
       do ICH3=1,2
       do ICH4=1,2
       self=self+(CS1S1S3S3(INH3,INH4,2,2)*CS1S3S3(INH3,ICH3,ICH4)*
     &  CS1S3S3(INH4,ICH4,ICH3)*FD211HHpHp(1,INH4,ICH4,ICH3))/32.
       enddo 
       enddo 
        ICH4=1
        ICH5=1        
       do ICH3=1,2
       do INH3=ihiggs,6
       self=self+(CS1S3S3(INH3,ICH3,ICH4)*CS1S3S3(INH3,ICH5,ICH3)*
     &  CS3S3S3S3(2,ICH4,2,ICH5)*FD211HpHpH(1,ICH5,ICH3,INH3))/16.
       enddo 
       enddo 
        INH3=6
        INH4=6       
       do ISt3=1,2
       do ISt4=1,2
       self=self+(3*CS1S1S3S3(INH3,INH4,2,2)*CSSS(INH3,ISt3,ISt4)*
     &  CSSS(INH4,ISt4,ISt3)*FD211HSTST(1,INH4,ISt4,ISt3))/32.
       enddo 
       enddo 
        ICH3=1
        ICH4=1       
       do ISt3=1,2
       self=self+(3*CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*
     &  CS3S3S3S3(2,ICH4,2,ICH3)*FD211HpSTSB(1,ICH4,ISt3))/16.
       enddo 

c
c topo14-GBC
c

        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-Mah(INH3) + Mah(INH4)) .gt. 1D-15) then 
                if( Abs(Mah(INH3) - MassHpm(ICH3))*Abs(Mah(INH4) - MassHpm(ICH3)) .gt. 1D-15 ) then 
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*(3*
     &  (FD111THpT(2,ICH3)*hy(INH4)*hyc(INH3)*Mah(INH3)**2-
     &  FD111THpT(4,ICH3)*hy(INH4)*hyc(INH3)*Mah(INH3)**2-
     &  FD111THT(2,INH4)*hy(INH4)*hyc(INH3)*Mah(INH3)**2+
     &  FD111THT(4,INH4)*hy(INH4)*hyc(INH3)*Mah(INH3)**2+
     &  FD111THpT(2,ICH3)*hy(INH3)*hyc(INH4)*Mah(INH3)**2-
     &  FD111THpT(4,ICH3)*hy(INH3)*hyc(INH4)*Mah(INH3)**2-
     &  FD111THT(2,INH4)*hy(INH3)*hyc(INH4)*Mah(INH3)**2+
     &  FD111THT(4,INH4)*hy(INH3)*hyc(INH4)*Mah(INH3)**2-M12*
     &  FD111THT(1,INH3)*hy(INH3)*hy(INH4)*Mah(INH4)**2-
     &  FD111THpT(2,ICH3)*hy(INH4)*hyc(INH3)*Mah(INH4)**2+
     &  FD111THpT(4,ICH3)*hy(INH4)*hyc(INH3)*Mah(INH4)**2+
     &  FD111THT(2,INH3)*hy(INH4)*hyc(INH3)*Mah(INH4)**2-
     &  FD111THT(4,INH3)*hy(INH4)*hyc(INH3)*Mah(INH4)**2-
     &  FD111THpT(2,ICH3)*hy(INH3)*hyc(INH4)*Mah(INH4)**2+
     &  FD111THpT(4,ICH3)*hy(INH3)*hyc(INH4)*Mah(INH4)**2+
     &  FD111THT(2,INH3)*hy(INH3)*hyc(INH4)*Mah(INH4)**2-
     &  FD111THT(4,INH3)*hy(INH3)*hyc(INH4)*Mah(INH4)**2-M12*
     &  FD111THT(1,INH3)*hyc(INH3)*hyc(INH4)*Mah(INH4)**2-M12*
     &  FD111THpT(1,ICH3)*(hy(INH3)*hy(INH4)+hyc(INH3)*hyc(INH4))*
     &  (Mah(INH3)**2-Mah(INH4)**2)+(-((FD111THT(2,INH3)-
     &  FD111THT(2,INH4)-FD111THT(4,INH3)+FD111THT(4,INH4))*(hy(INH4)*
     &  hyc(INH3)+hy(INH3)*hyc(INH4)))+M12*FD111THT(1,INH3)*(hy(INH3)*
     &  hy(INH4)+hyc(INH3)*hyc(INH4)))*MassHpm(ICH3)**2+M12*
     &  FD111THT(1,INH4)*(hy(INH3)*hy(INH4)+hyc(INH3)*hyc(INH4))*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2))+CFF12S1L(INH3)*(CFF12S1R(INH4)
     &  *((FD111ChaHpCha(2,ICH3)-FD111ChaHpCha(4,ICH3))*(Mah(INH3)**2-
     &  Mah(INH4)**2)+FD111ChaHCha(4,INH4)*(Mah(INH3)**2-MassHpm(ICH3)*
     &  *2)+(FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3))*(Mah(INH4)**2-
     &  MassHpm(ICH3)**2)+FD111ChaHCha(2,INH4)*(-Mah(INH3)**2+
     &  MassHpm(ICH3)**2))+MCha2*CFF12S1L(INH4)*(FD111ChaHpCha(1,ICH3)*
     &  (-Mah(INH3)**2+Mah(INH4)**2)+FD111ChaHCha(1,INH4)*(Mah(INH3)**
     &  2-MassHpm(ICH3)**2)+FD111ChaHCha(1,INH3)*(-Mah(INH4)**2+
     &  MassHpm(ICH3)**2)))+CFF12S1R(INH3)*(CFF12S1L(INH4)*
     &  ((FD111ChaHpCha(2,ICH3)-FD111ChaHpCha(4,ICH3))*(Mah(INH3)**2-
     &  Mah(INH4)**2)+FD111ChaHCha(4,INH4)*(Mah(INH3)**2-MassHpm(ICH3)*
     &  *2)+(FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3))*(Mah(INH4)**2-
     &  MassHpm(ICH3)**2)+FD111ChaHCha(2,INH4)*(-Mah(INH3)**2+
     &  MassHpm(ICH3)**2))+MCha2*CFF12S1R(INH4)*(FD111ChaHpCha(1,ICH3)*
     &  (-Mah(INH3)**2+Mah(INH4)**2)+FD111ChaHCha(1,INH4)*(Mah(INH3)**
     &  2-MassHpm(ICH3)**2)+FD111ChaHCha(1,INH3)*(-Mah(INH4)**2+
     &  MassHpm(ICH3)**2)))))/(8.*(Mah(INH3)**2-Mah(INH4)**2)*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)*(Mah(INH4)**2-MassHpm(ICH3)**2))
                endif
              else
                if( Mah(INH3) .gt. 1D-15) then 

                  if(abs(Mah(INH3)**2-MassHpm(ICH3)**2).gt.1D-10)then
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*(3*
     &  (FD111THpT(2,ICH3)-FD111THpT(4,ICH3)-FD111THT(2,INH3)+
     &  FD111THT(4,INH3))*(hy(INH4)*hyc(INH3)+hy(INH3)*hyc(INH4))-3*
     &  M12*FD111THpT(1,ICH3)*(hy(INH3)*hy(INH4)+hyc(INH3)*hyc(INH4))+
     &  3*M12*FD111THT(1,INH3)*(hy(INH3)*hy(INH4)+hyc(INH3)*hyc(INH4))+
     &  3*(-((FD211HTT(2,INH3)-FD211HTT(3,INH3))*(hy(INH4)*hyc(INH3)+
     &  hy(INH3)*hyc(INH4)))+M12*FD211HTT(1,INH3)*(hy(INH3)*hy(INH4)+
     &  hyc(INH3)*hyc(INH4)))*(-Mah(INH3)**2+MassHpm(ICH3)**2)+
     &  CFF12S1L(INH3)*(CFF12S1R(INH4)*(-FD111ChaHCha(2,INH3)+
     &  FD111ChaHCha(4,INH3)+FD111ChaHpCha(2,ICH3)-
     &  FD111ChaHpCha(4,ICH3)+(FD211HChaCha(2,INH3)-
     &  FD211HChaCha(3,INH3))*(Mah(INH3)**2-MassHpm(ICH3)**2))+MCha2*
     &  CFF12S1L(INH4)*(FD111ChaHCha(1,INH3)-FD111ChaHpCha(1,ICH3)+
     &  FD211HChaCha(1,INH3)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))+
     &  CFF12S1R(INH3)*(CFF12S1L(INH4)*(-FD111ChaHCha(2,INH3)+
     &  FD111ChaHCha(4,INH3)+FD111ChaHpCha(2,ICH3)-
     &  FD111ChaHpCha(4,ICH3)+(FD211HChaCha(2,INH3)-
     &  FD211HChaCha(3,INH3))*(Mah(INH3)**2-MassHpm(ICH3)**2))+MCha2*
     &  CFF12S1R(INH4)*(FD111ChaHCha(1,INH3)-FD111ChaHpCha(1,ICH3)+
     &  FD211HChaCha(1,INH3)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))))/(8.*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   !need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-Mah(INH3) + Mah(INH4)) .gt. 1D-15) then 
                if( Abs(Mah(INH3) - MassHpm(ICH3))*Abs(Mah(INH4) - MassHpm(ICH3)) .gt. 1D-15 ) then 
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*
     &  ((CFF11S1L(INe4,INe3,INH4)*CFF11S1R(INe3,INe4,INH3)+
     &  CFF11S1L(INe3,INe4,INH3)*CFF11S1R(INe4,INe3,INH4))*
     &  ((FD111ChiHpChi(2,INe3,ICH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))*(Mah(INH3)**2-Mah(INH4)**2)+
     &  FD111ChiHChi(4,INe3,INH4,INe4)*(Mah(INH3)**2-MassHpm(ICH3)**2)+
     &  (FD111ChiHChi(2,INe3,INH3,INe4)-FD111ChiHChi(4,INe3,INH3,INe4))
     &  *(Mah(INH4)**2-MassHpm(ICH3)**2)+FD111ChiHChi(2,INe3,INH4,INe4)
     &  *(-Mah(INH3)**2+MassHpm(ICH3)**2))+(CFF11S1L(INe3,INe4,INH3)*
     &  CFF11S1L(INe4,INe3,INH4)+CFF11S1R(INe3,INe4,INH3)*
     &  CFF11S1R(INe4,INe3,INH4))*(FD111ChiHpChi(1,INe3,ICH3,INe4)*(-
     &  Mah(INH3)**2+Mah(INH4)**2)+FD111ChiHChi(1,INe3,INH4,INe4)*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)+FD111ChiHChi(1,INe3,INH3,INe4)*
     &  (-Mah(INH4)**2+MassHpm(ICH3)**2))*MNeu(INe3)*MNeu(INe4)))/(16.*
     &  (Mah(INH3)**2-Mah(INH4)**2)*(Mah(INH3)**2-MassHpm(ICH3)**2)*
     &  (Mah(INH4)**2-MassHpm(ICH3)**2))
                endif
              else
                if( Mah(INH3) .gt. 1D-15) then 
                  if(abs(Mah(INH3)**2-MassHpm(ICH3)**2).gt.1D-10)then
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*(-
     &  ((CFF11S1L(INe4,INe3,INH4)*CFF11S1R(INe3,INe4,INH3)+
     &  CFF11S1L(INe3,INe4,INH3)*CFF11S1R(INe4,INe3,INH4))*
     &  (FD111ChiHChi(2,INe3,INH3,INe4)-FD111ChiHChi(4,INe3,INH3,INe4)-
     &  FD111ChiHpChi(2,INe3,ICH3,INe4)+FD111ChiHpChi(4,INe3,ICH3,INe4)
     &  -(FD211HNeuNeu(2,INH3,INe3,INe4)-FD211HNeuNeu(3,INH3,INe3,INe4)
     &  )*(Mah(INH3)**2-MassHpm(ICH3)**2)))+(CFF11S1L(INe3,INe4,INH3)*
     &  CFF11S1L(INe4,INe3,INH4)+CFF11S1R(INe3,INe4,INH3)*
     &  CFF11S1R(INe4,INe3,INH4))*(FD111ChiHChi(1,INe3,INH3,INe4)-
     &  FD111ChiHpChi(1,INe3,ICH3,INe4)+FD211HNeuNeu(1,INH3,INe3,INe4)*
     &  (-Mah(INH3)**2+MassHpm(ICH3)**2))*MNeu(INe3)*MNeu(INe4)))/(16.*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   !need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 

        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
              if( Abs(-MassHpm(ICH3) + MassHpm(ICH4)) .gt. 1D-15) then 
                if( Abs(-Mah(INH3) + MassHpm(ICH3))*Abs(-Mah(INH3) + MassHpm(ICH4)) .gt. 1D-15 ) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CTBS3(ICH3)*CTBS3C(ICH4)*(FD111THpB(4,ICH3)*Mah(INH3)**2-
     &  FD111THpB(4,ICH4)*Mah(INH3)**2+FD111THpB(4,ICH4)*MassHpm(ICH3)*
     &  *2+FD111THpB(2,ICH4)*(Mah(INH3)**2-MassHpm(ICH3)**2)-
     &  FD111THpB(4,ICH3)*MassHpm(ICH4)**2+(FD111THB(2,INH3)-
     &  FD111THB(4,INH3))*(MassHpm(ICH3)**2-MassHpm(ICH4)**2)+
     &  FD111THpB(2,ICH3)*(-Mah(INH3)**2+MassHpm(ICH4)**2)))/(8.*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)*(Mah(INH3)**2-MassHpm(ICH4)**2)
     &  *(MassHpm(ICH3)**2-MassHpm(ICH4)**2))
                endif
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
                  if(abs(Mah(INH3)**2-MassHpm(ICH3)**2).gt.1D-10)then
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CTBS3(ICH3)*CTBS3C(ICH4)*(FD111THB(2,INH3)-FD111THB(4,INH3)-
     &  FD111THpB(2,ICH3)+FD111THpB(4,ICH3)-(FD211HpTB(2,ICH3)-
     &  FD211HpTB(3,ICH3))*(Mah(INH3)**2-MassHpm(ICH3)**2)))/(8.*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   !need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INe3=ineu,3
       do INH3=ihiggs,6
              if( Abs(-MassHpm(ICH3) + MassHpm(ICH4)) .gt. 1D-15) then 
                if( Abs(-Mah(INH3) + MassHpm(ICH3))*Abs(-Mah(INH3) + MassHpm(ICH4)) .gt. 1D-15 ) then 
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  ((CF11F12S3L(INe3,ICH4)*DCONJG(CF11F12S3L(INe3,ICH3))+
     &  CF11F12S3R(INe3,ICH4)*DCONJG(CF11F12S3R(INe3,ICH3)))*
     &  (FD111ChiHpCha(4,INe3,ICH3)*Mah(INH3)**2-
     &  FD111ChiHpCha(4,INe3,ICH4)*Mah(INH3)**2+
     &  FD111ChiHpCha(4,INe3,ICH4)*MassHpm(ICH3)**2+
     &  FD111ChiHpCha(2,INe3,ICH4)*(Mah(INH3)**2-MassHpm(ICH3)**2)-
     &  FD111ChiHpCha(4,INe3,ICH3)*MassHpm(ICH4)**2+
     &  (FD111ChiHCha(2,INe3,INH3)-FD111ChiHCha(4,INe3,INH3))*
     &  (MassHpm(ICH3)**2-MassHpm(ICH4)**2)+FD111ChiHpCha(2,INe3,ICH3)*
     &  (-Mah(INH3)**2+MassHpm(ICH4)**2))+MCha*(CF11F12S3R(INe3,ICH4)*
     &  DCONJG(CF11F12S3L(INe3,ICH3))+CF11F12S3L(INe3,ICH4)*
     &  DCONJG(CF11F12S3R(INe3,ICH3)))*(FD111ChiHpCha(1,INe3,ICH4)*(-
     &  Mah(INH3)**2+MassHpm(ICH3)**2)+FD111ChiHpCha(1,INe3,ICH3)*
     &  (Mah(INH3)**2-MassHpm(ICH4)**2)+FD111ChiHCha(1,INe3,INH3)*(-
     &  MassHpm(ICH3)**2+MassHpm(ICH4)**2))*MNeu(INe3)))/(8.*(Mah(INH3)
     &  **2-MassHpm(ICH3)**2)*(Mah(INH3)**2-MassHpm(ICH4)**2)*
     &  (MassHpm(ICH3)**2-MassHpm(ICH4)**2))
                endif
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
                  if(abs(Mah(INH3)**2-MassHpm(ICH3)**2).gt.1D-10)then
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  ((CF11F12S3L(INe3,ICH4)*DCONJG(CF11F12S3L(INe3,ICH3))+
     &  CF11F12S3R(INe3,ICH4)*DCONJG(CF11F12S3R(INe3,ICH3)))*
     &  (FD111ChiHCha(2,INe3,INH3)-FD111ChiHCha(4,INe3,INH3)-
     &  FD111ChiHpCha(2,INe3,ICH3)+FD111ChiHpCha(4,INe3,ICH3)-
     &  (FD211HpChiCha(2,ICH3,INe3)-FD211HpChiCha(3,ICH3,INe3))*
     &  (Mah(INH3)**2-MassHpm(ICH3)**2))-MCha*(CF11F12S3R(INe3,ICH4)*
     &  DCONJG(CF11F12S3L(INe3,ICH3))+CF11F12S3L(INe3,ICH4)*
     &  DCONJG(CF11F12S3R(INe3,ICH3)))*(FD111ChiHCha(1,INe3,INH3)-
     &  FD111ChiHpCha(1,INe3,ICH3)+FD211HpChiCha(1,ICH3,INe3)*(-
     &  Mah(INH3)**2+MassHpm(ICH3)**2))*MNeu(INe3)))/(8.*(Mah(INH3)**2-
     &  MassHpm(ICH3)**2)**2)
                  else
                   !need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

c
c topo9-NoGBC
c

        ICH3=1
        ICH4=1
       do INH3=ihiggs,5
         if(Mah(INH3).gt.1D-10)then       
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CTBS3(ICH3)*CTBS3C(ICH4)*((-FD211HpTB(2,ICH3)+FD211HpTB(3,ICH3)
     &  )*Mah(INH3)**4+(FD111THB(2,INH3)-FD111THB(4,INH3)-
     &  FD111THpB(2,ICH3)+FD111THpB(4,ICH3))*Mah2(INH3)))/(8.*Mah(INH3)
     &  **4*Mah2(INH3))
          else
            ! need to consider
           endif
         enddo
        ICH3=1
        ICH4=1
       do INH3=ihiggs,5
         if(Mah(INH3).gt.1D-10)then
       do INe3=ineu,3       
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*(-
     &  ((CF11F12S3L(INe3,ICH4)*DCONJG(CF11F12S3L(INe3,ICH3))+
     &  CF11F12S3R(INe3,ICH4)*DCONJG(CF11F12S3R(INe3,ICH3)))*
     &  ((FD211HpChiCha(2,ICH3,INe3)-FD211HpChiCha(3,ICH3,INe3))*
     &  Mah(INH3)**4+(-FD111ChiHCha(2,INe3,INH3)+
     &  FD111ChiHCha(4,INe3,INH3)+FD111ChiHpCha(2,INe3,ICH3)-
     &  FD111ChiHpCha(4,INe3,ICH3))*Mah2(INH3)))+MCha*
     &  (CF11F12S3R(INe3,ICH4)*DCONJG(CF11F12S3L(INe3,ICH3))+
     &  CF11F12S3L(INe3,ICH4)*DCONJG(CF11F12S3R(INe3,ICH3)))*
     &  (FD211HpChiCha(1,ICH3,INe3)*Mah(INH3)**4+(-
     &  FD111ChiHCha(1,INe3,INH3)+FD111ChiHpCha(1,INe3,ICH3))*
     &  Mah2(INH3))*MNeu(INe3)))/(8.*Mah(INH3)**4*Mah2(INH3))
         enddo
            else
            !need to consider
            endif
       enddo 

c
c topo9-GBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
       do INH6=ihiggs,6
              if( Abs(-Mah(INH3) + Mah(INH6)) .gt. 1D-15) then 
                if( Abs(Mah(INH3) - MassHpm(ICH3))*Abs(Mah(INH6) - MassHpm(ICH3)) .gt. 1D-15 ) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S1(INH4,INH5,INH6)*
     &  CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH6,ICH3,2)*
     &  (FD111HHpH(1,INH4,ICH3,INH5)*(Mah(INH3)**2-Mah(INH6)**2)+
     &  FD111HHH(1,INH3,INH4,INH5)*(Mah(INH6)**2-MassHpm(ICH3)**2)+
     &  FD111HHH(1,INH4,INH6,INH5)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))
     &  /(32.*(Mah(INH3)**2-Mah(INH6)**2)*(Mah(INH3)**2-MassHpm(ICH3)**
     &  2)*(Mah(INH6)**2-MassHpm(ICH3)**2))
                endif
              else
                if( Mah(INH3) .gt. 1D-15) then 

                  if (abs((Mah(INH3)**2-MassHpm(ICH3)**2)).gt.1D-10) then
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S1(INH4,INH5,INH6)*
     &  CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH6,ICH3,2)*(-
     &  FD111HHH(1,INH3,INH4,INH5)+FD111HHpH(1,INH4,ICH3,INH5)+
     &  FD211HHH(1,INH3,INH4,INH5)*(Mah(INH3)**2-MassHpm(ICH3)**2)))
     &  /(32.*(Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   ! need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-MassHpm(ICH3) + MassHpm(ICH5)) .gt. 1D-15) then 
                if( Abs(-Mah(INH3) + MassHpm(ICH3))*Abs(-Mah(INH3) + MassHpm(ICH5)) .gt. 1D-15 ) then 
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH5,2)*
     &  CS1S3S3(INH4,ICH3,ICH4)*CS1S3S3(INH4,ICH4,ICH5)*
     &  (FD111HHpHp(1,INH4,ICH5,ICH4)*(Mah(INH3)**2-MassHpm(ICH3)**2)+
     &  FD111HHpH(1,INH3,ICH4,INH4)*(MassHpm(ICH3)**2-MassHpm(ICH5)**2)
     &  +FD111HHpHp(1,INH4,ICH3,ICH4)*(-Mah(INH3)**2+MassHpm(ICH5)**2))
     &  )/(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)*(Mah(INH3)**2-
     &  MassHpm(ICH5)**2)*(MassHpm(ICH3)**2-MassHpm(ICH5)**2))
                endif
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 

                  if (abs((Mah(INH3)**2-MassHpm(ICH3)**2)).gt.1D-10) then
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH5,2)*
     &  CS1S3S3(INH4,ICH3,ICH4)*CS1S3S3(INH4,ICH4,ICH5)*
     &  (FD111HHpH(1,INH3,ICH4,INH4)-FD111HHpHp(1,INH4,ICH3,ICH4)+
     &  FD211HpHHp(1,ICH3,INH4,ICH4)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   ! need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-Mah(INH3) + Mah(INH4)) .gt. 1D-15) then 
                if( Abs(Mah(INH3) - MassHpm(ICH3))*Abs(Mah(INH4) - MassHpm(ICH3)) .gt. 1D-15 ) then 
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH5,ICH4)*
     &  CS1S3S3(INH4,ICH3,2)*CS1S3S3(INH4,ICH4,ICH5)*
     &  (FD111HpHpHp(1,ICH3,ICH4,ICH5)*(Mah(INH3)**2-Mah(INH4)**2)+
     &  FD111HHpHp(1,INH3,ICH4,ICH5)*(Mah(INH4)**2-MassHpm(ICH3)**2)+
     &  FD111HHpHp(1,INH4,ICH4,ICH5)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-Mah(INH4)**2)*(Mah(INH3)**2-MassHpm(ICH3)**
     &  2)*(Mah(INH4)**2-MassHpm(ICH3)**2))
                endif
              else
                if( Mah(INH3) .gt. 1D-15) then 

                  if (abs((Mah(INH3)**2-MassHpm(ICH3)**2)).gt.1D-10) then
       self=self+(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH5,ICH4)*
     &  CS1S3S3(INH4,ICH3,2)*CS1S3S3(INH4,ICH4,ICH5)*(-
     &  FD111HHpHp(1,INH3,ICH4,ICH5)+FD111HpHpHp(1,ICH3,ICH4,ICH5)+
     &  FD211HHpHp(1,INH3,ICH4,ICH5)*(Mah(INH3)**2-MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   ! need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do ISt3=1,2
       do ISt4=1,2
              if( Abs(-Mah(INH3) + Mah(INH4)) .gt. 1D-15) then 
                if( Abs(Mah(INH3) - MassHpm(ICH3))*Abs(Mah(INH4) - MassHpm(ICH3)) .gt. 1D-15 ) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*
     &  CSSS(INH3,ISt4,ISt3)*CSSS(INH4,ISt3,ISt4)*
     &  (FD111HpSTST(1,ICH3,ISt3,ISt4)*(Mah(INH3)**2-Mah(INH4)**2)+
     &  FD111HSTST(1,INH3,ISt3,ISt4)*(Mah(INH4)**2-MassHpm(ICH3)**2)+
     &  FD111HSTST(1,INH4,ISt3,ISt4)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-Mah(INH4)**2)*(Mah(INH3)**2-MassHpm(ICH3)**
     &  2)*(Mah(INH4)**2-MassHpm(ICH3)**2))
                endif
              else
                if( Mah(INH3) .gt. 1D-15) then 

                  if (abs((Mah(INH3)**2-MassHpm(ICH3)**2)).gt.1D-10) then
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH4,ICH3,2)*
     &  CSSS(INH3,ISt4,ISt3)*CSSS(INH4,ISt3,ISt4)*
     &  (FD111HpSTST(1,ICH3,ISt3,ISt4)-FD111HSTST(1,INH3,ISt3,ISt4)+
     &  FD211HSTST(1,INH3,ISt3,ISt4)*(Mah(INH3)**2-MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   ! need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Abs(-MassHpm(ICH3) + MassHpm(ICH4)) .gt. 1D-15) then 
                if( Abs(-Mah(INH3) + MassHpm(ICH3))*Abs(-Mah(INH3) + MassHpm(ICH4)) .gt. 1D-15 ) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*
     &  (FD111HpSTSB(1,ICH4,ISt3)*(Mah(INH3)**2-MassHpm(ICH3)**2)+
     &  FD111HSTSB(1,INH3,ISt3)*(MassHpm(ICH3)**2-MassHpm(ICH4)**2)+
     &  FD111HpSTSB(1,ICH3,ISt3)*(-Mah(INH3)**2+MassHpm(ICH4)**2)))
     &  /(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)*(Mah(INH3)**2-
     &  MassHpm(ICH4)**2)*(MassHpm(ICH3)**2-MassHpm(ICH4)**2))
                endif
              else
                if( MassHpm(ICH3) .gt. 1D-15) then 
                  if (abs((Mah(INH3)**2-MassHpm(ICH3)**2)).gt.1D-10) then
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*(-
     &  FD111HpSTSB(1,ICH3,ISt3)+FD111HSTSB(1,INH3,ISt3)+
     &  FD211HpSTSB(1,ICH3,ISt3)*(-Mah(INH3)**2+MassHpm(ICH3)**2)))
     &  /(16.*(Mah(INH3)**2-MassHpm(ICH3)**2)**2)
                  else
                   ! need to consider
                  endif
                endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

c
c topo10-NoGBC
c

        ICH3=1
        ICH5=1
       do INH3=ihiggs,5
       do ICH4=1,2
       do INH4=ihiggs,6     
       self=self+-(CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH5,2)*
     &  CS1S3S3(INH4,ICH3,ICH4)*CS1S3S3(INH4,ICH4,ICH5)*
     &  (FD211HpHHp(1,ICH3,INH4,ICH4)*Mah(INH3)**4+(-
     &  FD111HHpH(1,INH3,ICH4,INH4)+FD111HHpHp(1,INH4,ICH3,ICH4))*
     &  Mah2(INH3)))/(16.*Mah(INH3)**4*Mah2(INH3))
         enddo
       enddo 
       enddo 

        ICH3=1
        ICH4=1
       do INH3=ihiggs,5
       do ISt3=1,2     
       self=self+(-3*CS1S3S3(INH3,2,ICH3)*CS1S3S3(INH3,ICH4,2)*
     &  CS3S13S(ICH3,ISt3)*CS3S13SC(ICH4,ISt3)*
     &  (FD211HpSTSB(1,ICH3,ISt3)*Mah(INH3)**4+
     &  (FD111HpSTSB(1,ICH3,ISt3)-FD111HSTSB(1,INH3,ISt3))*Mah2(INH3)))
     &  /(16.*Mah(INH3)**4*Mah2(INH3))
         enddo
       enddo 
c
c topo10-GBC
c

        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S3S3(INH4,INH5,ICH3,2)*
     &  CS1S3S3(INH3,2,ICH3)*(FD111HHH(1,INH3,INH4,INH5)-
     &  FD111HHpH(1,INH4,ICH3,INH5)))/(32.*(Mah(INH3)**2-MassHpm(ICH3)*
     &  *2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 

        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-Mah(INH4) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,ICH4,2)*CS1S3S3(INH3,ICH3,ICH4)*
     &  CS1S3S3(INH4,2,ICH3)*(FD111HHpH(1,INH3,ICH4,INH4)-
     &  FD111HHpHp(1,INH3,ICH3,ICH4)))/(16.*(Mah(INH4)**2-MassHpm(ICH3)
     &  **2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 

        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
              if( Abs(-Mah(INH3) + MassHpm(ICH5)) .gt. 1D-15) then 
       self=self+(CS1S3S3(INH3,2,ICH5)*CS1S3S3(INH3,ICH3,ICH4)*
     &  CS3S3S3S3(ICH4,ICH5,2,ICH3)*(FD111HHpHp(1,INH3,ICH3,ICH4)-
     &  FD111HpHpHp(1,ICH3,ICH5,ICH4)))/(16.*(Mah(INH3)**2-
     &  MassHpm(ICH5)**2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
       do ISt4=1,2
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS3S3S13S13(ICH3,2,ISt4,ISt3)*
     &  CSSS(INH3,ISt3,ISt4)*(-FD111HpSTST(1,ICH3,ISt3,ISt4)+
     &  FD111HSTST(1,INH3,ISt3,ISt4)))/(16.*(Mah(INH3)**2-MassHpm(ICH3)
     &  **2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CS1S3SSC(INH3,2,ISt3)*
     &  CS3S13S(ICH3,ISt3)*(-FD111HpSTSB(1,ICH3,ISt3)+
     &  FD111HSTSB(1,INH3,ISt3)))/(16.*(Mah(INH3)**2-MassHpm(ICH3)**2))
              endif
       enddo 
       enddo 
       enddo

c
c topo2-NoGBC
c 
c
c topo2-GBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S1S3S3(INH4,INH5,2,ICH3)*
     &  CS1S3S3(INH3,ICH3,2)*(FD111HHH(1,INH3,INH4,INH5)-
     &  FD111HHpH(1,INH4,ICH3,INH5)))/(32.*(Mah(INH3)**2-MassHpm(ICH3)*
     &  *2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
              if( Abs(-Mah(INH4) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(CS1S1S3S3(INH3,INH4,2,ICH4)*CS1S3S3(INH3,ICH4,ICH3)*
     &  CS1S3S3(INH4,ICH3,2)*(FD111HHpH(1,INH3,ICH4,INH4)-
     &  FD111HHpHp(1,INH3,ICH3,ICH4)))/(16.*(Mah(INH4)**2-MassHpm(ICH3)
     &  **2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
              if( Abs(-Mah(INH3) + MassHpm(ICH5)) .gt. 1D-15) then 
       self=self+(CS1S3S3(INH3,ICH3,ICH4)*CS1S3S3(INH3,ICH5,2)*
     &  CS3S3S3S3(2,ICH4,ICH3,ICH5)*(FD111HHpHp(1,INH3,ICH3,ICH4)-
     &  FD111HpHpHp(1,ICH3,ICH5,ICH4)))/(16.*(Mah(INH3)**2-
     &  MassHpm(ICH5)**2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
       do ISt4=1,2
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CS1S3S3(INH3,ICH3,2)*CS3S3S13S13(2,ICH3,ISt4,ISt3)*
     &  CSSS(INH3,ISt3,ISt4)*(-FD111HpSTST(1,ICH3,ISt3,ISt4)+
     &  FD111HSTST(1,INH3,ISt3,ISt4)))/(16.*(Mah(INH3)**2-MassHpm(ICH3)
     &  **2))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
              if( Abs(-Mah(INH3) + MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CS1S3S3(INH3,ICH3,2)*CS1S3SS(INH3,2,ISt3)*
     &  CS3S13SC(ICH3,ISt3)*(-FD111HpSTSB(1,ICH3,ISt3)+
     &  FD111HSTSB(1,INH3,ISt3)))/(16.*(Mah(INH3)**2-MassHpm(ICH3)**2))
              endif
       enddo 
       enddo 
       enddo 
c
c topo3-NoGBC
c 
c
c topo3-GBC
c
        
       do INe3=ineu,3
       do INe4=ineu,3
       do INe5=ineu,3
       do INH3=ihiggs,6
          if( Abs(-MNeu(INe3) + MNeu(INe5)) .gt. 1D-15) then 
             if(abs((MCha-MNeu(INe5))*(MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3L(INe5,2)*(CFF11S1R(INe3,INe4,INH3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(CFF11S1L(INe4,INe5,INH3)*MNeu(INe5)*
     &  (FD111ChiChiH(2,INe4,INe5,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(2,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(2,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  CFF11S1R(INe4,INe5,INH3)*MNeu(INe4)*
     &  (FD111ChiChiH(4,INe5,INe4,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(4,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(4,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2)))+
     &  CFF11S1L(INe3,INe4,INH3)*(CFF11S1L(INe4,INe5,INH3)*MNeu(INe4)*
     &  MNeu(INe5)*(MCha*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (FD111ChiChiH(1,INe4,INe5,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(1,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(1,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChiChiH(4,INe5,INe4,INH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChiH(4,INe3,INe4,INH3)*(MCha**2-
     &  MNeu(INe5)**2)+FD111ChaChiH(4,INe4,INH3)*(-MNeu(INe3)**2+
     &  MNeu(INe5)**2)))+CFF11S1R(INe4,INe5,INH3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (FD111ChiChiH(2,INe4,INe5,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(2,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(2,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChiChiH(5,INe5,INe4,INH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChiH(5,INe3,INe4,INH3)*(MCha**2-
     &  MNeu(INe5)**2)+FD111ChaChiH(5,INe4,INH3)*(-MNeu(INe3)**2+
     &  MNeu(INe5)**2)))))+CF11F12S3R(INe5,2)*(CFF11S1R(INe4,INe5,INH3)
     &  *MNeu(INe5)*(CFF11S1L(INe3,INe4,INH3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(FD111ChiChiH(2,INe4,INe5,INH3)*(-MCha**2+
     &  MNeu(INe3)**2)+FD111ChiChiH(2,INe3,INe4,INH3)*(MCha**2-
     &  MNeu(INe5)**2)+FD111ChaChiH(2,INe4,INH3)*(-MNeu(INe3)**2+
     &  MNeu(INe5)**2))+CFF11S1R(INe3,INe4,INH3)*MNeu(INe4)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*
     &  (FD111ChiChiH(1,INe4,INe5,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(1,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(1,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  DCONJG(CF11F12S3R(INe3,2))*(FD111ChiChiH(4,INe5,INe4,INH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChiH(4,INe3,INe4,INH3)*(MCha**2-
     &  MNeu(INe5)**2)+FD111ChaChiH(4,INe4,INH3)*(-MNeu(INe3)**2+
     &  MNeu(INe5)**2))))+CFF11S1L(INe4,INe5,INH3)*
     &  (CFF11S1L(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4)*
     &  (FD111ChiChiH(4,INe5,INe4,INH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChiH(4,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(4,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  CFF11S1R(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)*(FD111ChiChiH(2,INe4,INe5,INH3)*(-MCha**2+MNeu(INe3)
     &  **2)+FD111ChiChiH(2,INe3,INe4,INH3)*(MCha**2-MNeu(INe5)**2)+
     &  FD111ChaChiH(2,INe4,INH3)*(-MNeu(INe3)**2+MNeu(INe5)**2))+
     &  DCONJG(CF11F12S3R(INe3,2))*(FD111ChiChiH(5,INe5,INe4,INH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChiH(5,INe3,INe4,INH3)*(MCha**2-
     &  MNeu(INe5)**2)+FD111ChaChiH(5,INe4,INH3)*(-MNeu(INe3)**2+
     &  MNeu(INe5)**2))))))/(8.*(MCha-MNeu(INe3))*(MCha+MNeu(INe3))*
     &  (MCha-MNeu(INe5))*(MNeu(INe3)-MNeu(INe5))*(MCha+MNeu(INe5))*
     &  (MNeu(INe3)+MNeu(INe5)))
          else
          !need to consider
          endif
              else
             if(abs((MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3R(INe5,2)*(CFF11S1L(INe4,INe5,INH3)*
     &  (CFF11S1R(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)*(-FD111ChaChiH(2,INe4,INH3)+
     &  FD111ChiChiH(2,INe3,INe4,INH3)+FD211ChiChiH(2,INe3,INe4,INH3)*
     &  (MCha**2-MNeu(INe3)**2))+DCONJG(CF11F12S3R(INe3,2))*(-
     &  FD111ChaChiH(5,INe4,INH3)+FD111ChiChiH(5,INe3,INe4,INH3)+
     &  FD211ChiChiH(5,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2)))+
     &  CFF11S1L(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*(-
     &  FD111ChaChiH(4,INe4,INH3)+FD111ChiChiH(4,INe3,INe4,INH3)+
     &  FD211ChiChiH(4,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2))*
     &  MNeu(INe4))+CFF11S1R(INe4,INe5,INH3)*(CFF11S1L(INe3,INe4,INH3)*
     &  MNeu(INe3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*(-
     &  FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3)+
     &  FD211ChiChiH(2,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2))+
     &  CFF11S1R(INe3,INe4,INH3)*MNeu(INe4)*(DCONJG(CF11F12S3R(INe3,2))
     &  *MNeu(INe3)*(-FD111ChaChiH(4,INe4,INH3)+
     &  FD111ChiChiH(4,INe3,INe4,INH3)+FD211ChiChiH(4,INe3,INe4,INH3)*
     &  (MCha**2-MNeu(INe3)**2))+MCha*DCONJG(CF11F12S3L(INe3,2))*(-
     &  FD111ChaChiH(1,INe4,INH3)+FD111ChiChiH(1,INe3,INe4,INH3)+
     &  FD211ChiChiH(1,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2))*
     &  MNeu2(INe3))))+CF11F12S3L(INe5,2)*(CFF11S1R(INe3,INe4,INH3)*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(CFF11S1L(INe4,INe5,INH3)*MNeu(INe3)*(-
     &  FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3)+
     &  FD211ChiChiH(2,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2))+
     &  CFF11S1R(INe4,INe5,INH3)*(-FD111ChaChiH(4,INe4,INH3)+
     &  FD111ChiChiH(4,INe3,INe4,INH3)+FD211ChiChiH(4,INe3,INe4,INH3)*
     &  (MCha**2-MNeu(INe3)**2))*MNeu(INe4))+CFF11S1L(INe3,INe4,INH3)*
     &  (CFF11S1R(INe4,INe5,INH3)*(MCha*DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)*(-FD111ChaChiH(2,INe4,INH3)+
     &  FD111ChiChiH(2,INe3,INe4,INH3)+FD211ChiChiH(2,INe3,INe4,INH3)*
     &  (MCha**2-MNeu(INe3)**2))+DCONJG(CF11F12S3L(INe3,2))*(-
     &  FD111ChaChiH(5,INe4,INH3)+FD111ChiChiH(5,INe3,INe4,INH3)+
     &  FD211ChiChiH(5,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2)))+
     &  CFF11S1L(INe4,INe5,INH3)*MNeu(INe4)*(DCONJG(CF11F12S3L(INe3,2))
     &  *MNeu(INe3)*(-FD111ChaChiH(4,INe4,INH3)+
     &  FD111ChiChiH(4,INe3,INe4,INH3)+FD211ChiChiH(4,INe3,INe4,INH3)*
     &  (MCha**2-MNeu(INe3)**2))+MCha*DCONJG(CF11F12S3R(INe3,2))*(-
     &  FD111ChaChiH(1,INe4,INH3)+FD111ChiChiH(1,INe3,INe4,INH3)+
     &  FD211ChiChiH(1,INe3,INe4,INH3)*(MCha**2-MNeu(INe3)**2))*
     &  MNeu2(INe3)))))/(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 

c2
        
       do INe3=ineu,3
       do INH3=ihiggs,6
             if(abs((MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3L(INe3,2)*(-(MCha*CFF12S1R(INH3)**2*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(-FD111ChaChaH(4,INH3)+FD111ChiChaH(4,INe3,INH3)+
     &  FD211ChaChaH(4,INH3)*(MCha**2-MNeu(INe3)**2)))+CFF12S1L(INH3)*
     &  CFF12S1R(INH3)*(DCONJG(CF11F12S3L(INe3,2))*(MCha**2*
     &  FD111ChaChaH(2,INH3)+FD111ChaChaH(5,INH3)-MCha**2*
     &  FD111ChaChiH(2,INe3,INH3)-FD111ChiChaH(5,INe3,INH3)-MCha**4*
     &  FD211ChaChaH(2,INH3)-MCha**2*FD211ChaChaH(5,INH3)+MCha**2*
     &  FD211ChaChaH(2,INH3)*MNeu(INe3)**2+FD211ChaChaH(5,INH3)*
     &  MNeu(INe3)**2)+2*MCha*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)+
     &  FD211ChaChaH(2,INH3)*(-MCha**2+MNeu(INe3)**2)))+MCha**2*
     &  CFF12S1L(INH3)**2*(MCha*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (-FD111ChaChiH(1,INe3,INH3)+FD111ChaHCha(1,INH3)+
     &  FD211ChaChaH(1,INH3)*(-MCha**2+MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChaChaH(4,INH3)-
     &  FD111ChiChaH(4,INe3,INH3)+FD211ChaChaH(4,INH3)*(-MCha**2+
     &  MNeu(INe3)**2))))+CF11F12S3R(INe3,2)*(-(MCha*CFF12S1L(INH3)**2*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(-FD111ChaChaH(4,INH3)+FD111ChiChaH(4,INe3,INH3)+
     &  FD211ChaChaH(4,INH3)*(MCha**2-MNeu(INe3)**2)))+CFF12S1L(INH3)*
     &  CFF12S1R(INH3)*(DCONJG(CF11F12S3R(INe3,2))*(MCha**2*
     &  FD111ChaChaH(2,INH3)+FD111ChaChaH(5,INH3)-MCha**2*
     &  FD111ChaChiH(2,INe3,INH3)-FD111ChiChaH(5,INe3,INH3)-MCha**4*
     &  FD211ChaChaH(2,INH3)-MCha**2*FD211ChaChaH(5,INH3)+MCha**2*
     &  FD211ChaChaH(2,INH3)*MNeu(INe3)**2+FD211ChaChaH(5,INH3)*
     &  MNeu(INe3)**2)+2*MCha*DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*
     &  (FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)+
     &  FD211ChaChaH(2,INH3)*(-MCha**2+MNeu(INe3)**2)))+MCha**2*
     &  CFF12S1R(INH3)**2*(MCha*DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*
     &  (-FD111ChaChiH(1,INe3,INH3)+FD111ChaHCha(1,INH3)+
     &  FD211ChaChaH(1,INH3)*(-MCha**2+MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3R(INe3,2))*(FD111ChaChaH(4,INH3)-
     &  FD111ChiChaH(4,INe3,INH3)+FD211ChaChaH(4,INH3)*(-MCha**2+
     &  MNeu(INe3)**2)))))/(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
       enddo 
       enddo 
c3
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
             if(abs((MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3R(INe4,ICH3)*(-(CF11F12S3L(INe3,2)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(MCha*DCONJG(CF11F12S3R(INe4,ICH3))*(-
     &  FD111ChaChiHp(2,INe4,ICH3)+FD111ChiChiHp(2,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(2,INe4,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaChiHp(4,INe4,ICH3)+
     &  FD111ChiChiHp(4,INe3,INe4,ICH3)+FD211ChaChiHp(4,INe4,ICH3)*
     &  (MCha**2-MNeu(INe3)**2))*MNeu(INe4)))+CF11F12S3R(INe3,2)*
     &  (DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*(MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChaChiHp(2,INe4,ICH3)-
     &  FD111ChiChiHp(2,INe3,INe4,ICH3)+FD211ChaChiHp(2,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2))+MCha2*DCONJG(CF11F12S3L(INe4,ICH3))*
     &  (FD111ChaChiHp(1,INe4,ICH3)-FD111ChiChiHp(1,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2))*MNeu(INe4)
     &  )+DCONJG(CF11F12S3R(INe3,2))*(DCONJG(CF11F12S3R(INe4,ICH3))*
     &  (FD111ChaChiHp(5,INe4,ICH3)-FD111ChiChiHp(5,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(5,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChaChiHp(4,INe4,ICH3)-
     &  FD111ChiChiHp(4,INe3,INe4,ICH3)+FD211ChaChiHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2))*MNeu(INe4))))+CF11F12S3L(INe4,ICH3)*(-
     &  (CF11F12S3R(INe3,2)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*(MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaChiHp(2,INe4,ICH3)+
     &  FD111ChiChiHp(2,INe3,INe4,ICH3)+FD211ChaChiHp(2,INe4,ICH3)*
     &  (MCha**2-MNeu(INe3)**2))+DCONJG(CF11F12S3R(INe4,ICH3))*(-
     &  FD111ChaChiHp(4,INe4,ICH3)+FD111ChiChiHp(4,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(4,INe4,ICH3)*(MCha**2-MNeu(INe3)**2))*MNeu(INe4))
     &  )+CF11F12S3L(INe3,2)*(DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (MCha*DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChaChiHp(2,INe4,ICH3)
     &  -FD111ChiChiHp(2,INe3,INe4,ICH3)+FD211ChaChiHp(2,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2))+MCha2*DCONJG(CF11F12S3R(INe4,ICH3))*
     &  (FD111ChaChiHp(1,INe4,ICH3)-FD111ChiChiHp(1,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2))*MNeu(INe4)
     &  )+DCONJG(CF11F12S3L(INe3,2))*(DCONJG(CF11F12S3L(INe4,ICH3))*
     &  (FD111ChaChiHp(5,INe4,ICH3)-FD111ChiChiHp(5,INe3,INe4,ICH3)+
     &  FD211ChaChiHp(5,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChaChiHp(4,INe4,ICH3)-
     &  FD111ChiChiHp(4,INe3,INe4,ICH3)+FD211ChaChiHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2))*MNeu(INe4)))))/(8.*(MCha**2-MNeu(INe3)*
     &  *2)**2)
          else
          !need to consider
          endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
          if( Abs(-MNeu(INe3) + MNeu(INe4)) .gt. 1D-15) then 
             if(abs((MCha-MNeu(INe4))*(MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3L(INe3,ICH3)*(CF11F12S3R(INe4,2)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(DCONJG(CF11F12S3L(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChiChaHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+CF11F12S3L(INe4,2)*(DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)*(MCha2*DCONJG(CF11F12S3R(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(1,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(1,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChaChiHp(2,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+DCONJG(CF11F12S3L(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChiChaHp(4,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChaChiHp(8,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(8,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(5,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))))+CF11F12S3R(INe3,ICH3)*(CF11F12S3L(INe4,2)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(DCONJG(CF11F12S3R(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChiChaHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+CF11F12S3R(INe4,2)*(DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)*(MCha2*DCONJG(CF11F12S3L(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(1,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(1,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChaChiHp(2,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+DCONJG(CF11F12S3R(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*MNeu(INe4)*
     &  (FD111ChiChaHp(4,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChaChiHp(8,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(8,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(5,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2))))))/(8.*(MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-
     &  MNeu(INe4))*(MNeu(INe3)-MNeu(INe4))*(MCha+MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))
          else
          !need to consider
          endif
              else
             if(abs((MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3R(INe4,2)*(CF11F12S3L(INe3,ICH3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(DCONJG(CF11F12S3L(INe4,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChiChaHp(4,INe3,ICH3)+FD211ChiChaHpm(4,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+CF11F12S3R(INe3,ICH3)*
     &  (DCONJG(CF11F12S3R(INe3,2))*(MCha*DCONJG(CF11F12S3L(INe4,ICH3))
     &  *MNeu(INe3)*(-FD111ChaChaHp(4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)+
     &  FD211ChiChaHpm(4,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(-FD111ChaChaHp(5,ICH3)+
     &  FD111ChaChiHp(8,INe3,ICH3)+FD211ChiChaHpm(5,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+DCONJG(CF11F12S3L(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha2*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD211ChiChaHpm(1,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))*MNeu2(INe3))))+CF11F12S3L(INe4,2)*
     &  (CF11F12S3R(INe3,ICH3)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*
     &  (DCONJG(CF11F12S3R(INe4,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChiChaHp(4,INe3,ICH3)+FD211ChiChaHpm(4,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+CF11F12S3L(INe3,ICH3)*
     &  (DCONJG(CF11F12S3L(INe3,2))*(MCha*DCONJG(CF11F12S3R(INe4,ICH3))
     &  *MNeu(INe3)*(-FD111ChaChaHp(4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)+
     &  FD211ChiChaHpm(4,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaChaHp(5,ICH3)+
     &  FD111ChaChiHp(8,INe3,ICH3)+FD211ChiChaHpm(5,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+DCONJG(CF11F12S3R(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha2*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD211ChiChaHpm(1,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))*MNeu2(INe3)))))/(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
              endif
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
          if( Abs(-MNeu(INe3) + MNeu(INe4)) .gt. 1D-15) then 
             if(abs((MCha-MNeu(INe4))*(MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3L(INe4,ICH3)*(CF11F12S3L(INe4,2)*MNeu(INe4)*
     &  (DCONJG(CF11F12S3L(INe3,ICH3))*(MCha*DCONJG(CF11F12S3R(INe3,2))
     &  +DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*
     &  (FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(MCha2*DCONJG(CF11F12S3R(INe3,2))
     &  *MNeu(INe3)*(FD111ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**
     &  2)+FD111ChaChiHp(1,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(1,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChiChaHp(4,INe4,ICH3)*(-MCha**
     &  2+MNeu(INe3)**2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-MNeu(INe4)
     &  **2)+FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))))+
     &  CF11F12S3R(INe4,2)*(MCha*DCONJG(CF11F12S3R(INe3,ICH3))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(FD111ChiChaHp(4,INe4,ICH3)*(-MCha**2+MNeu(INe3)**
     &  2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(MCha*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)*(FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)
     &  +FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3R(INe3,2))*(FD111ChaChiHp(8,INe4,ICH3)*(-MCha**
     &  2+MNeu(INe3)**2)+FD111ChaChiHp(8,INe3,ICH3)*(MCha**2-MNeu(INe4)
     &  **2)+FD111ChaChaHp(5,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2)))))+
     &  CF11F12S3R(INe4,ICH3)*(CF11F12S3R(INe4,2)*MNeu(INe4)*
     &  (DCONJG(CF11F12S3L(INe3,2))*(MCha2*DCONJG(CF11F12S3L(INe3,ICH3)
     &  )*MNeu(INe3)*(FD111ChaChiHp(1,INe4,ICH3)*(-MCha**2+MNeu(INe3)**
     &  2)+FD111ChaChiHp(1,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(1,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(FD111ChaChiHp(2,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+DCONJG(CF11F12S3R(INe3,2))*(DCONJG(CF11F12S3R(INe3,ICH3)
     &  )*MNeu(INe3)*(FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**
     &  2)+FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(FD111ChiChaHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2))))+CF11F12S3L(INe4,2)*(MCha*DCONJG(CF11F12S3R(INe3,2))*
     &  (DCONJG(CF11F12S3R(INe3,ICH3))*MNeu(INe3)*
     &  (FD111ChaChiHp(2,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChaChiHp(2,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(2,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(FD111ChiChaHp(4,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2)))+DCONJG(CF11F12S3L(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*MNeu(INe3)*
     &  (FD111ChiChaHp(4,INe4,ICH3)*(-MCha**2+MNeu(INe3)**2)+
     &  FD111ChiChaHp(4,INe3,ICH3)*(MCha**2-MNeu(INe4)**2)+
     &  FD111ChaChaHp(4,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)**2))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(FD111ChaChiHp(8,INe4,ICH3)*(-
     &  MCha**2+MNeu(INe3)**2)+FD111ChaChiHp(8,INe3,ICH3)*(MCha**2-
     &  MNeu(INe4)**2)+FD111ChaChaHp(5,ICH3)*(-MNeu(INe3)**2+MNeu(INe4)
     &  **2))))))/(8.*(MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-
     &  MNeu(INe4))*(MNeu(INe3)-MNeu(INe4))*(MCha+MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))
          else
          !need to consider
          endif
              else
             if(abs((MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(CF11F12S3R(INe4,2)*(CF11F12S3L(INe4,ICH3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*(DCONJG(CF11F12S3L(INe3,ICH3))*
     &  MNeu(INe3)*(-FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChiChaHp(4,INe3,ICH3)+FD211ChiChaHpm(4,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+DCONJG(CF11F12S3R(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)+
     &  FD211ChiChaHpm(4,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaChaHp(5,ICH3)+
     &  FD111ChaChiHp(8,INe3,ICH3)+FD211ChiChaHpm(5,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))))+CF11F12S3R(INe4,ICH3)*
     &  (DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (DCONJG(CF11F12S3R(INe3,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChiChaHp(4,INe3,ICH3)+FD211ChiChaHpm(4,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2)))+DCONJG(CF11F12S3L(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*MNeu(INe3)*(-
     &  FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha2*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD211ChiChaHpm(1,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))*MNeu2(INe3))))+CF11F12S3L(INe4,2)*
     &  (CF11F12S3R(INe4,ICH3)*(MCha*DCONJG(CF11F12S3L(INe3,ICH3))*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(-FD111ChaChaHp(4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)+
     &  FD211ChiChaHpm(4,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(MCha*DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)*(-FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaChaHp(5,ICH3)+
     &  FD111ChaChiHp(8,INe3,ICH3)+FD211ChiChaHpm(5,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))))+CF11F12S3L(INe4,ICH3)*
     &  (DCONJG(CF11F12S3L(INe3,ICH3))*MNeu(INe3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(-FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD211ChiChaHpm(2,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(MCha*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)*(-FD111ChaChaHp(4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)+
     &  FD211ChiChaHpm(4,INe3,ICH3)*(MCha**2-MNeu(INe3)**2))+MCha2*
     &  DCONJG(CF11F12S3R(INe3,2))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD211ChiChaHpm(1,INe3,ICH3)*(MCha**
     &  2-MNeu(INe3)**2))*MNeu2(INe3)))))/(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
              endif
       enddo 
       enddo 
       enddo 
c4
        
       do INe3=ineu,3
       do INe4=ineu,3
       do ISt3=1,2
          if( Abs(-MNeu(INe3) + MNeu(INe4)) .gt. 1D-15) then 
             if(abs((MCha-MNeu(INe4))*(MCha-MNeu(INe3))) .gt.1D-10)then
       self=self+(3*(-(((CF11F12S3L(INe4,2)*CF11FSR(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)*DCONJG(CF11F12S3L(INe3,2))+
     &  CF11F12S3R(INe4,2)*CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChaST(8,ISt3))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+
     &  MNeu(INe4))))-((CF11F12S3L(INe4,2)*CF11FSL(INe3,ISt3)*
     &  CF11FSLC(INe4,ISt3)*DCONJG(CF11F12S3L(INe3,2))+
     &  CF11F12S3R(INe4,2)*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChaST(8,ISt3))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+
     &  MNeu(INe4)))-(M1*MCha*(CF11F12S3R(INe4,2)*CF11FSLC(INe3,ISt3)*
     &  CF11FSR(INe4,ISt3)*DCONJG(CF11F12S3L(INe3,2))+
     &  CF11F12S3L(INe4,2)*CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChaST(1,ISt3)*MNeu(INe3)*
     &  MNeu(INe4))/((MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-
     &  MNeu(INe4))*(MCha+MNeu(INe4)))-(M1*MCha*(CF11F12S3R(INe4,2)*
     &  CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3L(INe4,2)*
     &  CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChaST(1,ISt3)*MNeu(INe3)*
     &  MNeu(INe4))/((MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-
     &  MNeu(INe4))*(MCha+MNeu(INe4)))+((CF11F12S3L(INe4,2)*
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3R(INe4,2)*
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111ChiTST(5,INe3,ISt3))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+((CF11F12S3L(INe4,2)*
     &  CF11FSL(INe3,ISt3)*CF11FSLC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3R(INe4,2)*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111ChiTST(5,INe3,ISt3))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+(M1*MCha*(CF11F12S3R(INe4,2)*
     &  CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3L(INe4,2)*
     &  CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChiST(1,INe3,ISt3)*MNeu(INe3)
     &  *MNeu(INe4))/((MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-
     &  MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))+(M1*MCha*
     &  (CF11F12S3R(INe4,2)*CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3L(INe4,2)*
     &  CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChiST(1,INe3,ISt3)*MNeu(INe3)
     &  *MNeu(INe4))/((MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-
     &  MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))+((CF11F12S3L(INe4,2)*
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3R(INe4,2)*
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111ChiTST(5,INe4,ISt3))/((MCha-
     &  MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)+MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+((CF11F12S3L(INe4,2)*
     &  CF11FSL(INe3,ISt3)*CF11FSLC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3R(INe4,2)*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111ChiTST(5,INe4,ISt3))/((MCha-
     &  MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)+MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+(M1*MCha*(CF11F12S3R(INe4,2)*
     &  CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3L(INe4,2)*
     &  CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChiST(1,INe4,ISt3)*MNeu(INe3)
     &  *MNeu(INe4))/((MCha-MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)+
     &  MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))+(M1*MCha*
     &  (CF11F12S3R(INe4,2)*CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))+CF11F12S3L(INe4,2)*
     &  CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2)))*FD111TChiST(1,INe4,ISt3)*MNeu(INe3)
     &  *MNeu(INe4))/((MCha-MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)+
     &  MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))-(M1*FD111TChaST(3,ISt3)*
     &  (CF11F12S3L(INe4,2)*(CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+MNeu(INe4)))+(M1*
     &  FD111ChiTST(4,INe3,ISt3)*(CF11F12S3L(INe4,2)*
     &  (CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*(MNeu(INe3)+
     &  MNeu(INe4)))+(M1*FD111ChiTST(4,INe4,ISt3)*(CF11F12S3L(INe4,2)*
     &  (CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe4))*
     &  (MCha+MNeu(INe4))*(-MNeu(INe3)+MNeu(INe4))*(MNeu(INe3)+
     &  MNeu(INe4)))-(M1*FD111TChaST(3,ISt3)*(CF11F12S3L(INe4,2)*
     &  (CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+MNeu(INe4)))+(M1*
     &  FD111ChiTST(4,INe3,ISt3)*(CF11F12S3L(INe4,2)*
     &  (CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*(MNeu(INe3)+
     &  MNeu(INe4)))+(M1*FD111ChiTST(4,INe4,ISt3)*(CF11F12S3L(INe4,2)*
     &  (CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe4))+CF11F12S3R(INe4,2)*
     &  (CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe4))))/((MCha-MNeu(INe4))*
     &  (MCha+MNeu(INe4))*(-MNeu(INe3)+MNeu(INe4))*(MNeu(INe3)+
     &  MNeu(INe4)))-(FD111TChaST(2,ISt3)*(CF11F12S3L(INe4,2)*(MCha*
     &  CF11FSL(INe3,ISt3)*CF11FSLC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+CF11FSR(INe3,ISt3)*
     &  CF11FSRC(INe4,ISt3)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*MNeu(INe4))+
     &  CF11F12S3R(INe4,2)*(MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)
     &  *DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+CF11FSL(INe3,ISt3)*
     &  CF11FSLC(INe4,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4))))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+
     &  MNeu(INe4)))+(FD111TChiST(2,INe3,ISt3)*(CF11F12S3L(INe4,2)*
     &  (MCha*CF11FSL(INe3,ISt3)*CF11FSLC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+CF11FSR(INe3,ISt3)*
     &  CF11FSRC(INe4,ISt3)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*MNeu(INe4))+
     &  CF11F12S3R(INe4,2)*(MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)
     &  *DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+CF11FSL(INe3,ISt3)*
     &  CF11FSLC(INe4,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4))))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+(FD111TChiST(2,INe4,ISt3)*
     &  (CF11F12S3L(INe4,2)*(MCha*CF11FSL(INe3,ISt3)*
     &  CF11FSLC(INe4,ISt3)*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*MNeu(INe4))+CF11F12S3R(INe4,2)*(MCha*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+CF11FSL(INe3,ISt3)*
     &  CF11FSLC(INe4,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4))))/((MCha-
     &  MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)+MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))-(FD111TChaST(2,ISt3)*
     &  (CF11F12S3L(INe4,2)*(MCha*CF11FSR(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*MNeu(INe4))+CF11F12S3R(INe4,2)*(MCha*
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+CF11FSR(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4))))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+
     &  MNeu(INe4)))+(FD111TChiST(2,INe3,ISt3)*(CF11F12S3L(INe4,2)*
     &  (MCha*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+CF11FSL(INe4,ISt3)*
     &  CF11FSLC(INe3,ISt3)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*MNeu(INe4))+
     &  CF11F12S3R(INe4,2)*(MCha*CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)
     &  *DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)+CF11FSR(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*MNeu(INe4))))/((MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3))*(MNeu(INe3)-MNeu(INe4))*
     &  (MNeu(INe3)+MNeu(INe4)))+(FD111TChiST(2,INe4,ISt3)*
     &  (CF11F12S3L(INe4,2)*(MCha*CF11FSR(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)+
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*MNeu(INe4))+CF11F12S3R(INe4,2)*(MCha*
     &  CF11FSL(INe4,ISt3)*CF11FSLC(INe3,ISt3)*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)+CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))
     &  *MNeu(INe4))))/((MCha-MNeu(INe4))*(MCha+MNeu(INe4))*(-MNeu(INe3)
     &  +MNeu(INe4))*(MNeu(INe3)+MNeu(INe4)))))/8.
                  else
                  !need to consider
                  endif
              else
                  if (abs(MCha-MNeu(INe3)) .gt.1D-10)then
       self=self+(3*(CF11F12S3R(INe4,2)*(CF11FSR(INe3,ISt3)*
     &  CF11FSRC(INe4,ISt3)*DCONJG(CF11F12S3R(INe3,2))*
     &  FD111ChiTST(5,INe3,ISt3)-CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)
     &  *DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(8,ISt3)+MCha**2*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(5,INe3,ISt3)+M1*
     &  CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111ChiTST(4,INe3,ISt3)*MNeu(INe3)+
     &  M1*CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111ChiTST(4,INe3,ISt3)*MNeu(INe3)-
     &  MCha*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)-MCha*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)-M1*
     &  CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(3,ISt3)*MNeu(INe3)-M1*
     &  CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(3,ISt3)*MNeu(INe3)+MCha*
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha**3*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha**3*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)+
     &  M1*MCha**2*CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)+
     &  M1*MCha**2*CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)-
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)**2+
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)*
     &  *2+MCha**2*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *2-CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(5,INe3,ISt3)*MNeu(INe3)*
     &  *2-MCha*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *3-MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *3-M1*CF11FSLC(INe4,ISt3)*CF11FSR(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)*
     &  *3-M1*CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)*
     &  *3-CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *4+CF11FSL(INe3,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*(CF11FSLC(INe4,ISt3)*
     &  MNeu(INe3)*(-FD111TChaST(2,ISt3)+FD111TChiST(2,INe3,ISt3)+
     &  FD211ChiTST(2,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+M1*
     &  CF11FSRC(INe4,ISt3)*(FD111ChiTST(4,INe3,ISt3)-
     &  FD111TChaST(3,ISt3)+FD211ChiTST(4,INe3,ISt3)*(MCha**2-
     &  MNeu(INe3)**2)))+CF11FSL(INe4,ISt3)*(M1*CF11FSRC(INe3,ISt3)*
     &  (MCha*DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(FD111ChiTST(4,INe3,ISt3)-FD111TChaST(3,ISt3)+
     &  FD211ChiTST(4,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+
     &  CF11FSLC(INe3,ISt3)*(MCha*DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)
     &  *(-FD111TChaST(2,ISt3)+FD111TChiST(2,INe3,ISt3)+
     &  FD211ChiTST(2,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3R(INe3,2))*(FD111ChiTST(5,INe3,ISt3)-
     &  FD111TChaST(8,ISt3)+FD211ChiTST(5,INe3,ISt3)*(MCha**2-
     &  MNeu(INe3)**2))))+M1*MCha*(CF11FSLC(INe4,ISt3)*
     &  CF11FSR(INe3,ISt3)+CF11FSLC(INe3,ISt3)*CF11FSR(INe4,ISt3))*
     &  DCONJG(CF11F12S3L(INe3,2))*(-FD111TChaST(1,ISt3)+
     &  FD111TChiST(1,INe3,ISt3)+FD211ChiTST(1,INe3,ISt3)*(MCha**2-
     &  MNeu(INe3)**2))*MNeu2(INe3))+CF11F12S3L(INe4,2)*
     &  (CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111ChiTST(5,INe3,ISt3)-
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(8,ISt3)+MCha**2*
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(5,INe3,ISt3)+M1*
     &  CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111ChiTST(4,INe3,ISt3)*MNeu(INe3)+
     &  M1*CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111ChiTST(4,INe3,ISt3)*MNeu(INe3)-
     &  MCha*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)-MCha*
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)-M1*
     &  CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(3,ISt3)*MNeu(INe3)-M1*
     &  CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(3,ISt3)*MNeu(INe3)+MCha*
     &  CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha**3*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)+
     &  MCha**3*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)+
     &  M1*MCha**2*CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)+
     &  M1*MCha**2*CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)-
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChaST(2,ISt3)*MNeu(INe3)**2+
     &  CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChiST(2,INe3,ISt3)*MNeu(INe3)*
     &  *2+MCha**2*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *2-CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(5,INe3,ISt3)*MNeu(INe3)*
     &  *2-MCha*CF11FSR(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *3-MCha*CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *3-M1*CF11FSL(INe4,ISt3)*CF11FSRC(INe3,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)*
     &  *3-M1*CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(4,INe3,ISt3)*MNeu(INe3)*
     &  *3-CF11FSR(INe3,ISt3)*CF11FSRC(INe4,ISt3)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD211ChiTST(2,INe3,ISt3)*MNeu(INe3)*
     &  *4+CF11FSLC(INe3,ISt3)*(MCha*DCONJG(CF11F12S3R(INe3,2))+
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3))*(CF11FSL(INe4,ISt3)*
     &  MNeu(INe3)*(-FD111TChaST(2,ISt3)+FD111TChiST(2,INe3,ISt3)+
     &  FD211ChiTST(2,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+M1*
     &  CF11FSR(INe4,ISt3)*(FD111ChiTST(4,INe3,ISt3)-
     &  FD111TChaST(3,ISt3)+FD211ChiTST(4,INe3,ISt3)*(MCha**2-
     &  MNeu(INe3)**2)))+CF11FSLC(INe4,ISt3)*(M1*CF11FSR(INe3,ISt3)*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))*(FD111ChiTST(4,INe3,ISt3)-FD111TChaST(3,ISt3)+
     &  FD211ChiTST(4,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+
     &  CF11FSL(INe3,ISt3)*(MCha*DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)*
     &  (-FD111TChaST(2,ISt3)+FD111TChiST(2,INe3,ISt3)+
     &  FD211ChiTST(2,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))+
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChiTST(5,INe3,ISt3)-
     &  FD111TChaST(8,ISt3)+FD211ChiTST(5,INe3,ISt3)*(MCha**2-
     &  MNeu(INe3)**2))))+M1*MCha*(CF11FSL(INe4,ISt3)*
     &  CF11FSRC(INe3,ISt3)+CF11FSL(INe3,ISt3)*CF11FSRC(INe4,ISt3))*
     &  DCONJG(CF11F12S3R(INe3,2))*(-FD111TChaST(1,ISt3)+FD111TChiST(1,INe3,ISt3)
     &  +FD211ChiTST(1,INe3,ISt3)*(MCha**2-MNeu(INe3)**2))*MNeu2(INe3))))
     &  /(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
              endif
       enddo 
       enddo 
       enddo 
c5
        
       do INe3=ineu,3
       do ISt3=1,2
         if (abs(MCha-MNeu(INe3)) .gt.1D-10)then
       self=self+(-3*CF12F4S(ISt3)*CF12F4SC(ISt3)*(MCha*
     &  CF11F12S3L(INe3,2)*(MCha*DCONJG(CF11F12S3L(INe3,2))+
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3))*(-FD111BChaST(2,ISt3)+
     &  FD111BChiST(2,INe3,ISt3)+FD211ChaBST(2,ISt3)*(MCha**2-
     &  MNeu(INe3)**2))+CF11F12S3R(INe3,2)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*(-FD111BChaST(2,ISt3)+
     &  FD111BChiST(2,INe3,ISt3)+FD211ChaBST(2,ISt3)*(MCha**2-
     &  MNeu(INe3)**2))+DCONJG(CF11F12S3R(INe3,2))*(-
     &  FD111BChaST(8,ISt3)+FD111BChiST(8,INe3,ISt3)+
     &  FD211ChaBST(5,ISt3)*(MCha**2-MNeu(INe3)**2)))))/(8.*(MCha**2-MNeu(INe3)**2)**2)
          else
          !need to consider
          endif
       enddo 
       enddo 
c6 =0

c7
        
       do INe3=ineu,3
         if (abs(MCha-MNeu(INe3)) .gt.1D-10)then
       self=self+(-3*CF12F3S*CF12F3SC*(MCha*CF11F12S3L(INe3,2)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))*(FD111ChiTSB(2,INe3)-FD111TChaSB(2)+MCha**2*
     &  FD211ChaTSB(2)-FD211ChaTSB(2)*MNeu(INe3)**2)+CF11F12S3R(INe3,2)
     &  *(MCha*DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)*
     &  (FD111ChiTSB(2,INe3)-FD111TChaSB(2)+MCha**2*FD211ChaTSB(2)-
     &  FD211ChaTSB(2)*MNeu(INe3)**2)+DCONJG(CF11F12S3R(INe3,2))*
     &  (FD111ChiTSB(5,INe3)-FD111TChaSB(8)+MCha**2*FD211ChaTSB(5)-
     &  FD211ChaTSB(5)*MNeu(INe3)**2))))/(8.*(MCha**2-MNeu(INe3)**2)**
     &  2)
          else
          !need to consider
          endif
       enddo 


c 
c topo8-NoGBC
c
c 
c topo8-GBC
c
        
       do INe3=ineu,3
       do INe4=ineu,3
       do INH3=ihiggs,6
        if(abs((MCha-MNeu(INe3))*(MCha-MNeu(INe4))).gt.1D-10)then
       self=self+(CF11F12S3R(INe4,2)*(CFF11S1L(INe4,INe3,INH3)*(-
     &  (CFF12S1R(INH3)*(FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)
     &  -FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3))*
     &  (MCha2*DCONJG(CF11F12S3L(INe3,2))+MCha*
     &  DCONJG(CF11F12S3R(INe3,2))*MNeu(INe3)))+CFF12S1L(INH3)*
     &  (DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaChaH(6,INH3)+
     &  FD111ChaChiH(6,INe3,INH3)+FD111ChaChiH(6,INe4,INH3)-
     &  FD111ChiChiH(6,INe3,INe4,INH3))+MCha*DCONJG(CF11F12S3R(INe3,2))
     &  *(-FD111ChaChaH(4,INH3)+FD111ChaChiH(4,INe3,INH3)+
     &  FD111ChiChaH(4,INe4,INH3)-FD111ChiChiH(4,INe4,INe3,INH3))*
     &  MNeu(INe3)))+CFF11S1R(INe4,INe3,INH3)*(-(CFF12S1L(INH3)*
     &  (FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)-
     &  FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3))*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)))+CFF12S1R(INH3)*(MCha*DCONJG(CF11F12S3R(INe3,2))*(-
     &  FD111ChaChaH(4,INH3)+FD111ChaChiH(4,INe4,INH3)+
     &  FD111ChiChaH(4,INe3,INH3)-FD111ChiChiH(4,INe3,INe4,INH3))+
     &  MCha2*DCONJG(CF11F12S3L(INe3,2))*(FD111ChaChiH(1,INe3,INH3)+
     &  FD111ChaChiH(1,INe4,INH3)-FD111ChaHCha(1,INH3)-
     &  FD111ChiChiH(1,INe3,INe4,INH3))*MNeu(INe3)))*MNeu(INe4))+
     &  CF11F12S3L(INe4,2)*(CFF11S1R(INe4,INe3,INH3)*(-(CFF12S1L(INH3)*
     &  (FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)-
     &  FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3))*
     &  (MCha2*DCONJG(CF11F12S3R(INe3,2))+MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*MNeu(INe3)))+CFF12S1R(INH3)*
     &  (DCONJG(CF11F12S3R(INe3,2))*(-FD111ChaChaH(6,INH3)+
     &  FD111ChaChiH(6,INe3,INH3)+FD111ChaChiH(6,INe4,INH3)-
     &  FD111ChiChiH(6,INe3,INe4,INH3))+MCha*DCONJG(CF11F12S3L(INe3,2))
     &  *(-FD111ChaChaH(4,INH3)+FD111ChaChiH(4,INe3,INH3)+
     &  FD111ChiChaH(4,INe4,INH3)-FD111ChiChiH(4,INe4,INe3,INH3))*
     &  MNeu(INe3)))+CFF11S1L(INe4,INe3,INH3)*(-(CFF12S1R(INH3)*
     &  (FD111ChaChaH(2,INH3)-FD111ChaChiH(2,INe3,INH3)-
     &  FD111ChaChiH(2,INe4,INH3)+FD111ChiChiH(2,INe3,INe4,INH3))*
     &  (MCha*DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)))+CFF12S1L(INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))*(-
     &  FD111ChaChaH(4,INH3)+FD111ChaChiH(4,INe4,INH3)+
     &  FD111ChiChaH(4,INe3,INH3)-FD111ChiChiH(4,INe3,INe4,INH3))+
     &  MCha2*DCONJG(CF11F12S3R(INe3,2))*(FD111ChaChiH(1,INe3,INH3)+
     &  FD111ChaChiH(1,INe4,INH3)-FD111ChaHCha(1,INH3)-
     &  FD111ChiChiH(1,INe3,INe4,INH3))*MNeu(INe3)))*MNeu(INe4)))/(8.*
     &  (MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+MNeu(INe4)))
         else
          !need to consider
         endif
       enddo 
       enddo 
       enddo 

       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
        if(abs((MCha-MNeu(INe3))*(MCha-MNeu(INe4))).gt.1D-10)then
       self=self+(CF11F12S3L(INe4,ICH3)*(CF11F12S3R(INe4,2)*(-
     &  (DCONJG(CF11F12S3L(INe3,ICH3))*(FD111ChaChaHp(2,ICH3)-
     &  FD111ChaChiHp(2,INe3,ICH3)-FD111ChaChiHp(2,INe4,ICH3)+
     &  FD111ChiChiHp(2,INe3,INe4,ICH3))*(MCha2*
     &  DCONJG(CF11F12S3R(INe3,2))+MCha*DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)))+DCONJG(CF11F12S3R(INe3,ICH3))*
     &  (DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaChaHp(6,ICH3)+
     &  FD111ChaChiHp(6,INe3,ICH3)+FD111ChaChiHp(6,INe4,ICH3)-
     &  FD111ChiChiHp(6,INe3,INe4,ICH3))+MCha*DCONJG(CF11F12S3R(INe3,2)
     &  )*(-FD111ChaChaHp(4,ICH3)+FD111ChaChiHp(4,INe3,ICH3)+
     &  FD111ChiChaHp(4,INe4,ICH3)-FD111ChiChiHp(4,INe4,INe3,ICH3))*
     &  MNeu(INe3)))+CF11F12S3L(INe4,2)*(-(DCONJG(CF11F12S3L(INe3,ICH3)
     &  )*(FD111ChaChaHp(2,ICH3)-FD111ChaChiHp(2,INe3,ICH3)-
     &  FD111ChaChiHp(2,INe4,ICH3)+FD111ChiChiHp(2,INe3,INe4,ICH3))*
     &  (MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3)))+DCONJG(CF11F12S3R(INe3,ICH3))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChaChiHp(4,INe4,ICH3)+FD111ChiChaHp(4,INe3,ICH3)-
     &  FD111ChiChiHp(4,INe3,INe4,ICH3))+MCha2*
     &  DCONJG(CF11F12S3R(INe3,2))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD111ChaChiHp(1,INe4,ICH3)-
     &  FD111ChiChiHp(1,INe3,INe4,ICH3))*MNeu(INe3)))*MNeu(INe4))+
     &  CF11F12S3R(INe4,ICH3)*(CF11F12S3L(INe4,2)*
     &  (DCONJG(CF11F12S3R(INe3,2))*(DCONJG(CF11F12S3L(INe3,ICH3))*(-
     &  FD111ChaChaHp(6,ICH3)+FD111ChaChiHp(6,INe3,ICH3)+
     &  FD111ChaChiHp(6,INe4,ICH3)-FD111ChiChiHp(6,INe3,INe4,ICH3))+
     &  MCha*DCONJG(CF11F12S3R(INe3,ICH3))*(-FD111ChaChaHp(2,ICH3)+
     &  FD111ChaChiHp(2,INe3,ICH3)+FD111ChaChiHp(2,INe4,ICH3)-
     &  FD111ChiChiHp(2,INe3,INe4,ICH3))*MNeu(INe3))+
     &  DCONJG(CF11F12S3L(INe3,2))*(MCha2*DCONJG(CF11F12S3R(INe3,ICH3))
     &  *(-FD111ChaChaHp(2,ICH3)+FD111ChaChiHp(2,INe3,ICH3)+
     &  FD111ChaChiHp(2,INe4,ICH3)-FD111ChiChiHp(2,INe3,INe4,ICH3))+
     &  MCha*DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaChaHp(4,ICH3)+
     &  FD111ChaChiHp(4,INe3,ICH3)+FD111ChiChaHp(4,INe4,ICH3)-
     &  FD111ChiChiHp(4,INe4,INe3,ICH3))*MNeu(INe3)))+
     &  CF11F12S3R(INe4,2)*(DCONJG(CF11F12S3L(INe3,2))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(-FD111ChaChaHp(2,ICH3)+
     &  FD111ChaChiHp(2,INe3,ICH3)+FD111ChaChiHp(2,INe4,ICH3)-
     &  FD111ChiChiHp(2,INe3,INe4,ICH3))+MCha2*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaChaHp(1,ICH3)+
     &  FD111ChaChiHp(1,INe3,ICH3)+FD111ChaChiHp(1,INe4,ICH3)-
     &  FD111ChiChiHp(1,INe3,INe4,ICH3))*MNeu(INe3))+
     &  DCONJG(CF11F12S3R(INe3,2))*(MCha*DCONJG(CF11F12S3L(INe3,ICH3))*
     &  (-FD111ChaChaHp(4,ICH3)+FD111ChaChiHp(4,INe4,ICH3)+
     &  FD111ChiChaHp(4,INe3,ICH3)-FD111ChiChiHp(4,INe3,INe4,ICH3))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(-FD111ChaChaHp(2,ICH3)+
     &  FD111ChaChiHp(2,INe3,ICH3)+FD111ChaChiHp(2,INe4,ICH3)-
     &  FD111ChiChiHp(2,INe3,INe4,ICH3))*MNeu(INe3)))*MNeu(INe4)))/(8.*
     &  (MCha-MNeu(INe3))*(MCha+MNeu(INe3))*(MCha-MNeu(INe4))*(MCha+MNeu(INe4)))
         else
          !need to consider
         endif
       enddo 
       enddo 
       enddo 

       do INe3=ineu,3
       do ISt3=1,2
        if(abs((MCha-MNeu(INe3))).gt.1D-10)then
       self=self+(3*(CTBS3R*M1*CF11F12S3R(INe3,2)*CF11FSRC(INe3,ISt3)*
     &  CF12F4S(ISt3)*FD111BChaST(6,ISt3)+M1*CF11FSR(INe3,ISt3)*
     &  CF12F4SC(ISt3)*CTBS3C(2)*DCONJG(CF11F12S3R(INe3,2))*
     &  FD111BChaST(6,ISt3)-CTBS3R*M1*CF11F12S3R(INe3,2)*
     &  CF11FSRC(INe3,ISt3)*CF12F4S(ISt3)*FD111BChiST(6,INe3,ISt3)-M1*
     &  CF11FSR(INe3,ISt3)*CF12F4SC(ISt3)*CTBS3C(2)*
     &  DCONJG(CF11F12S3R(INe3,2))*FD111BChiST(6,INe3,ISt3)-CTBS3R*M1*
     &  CF11F12S3R(INe3,2)*CF11FSRC(INe3,ISt3)*CF12F4S(ISt3)*
     &  FD111TChaST(6,ISt3)-M1*CF11FSR(INe3,ISt3)*CF12F4SC(ISt3)*
     &  CTBS3C(2)*DCONJG(CF11F12S3R(INe3,2))*FD111TChaST(6,ISt3)+
     &  CTBS3R*M1*CF11F12S3R(INe3,2)*CF11FSRC(INe3,ISt3)*CF12F4S(ISt3)*
     &  FD111TChiST(6,INe3,ISt3)+M1*CF11FSR(INe3,ISt3)*CF12F4SC(ISt3)*
     &  CTBS3C(2)*DCONJG(CF11F12S3R(INe3,2))*FD111TChiST(6,INe3,ISt3)+
     &  CTBS3R*M12*CF11F12S3R(INe3,2)*CF11FSLC(INe3,ISt3)*CF12F4S(ISt3)
     &  *FD111BChaST(2,ISt3)*MNeu(INe3)+M1*MCha*CF11FSR(INe3,ISt3)*
     &  CF12F4SC(ISt3)*CTBS3C(2)*DCONJG(CF11F12S3L(INe3,2))*
     &  FD111BChaST(4,ISt3)*MNeu(INe3)-CTBS3R*M12*CF11F12S3R(INe3,2)*
     &  CF11FSLC(INe3,ISt3)*CF12F4S(ISt3)*FD111BChiST(2,INe3,ISt3)*
     &  MNeu(INe3)-M1*MCha*CF11FSR(INe3,ISt3)*CF12F4SC(ISt3)*CTBS3C(2)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111BChiST(4,INe3,ISt3)*MNeu(INe3)-
     &  CTBS3R*M12*CF11F12S3R(INe3,2)*CF11FSLC(INe3,ISt3)*CF12F4S(ISt3)
     &  *FD111TChaST(2,ISt3)*MNeu(INe3)-M1*MCha*CF11FSR(INe3,ISt3)*
     &  CF12F4SC(ISt3)*CTBS3C(2)*DCONJG(CF11F12S3L(INe3,2))*
     &  FD111TChaST(4,ISt3)*MNeu(INe3)+CTBS3R*M12*CF11F12S3R(INe3,2)*
     &  CF11FSLC(INe3,ISt3)*CF12F4S(ISt3)*FD111TChiST(2,INe3,ISt3)*
     &  MNeu(INe3)+M1*MCha*CF11FSR(INe3,ISt3)*CF12F4SC(ISt3)*CTBS3C(2)*
     &  DCONJG(CF11F12S3L(INe3,2))*FD111TChiST(4,INe3,ISt3)*MNeu(INe3)+
     &  M12*CF11FSL(INe3,ISt3)*CF12F4SC(ISt3)*CTBS3C(2)*
     &  (FD111BChaST(2,ISt3)-FD111BChiST(2,INe3,ISt3)-
     &  FD111TChaST(2,ISt3)+FD111TChiST(2,INe3,ISt3))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CTBS3R*MCha*CF11F12S3L(INe3,2)*CF12F4S(ISt3)*(M12*
     &  CF11FSLC(INe3,ISt3)*(FD111BChaST(2,ISt3)-
     &  FD111BChiST(2,INe3,ISt3)-FD111TChaST(2,ISt3)+
     &  FD111TChiST(2,INe3,ISt3))+M1*CF11FSRC(INe3,ISt3)*
     &  (FD111BChaST(4,ISt3)-FD111BChiST(4,INe3,ISt3)-
     &  FD111TChaST(4,ISt3)+FD111TChiST(4,INe3,ISt3))*MNeu(INe3))))
     &  /(8.*M1*M12*(MCha-MNeu(INe3))*(MCha+MNeu(INe3)))
         else
          !need to consider
         endif
       enddo 
       enddo 
c 
c topo4-NoGBC
c
c 
c topo4-GBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CTBS3R*CS1S3S3(INH3,ICH3,2)*CTBS3C(ICH3)*
     &  (FD111THpT(4,ICH3)*hy(INH3)-FD111THT(4,INH3)*hy(INH3)+
     &  FD111BHpT(2,ICH3)*hyc(INH3)-FD111BHT(2,INH3)*hyc(INH3)-
     &  FD111THpT(2,ICH3)*hyc(INH3)+FD111THpT(4,ICH3)*hyc(INH3)+
     &  FD111THT(2,INH3)*hyc(INH3)-FD111THT(4,INH3)*hyc(INH3)-
     &  FD111BHpT(4,ICH3)*(hy(INH3)+hyc(INH3))+FD111BHT(4,INH3)*
     &  (hy(INH3)+hyc(INH3))))/(8.*M1*(Mah(INH3)-MassHpm(ICH3))*
     &  (Mah(INH3)+MassHpm(ICH3)))
              endif
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
           if (abs(Mcha2-MNeu2(INe3)).gt.1D-10)then
       self=self+(CS1S3S3(INH3,ICH3,2)*(CF11F12S3R(INe4,ICH3)*
     &  (CFF11S1L(INe3,INe4,INH3)*(FD111ChaHChi(2,INH3,INe4)-
     &  FD111ChaHChi(4,INH3,INe4)-FD111ChaHpChi(2,ICH3,INe4)+
     &  FD111ChaHpChi(4,ICH3,INe4)-FD111ChiHChi(2,INe3,INH3,INe4)+
     &  FD111ChiHChi(4,INe3,INH3,INe4)+FD111ChiHpChi(2,INe3,ICH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CFF11S1R(INe3,INe4,INH3)*(DCONJG(CF11F12S3R(INe3,2)
     &  )*(-FD111ChaHChi(4,INH3,INe4)+FD111ChaHpChi(4,ICH3,INe4)+
     &  FD111ChiHChi(4,INe3,INH3,INe4)-FD111ChiHpChi(4,INe3,ICH3,INe4))
     &  +MCha*DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaHChi(1,INH3,INe4)+
     &  FD111ChaHpChi(1,ICH3,INe4)+FD111ChiHChi(1,INe3,INH3,INe4)-
     &  FD111ChiHpChi(1,INe3,ICH3,INe4))*MNeu(INe3))*MNeu(INe4))+
     &  CF11F12S3L(INe4,ICH3)*(CFF11S1R(INe3,INe4,INH3)*
     &  (FD111ChaHChi(2,INH3,INe4)-FD111ChaHChi(4,INH3,INe4)-
     &  FD111ChaHpChi(2,ICH3,INe4)+FD111ChaHpChi(4,ICH3,INe4)-
     &  FD111ChiHChi(2,INe3,INH3,INe4)+FD111ChiHChi(4,INe3,INH3,INe4)+
     &  FD111ChiHpChi(2,INe3,ICH3,INe4)-FD111ChiHpChi(4,INe3,ICH3,INe4)
     &  )*(MCha*DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CFF11S1L(INe3,INe4,INH3)*(DCONJG(CF11F12S3L(INe3,2)
     &  )*(-FD111ChaHChi(4,INH3,INe4)+FD111ChaHpChi(4,ICH3,INe4)+
     &  FD111ChiHChi(4,INe3,INH3,INe4)-FD111ChiHpChi(4,INe3,ICH3,INe4))
     &  +MCha*DCONJG(CF11F12S3R(INe3,2))*(-FD111ChaHChi(1,INH3,INe4)+
     &  FD111ChaHpChi(1,ICH3,INe4)+FD111ChiHChi(1,INe3,INH3,INe4)-
     &  FD111ChiHpChi(1,INe3,ICH3,INe4))*MNeu(INe3))*MNeu(INe4))))/(8.*
     &  (Mah(INH3)-MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*(MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3)))
         else
         ! need to consider
         endif
              endif
       enddo 
       enddo 
       enddo 
       enddo
        
       do ICH3=1,2
       do INe3=ineu,3
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
           if (abs(Mcha2-MNeu2(INe3)).gt.1D-10)then
       self=self+(CS1S3S3(INH3,ICH3,2)*(CF11F12S3R(INe3,ICH3)*
     &  (CFF12S1L(INH3)*(FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3)-
     &  FD111ChaHpCha(2,ICH3)+FD111ChaHpCha(4,ICH3)-
     &  FD111ChiHCha(2,INe3,INH3)+FD111ChiHCha(4,INe3,INH3)+
     &  FD111ChiHpCha(2,INe3,ICH3)-FD111ChiHpCha(4,INe3,ICH3))*(MCha*
     &  DCONJG(CF11F12S3R(INe3,2))+DCONJG(CF11F12S3L(INe3,2))*
     &  MNeu(INe3))+CFF12S1R(INH3)*(MCha*DCONJG(CF11F12S3R(INe3,2))*(-
     &  FD111ChaHCha(4,INH3)+FD111ChaHpCha(4,ICH3)+
     &  FD111ChiHCha(4,INe3,INH3)-FD111ChiHpCha(4,INe3,ICH3))+MCha2*
     &  DCONJG(CF11F12S3L(INe3,2))*(-FD111ChaHCha(1,INH3)+
     &  FD111ChaHpCha(1,ICH3)+FD111ChiHCha(1,INe3,INH3)-
     &  FD111ChiHpCha(1,INe3,ICH3))*MNeu(INe3)))+CF11F12S3L(INe3,ICH3)*
     &  (CFF12S1R(INH3)*(FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3)-
     &  FD111ChaHpCha(2,ICH3)+FD111ChaHpCha(4,ICH3)-
     &  FD111ChiHCha(2,INe3,INH3)+FD111ChiHCha(4,INe3,INH3)+
     &  FD111ChiHpCha(2,INe3,ICH3)-FD111ChiHpCha(4,INe3,ICH3))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3))+CFF12S1L(INH3)*(MCha*DCONJG(CF11F12S3L(INe3,2))*(-
     &  FD111ChaHCha(4,INH3)+FD111ChaHpCha(4,ICH3)+
     &  FD111ChiHCha(4,INe3,INH3)-FD111ChiHpCha(4,INe3,ICH3))+MCha2*
     &  DCONJG(CF11F12S3R(INe3,2))*(-FD111ChaHCha(1,INH3)+
     &  FD111ChaHpCha(1,ICH3)+FD111ChiHCha(1,INe3,INH3)-
     &  FD111ChiHpCha(1,INe3,ICH3))*MNeu(INe3)))))/(8.*(Mah(INH3)-
     &  MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*(MCha-MNeu(INe3))*(MCha+MNeu(INe3)))
         else
         ! need to consider
         endif
              endif
       enddo 
       enddo 
       enddo 
        
       do INe3=ineu,3
       do ISt3=1,2
           if (abs(Mcha2-MNeu2(INe3)).gt.1D-10)then
       self=self+(3*CF12F3S*CS3S14S13(ISt3)*(-(CF11FSRC(INe3,ISt3)*
     &  (FD111ChaSTT(2,ISt3)-FD111ChaSTT(4,ISt3)-FD111ChiSBT(4,INe3)-
     &  FD111ChiStT(2,INe3,ISt3)+FD111ChiStT(4,INe3,ISt3)-
     &  FD111SBChaT(2)+FD111SBChaT(3)+FD111SBChiT(2,INe3))*(MCha*
     &  DCONJG(CF11F12S3L(INe3,2))+DCONJG(CF11F12S3R(INe3,2))*
     &  MNeu(INe3)))+M1*CF11FSLC(INe3,ISt3)*(DCONJG(CF11F12S3R(INe3,2))
     &  *(FD111ChaSTT(4,ISt3)+FD111ChiSBT(4,INe3)-
     &  FD111ChiStT(4,INe3,ISt3)-FD111SBChaT(3))+MCha*
     &  DCONJG(CF11F12S3L(INe3,2))*(FD111ChaSTT(1,ISt3)-
     &  FD111ChiStT(1,INe3,ISt3)-FD111SBChaT(1)+FD111SBChiT(1,INe3))*
     &  MNeu(INe3))))/(8.*(MSQ-MassSu(ISt3))*(MSQ+MassSu(ISt3))*(MCha-
     &  MNeu(INe3))*(MCha+MNeu(INe3)))
         else
         ! need to consider
         endif
       enddo 
       enddo 

c 
c topo5-NoGBC
c 
c 
c topo5-GBC
c
        
       do ICH3=1,2
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(3*CS1S3S3(INH3,2,ICH3)*CTBS3(ICH3)*CTBS3C(2)*
     &  (FD111BHpT(2,ICH3)*hy(INH3)-FD111BHT(2,INH3)*hy(INH3)+
     &  FD111BHT(4,INH3)*hy(INH3)-FD111THpT(2,ICH3)*hy(INH3)+
     &  FD111THpT(4,ICH3)*hy(INH3)+FD111THT(2,INH3)*hy(INH3)-
     &  FD111THT(4,INH3)*hy(INH3)+FD111BHT(4,INH3)*hyc(INH3)+
     &  FD111THpT(4,ICH3)*hyc(INH3)-FD111THT(4,INH3)*hyc(INH3)-
     &  FD111BHpT(4,ICH3)*(hy(INH3)+hyc(INH3))))/(8.*M1*(Mah(INH3)-
     &  MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3)))
              endif
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INe4=ineu,3
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
           if(abs(MCha-MNeu(INe3)).gt.1D-10)then
       self=self+(CS1S3S3(INH3,2,ICH3)*(CF11F12S3L(INe3,2)*
     &  (CFF11S1L(INe3,INe4,INH3)*MNeu(INe3)*
     &  (DCONJG(CF11F12S3L(INe4,ICH3))*(FD111ChaHChi(2,INH3,INe4)-
     &  FD111ChaHChi(4,INH3,INe4)-FD111ChaHpChi(2,ICH3,INe4)+
     &  FD111ChaHpChi(4,ICH3,INe4)-FD111ChiHChi(2,INe3,INH3,INe4)+
     &  FD111ChiHChi(4,INe3,INH3,INe4)+FD111ChiHpChi(2,INe3,ICH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))+MCha*
     &  DCONJG(CF11F12S3R(INe4,ICH3))*(-FD111ChaHChi(1,INH3,INe4)+
     &  FD111ChaHpChi(1,ICH3,INe4)+FD111ChiHChi(1,INe3,INH3,INe4)-
     &  FD111ChiHpChi(1,INe3,ICH3,INe4))*MNeu(INe4))+
     &  CFF11S1R(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3R(INe4,ICH3))*
     &  (FD111ChaHChi(2,INH3,INe4)-FD111ChaHChi(4,INH3,INe4)-
     &  FD111ChaHpChi(2,ICH3,INe4)+FD111ChaHpChi(4,ICH3,INe4)-
     &  FD111ChiHChi(2,INe3,INH3,INe4)+FD111ChiHChi(4,INe3,INH3,INe4)+
     &  FD111ChiHpChi(2,INe3,ICH3,INe4)-FD111ChiHpChi(4,INe3,ICH3,INe4)
     &  )+DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaHChi(4,INH3,INe4)+
     &  FD111ChaHpChi(4,ICH3,INe4)+FD111ChiHChi(4,INe3,INH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))*MNeu(INe4)))+
     &  CF11F12S3R(INe3,2)*(CFF11S1R(INe3,INe4,INH3)*MNeu(INe3)*
     &  (DCONJG(CF11F12S3R(INe4,ICH3))*(FD111ChaHChi(2,INH3,INe4)-
     &  FD111ChaHChi(4,INH3,INe4)-FD111ChaHpChi(2,ICH3,INe4)+
     &  FD111ChaHpChi(4,ICH3,INe4)-FD111ChiHChi(2,INe3,INH3,INe4)+
     &  FD111ChiHChi(4,INe3,INH3,INe4)+FD111ChiHpChi(2,INe3,ICH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))+MCha*
     &  DCONJG(CF11F12S3L(INe4,ICH3))*(-FD111ChaHChi(1,INH3,INe4)+
     &  FD111ChaHpChi(1,ICH3,INe4)+FD111ChiHChi(1,INe3,INH3,INe4)-
     &  FD111ChiHpChi(1,INe3,ICH3,INe4))*MNeu(INe4))+
     &  CFF11S1L(INe3,INe4,INH3)*(MCha*DCONJG(CF11F12S3L(INe4,ICH3))*
     &  (FD111ChaHChi(2,INH3,INe4)-FD111ChaHChi(4,INH3,INe4)-
     &  FD111ChaHpChi(2,ICH3,INe4)+FD111ChaHpChi(4,ICH3,INe4)-
     &  FD111ChiHChi(2,INe3,INH3,INe4)+FD111ChiHChi(4,INe3,INH3,INe4)+
     &  FD111ChiHpChi(2,INe3,ICH3,INe4)-FD111ChiHpChi(4,INe3,ICH3,INe4)
     &  )+DCONJG(CF11F12S3R(INe4,ICH3))*(-FD111ChaHChi(4,INH3,INe4)+
     &  FD111ChaHpChi(4,ICH3,INe4)+FD111ChiHChi(4,INe3,INH3,INe4)-
     &  FD111ChiHpChi(4,INe3,ICH3,INe4))*MNeu(INe4)))))/(8.*(Mah(INH3)-
     &  MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*(MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3)))
            else
             !need to consider
            endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
        
       do ICH3=1,2
       do INe3=ineu,3
       do INH3=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
           if(abs(MCha-MNeu(INe3)).gt.1D-10)then
       self=self+(CS1S3S3(INH3,2,ICH3)*(CF11F12S3R(INe3,2)*
     &  (CFF12S1R(INH3)*(MCha*DCONJG(CF11F12S3R(INe3,ICH3))*
     &  (FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3)-
     &  FD111ChaHpCha(2,ICH3)+FD111ChaHpCha(4,ICH3)-
     &  FD111ChiHCha(2,INe3,INH3)+FD111ChiHCha(4,INe3,INH3)+
     &  FD111ChiHpCha(2,INe3,ICH3)-FD111ChiHpCha(4,INe3,ICH3))+MCha2*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(-FD111ChaHCha(1,INH3)+
     &  FD111ChaHpCha(1,ICH3)+FD111ChiHCha(1,INe3,INH3)-
     &  FD111ChiHpCha(1,INe3,ICH3))*MNeu(INe3))+CFF12S1L(INH3)*(MCha*
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(-FD111ChaHCha(4,INH3)+
     &  FD111ChaHpCha(4,ICH3)+FD111ChiHCha(4,INe3,INH3)-
     &  FD111ChiHpCha(4,INe3,ICH3))+DCONJG(CF11F12S3L(INe3,ICH3))*
     &  (FD111ChaHCha(2,INH3)-FD111ChaHCha(4,INH3)-
     &  FD111ChaHpCha(2,ICH3)+FD111ChaHpCha(4,ICH3)-
     &  FD111ChiHCha(2,INe3,INH3)+FD111ChiHCha(4,INe3,INH3)+
     &  FD111ChiHpCha(2,INe3,ICH3)-FD111ChiHpCha(4,INe3,ICH3))*
     &  MNeu(INe3)))+CF11F12S3L(INe3,2)*(CFF12S1L(INH3)*(MCha*
     &  DCONJG(CF11F12S3L(INe3,ICH3))*(FD111ChaHCha(2,INH3)-
     &  FD111ChaHCha(4,INH3)-FD111ChaHpCha(2,ICH3)+
     &  FD111ChaHpCha(4,ICH3)-FD111ChiHCha(2,INe3,INH3)+
     &  FD111ChiHCha(4,INe3,INH3)+FD111ChiHpCha(2,INe3,ICH3)-
     &  FD111ChiHpCha(4,INe3,ICH3))+MCha2*DCONJG(CF11F12S3R(INe3,ICH3))
     &  *(-FD111ChaHCha(1,INH3)+FD111ChaHpCha(1,ICH3)+
     &  FD111ChiHCha(1,INe3,INH3)-FD111ChiHpCha(1,INe3,ICH3))*
     &  MNeu(INe3))+CFF12S1R(INH3)*(MCha*DCONJG(CF11F12S3L(INe3,ICH3))*
     &  (-FD111ChaHCha(4,INH3)+FD111ChaHpCha(4,ICH3)+
     &  FD111ChiHCha(4,INe3,INH3)-FD111ChiHpCha(4,INe3,ICH3))+
     &  DCONJG(CF11F12S3R(INe3,ICH3))*(FD111ChaHCha(2,INH3)-
     &  FD111ChaHCha(4,INH3)-FD111ChaHpCha(2,ICH3)+
     &  FD111ChaHpCha(4,ICH3)-FD111ChiHCha(2,INe3,INH3)+
     &  FD111ChiHCha(4,INe3,INH3)+FD111ChiHpCha(2,INe3,ICH3)-
     &  FD111ChiHpCha(4,INe3,ICH3))*MNeu(INe3)))))/(8.*(Mah(INH3)-
     &  MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*(MCha-MNeu(INe3))*(MCha+MNeu(INe3)))

            else
             !need to consider
            endif
              endif
       enddo 
       enddo 
       enddo 
        
       do INe3=ineu,3
       do ISt3=1,2
           if(abs(MCha-MNeu(INe3)).gt.1D-10)then
       self=self+(3*CF12F3SC*CS3S13S14(ISt3)*(MCha*CF11F12S3L(INe3,2)*
     &  (CF11FSR(INe3,ISt3)*(-FD111ChaSTT(2,ISt3)+FD111ChaSTT(4,ISt3)+
     &  FD111ChiSBT(4,INe3)+FD111ChiStT(2,INe3,ISt3)-
     &  FD111ChiStT(4,INe3,ISt3)+FD111SBChaT(2)-FD111SBChaT(3)-
     &  FD111SBChiT(2,INe3))+M1*CF11FSL(INe3,ISt3)*(FD111ChaSTT(1,ISt3)
     &  -FD111ChiStT(1,INe3,ISt3)-FD111SBChaT(1)+FD111SBChiT(1,INe3))*
     &  MNeu(INe3))+CF11F12S3R(INe3,2)*(M1*CF11FSL(INe3,ISt3)*
     &  (FD111ChaSTT(4,ISt3)+FD111ChiSBT(4,INe3)-
     &  FD111ChiStT(4,INe3,ISt3)-FD111SBChaT(3))+CF11FSR(INe3,ISt3)*(-
     &  FD111ChaSTT(2,ISt3)+FD111ChaSTT(4,ISt3)+FD111ChiSBT(4,INe3)+
     &  FD111ChiStT(2,INe3,ISt3)-FD111ChiStT(4,INe3,ISt3)+
     &  FD111SBChaT(2)-FD111SBChaT(3)-FD111SBChiT(2,INe3))*MNeu(INe3)))
     &  )/(8.*(MSQ-MassSu(ISt3))*(MSQ+MassSu(ISt3))*(MCha-MNeu(INe3))*
     &  (MCha+MNeu(INe3)))
            else
             !need to consider
            endif
       enddo 
       enddo 

c 
c topo6-NoGBC
c
c 
c topo6-GBC
c
        
       do ICH3=1,2
       do ICH4=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
       do INH5=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
          if( Abs(Mah(INH4) - MassHpm(ICH4)) .gt. 1D-15) then 
       self=self+(CS1S1S1(INH3,INH4,INH5)*CS1S3S3(INH3,2,ICH3)*
     &  CS1S3S3(INH4,ICH4,2)*CS1S3S3(INH5,ICH3,ICH4)*
     &  (FD111HHH(1,INH3,INH4,INH5)-FD111HHpH(1,INH3,ICH4,INH5)-
     &  FD111HHpH(1,INH4,ICH3,INH5)+FD111HpHpH(1,ICH3,ICH4,INH5)))
     &  /(16.*(Mah(INH3)-MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*
     &  (Mah(INH4)-MassHpm(ICH4))*(Mah(INH4)+MassHpm(ICH4)))
              endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 
 
        
        !finite
        
       do ICH3=1,2
       do ICH4=1,2
       do ICH5=1,2
       do INH3=ihiggs,6
       do INH4=ihiggs,6
          if( Abs(Mah(INH3) - MassHpm(ICH5)) .gt. 1D-15) then 
          if( Abs(Mah(INH4) - MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(CS1S3S3(INH3,2,ICH5)*CS1S3S3(INH3,ICH4,ICH3)*
     &  CS1S3S3(INH4,ICH3,2)*CS1S3S3(INH4,ICH5,ICH4)*
     &  (FD111HHpH(1,INH3,ICH4,INH4)-FD111HHpHp(1,INH3,ICH3,ICH4)-
     &  FD111HHpHp(1,INH4,ICH5,ICH4)+FD111HpHpHp(1,ICH3,ICH5,ICH4)))
     &  /(16.*(Mah(INH4)-MassHpm(ICH3))*(Mah(INH4)+MassHpm(ICH3))*
     &  (Mah(INH3)-MassHpm(ICH5))*(Mah(INH3)+MassHpm(ICH5)))
              endif
              endif
       enddo 
       enddo 
       enddo 
       enddo 
       enddo 

        !finite
        
       do ICH3=1,2
       do INH3=ihiggs,6
       do ISt3=1,2
       do ISt4=1,2
          if( Abs(Mah(INH3) - MassHpm(ICH3)) .gt. 1D-15) then 
       self=self+(-3*(CS1S3S3(INH3,2,ICH3)*CS3S13S(ICH3,ISt4)*
     &  CS3S14S13(ISt3)*CSSS(INH3,ISt3,ISt4)+CS1S3S3(INH3,ICH3,2)*
     &  CS3S13S14(ISt3)*CS3S13SC(ICH3,ISt4)*CSSS(INH3,ISt4,ISt3))*
     &  (FD111HpSTSB(1,ICH3,ISt4)-FD111HpSTST(1,ICH3,ISt3,ISt4)-
     &  FD111HSTSB(1,INH3,ISt4)+FD111HSTST(1,INH3,ISt3,ISt4)))/(16.*
     &  (Mah(INH3)-MassHpm(ICH3))*(Mah(INH3)+MassHpm(ICH3))*(MSQ-
     &  MassSu(ISt3))*(MSQ+MassSu(ISt3)))
              endif
       enddo 
       enddo 
       enddo 
       enddo 
c 
c topo7-NoGBC
c
      self=self*fac+selfCT*fac1
       SCH=self
#ifdef printMT4
      Write(16,110) "TwoLoopChargedSelfNMSSM[Sum][Nhung]="
      write(16,111) self
110   format(A)   
111   format(F24.14)
#endif
      end subroutine CHself_atlambda
c
c======================================================================
c
c
c**********************************************************************
c
      subroutine counterM_atlambda(CHself,dtad2MT4,dMH2MT4,dv42)
      implicit none
      double precision CHself,dMHp2MT4,dtad2MT4(6),dMH2MT4(6,6),dv42
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision ImAkappaDR,ImAlambdaDR,ReAkappaDR,ReAlambdaDR 
      double precision phiIL,beta,dReAlambda
      double precision selfCHatat,dMHp2atat
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /MHpmatat/selfCHatat,dMHp2atat
#include "TLMT4.h"
#include "TLMT4lambda.h"
      logical MHpmInput,CPcon,mQrun,onelswitch
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch


      phiIl=phis+phil+phiu
      beta=atan(TB)
      dMHG1L(3) =(-(SB**2*dtad1MT4(1,3)) + CB*SB*dtad1MT4(2,3) + 
     -    IC*(CB**2*dtad1MT4(4,3) + SB**2*dtad1MT4(4,3)))/(SB*vv)
      dMHp2MT4 =CHself-dZCHiggs4(2,2)*dMHp1MT4(3)-dMHG1L(3)*dZCHiggs4(2,1)

      if(MHpmInput) then
      dReAlambda=(dv4(2)*(SB*Cos(phiIL)*
     -        (2*SB**3*dtad1MT4(1,2) + 2*CB**3*dtad1MT4(2,2) + 
     -          abslambda**2*CB*SB*vv2*dv4(2)) - 
     -       2*dtad1MT4(4,2)*Sin(phiIL)))/(2.*SB*vv2*abs(MUE)) + 
     -  (CB*SB**2*vv*Cos(phiIL)*dMHp2MT4 + 
     -     SB*Cos(phiIL)*(abslambda**2*CB*dv42*SB*vv2 - 
     -        SB**3*dtad2MT4(1) - CB**3*dtad2MT4(2)) + 
     -     dtad2MT4(4)*Sin(phiIL))/(SB*vv*abs(MUE))

        ! a factor of 1/2 need to be justify

      ReAlambdaDR=ReAlambdaDR+dReAlambda

      ImAlambdaDR=ImAlambdaDR+(dtad2MT4(4))/(SB*vv*Abs(MUE)*Cos(phiIL))
     -         - (dReAlambda)*Tan(phiIL)+
     -         (-dtad1MT4(4,2)*dv4(2))/(SB*vv**2*Abs(MUE)*cos(phiIL))

      else
      selfCHatat=dMHp2MT4
      dMHp2MT4=(dv4(2)*(-2*Cos(phiIL)*SB*
     -        (SB**3*dtad1MT4(1,2) + CB**3*dtad1MT4(2,2)) - 
     -       abslambda**2*CB*Cos(phiIL)*SB**2*vv**2*dv4(2) + 
     -       2*dtad1MT4(4,2)*Sin(phiIL)))/
     -   (2.*CB*Cos(phiIL)*SB**2*vv**2) + 
     -  (-2*abslambda**2*CB*Cos(phiIL)*dv42*SB**2*vv**2 + 
     -     2*Cos(phiIL)*SB*(SB**3*dtad2MT4(1) + CB**3*dtad2MT4(2)) - 
     -     2*dtad2MT4(4)*Sin(phiIL))/(2.*CB*Cos(phiIL)*SB**2*vv)
c Nhung corrected the missing piece here
      dMHp2atat=dMHp2MT4
c end Nhung corrected the missing piece here
      ImAlambdaDR=ImAlambdaDR+(dtad2MT4(4))/(SB*vv*Abs(MUE)*Cos(phiIL))+
     -         (-dtad1MT4(4,2)*dv4(2))/(SB*vv**2*Abs(MUE)*cos(phiIL))
      endif


      if(abskappa.gt.1D-14)then
      ImAkappaDR=ImAkappaDR+(abslambda**2*(abslambda*CB*vv*dtad2MT4(4) - 
     -  abs(MUE)*(6*abskappa*CB*dv42*SB*Sinphiy*vv*abs(MUE) + 
     -  sqrt2*dtad2MT4(6))))/(2.*abskappa*Cosk3s*abs(MUE)**3) - 
     -  (abslambda**3*CB*dv4(2)*
     -     ((4*abskappa*SB*Sinphiy*sqrt2*abs(MUE)**2*dv4(2))/
     -     abslambda + Cos(phiIL)*sqrt2*(-2*dtad1MT4(4,2) + 
     -     (2*abskappa*SB*Sinphiy*abs(MUE)**2*dv4(2))/
     -           abslambda)/cos(phiIL)))/
     -   (4.*Sqrt2*abskappa*Cosk3s*abs(MUE)**3)
      else
      ImAkappaDR=ImAkappaDR
      endif

#ifdef printMT4
      Write(*,*) "______charged Higgs CT finite___________"
      write(*,*)'dMHp2MT4=',dMHp2MT4
#endif   


c Higgs mass CT matrix
       dMH2MT4(1:6,1:6)=0D0
       dMH2MT4(1,1)=dMH2MT4(1,1)+(2*CB**3*SB*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(2,3)*(-((CB**5*dTanB4)/vv)+(CB2*SB*
     &  dv4(1))/vv**2)+dtad1MT4(1,3)*((CB**4*SB*dTanB4)/vv-(CB*
     &  (CB2+2*SB2)*dv4(1))/vv**2)-(CB*(CB2+2*SB2)*dtad1MT4(1,2)*dv4(2)
     &  )/vv**2+(CB2*SB*dtad1MT4(2,2)*dv4(2))/vv**2+(la2*SB2*dv4(2)**2)
     &  /2.-(CB*(CB2+2*SB2)*dtad1MT4(1,1)*dv4(3))/vv**2+(CB2*SB*
     &  dtad1MT4(2,1)*dv4(3))/vv**2+(2*abslambda*SB2*vv*dabsla14(1)+2*
     &  CB**3*la2*SB*vv*dTanB4)*dv4(3)+la2*SB2*dv4(1)*dv4(3))
       dMH2MT4(1,2)=dMH2MT4(1,2)+(CB2*(-CB2+SB2)*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(2,3)*(-((CB**4*SB*dTanB4)/vv)-(CB**
     &  3*dv4(1))/vv**2)+dtad1MT4(1,3)*((CB**3*SB2*dTanB4)/vv-
     &  (SB**3*dv4(1))/vv**2)-(SB**3*dtad1MT4(1,2)*dv4(2))/vv**2-(CB**
     &  3*dtad1MT4(2,2)*dv4(2))/vv**2+(CB*la2*SB*dv4(2)**2)/2.-(SB**3*
     &  dtad1MT4(1,1)*dv4(3))/vv**2-(CB**3*dtad1MT4(2,1)*dv4(3))/vv**2+
     &  (2*abslambda*CB*SB*vv*dabsla14(1)+CB2*la2*(CB2-SB2)*vv*
     &  dTanB4)*dv4(3)+CB*la2*SB*dv4(1)*dv4(3))
       dMH2MT4(1,3)=dMH2MT4(1,3)+((CB2*SB*(-2*CB2+SB2)*vv*
     &  dMHp1MT4(3)*dTanB4)/vS-(CB*SB2*dMHp1MT4(2)*dv4(2))/vS-(3*
     &  CB*la2*SB2*vv*dv4(2)**2)/(2.*vS)+dv4(1)*(-((CB*SB2*dMHp1MT4(3))
     &  /vS)-(3*CB*la2*SB2*vv*dv4(3))/vS)+(CB*SB2*vv*dMHp1MT4(3)*
     &  dvS14(1))/vS**2+dtad1MT4(2,3)*((CB**4*(CB2-SB2)*dTanB4)
     &  /vS-(CB**3*SB*dvS14(1))/vS**2)+dtad1MT4(1,3)*((2*CB**3*SB**3*
     &  dTanB4)/vS-(SB**4*dvS14(1))/vS**2)+dv4(3)*(-(abslambda*
     &  Cosphiy*SB*vS*dabska14(1))/2.+(-(abskappa*Cosphiy*SB*vS)/2.+
     &  abslambda*CB*(2*vS-(3*SB2*vv2)/vS))*dabsla14(1)-(CB*SB2*
     &  dMHp1MT4(1))/vS-(CB2*(-2*MHp2*SB**3+abskappa*abslambda*CB**5*
     &  Cosphiy*vS**2+abskappa*abslambda*CB*Cosphiy*SB**4*vS**2+2*
     &  abskappa*abslambda*CB**3*Cosphiy*SB2*vS**2+4*CB2*SB*(MHp2+la2*
     &  SB2*vS**2)+6*CB**6*la2*SB*vv2+la2*SB**5*(2*vS**2-3*SB2*vv2)+CB*
     &  *4*la2*SB*(2*vS**2+9*SB2*vv2))*dTanB4)/(2.*vS)+(-
     &  (abskappa*abslambda*Cosphiy*SB)/2.+(CB*MHp2*SB2)/vS**2+la2*(CB+
     &  (3*CB*SB2*vv2)/(2.*vS**2)))*dvS14(1)))
       dMH2MT4(1,4)=0+dMH2MT4(1,4)
       dMH2MT4(1,5)=dMH2MT4(1,5)+(dtad1MT4(4,3)*(-((CB**3*
     &  dTanB4)/(SB**2*vv))-dv4(1)/(SB*vv**2))-(dtad1MT4(4,2)*
     &  dv4(2))/(SB*vv**2)-(dtad1MT4(4,1)*dv4(3))/(SB*vv**2))
       dMH2MT4(1,6)=dMH2MT4(1,6)+(-((dtad1MT4(4,3)*dvS14(1))/vS**2)
     &  +dv4(3)*((-3*abslambda*SB*Sinphiy*vS*dabska14(1))/2.-(3*
     &  abskappa*SB*Sinphiy*vS*dabsla14(1))/2.-(3*abskappa*abslambda*
     &  CB**3*Sinphiy*vS*dTanB4)/2.-(3*abskappa*abslambda*SB*
     &  Sinphiy*dvS14(1))/2.))
       dMH2MT4(2,2)=dMH2MT4(2,2)+(-2*CB**3*SB*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(1,3)*((CB2*SB**3*dTanB4)/vv+(CB*
     &  SB2*dv4(1))/vv**2)+dtad1MT4(2,3)*(-((CB**3*SB2*dTanB4)
     &  /vv)-(SB*(2*CB2+SB2)*dv4(1))/vv**2)+(CB*SB2*dtad1MT4(1,2)*
     &  dv4(2))/vv**2-(SB*(2*CB2+SB2)*dtad1MT4(2,2)*dv4(2))/vv**2+(CB2*
     &  la2*dv4(2)**2)/2.+(CB*SB2*dtad1MT4(1,1)*dv4(3))/vv**2-(SB*(2*
     &  CB2+SB2)*dtad1MT4(2,1)*dv4(3))/vv**2+(2*abslambda*CB2*vv*
     &  dabsla14(1)-2*CB**3*la2*SB*vv*dTanB4)*dv4(3)+CB2*la2*
     &  dv4(1)*dv4(3))
       dMH2MT4(2,3)=dMH2MT4(2,3)+(-((CB**3*(CB2-2*SB2)*vv*
     &  dMHp1MT4(3)*dTanB4)/vS)-(CB2*SB*dMHp1MT4(2)*dv4(2))/vS-
     &  (3*CB2*la2*SB*vv*dv4(2)**2)/(2.*vS)+dv4(1)*(-((CB2*SB*
     &  dMHp1MT4(3))/vS)-(3*CB2*la2*SB*vv*dv4(3))/vS)+(CB2*SB*vv*
     &  dMHp1MT4(3)*dvS14(1))/vS**2+dtad1MT4(2,3)*((-2*CB**5*SB*
     &  dTanB4)/vS-(CB**4*dvS14(1))/vS**2)+dtad1MT4(1,3)*((CB2*
     &  (CB2-SB2)*SB2*dTanB4)/vS-(CB*SB**3*dvS14(1))/vS**2)+
     &  dv4(3)*(-(abslambda*CB*Cosphiy*vS*dabska14(1))/2.+(-(abskappa*
     &  CB*Cosphiy*vS)/2.+2*abslambda*SB*vS-(3*abslambda*CB2*SB*vv2)
     &  /vS)*dabsla14(1)-(CB2*SB*dMHp1MT4(1))/vS+(CB2*(-2*CB**5*MHp2+
     &  abskappa*abslambda*Cosphiy*SB*vS**2+CB**3*(2*MHp2*SB2-3*la2*
     &  vv2)+CB*(4*MHp2*SB**4+2*la2*(vS**2+3*SB2*vv2)))*dTanB4)
     &  /(2.*vS)+(-(abskappa*abslambda*CB*Cosphiy)/2.+(CB2*MHp2*SB)/vS*
     &  *2+la2*(SB+(3*CB2*SB*vv2)/(2.*vS**2)))*dvS14(1)))
       dMH2MT4(2,4)=dMH2MT4(2,4)+(dtad1MT4(4,3)*(-((CB**3*
     &  dTanB4)/(SB**2*vv))-dv4(1)/(SB*vv**2))-(dtad1MT4(4,2)*
     &  dv4(2))/(SB*vv**2)-(dtad1MT4(4,1)*dv4(3))/(SB*vv**2))
       dMH2MT4(2,5)=0+dMH2MT4(2,5)
       dMH2MT4(2,6)=dMH2MT4(2,6)+(dv4(3)*((-3*abslambda*CB*Sinphiy*
     &  vS*dabska14(1))/2.-(3*abskappa*CB*Sinphiy*vS*dabsla14(1))/2.+
     &  (3*abskappa*abslambda*CB2*SB*Sinphiy*vS*dTanB4)/2.-(3*
     &  abskappa*abslambda*CB*Sinphiy*dvS14(1))/2.)+dtad1MT4(4,3)*(-
     &  ((CB2*dTanB4)/(SB**2*vS))-(CB*dvS14(1))/(SB*vS**2)))
       dMH2MT4(3,3)=dMH2MT4(3,3)+((2*CB**3*SB*(CB2-SB2)*vv2*
     &  dMHp1MT4(3)*dTanB4)/vS**2+(2*CB2*SB2*vv*dMHp1MT4(2)*
     &  dv4(2))/vS**2-(CB*SB**4*dtad1MT4(1,2)*dv4(2))/vS**2-(CB**4*SB*
     &  dtad1MT4(2,2)*dv4(2))/vS**2-(CB*Sink3s*dtad1MT4(4,2)*dv4(2))
     &  /(Cosk3s*vS**2)+(CB*SB*(abskappa*abslambda*(-Cosphiy+(3*Sink3s*
     &  Sinphiy)/Cosk3s)+(2*CB*SB*(MHp2+3*la2*vv2))/vS**2)*dv4(2)**2)
     &  /2.-(CB*SB**4*dtad1MT4(1,1)*dv4(3))/vS**2-(CB**4*SB*
     &  dtad1MT4(2,1)*dv4(3))/vS**2-(CB*Sink3s*dtad1MT4(4,1)*dv4(3))
     &  /(Cosk3s*vS**2)+dv4(1)*((2*CB2*SB2*vv*dMHp1MT4(3))/vS**2+CB*SB*
     &  (abskappa*abslambda*(-Cosphiy+(3*Sink3s*Sinphiy)/Cosk3s)+(2*CB*
     &  SB*(MHp2+3*la2*vv2))/vS**2)*dv4(3))-(2*CB2*SB2*vv2*dMHp1MT4(3)*
     &  dvS14(1))/vS**3-(dtad1MT4(3,3)*dvS14(1))/vS**2-(Sink3s*
     &  dtad1MT4(6,3)*dvS14(1))/(Cosk3s*vS**2)+dtad1MT4(2,3)*(-((CB**5*
     &  (CB2-2*SB2)*vv*dTanB4)/vS**2)-(CB**4*SB*dv4(1))/vS**2+(2*
     &  CB**4*SB*vv*dvS14(1))/vS**3)+dtad1MT4(1,3)*((CB2*SB**3*(-2*CB2+
     &  SB2)*vv*dTanB4)/vS**2-(CB*SB**4*dv4(1))/vS**2+(2*CB*SB**
     &  4*vv*dvS14(1))/vS**3)+dtad1MT4(4,3)*((CB2*SB*Sink3s*vv*
     &  dTanB4)/(Cosk3s*vS**2)-(CB*Sink3s*dv4(1))/(Cosk3s*vS**2)+
     &  (2*CB*Sink3s*vv*dvS14(1))/(Cosk3s*vS**3))+dv4(3)*((abslambda*
     &  CB*SB*(-(Cosk3s*Cosphiy)+3*Sink3s*Sinphiy)*vv*dabska14(1))
     &  /Cosk3s+CB*SB*vv*(-(abskappa*Cosphiy)+(3*abskappa*Sink3s*
     &  Sinphiy)/Cosk3s+(4*abslambda*CB*SB*vv2)/vS**2)*dabsla14(1)+(2*
     &  CB2*SB2*vv*dMHp1MT4(1))/vS**2+(CB2*(CB2-SB2)*vv*(-2*abskappa*
     &  abslambda*CB2*SB2*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vS**2+
     &  abskappa*abslambda*CB**4*(-(Cosk3s*Cosphiy)+3*Sink3s*Sinphiy)*
     &  vS**2+abskappa*abslambda*SB**4*(-(Cosk3s*Cosphiy)+3*Sink3s*
     &  Sinphiy)*vS**2+4*CB**5*Cosk3s*la2*SB*vv2+8*CB**3*Cosk3s*la2*SB*
     &  *3*vv2+4*CB*Cosk3s*SB*(MHp2+la2*SB**4*vv2))*dTanB4)
     &  /(Cosk3s*vS**2)-(4*CB2*SB2*vv*(MHp2+la2*vv2)*dvS14(1))/vS**3))
       dMH2MT4(3,4)=dMH2MT4(3,4)+(-((dtad1MT4(4,3)*dvS14(1))/vS**2)
     &  +dv4(3)*((abslambda*SB*Sinphiy*vS*dabska14(1))/2.+(abskappa*SB*
     &  Sinphiy*vS*dabsla14(1))/2.+(abskappa*abslambda*CB**3*Sinphiy*
     &  vS*dTanB4)/2.+(abskappa*abslambda*SB*Sinphiy*dvS14(1))
     &  /2.))
       dMH2MT4(3,5)=dMH2MT4(3,5)+(dv4(3)*((abslambda*CB*Sinphiy*vS*
     &  dabska14(1))/2.+(abskappa*CB*Sinphiy*vS*dabsla14(1))/2.-
     &  (abskappa*abslambda*CB2*SB*Sinphiy*vS*dTanB4)/2.+
     &  (abskappa*abslambda*CB*Sinphiy*dvS14(1))/2.)+dtad1MT4(4,3)*(-
     &  ((CB2*dTanB4)/(SB**2*vS))-(CB*dvS14(1))/(SB*vS**2)))
       dMH2MT4(3,6)=dMH2MT4(3,6)+((-2*CB*dtad1MT4(4,2)*dv4(2))/vS**
     &  2+2*abskappa*abslambda*CB*SB*Sinphiy*dv4(2)**2-(2*CB*
     &  dtad1MT4(4,1)*dv4(3))/vS**2+(4*abslambda*CB*SB*Sinphiy*vv*
     &  dabska14(1)+4*abskappa*CB*SB*Sinphiy*vv*dabsla14(1)+4*abskappa*
     &  abslambda*CB2*(CB2-SB2)*Sinphiy*vv*dTanB4)*dv4(3)+4*
     &  abskappa*abslambda*CB*SB*Sinphiy*dv4(1)*dv4(3)-(2*dtad1MT4(6,3)
     &  *dvS14(1))/vS**2+dtad1MT4(4,3)*((2*CB2*SB*vv*dTanB4)/vS**
     &  2-(2*CB*dv4(1))/vS**2+(4*CB*vv*dvS14(1))/vS**3))
       dMH2MT4(4,4)=dMH2MT4(4,4)+(2*CB**3*SB*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(2,3)*(-((CB**5*dTanB4)/vv)+(CB2*SB*
     &  dv4(1))/vv**2)+dtad1MT4(1,3)*((CB**4*SB*dTanB4)/vv-(CB*
     &  (CB2+2*SB2)*dv4(1))/vv**2)-(CB*(CB2+2*SB2)*dtad1MT4(1,2)*dv4(2)
     &  )/vv**2+(CB2*SB*dtad1MT4(2,2)*dv4(2))/vv**2+(la2*SB2*dv4(2)**2)
     &  /2.-(CB*(CB2+2*SB2)*dtad1MT4(1,1)*dv4(3))/vv**2+(CB2*SB*
     &  dtad1MT4(2,1)*dv4(3))/vv**2+(2*abslambda*SB2*vv*dabsla14(1)+2*
     &  CB**3*la2*SB*vv*dTanB4)*dv4(3)+la2*SB2*dv4(1)*dv4(3))
       dMH2MT4(4,5)=dMH2MT4(4,5)+(CB2*(CB2-SB2)*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(2,3)*((CB**4*SB*dTanB4)/vv+(CB**3*
     &  dv4(1))/vv**2)+dtad1MT4(1,3)*(-((CB**3*SB2*dTanB4)/vv)+
     &  (SB**3*dv4(1))/vv**2)+(SB**3*dtad1MT4(1,2)*dv4(2))/vv**2+(CB**
     &  3*dtad1MT4(2,2)*dv4(2))/vv**2+(CB*la2*SB*dv4(2)**2)/2.+(SB**3*
     &  dtad1MT4(1,1)*dv4(3))/vv**2+(CB**3*dtad1MT4(2,1)*dv4(3))/vv**2+
     &  (2*abslambda*CB*SB*vv*dabsla14(1)+CB2*la2*(CB2-SB2)*vv*
     &  dTanB4)*dv4(3)+CB*la2*SB*dv4(1)*dv4(3))
       dMH2MT4(4,6)=dMH2MT4(4,6)+((CB2*SB*(2*CB2-SB2)*vv*
     &  dMHp1MT4(3)*dTanB4)/vS+(CB*SB2*dMHp1MT4(2)*dv4(2))/vS+(3*
     &  CB*la2*SB2*vv*dv4(2)**2)/(2.*vS)+dv4(1)*((CB*SB2*dMHp1MT4(3))
     &  /vS+(3*CB*la2*SB2*vv*dv4(3))/vS)-(CB*SB2*vv*dMHp1MT4(3)*
     &  dvS14(1))/vS**2+dtad1MT4(2,3)*((CB**4*(-CB2+SB2)*dTanB4)
     &  /vS+(CB**3*SB*dvS14(1))/vS**2)+dtad1MT4(1,3)*((-2*CB**3*SB**3*
     &  dTanB4)/vS+(SB**4*dvS14(1))/vS**2)+dv4(3)*((-3*abslambda*
     &  Cosphiy*SB*vS*dabska14(1))/2.+((-3*abskappa*Cosphiy*SB*vS)/2.+
     &  (3*abslambda*CB*SB2*vv2)/vS)*dabsla14(1)+(CB*SB2*dMHp1MT4(1))
     &  /vS+(CB2*(4*CB2*MHp2*SB-2*MHp2*SB**3-3*abskappa*abslambda*CB**
     &  5*Cosphiy*vS**2-3*abskappa*abslambda*CB*Cosphiy*SB**4*vS**2-6*
     &  abskappa*abslambda*CB**3*Cosphiy*SB2*vS**2+6*CB**6*la2*SB*vv2+
     &  9*CB**4*la2*SB**3*vv2-3*la2*SB**7*vv2)*dTanB4)/(2.*vS)+
     &  (SB*(-3*abskappa*abslambda*Cosphiy-(CB*SB*(2*MHp2+3*la2*vv2))
     &  /vS**2)*dvS14(1))/2.))
       dMH2MT4(5,5)=dMH2MT4(5,5)+(-2*CB**3*SB*dMHp1MT4(3)*
     &  dTanB4+dtad1MT4(1,3)*((CB2*SB**3*dTanB4)/vv+(CB*
     &  SB2*dv4(1))/vv**2)+dtad1MT4(2,3)*(-((CB**3*SB2*dTanB4)
     &  /vv)-(SB*(2*CB2+SB2)*dv4(1))/vv**2)+(CB*SB2*dtad1MT4(1,2)*
     &  dv4(2))/vv**2-(SB*(2*CB2+SB2)*dtad1MT4(2,2)*dv4(2))/vv**2+(CB2*
     &  la2*dv4(2)**2)/2.+(CB*SB2*dtad1MT4(1,1)*dv4(3))/vv**2-(SB*(2*
     &  CB2+SB2)*dtad1MT4(2,1)*dv4(3))/vv**2+(2*abslambda*CB2*vv*
     &  dabsla14(1)-2*CB**3*la2*SB*vv*dTanB4)*dv4(3)+CB2*la2*
     &  dv4(1)*dv4(3))
       dMH2MT4(5,6)=dMH2MT4(5,6)+((CB**3*(CB2-2*SB2)*vv*dMHp1MT4(3)
     &  *dTanB4)/vS+(CB2*SB*dMHp1MT4(2)*dv4(2))/vS+(3*CB2*la2*SB*
     &  vv*dv4(2)**2)/(2.*vS)+dv4(1)*((CB2*SB*dMHp1MT4(3))/vS+(3*CB2*
     &  la2*SB*vv*dv4(3))/vS)-(CB2*SB*vv*dMHp1MT4(3)*dvS14(1))/vS**2+
     &  dtad1MT4(2,3)*((2*CB**5*SB*dTanB4)/vS+(CB**4*dvS14(1))
     &  /vS**2)+dtad1MT4(1,3)*((CB2*SB2*(-CB2+SB2)*dTanB4)/vS+
     &  (CB*SB**3*dvS14(1))/vS**2)+dv4(3)*((-3*abslambda*CB*Cosphiy*vS*
     &  dabska14(1))/2.+((-3*abskappa*CB*Cosphiy*vS)/2.+(3*abslambda*
     &  CB2*SB*vv2)/vS)*dabsla14(1)+(CB2*SB*dMHp1MT4(1))/vS+(CB2*(3*
     &  abskappa*abslambda*CB**4*Cosphiy*SB*vS**2+6*abskappa*abslambda*
     &  CB2*Cosphiy*SB**3*vS**2+3*abskappa*abslambda*Cosphiy*SB**5*vS**
     &  2+3*CB**7*la2*vv2+CB**3*(2*MHp2-9*la2*SB**4*vv2)-2*CB*(2*MHp2*
     &  SB2+3*la2*SB**6*vv2))*dTanB4)/(2.*vS)+(CB*(-3*abskappa*
     &  abslambda*Cosphiy-(CB*SB*(2*MHp2+3*la2*vv2))/vS**2)*dvS14(1))
     &  /2.))
       dMH2MT4(6,6)=dMH2MT4(6,6)+((2*CB**3*SB*(CB2-SB2)*vv2*
     &  dMHp1MT4(3)*dTanB4)/vS**2+(2*CB2*SB2*vv*dMHp1MT4(2)*
     &  dv4(2))/vS**2-(CB*SB**4*dtad1MT4(1,2)*dv4(2))/vS**2-(CB**4*SB*
     &  dtad1MT4(2,2)*dv4(2))/vS**2+(3*CB*Sink3s*dtad1MT4(4,2)*dv4(2))
     &  /(Cosk3s*vS**2)+(CB*SB*(3*abskappa*abslambda*(Cosphiy-(3*
     &  Sink3s*Sinphiy)/Cosk3s)+(2*CB*SB*(MHp2+3*la2*vv2))/vS**2)*
     &  dv4(2)**2)/2.-(CB*SB**4*dtad1MT4(1,1)*dv4(3))/vS**2-(CB**4*SB*
     &  dtad1MT4(2,1)*dv4(3))/vS**2+(3*CB*Sink3s*dtad1MT4(4,1)*dv4(3))
     &  /(Cosk3s*vS**2)+dv4(1)*((2*CB2*SB2*vv*dMHp1MT4(3))/vS**2+CB*SB*
     &  (3*abskappa*abslambda*(Cosphiy-(3*Sink3s*Sinphiy)/Cosk3s)+(2*
     &  CB*SB*(MHp2+3*la2*vv2))/vS**2)*dv4(3))-(2*CB2*SB2*vv2*
     &  dMHp1MT4(3)*dvS14(1))/vS**3-(dtad1MT4(3,3)*dvS14(1))/vS**2+(3*
     &  Sink3s*dtad1MT4(6,3)*dvS14(1))/(Cosk3s*vS**2)+dtad1MT4(2,3)*(-
     &  ((CB**5*(CB2-2*SB2)*vv*dTanB4)/vS**2)-(CB**4*SB*dv4(1))
     &  /vS**2+(2*CB**4*SB*vv*dvS14(1))/vS**3)+dtad1MT4(1,3)*((CB2*SB**
     &  3*(-2*CB2+SB2)*vv*dTanB4)/vS**2-(CB*SB**4*dv4(1))/vS**2+
     &  (2*CB*SB**4*vv*dvS14(1))/vS**3)+dtad1MT4(4,3)*((-3*CB2*SB*
     &  Sink3s*vv*dTanB4)/(Cosk3s*vS**2)+(3*CB*Sink3s*dv4(1))
     &  /(Cosk3s*vS**2)-(6*CB*Sink3s*vv*dvS14(1))/(Cosk3s*vS**3))+
     &  dv4(3)*((3*abslambda*CB*SB*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*
     &  vv*dabska14(1))/Cosk3s+CB*SB*vv*(3*abskappa*Cosphiy-(9*
     &  abskappa*Sink3s*Sinphiy)/Cosk3s+(4*abslambda*CB*SB*vv2)/vS**2)*
     &  dabsla14(1)+(2*CB2*SB2*vv*dMHp1MT4(1))/vS**2+(CB2*(CB2-SB2)*vv*
     &  (3*abskappa*abslambda*CB**4*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*
     &  vS**2+3*abskappa*abslambda*SB**4*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vS**2+6*abskappa*abslambda*CB2*SB2*(Cosk3s*Cosphiy-3*
     &  Sink3s*Sinphiy)*vS**2+4*CB**5*Cosk3s*la2*SB*vv2+8*CB**3*Cosk3s*
     &  la2*SB**3*vv2+4*CB*Cosk3s*SB*(MHp2+la2*SB**4*vv2))*dTanB4
     &  )/(Cosk3s*vS**2)-(4*CB2*SB2*vv*(MHp2+la2*vv2)*dvS14(1))/vS**3))
     &  
c
c============= 
c

       dMH2MT4(1,1)=dMH2MT4(1,1)+(SB2*dMHp1MT4(3)+((CB**3+2*CB*SB2)
     &  *dtad1MT4(1,3))/vv-(CB2*SB*dtad1MT4(2,3))/vv+(la2*SB2*vv2*
     &  dv4(3))/vv)*Zfactor1LIN(1,1)
       dMH2MT4(1,2)=dMH2MT4(1,2)+(-(CB*dZHu4*SB*dMHp1MT4(3))/2.+
     &  (dZHu4*SB**3*dtad1MT4(1,3))/(2.*vv)+(CB**3*dZHu4*dtad1MT4(2,3))
     &  /(2.*vv)+(CB*dZHu4*la2*SB*vv2*dv4(3))/(2.*vv)+(-(CB*SB*
     &  dMHp1MT4(3))/2.+(SB**3*dtad1MT4(1,3))/(2.*vv)+(CB**3*
     &  dtad1MT4(2,3))/(2.*vv)+(CB*la2*SB*vv2*dv4(3))/(2.*vv))*
     &  Zfactor1LIN(1,1)+(-(CB*SB*dMHp1MT4(3))/2.+(SB**3*dtad1MT4(1,3))
     &  /(2.*vv)+(CB**3*dtad1MT4(2,3))/(2.*vv)+(CB*la2*SB*vv2*dv4(3))
     &  /(2.*vv))*Zfactor1LIN(2,2))
       dMH2MT4(1,3)=dMH2MT4(1,3)+((-(CB*SB**2*vv*dMHp1MT4(3))/(2.*
     &  vS)+(SB**4*dtad1MT4(1,3))/(2.*vS)+(CB**3*SB*dtad1MT4(2,3))/(2.*
     &  vS)-((2*CB*MHp2*SB**2-2*CB*la2*vS**2+abskappa*abslambda*
     &  Cosphiy*SB*vS**2+3*CB*la2*SB2*vv2)*dv4(3))/(4.*vS))*
     &  Zfactor1LIN(1,1)+(-(CB*SB**2*vv*dMHp1MT4(3))/(2.*vS)+(SB**4*
     &  dtad1MT4(1,3))/(2.*vS)+(CB**3*SB*dtad1MT4(2,3))/(2.*vS)-((2*CB*
     &  MHp2*SB**2-2*CB*la2*vS**2+abskappa*abslambda*Cosphiy*SB*vS**2+
     &  3*CB*la2*SB2*vv2)*dv4(3))/(4.*vS))*Zfactor1LIN(3,3))
       dMH2MT4(1,4)=0+dMH2MT4(1,4)
       dMH2MT4(1,5)=dMH2MT4(1,5)+((dZHu4*dtad1MT4(4,3))/(2.*SB*vv)+
     &  (dtad1MT4(4,3)*Zfactor1LIN(1,1))/(2.*SB*vv)+(dtad1MT4(4,3)*
     &  Zfactor1LIN(2,2))/(2.*SB*vv))
       dMH2MT4(1,6)=dMH2MT4(1,6)+((dtad1MT4(4,3)/(2.*vS)-(3*
     &  abskappa*abslambda*SB*Sinphiy*vS*dv4(3))/4.)*Zfactor1LIN(1,1)+
     &  (dtad1MT4(4,3)/(2.*vS)-(3*abskappa*abslambda*SB*Sinphiy*vS*
     &  dv4(3))/4.)*Zfactor1LIN(3,3))
       dMH2MT4(2,2)=dMH2MT4(2,2)+(CB2*dZHu4*dMHp1MT4(3)-(CB*dZHu4*
     &  SB2*dtad1MT4(1,3))/vv+(dZHu4*SB*(2*CB2+SB**2)*dtad1MT4(2,3))
     &  /vv+(CB2*dZHu4*la2*vv2*dv4(3))/vv+(CB2*dMHp1MT4(3)-(CB*SB2*
     &  dtad1MT4(1,3))/vv+((2*CB2*SB+SB**3)*dtad1MT4(2,3))/vv+(CB2*la2*
     &  vv2*dv4(3))/vv)*Zfactor1LIN(2,2))
       dMH2MT4(2,3)=dMH2MT4(2,3)+(-(CB**2*dZHu4*SB*vv*dMHp1MT4(3))
     &  /(2.*vS)+(CB*dZHu4*SB**3*dtad1MT4(1,3))/(2.*vS)+(CB**4*dZHu4*
     &  dtad1MT4(2,3))/(2.*vS)-(dZHu4*(2*CB**2*MHp2*SB+abskappa*
     &  abslambda*CB*Cosphiy*vS**2+la2*SB*(-2*vS**2+3*CB2*vv2))*dv4(3))
     &  /(4.*vS)+(-(CB**2*SB*vv*dMHp1MT4(3))/(2.*vS)+(CB*SB**3*
     &  dtad1MT4(1,3))/(2.*vS)+(CB**4*dtad1MT4(2,3))/(2.*vS)-((2*CB**2*
     &  MHp2*SB+abskappa*abslambda*CB*Cosphiy*vS**2-2*la2*SB*vS**2+3*
     &  CB2*la2*SB*vv2)*dv4(3))/(4.*vS))*Zfactor1LIN(2,2)+(-(CB**2*SB*
     &  vv*dMHp1MT4(3))/(2.*vS)+(CB*SB**3*dtad1MT4(1,3))/(2.*vS)+(CB**
     &  4*dtad1MT4(2,3))/(2.*vS)-((2*CB**2*MHp2*SB+abskappa*abslambda*
     &  CB*Cosphiy*vS**2-2*la2*SB*vS**2+3*CB2*la2*SB*vv2)*dv4(3))/(4.*
     &  vS))*Zfactor1LIN(3,3))
       dMH2MT4(2,4)=dMH2MT4(2,4)+((dZHu4*dtad1MT4(4,3))/(2.*SB*vv)+
     &  (dtad1MT4(4,3)*Zfactor1LIN(1,1))/(2.*SB*vv)+(dtad1MT4(4,3)*
     &  Zfactor1LIN(2,2))/(2.*SB*vv))
       dMH2MT4(2,5)=0+dMH2MT4(2,5)
       dMH2MT4(2,6)=dMH2MT4(2,6)+((CB*dZHu4*dtad1MT4(4,3))/(2.*SB*
     &  vS)-(3*abskappa*abslambda*CB*dZHu4*Sinphiy*vS*dv4(3))/4.+((CB*
     &  dtad1MT4(4,3))/(2.*SB*vS)-(3*abskappa*abslambda*CB*Sinphiy*vS*
     &  dv4(3))/4.)*Zfactor1LIN(2,2)+((CB*dtad1MT4(4,3))/(2.*SB*vS)-(3*
     &  abskappa*abslambda*CB*Sinphiy*vS*dv4(3))/4.)*Zfactor1LIN(3,3))
       dMH2MT4(3,3)=dMH2MT4(3,3)+((CB2*SB2*vv2*dMHp1MT4(3))/vS**2-
     &  (CB*SB**4*vv*dtad1MT4(1,3))/vS**2-(CB**4*SB*vv*dtad1MT4(2,3))
     &  /vS**2+dtad1MT4(3,3)/vS-(CB*Sink3s*vv*dtad1MT4(4,3))/(Cosk3s*
     &  vS**2)+(Sink3s*dtad1MT4(6,3))/(Cosk3s*vS)+(vv*(abskappa*
     &  abslambda*CB*SB*(-(Cosk3s*Cosphiy)+3*Sink3s*Sinphiy)*vS**2+2*
     &  CB2*Cosk3s*SB2*(MHp2+la2*vv**2))*dv4(3))/(Cosk3s*vS**2))*
     &  Zfactor1LIN(3,3)
       dMH2MT4(3,4)=dMH2MT4(3,4)+((dtad1MT4(4,3)/(2.*vS)+(abskappa*
     &  abslambda*SB*Sinphiy*vS*dv4(3))/4.)*Zfactor1LIN(1,1)+
     &  (dtad1MT4(4,3)/(2.*vS)+(abskappa*abslambda*SB*Sinphiy*vS*dv4(3)
     &  )/4.)*Zfactor1LIN(3,3))
       dMH2MT4(3,5)=dMH2MT4(3,5)+((CB*dZHu4*dtad1MT4(4,3))/(2.*SB*
     &  vS)+(abskappa*abslambda*CB*dZHu4*Sinphiy*vS*dv4(3))/4.+((CB*
     &  dtad1MT4(4,3))/(2.*SB*vS)+(abskappa*abslambda*CB*Sinphiy*vS*
     &  dv4(3))/4.)*Zfactor1LIN(2,2)+((CB*dtad1MT4(4,3))/(2.*SB*vS)+
     &  (abskappa*abslambda*CB*Sinphiy*vS*dv4(3))/4.)*Zfactor1LIN(3,3))
       dMH2MT4(3,6)=dMH2MT4(3,6)+((-2*CB*vv*dtad1MT4(4,3))/vS**2+
     &  (2*dtad1MT4(6,3))/vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*
     &  dv4(3))*Zfactor1LIN(3,3)
       dMH2MT4(4,4)=dMH2MT4(4,4)+(SB2*dMHp1MT4(3)+((CB**3+2*CB*SB2)
     &  *dtad1MT4(1,3))/vv-(CB2*SB*dtad1MT4(2,3))/vv+(la2*SB2*vv2*
     &  dv4(3))/vv)*Zfactor1LIN(1,1)
       dMH2MT4(4,5)=dMH2MT4(4,5)+((CB*dZHu4*SB*dMHp1MT4(3))/2.-
     &  (dZHu4*SB**3*dtad1MT4(1,3))/(2.*vv)-(CB**3*dZHu4*dtad1MT4(2,3))
     &  /(2.*vv)+(CB*dZHu4*la2*SB*vv2*dv4(3))/(2.*vv)+((CB*SB*
     &  dMHp1MT4(3))/2.-(SB**3*dtad1MT4(1,3))/(2.*vv)-(CB**3*
     &  dtad1MT4(2,3))/(2.*vv)+(CB*la2*SB*vv2*dv4(3))/(2.*vv))*
     &  Zfactor1LIN(1,1)+((CB*SB*dMHp1MT4(3))/2.-(SB**3*dtad1MT4(1,3))
     &  /(2.*vv)-(CB**3*dtad1MT4(2,3))/(2.*vv)+(CB*la2*SB*vv2*dv4(3))
     &  /(2.*vv))*Zfactor1LIN(2,2))
       dMH2MT4(4,6)=dMH2MT4(4,6)+(((CB*SB**2*vv*dMHp1MT4(3))/(2.*
     &  vS)-(SB**4*dtad1MT4(1,3))/(2.*vS)-(CB**3*SB*dtad1MT4(2,3))/(2.*
     &  vS)+((2*CB*MHp2*SB**2-3*abskappa*abslambda*Cosphiy*SB*vS**2+3*
     &  CB*la2*SB2*vv2)*dv4(3))/(4.*vS))*Zfactor1LIN(1,1)+((CB*SB**2*
     &  vv*dMHp1MT4(3))/(2.*vS)-(SB**4*dtad1MT4(1,3))/(2.*vS)-(CB**3*
     &  SB*dtad1MT4(2,3))/(2.*vS)+((2*CB*MHp2*SB**2-3*abskappa*
     &  abslambda*Cosphiy*SB*vS**2+3*CB*la2*SB2*vv2)*dv4(3))/(4.*vS))*
     &  Zfactor1LIN(3,3))
       dMH2MT4(5,5)=dMH2MT4(5,5)+(CB2*dZHu4*dMHp1MT4(3)-(CB*dZHu4*
     &  SB2*dtad1MT4(1,3))/vv+(dZHu4*SB*(2*CB2+SB**2)*dtad1MT4(2,3))
     &  /vv+(CB2*dZHu4*la2*vv2*dv4(3))/vv+(CB2*dMHp1MT4(3)-(CB*SB2*
     &  dtad1MT4(1,3))/vv+((2*CB2*SB+SB**3)*dtad1MT4(2,3))/vv+(CB2*la2*
     &  vv2*dv4(3))/vv)*Zfactor1LIN(2,2))
       dMH2MT4(5,6)=dMH2MT4(5,6)+((CB**2*dZHu4*SB*vv*dMHp1MT4(3))
     &  /(2.*vS)-(CB*dZHu4*SB**3*dtad1MT4(1,3))/(2.*vS)-(CB**4*dZHu4*
     &  dtad1MT4(2,3))/(2.*vS)+(CB*dZHu4*(-3*abskappa*abslambda*
     &  Cosphiy*vS**2+CB*SB*(2*MHp2+3*la2*vv2))*dv4(3))/(4.*vS)+((CB**
     &  2*SB*vv*dMHp1MT4(3))/(2.*vS)-(CB*SB**3*dtad1MT4(1,3))/(2.*vS)-
     &  (CB**4*dtad1MT4(2,3))/(2.*vS)+(CB*(-3*abskappa*abslambda*
     &  Cosphiy*vS**2+CB*SB*(2*MHp2+3*la2*vv2))*dv4(3))/(4.*vS))*
     &  Zfactor1LIN(2,2)+((CB**2*SB*vv*dMHp1MT4(3))/(2.*vS)-(CB*SB**3*
     &  dtad1MT4(1,3))/(2.*vS)-(CB**4*dtad1MT4(2,3))/(2.*vS)+(CB*(-3*
     &  abskappa*abslambda*Cosphiy*vS**2+CB*SB*(2*MHp2+3*la2*vv2))*
     &  dv4(3))/(4.*vS))*Zfactor1LIN(3,3))
       dMH2MT4(6,6)=dMH2MT4(6,6)+((CB2*SB2*vv2*dMHp1MT4(3))/vS**2-
     &  (CB*SB**4*vv*dtad1MT4(1,3))/vS**2-(CB**4*SB*vv*dtad1MT4(2,3))
     &  /vS**2+dtad1MT4(3,3)/vS+(3*CB*Sink3s*vv*dtad1MT4(4,3))/(Cosk3s*
     &  vS**2)-(3*Sink3s*dtad1MT4(6,3))/(Cosk3s*vS)+(vv*(3*abskappa*
     &  abslambda*CB*SB*(Cosk3s*Cosphiy-3*Sink3s*Sinphiy)*vS**2+2*CB2*
     &  Cosk3s*SB2*(MHp2+la2*vv**2))*dv4(3))/(Cosk3s*vS**2))*
     &  Zfactor1LIN(3,3)
c
c============= finite
c

       dMH2MT4(1,1)=dMH2MT4(1,1)+(SB2*dMHp2MT4+((2*CB**3+4*CB*
     &  SB2)*dtad2MT4(1))/(2.*vv)-(CB2*SB*dtad2MT4(2))/vv+(-(2*CB2*
     &  la2*SB**4*vv**3+CB**4*la2*SB2*vv**3+SB2*vv*(2*MHp2+la2*SB**4*
     &  vv2))/(2.*vv**2)+(6*CB2*la2*SB**4*vv2+3*CB**4*la2*SB2*vv2+SB*
     &  (2*la2*SB**5*vv2+SB*(2*MHp2+la2*SB**4*vv2)))/(2.*vv))*dv42)

       dMH2MT4(1,2)=dMH2MT4(1,2)+(-(CB*SB*dMHp2MT4)+(SB**3*
     &  dtad2MT4(1))/vv+(CB**3*dtad2MT4(2))/vv+(-(2*CB**3*la2*SB**
     &  3*vv**3+CB*la2*SB**5*vv**3+CB*SB*vv*(-2*MHp2+CB**4*la2*vv2))
     &  /(2.*vv**2)+(6*CB**3*la2*SB**3*vv2+3*CB*la2*SB**5*vv2+SB*(2*CB*
     &  *5*la2*vv2+CB*(-2*MHp2+CB**4*la2*vv2)))/(2.*vv))*dv42)
       dMH2MT4(1,3)=dMH2MT4(1,3)+(-((CB*SB2*vv*dMHp2MT4)/vS)+
     &  (SB**4*dtad2MT4(1))/vS+(CB**3*SB*dtad2MT4(2))/vS+((2*CB**5*
     &  la2*vS**2+4*CB**3*la2*SB2*vS**2-abskappa*abslambda*Cosphiy*SB*
     &  (CB2+SB2)**2*vS**2-6*CB**3*la2*SB**4*vv2-3*CB*la2*SB**6*vv2+
     &  SB2*(2*CB*la2*SB2*vS**2-2*CB**5*la2*vv2-CB*(2*MHp2+CB**4*la2*
     &  vv2)))*dv42)/(2.*vS))
       dMH2MT4(1,4)=0+dMH2MT4(1,4)
       dMH2MT4(1,5)=dMH2MT4(1,5)+dtad2MT4(4)/(SB*vv)
       dMH2MT4(1,6)=dMH2MT4(1,6)+(dtad2MT4(4)/vS-(3*abskappa*
     &  abslambda*SB*Sinphiy*vS*dv42)/2.)
       dMH2MT4(2,2)=dMH2MT4(2,2)+(CB2*dMHp2MT4-(CB*SB2*
     &  dtad2MT4(1))/vv+((4*CB2*SB+2*SB**3)*dtad2MT4(2))/(2.*vv)+
     &  ((3*CB**6*la2*vv2+2*CB2*la2*SB**4*vv2+6*CB**4*la2*SB2*vv2+CB2*
     &  (2*MHp2+la2*SB**4*vv2))/(2.*vv)-(CB**6*la2*vv**3+2*CB**4*la2*
     &  SB2*vv**3+CB2*vv*(2*MHp2+la2*SB**4*vv2))/(2.*vv**2))*dv42)
       dMH2MT4(2,3)=dMH2MT4(2,3)+(-((CB2*SB*vv*dMHp2MT4)/vS)+
     &  (CB*SB**3*dtad2MT4(1))/vS+(CB**4*dtad2MT4(2))/vS-((-4*CB2*
     &  la2*SB**3*vS**2-2*la2*SB**5*vS**2+abskappa*abslambda*CB*
     &  Cosphiy*(CB2+SB2)**2*vS**2+3*CB**6*la2*SB*vv2+6*CB**4*la2*SB**
     &  3*vv2+CB2*(-2*CB2*la2*SB*vS**2+2*la2*SB**5*vv2+SB*(2*MHp2+la2*
     &  SB**4*vv2)))*dv42)/(2.*vS))
       dMH2MT4(2,4)=dMH2MT4(2,4)+dtad2MT4(4)/(SB*vv)
       dMH2MT4(2,5)=0+dMH2MT4(2,5)
       dMH2MT4(2,6)=dMH2MT4(2,6)+((CB*dtad2MT4(4))/(SB*vS)-(3*
     &  abskappa*abslambda*CB*Sinphiy*vS*dv42)/2.)
       dMH2MT4(3,3)=dMH2MT4(3,3)+((CB2*SB2*vv2*dMHp2MT4)/vS**2-
     &  (CB*SB**4*vv*dtad2MT4(1))/vS**2-(CB**4*SB*vv*dtad2MT4(2))
     &  /vS**2+((2*CB**4*vS+2*SB**4*vS+4*CB2*SB2*vS)*dtad2MT4(3))
     &  /(2.*vS**2)-(CB*Sink3s*vv*dtad2MT4(4))/(Cosk3s*vS**2)+
     &  (Sink3s*dtad2MT4(6))/(Cosk3s*vS)+((4*CB2*MHp2*SB2*vv-(2*
     &  abskappa*abslambda*CB*SB*(CB2+SB2)**2*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vS**2*vv)/Cosk3s+8*CB**4*la2*SB**4*vv**3+4*CB2*la2*SB*
     &  *6*vv**3+4*CB**6*la2*SB2*vv**3)*dv42)/(2.*vS**2))
       dMH2MT4(3,4)=dMH2MT4(3,4)+(dtad2MT4(4)/vS+(abskappa*
     &  abslambda*SB*Sinphiy*vS*dv42)/2.)
       dMH2MT4(3,5)=dMH2MT4(3,5)+((CB*dtad2MT4(4))/(SB*vS)+
     &  (abskappa*abslambda*CB*Sinphiy*vS*dv42)/2.)
       dMH2MT4(3,6)=dMH2MT4(3,6)+((-2*CB*vv*dtad2MT4(4))/vS**2+
     &  (2*dtad2MT4(6))/vS+4*abskappa*abslambda*CB*SB*Sinphiy*vv*
     &  dv42)
       dMH2MT4(4,4)=dMH2MT4(4,4)+(SB2*dMHp2MT4+((2*CB**3+4*CB*
     &  SB2)*dtad2MT4(1))/(2.*vv)-(CB2*SB*dtad2MT4(2))/vv+(-(2*CB2*
     &  la2*SB**4*vv**3+CB**4*la2*SB2*vv**3+SB2*vv*(2*MHp2+la2*SB**4*
     &  vv2))/(2.*vv**2)+(6*CB2*la2*SB**4*vv2+3*CB**4*la2*SB2*vv2+SB*
     &  (2*la2*SB**5*vv2+SB*(2*MHp2+la2*SB**4*vv2)))/(2.*vv))*dv42)
       dMH2MT4(4,5)=dMH2MT4(4,5)+(CB*SB*dMHp2MT4-(SB**3*
     &  dtad2MT4(1))/vv-(CB**3*dtad2MT4(2))/vv+(-(2*CB**3*la2*SB**
     &  3*vv**3+CB*la2*SB**5*vv**3+CB*SB*vv*(2*MHp2+CB**4*la2*vv2))
     &  /(2.*vv**2)+(6*CB**3*la2*SB**3*vv2+3*CB*la2*SB**5*vv2+SB*(2*CB*
     &  *5*la2*vv2+CB*(2*MHp2+CB**4*la2*vv2)))/(2.*vv))*dv42)
       dMH2MT4(4,6)=dMH2MT4(4,6)+((CB*SB2*vv*dMHp2MT4)/vS-(SB**
     &  4*dtad2MT4(1))/vS-(CB**3*SB*dtad2MT4(2))/vS+(SB*(-3*
     &  abskappa*abslambda*Cosphiy*(CB2+SB2)**2*vS**2+6*CB**3*la2*SB**
     &  3*vv2+3*CB*la2*SB**5*vv2+SB*(2*CB**5*la2*vv2+CB*(2*MHp2+CB**4*
     &  la2*vv2)))*dv42)/(2.*vS))
       dMH2MT4(5,5)=dMH2MT4(5,5)+(CB2*dMHp2MT4-(CB*SB2*
     &  dtad2MT4(1))/vv+((4*CB2*SB+2*SB**3)*dtad2MT4(2))/(2.*vv)+
     &  ((3*CB**6*la2*vv2+2*CB2*la2*SB**4*vv2+6*CB**4*la2*SB2*vv2+CB2*
     &  (2*MHp2+la2*SB**4*vv2))/(2.*vv)-(CB**6*la2*vv**3+2*CB**4*la2*
     &  SB2*vv**3+CB2*vv*(2*MHp2+la2*SB**4*vv2))/(2.*vv**2))*dv42)
       dMH2MT4(5,6)=dMH2MT4(5,6)+((CB2*SB*vv*dMHp2MT4)/vS-(CB*
     &  SB**3*dtad2MT4(1))/vS-(CB**4*dtad2MT4(2))/vS+(CB*(-3*
     &  abskappa*abslambda*Cosphiy*(CB2+SB2)**2*vS**2+6*CB**3*la2*SB**
     &  3*vv2+3*CB*la2*SB**5*vv2+SB*(2*CB**5*la2*vv2+CB*(2*MHp2+CB**4*
     &  la2*vv2)))*dv42)/(2.*vS))
       dMH2MT4(6,6)=dMH2MT4(6,6)+((CB2*SB2*vv2*dMHp2MT4)/vS**2-
     &  (CB*SB**4*vv*dtad2MT4(1))/vS**2-(CB**4*SB*vv*dtad2MT4(2))
     &  /vS**2+((2*CB**4*vS+2*SB**4*vS+4*CB2*SB2*vS)*dtad2MT4(3))
     &  /(2.*vS**2)+(3*CB*Sink3s*vv*dtad2MT4(4))/(Cosk3s*vS**2)-(3*
     &  Sink3s*dtad2MT4(6))/(Cosk3s*vS)+((4*CB2*MHp2*SB2*vv+(6*
     &  abskappa*abslambda*CB*SB*(CB2+SB2)**2*(Cosk3s*Cosphiy-3*Sink3s*
     &  Sinphiy)*vS**2*vv)/Cosk3s+8*CB**4*la2*SB**4*vv**3+4*CB2*la2*SB*
     &  *6*vv**3+4*CB**6*la2*SB2*vv**3)*dv42)/(2.*vS**2))

      end subroutine counterM_atlambda
